<?php
namespace CustomerPortal\Repositories;


class Call extends \Doctrine\ORM\EntityRepository {

    public function findMyCalls($userId, $type, $whereFilters = '1=1') {
        $query = "SELECT C.ID, C.REFERENCE, C.CLASSIFICATION, C.REPORTED_BY, C.DESCRIPTION, TO_CHAR(C.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                       C.PRODUCT, C.TYPE, C.CALL_RELATIONSHIP, C.CAUSE, C.UPDATED, C.IS_ACTIVE, TO_CHAR(C.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT
                FROM CALL C
                LEFT JOIN CALL_LABEL_RELATION CLR ON  CLR.CALL = C.ID
                WHERE (C.REPORTED_BY = :USER_ID AND C.TYPE = :CALL_TYPE) AND (".$whereFilters.")";

        $groupBy = " GROUP BY C.ID, C.REFERENCE, C.CLASSIFICATION, C.REPORTED_BY, C.DESCRIPTION, C.UPDATED_AT,
                    C.PRODUCT, C.TYPE, C.CALL_RELATIONSHIP, C.CAUSE, C.UPDATED, C.IS_ACTIVE, C.CREATED_AT";

        $orderBy = " ORDER BY C.ID";

        $params = array(
            "USER_ID" => $userId,
            "CALL_TYPE" => $type
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query . $groupBy . $orderBy, $params);
    }

    public function findMyIssueTypeProductCalls($userId, $whereFilters = '1=1') {
        return $this->findMyCalls($userId, 'I', $whereFilters);
    }

    public function findMyRichClientTypeProductCalls($userId, $whereFilters = '1=1') {
        return $this->findMyCalls($userId, 'S', $whereFilters);
    }

    public function findMySupportCalls($userId, $whereFilters = "1=1"){
        $query = "SELECT C.ID, C.REFERENCE, C.CLASSIFICATION, C.REPORTED_BY, C.DESCRIPTION, C.PRODUCT, C.CAUSE, C.IS_ACTIVE,
                       C.CALL_RELATIONSHIP,  C.TYPE, S.NMSISTEMA AS PRODUCT_NAME, MOTI.NMMOTICONT AS CAUSE_NAME,
                       TO_CHAR(C.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
                       TO_CHAR(C.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                       TO_CHAR(MAX(EVCONT.DTHREVOLCONT), 'DD/MM/YYYY') AS UPDATED
                FROM CALL C
                JOIN SISTEMA S
                  ON S.CDSISTEMA = C.PRODUCT
                JOIN MOTICONT MOTI
                  ON MOTI.CDMOTICONT = C.CAUSE
                JOIN CONTATO CONT
                  ON CONT.CDCASO = C.CALL_RELATIONSHIP
                LEFT JOIN EVOLCONT EVCONT
                  ON EVCONT.NRCONTATO = CONT.NRCONTATO AND EVCONT.CDOPEREVOL <> C.REPORTED_BY
                LEFT JOIN CALL_LABEL_RELATION CLR
                  ON CLR.CALL = C.ID
                WHERE C.REPORTED_BY = :USER_ID AND C.TYPE = :CALL_TYPE AND (". $whereFilters .")";

        $groupBy = " GROUP BY C.ID, C.REFERENCE, C.CLASSIFICATION, C.REPORTED_BY, C.DESCRIPTION, C.PRODUCT, C.CAUSE, C.IS_ACTIVE,
                         C.CALL_RELATIONSHIP, C.TYPE, S.NMSISTEMA, MOTI.NMMOTICONT, C.CREATED_AT, C.UPDATED_AT";

        $orderBy = " ORDER BY C.ID";

        $params = array(
            "USER_ID" => $userId,
            "CALL_TYPE" => 'C'
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query . $groupBy . $orderBy, $params);
    }

    public function findCall($id){
        $query = "SELECT ID, REFERENCE, CLASSIFICATION, REPORTED_BY, DESCRIPTION, TO_CHAR(UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                       PRODUCT, C.TYPE, CALL_RELATIONSHIP, CAUSE, UPDATED, IS_ACTIVE,
                       (
                          SELECT LISTAGG(CLR.LABEL, ',') WITHIN GROUP (ORDER BY CLR.ID)
                          FROM CALL_LABEL_RELATION CLR
                          WHERE CLR.CALL = C.ID
                       ) AS CALL_LABELS
                FROM CALL C
                WHERE C.ID = :CALL_ID";

        $params = array(
            "CALL_ID" => $id
        );

        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }

    public function findSupportCallCause($cdOperador, $referenceProduct){
        $query = "
            SELECT DISTINCT * FROM (
                SELECT MO.CDMOTICONT AS ID,
                        MO.NMMOTICONT AS NAME,
                        MO.NRORG      AS NRORG
                FROM MOTICONT MO
                INNER JOIN CRM_PERFIMOTICONT PE ON MO.CDMOTICONT = PE.CDMOTICONT AND MO.NRORG = PE.NRORG
                INNER JOIN CRM_PERFIOPER PO ON PO.NRSEQPERFATIV = PE.NRSEQPERFATIV  AND PO.NRORG = PE.NRORG
                                            AND PO.CDOPERADOR = :CDOPERADOR
                WHERE MO.IDATIVO = 'S'
                AND EXISTS (
                  SELECT *
                  FROM CRM_RELACIONAMOTIVOSISTEMA C
                  WHERE MO.CDMOTICONT = C.CDMOTICONT AND C.CDSISTEMA = :CDSISTEMA
                )
                UNION ALL
                    SELECT MO.CDMOTICONT AS ID,
                           MO.NMMOTICONT AS NAME,
                           MO.NRORG      AS NRORG
                    FROM MOTICONT MO
                    INNER JOIN CRM_PERFIMOTICONT PE ON MO.CDMOTICONT = PE.CDMOTICONT AND MO.NRORG = PE.NRORG
                    INNER JOIN CRM_PERFIOPER PO ON PO.NRSEQPERFATIV = PE.NRSEQPERFATIV AND PO.NRORG = PE.NRORG
                                                AND PO.CDOPERADOR = :CDOPERADOR
                    WHERE MO.IDATIVO = 'S'
                    AND NOT EXISTS (
                      SELECT *
                      FROM CRM_RELACIONAMOTIVOSISTEMA C
                      WHERE C.CDSISTEMA = :CDSISTEMA
                    )
                UNION ALL
                    SELECT MO.CDMOTICONT AS ID,
                           MO.NMMOTICONT AS NAME,
                           MO.NRORG      AS NRORG
                    FROM MOTICONT MO
                    INNER JOIN CONTATO CO ON MO.CDMOTICONT = CO.CDMOTICONT AND MO.NRORG = CO.NRORG
                    WHERE CO.CDCASO = ''
            )";

        $params = array(
            "CDOPERADOR" => $cdOperador,
            "CDSISTEMA" => $referenceProduct
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findSupportCallReferenceProducts($cdOperador, $nrOrg){
        $query = "
            SELECT DISTINCT * FROM (
                SELECT SI.CDSISTEMA AS ID,
                        SI.NMSISTEMA AS NAME,
                        SI.NRORG AS NRORG
                FROM SISTEMA SI, CRM_PERFISISTEMA PE, CRM_PERFIOPER PO
                WHERE SI.CDSISTEMA = PE.CDSISTEMA
                    AND SI.NRORG = PE.NRORG
                    AND PO.NRSEQPERFATIV = PE.NRSEQPERFATIV
                    AND PO.NRORG = PE.NRORG
                    AND PO.CDOPERADOR = :CDOPERADOR
                    AND SI.NRORG = :NRORG
                    AND SI.IDATIVO = 'S'
                    AND SI.IDTECNOSIST = 'WEB'
                    AND SI.IDTIPLINPSIST = 'PROD'
                UNION ALL
                    SELECT SI.CDSISTEMA AS ID,
                           SI.NMSISTEMA AS NAME,
                           SI.NRORG     AS NRORG
                    FROM SISTEMA SI
                    INNER JOIN CONTATO CO ON SI.CDSISTEMA = CO.CDSISTEMA AND SI.NRORG = CO.NRORG
                    WHERE CO.CDCASO = ''
            )
            ORDER BY NAME";

        $params = array(
            'CDOPERADOR'    => $cdOperador,
            'NRORG'         => $nrOrg
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function countCallsByType($userId) {
        $query = "
            SELECT C.TYPE AS TYPE, COUNT(C.ID) AS QUANTITY
            FROM CALL C
            WHERE C.REPORTED_BY = :USER_ID
            GROUP BY C.TYPE
            ORDER BY C.TYPE
        ";

        $params = array(
            "USER_ID" => $userId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function countCallsByLabel($userId) {
        $query = "
            SELECT NVL(CL.DESCRIPTION, 'Sem Rotulo') AS LABEL, COUNT(C.ID) AS QUANTITY
            FROM CALL C
            LEFT JOIN CALL_LABEL_RELATION CLR ON CLR.CALL = C.ID
            LEFT JOIN CALL_LABEL CL ON CLR.LABEL = CL.ID
            WHERE C.REPORTED_BY = :USER_ID
            GROUP BY CL.DESCRIPTION
            ORDER BY CL.DESCRIPTION
        ";

        $params = array(
            "USER_ID" => $userId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}