<?php
namespace Teknisa\Logic\SIT;

use Geral\Util\DateUtil;
use Planejamento\Entities\Nutrprod;
use Planejamento\Entities\Grupnutr;
use Geral\Entities\Organizacao;
use Setforms2\Shared\Registry;
use Teknisa\Entities\SitProdutoint;
use Teknisa\Logic\SIT\Exception\ExceptionDePara;
use Geral\Entities\Operador;
use Geral\Util\Ambiente;
use Geral\Util\AcumuladorDeMensagens;
use Geral\Util\Mensagem;
use Geral\Entities\Produto;
use \Teknisa\Entities\SitLogtek;

class Depara
{
    const NRORG = '1';
    
    /** Numero maximo de retorno de produtos com  nutrientes para
     * evitar exceder memoria.
     */
    const NUMMAXRETORNO = '100';


    public function recebeListaDe(Request $request, Response $response){
       	$manager = \Setforms2\Shared\Registry::getEntityManager();

		$con = $manager->getConnection();
		$con->beginTransaction();

        try {
            $prodsInt = $request->getData();
            if (empty($prodsInt)) {
                ExceptionDePara::erroListaDeNaoRecon();
            }

            $nrorgInt = $prodsInt['NRORGINT'];
            $this->validaOrganizacao($nrorgInt);

            $qtdProdEnviados = $prodsInt['QTPRODENV'];
            $qtdProdProc = count($prodsInt['PRODUTOS']);
            if ($qtdProdEnviados <> $qtdProdProc) {
                ExceptionDePara::erroQtdProdRecebidos(array($qtdProdEnviados, $qtdProdProc));
            }

            $remessa  = $prodsInt['NRREMESSA'];
            $objDePara = SitProdutoint::getObjectByWhere(
                'NRORGINT = ? AND NRORG = ? AND NRREMESSA = ?',
                array($nrorgInt, self::NRORG, $remessa)
            );

            if (!$objDePara) {
                if ($qtdProdEnviados > 0) {
                    /** @var $buscaOperAleat Operador */
                    $buscaOperAleat = Operador::getObjectByWhere('NRORG = ?', array(self::NRORG));
                    $operador = $buscaOperAleat->getCdoperador();

                    foreach ($prodsInt['PRODUTOS'] as $prod) {
                        $novoProduto = new SitProdutoint();
                        try {
                            $novoProduto->insereProdutoDe(
                                $nrorgInt, $remessa, $prod['CDPRODUTO'],
                                utf8_decode($prod['NMPRODUTO']), $prod['SGUNIDADE'],
                                $prod['VRPESOUNID'], $prod['OBSERVACAO'],
                                $prod['DATA'], $operador, self::NRORG
                            );

                            $manager->flush();
                        } catch (\Exception $e) {
                            return var_dump($prod);
                        }
                    }
                }
            }

            $this->verificaProdutosLog($nrorgInt, $remessa);
            $manager->flush();

            $this->obtemEventuaisAtualizacoes($nrorgInt, $remessa, $qtdProdProc, $response);
            $con->commit();
            
            return true;
        } catch (\Exception $e) {
            $con->rollBack();
            throw $e;
        }
    }


    private function obtemAtualizacoes(SitProdutoint $objSitProduto, &$arrayResult){
    	$produto = $objSitProduto->getCdproduto();
    	
        /** @var $objProdNutri NutrProd[] */
        $objProdNutri = Nutrprod::getObjectsByWhere(
            'CDPRODUTO = ? AND NRORG = ?',
            array($produto, self::NRORG),
            'CDNUTRIENTE'
        );

        $arrayResult['PRODUTOS'][] = array(
            'NRREMESSA'  => $objSitProduto->getNrremessa(),
            'CDPRODUTO'  => $objSitProduto->getCdprodutoint(),
        	'VRPESOUNID' => $objSitProduto->getQtpesoint(),
            'IDATIVO'    => $objSitProduto->getIdativo(),
        );

        // Se o produto tiver sido excluido nao processa os nutrientes
        if ($objSitProduto->getIdativo() === 'N') return false;

        foreach ($objProdNutri as $nutriente) { 
            $dtUltAtuNutri = now();
            
            $fator = $nutriente->getNutriente()->getVrfatornutr();
            if (empty($fator) === true) {
                $fator = 0;
            }
            
            $qtdNutr = $nutriente->getQtnutrprod();
            if (empty($qtdNutr) === true) {
                $qtdNutr = 0;
            }
            
            $calorias = $nutriente->getNutriente()->getIdcalorias();
            if (empty($calorias) === true) {
            	$calorias = 'N';
            }
            
            $nutrRelev = $nutriente->getNutriente()->getIdnutrrelev();
            if (empty($nutrRelev) === true) {
            	$nutrRelev = 'N';
            }      
                     
            $grupoNutri = $nutriente->getNutriente()->getCdgrupnutr();
            if (empty($grupoNutri) === false ) {
                $i = count($arrayResult['PRODUTOS'])-1;
	            //$fonte = utf8_encode($nutriente->getDsfontnutr());
            	$fonte = $nutriente->getDsfontnutr();
	            	            
	            /** $objGrupNutr Grupnutr */
	            $objGrupNutr = Grupnutr::getObject(array('nrorg' => $nutriente->getNrorg(), 'cdgrupnutr' => $grupoNutri));
	            $arrayResult['PRODUTOS'][$i][] = array(
                    'CDGRUPNUTR'  => $grupoNutri,
                    'NMGRUPNUTR'  => $objGrupNutr->getNmgrupnutr(),
	                'CDNUTRIENTE' => $nutriente->getNutriente()->getCdnutriente(),
	                'NMNUTRIENTE' => $nutriente->getNutriente()->getNmnutriente(),
	                'NMREDUZIDO'  => $nutriente->getNutriente()->getNmreduzido(),
	                'DTULTATUNUTR'=> $dtUltAtuNutri,
	                'DSFONTNUTR'  => $fonte,
	                'IDTIPONUTR'  => $nutriente->getNutriente()->getIdtiponutr(),
	                'SGUNIDADE'   => $nutriente->getNutriente()->getSgunidade(),
	                'IDCALORIAS'  => $calorias,
	                'IDNUTRRELEV' => $nutrRelev,
	                'VRFATORNUTR' => $fator,
	                'QTNUTRPROD'  => $qtdNutr,
	            );
            }
        }
    }
    
    
    public function obtemEventuaisAtualizacoes($nrorgInt, $remessa, $qtdProdProc, Response $response){
        /** @var $objSitProdInt SitProdutoint[] */
;        $objSitProdInt = SitProdutoint::getObjectsByWhere(
                'NRORGINT = ? AND DTLIBERACAO IS NOT NULL
                AND DTENVIO IS NULL AND ROWNUM <= ?',
                array($nrorgInt, self::NUMMAXRETORNO)
        );

        $arrayResult = array(
            'NRORG'     => $nrorgInt,
            'NRREMESSA' => $remessa,
            'QTPRODREC' => $qtdProdProc,
            'QTPRODRET' => count($objSitProdInt),
            'PRODUTOS'  => array(),
        );

        if ($objSitProdInt) {
            foreach ($objSitProdInt as $produtoInt) {
//                if (($this->obtemAtualizacoes($produtoInt, $arrayResult)) === false) continue;
                $this->obtemAtualizacoes($produtoInt, $arrayResult);

                $manager = \Setforms2\Shared\Registry::getEntityManager();
                $con = $manager->getConnection();
                $con->beginTransaction();

                try {
                    $produtoInt->atualizaEnvio(self::NRORG);
                    $manager->flush();

                    $this->geraLogRetorno($produtoInt, $remessa, $produtoInt->getNrremessa());
                    $manager->flush();

                    $con->commit();
                } catch (\Exception $e) {
                    $con->rollBack();
                    throw $e;
                }
            }
        }

        $response->setData($arrayResult);
        return true;
    }


    public static function verificaNutrientesPara($produto){
        /** @var $nutrProd NutrProd */
        $nutrProd = Nutrprod::getObjectByWhere(
            'CDPRODUTO = ? AND NRORG = ?',
            array($produto, self::NRORG)
        );

        if ($nutrProd instanceof Nutrprod) {
            return true;
        } else {
            return false;
        }
    }


    public static function geraLiberacaoPara($listaProdutos){
        try {
            $prodErro = array();
            foreach ($listaProdutos as $produto) {
                $validaNutr = self::verificaNutrientesPara($produto['CDPRODUTO']);
                if ($validaNutr === false) {
                    $prodErro[] = $produto;
                    continue;
                }
                
                /** @var $objsSitProd SitProdutoint[] */
                $objsSitProd = SitProdutoint::getObjectsByWhere(
                    'CDPRODUTO = ?',
                    array($produto['CDPRODUTO'])
                );

                foreach ($objsSitProd as $objSitProd) {
                    $con = Registry::getEntityManager();

                    $objSitProd->setDtliberacao(DateUtil::getDataAtual());
                    $objSitProd->setDtenvio(null);
                    $objSitProd->setDsobsint(null);
                    $objSitProd->persist();
                    $con->flush();
                }
            }

            return $prodErro;
        } catch (\Exception $e) {
            $con->rollback();
            return false;
        }
    }


    public static function validaOrganizacao($nrorgInt){
        /** @var $objOrg Organizacao */
        $objOrg = Organizacao::getObject(array('nrorg' => $nrorgInt));
        if ($objOrg instanceof Organizacao) {
            $ativo = $objOrg->getIdativo();

            if ($ativo === 'S') {
                return true;
            } 
        } 
        
        ExceptionDePara::erroOrgInvalidoDe();
    }
    
    
    public static function geraAssociacaoAutomatica($produtos, $produtoPara){		
    	$nrorg = Ambiente::getNrorg();
    	$operador = Ambiente::getCdOperador();
    	
    	// Prepara lista dos produtos De
    	$produtoDe = array();
    	foreach ($produtos as $listaProdutos) {
    		/** @var $prodDe \Teknisa\Entities\SitProdutoint */
    		$prodDe = SitProdutoint::getObjectByWhere(
    			'CDPRODUTOINT = ? AND NRORGINT = ? AND NRREMESSA = ?',
                array($listaProdutos['CDPRODUTOINT'], $listaProdutos['NRORGINT'], $listaProdutos['NRREMESSA'])
    		);

            //$produtoPara2 = $produtoPara;

    		// Verifica se o produto Para est� com nutriente
    		$nutriente = self::verificaNutrientesPara($produtoPara);
    		if ($nutriente === true) {
    			$dtLiberacao = DateUtil::getDataAtual();
            } else {
    			$dtLiberacao  = null;
                //$produtoPara2 = null;
    		}

            try {
                $con = Registry::getEntityManager();
                $prodDe->atualizaProdutoPara(
                    $produtoPara, $dtLiberacao, $nrorg,
                    $operador, $listaProdutos['QTPESOINT']
                );
                $con->flush();
            } catch (\Exception $e) {
                tk_warning($e->getMessage());
            }
    	}
    	
    	if ($dtLiberacao) {
    		AcumuladorDeMensagens::getInstance()->addMensagem('msg_sit00100_prodLiberadoNutri', array($produtoPara), Mensagem::VALIDACAO);
    	} else {
    		/** @var $objProd Produto */
    		$objProd = Produto::getObjectByWhere('CDPRODUTO = ? AND NRORG = ?', array($produtoPara, $nrorg));
    		AcumuladorDeMensagens::getInstance()->addMensagem('msg_sit00100_prodNaoLiberadoNutri', array($objProd->getNmproduto()), Mensagem::ERRO);
    	}
    }
    
    
    public function verificaProdutosLog($nrorgInt, $remessaInt){
        /** @var $objsLog SitLogtek []*/
        $objsLog = SitLogtek::getObjectsByWhere(
                'NRREMESSAINT = ? AND NRORGINT = ?',
                array($remessaInt, $nrorgInt)
        );

        if (count($objsLog) > 0) {
            foreach ($objsLog as $objLog) {
                /** @var $objDePara SitProdutoint */
                $objDePara = SitProdutoint::getObjectByWhere(
                    'NRORGINT = ? AND CDPRODUTOINT = ? AND NRREMESSA = ?',
                    array($nrorgInt, $objLog->getCdprodutoint(), $objLog->getNrremessaori())
                );

                $objDePara->atualizaEnvioPeloLog(self::NRORG);
                $objLog->delete();
            }
        }

    	return true;
    }  


    public function geraLogRetorno(SitProdutoint $produtoInt, $remessa, $remessaOri) {
        $objLog = new SitLogtek();
        $objLog->insereLog(
            $produtoInt->getNrorgint(), $remessa, $produtoInt->getCdprodutoint(), $remessaOri
        );

    	return true;
    }

    public static function deletaRegistro($nrorgint, $cdprodutoint, $nrremessa) {
        $reg = SitProdutoint::getObjectByWhere(
            "NRORGINT = ? AND CDPRODUTOINT = ? AND NRREMESSA = ?",
            array($nrorgint, $cdprodutoint, $nrremessa)
        );

        if (empty($reg) === false) {
            SitProdutoint::excluiRegistro($reg);
        }
    }

    public static function saveSitprodutoint(
        $state, $nrorg, $nrremessa, $nrorgint, $idativo, $cdproduto,
        $cdprodutoint, $nmprodutoint, $cdunidadeint, $qtpesoint, $dsobsint,
        $dtrecebimento, $dtliberacao, $dtenvio, $dtinclusao, $dtultatu,
        $cdoperinclusao, $cdoperultatu, $nrorginclusao, $nrorgultatu
    ) {
        $conn = Registry::getEntityManager();
        try {
            $conn->beginTransaction();
            if ($state == 'I') {
//                $obj = new SitProdutoint();
//                $obj->customSave(
//                    $nrorg, $nrremessa, $nrorgint, $idativo, $cdproduto,
//                    $cdprodutoint, $nmprodutoint, $cdunidadeint, $qtpesoint, $dsobsint,
//                    $dtrecebimento, $dtliberacao, $dtenvio, $dtinclusao, $dtultatu,
//                    $cdoperinclusao, $cdoperultatu, $nrorginclusao, $nrorgultatu
//                );
            } else if ($state == 'U') {
                $obj = SitProdutoint::getObjectByWhere(
                    "NRORG = ? AND CDPRODUTOINT = ? AND NRORGINT = ?",
                    array($nrorg, $cdprodutoint, $nrorgint)
                );
                if ($obj instanceof SitProdutoint) {
                    $obj->customUpdate(
                        $idativo, $cdproduto, $qtpesoint, $dtultatu,
                        $cdoperultatu, $nrorgultatu, $dtliberacao, $dtenvio
                    );
                }
            }
            $conn->flush();
            $conn->commit();
        }
        catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }
}
