CREATE TABLE RATFISNCM(
   CDCLASFISC      VARCHAR2(08)               NOT NULL ,
   CDRATEIOPADRAO  VARCHAR2(04)               NOT NULL ,
   IDENTRSAIDOP    VARCHAR2(01)               NOT NULL 
	  CONSTRAINT CKC_IDENTRSAIDOP_RATFISNCM CHECK (IDENTRSAIDOP IN ('E','S')),
   NRORG 	       NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO 	       VARCHAR2(1)   DEFAULT 'S' NOT NULL,
      CONSTRAINT CKC_IDATIVO_RATFISNCM CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO 	   DATE 	                  NULL,
   DTULTATU 	   DATE 	                  NULL,
   NRORGINCLUSAO   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERINCLUSAO  VARCHAR2(12)	 DEFAULT '1'  NULL,
   NRORGULTATU 	   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERULTATU    VARCHAR2(12)  DEFAULT '1',
   CONSTRAINT PK_RATFISNCM PRIMARY KEY (CDCLASFISC, CDRATEIOPADRAO, IDENTRSAIDOP)
)
/

CREATE INDEX FIS00837_FK ON RATFISNCM (CDRATEIOPADRAO ASC)
/

ALTER TABLE RATFISNCM
   ADD CONSTRAINT FKFIS00837_RATEIOPADRAOFIS FOREIGN KEY (CDRATEIOPADRAO)
       REFERENCES RATEIOPADRAOFIS (CDRATEIOPADRAO) ON DELETE CASCADE
/

CREATE INDEX FIS00838_FK ON RATFISNCM (CDCLASFISC ASC)
/
	   
ALTER TABLE RATFISNCM
   ADD CONSTRAINT FKFIS00838_CLASFISCAL FOREIGN KEY (CDCLASFISC)
      REFERENCES CLASFISCAL (CDCLASFISC)
/	

ALTER TABLE RATEIOPADRAOFIS ADD IDTPRATEIO VARCHAR2(01) DEFAULT 'N' NOT NULL 
   CONSTRAINT CKC_IDTPRATEIO_RATFISCST CHECK (IDTPRATEIO IN ('N','F')) 
/

CREATE TABLE FORMAJUSTFISC(
   CDFORMULA       VARCHAR2(10)              NOT NULL,
   NMFORMULA       VARCHAR2(20)              NOT NULL,
   DSFORMULA       VARCHAR2(200)             NOT NULL,
   CDCRITAJUSTE    VARCHAR2(04)              NOT NULL,
   NRORG 	       NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO 	       VARCHAR2(1)   DEFAULT 'S' NOT NULL,
      CONSTRAINT CKC_IDATIVO_FORMAJUSTFISC CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO 	   DATE 	                 NULL,
   DTULTATU 	   DATE 	                 NULL,
   NRORGINCLUSAO   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERINCLUSAO  VARCHAR2(12)	 DEFAULT '1' NULL,
   NRORGULTATU 	   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERULTATU    VARCHAR2(12)  DEFAULT '1',
   CONSTRAINT PK_FORMAJUSTFISC PRIMARY KEY (CDFORMULA)
)
/

CREATE INDEX FIS00839_FK ON FORMAJUSTFISC (CDCRITAJUSTE ASC)
/

ALTER TABLE FORMAJUSTFISC
   ADD CONSTRAINT FKFIS00839_RATEIOPADRAOFIS FOREIGN KEY (CDCRITAJUSTE)
      REFERENCES RATEIOPADRAOFIS (CDRATEIOPADRAO)
/

CREATE TABLE AJUSTFISCAUTO(
   CDAJUSTFISC     VARCHAR2(10)               NOT NULL,
   DSAJUSTFISC     VARCHAR2(200)              NULL,
   CDIMPOSTONFE    VARCHAR2(02)               NOT NULL,
   IDTPAJUSTE      VARCHAR2(03)  DEFAULT 'OC' NOT NULL
      CONSTRAINT CKC_IDTPAJUSTE_AJUSTFISCAUTO CHECK(IDTPAJUSTE IN ('DE','OD','DD','OC','ED','EC','VI')),
   IDTIPOCALC      VARCHAR2(01)  DEFAULT 'T'  NOT NULL
      CONSTRAINT CKC_IDTIPOCALC_AJUSTFISCAUTO CHECK (IDTIPOCALC IN ('T','D')), -- "Valor Total" = "T", "Valor por Documento"= "D"
   IDTIPOCALCDOC   VARCHAR2(01)  DEFAULT 'T'  NOT NULL
      CONSTRAINT CKC_IDTIPOCALCDOC_AJUSTFISCAUT CHECK (IDTIPOCALCDOC IN ('T','D','I')), -- "Ajuste com Valor Total"="T", "Ajuste por Documento"="D","Ajuste por Item"="I"
   IDTIPOCONT      VARCHAR2(01)  DEFAULT 'N'  NOT NULL
      CONSTRAINT CKC_IDTIPOCONT_AJUSTFISCAUTO CHECK (IDTIPOCONT IN ('N','R','C')), -- "No" = "N", "Sim - Padro de Rateio" = "R", "Sim - Prprio Clculo" = "C"
   CDCONTRATEIO    VARCHAR2(04)               NULL, 
   CDAJUSTEEEFD    NUMBER                     NULL,
   DSAJUSTEEFDICMS VARCHAR2(400)              NULL,
   CDFORMULA       VARCHAR2(10)               NOT NULL ,
   NRORG 	       NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg') NOT NULL,
   IDATIVO 	       VARCHAR2(1)   DEFAULT 'S' NOT NULL,
      CONSTRAINT CKC_IDATIVO_AJUSTFISCAUTO CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO 	   DATE 	                  NULL,
   DTULTATU 	   DATE 	                  NULL,
   NRORGINCLUSAO   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERINCLUSAO  VARCHAR2(12)	 DEFAULT '1'  NULL,
   NRORGULTATU 	   NUMBER 	     DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NULL,
   CDOPERULTATU    VARCHAR2(12)  DEFAULT '1',   
   CONSTRAINT PK_AJUSTFISCAUTO PRIMARY KEY (CDAJUSTFISC)
)
/

CREATE INDEX FIS00840_FK ON AJUSTFISCAUTO (CDCONTRATEIO ASC)
/

ALTER TABLE AJUSTFISCAUTO
   ADD CONSTRAINT FKFIS00840_RATEIOPADRAOFIS FOREIGN KEY (CDCONTRATEIO)
      REFERENCES RATEIOPADRAOFIS (CDRATEIOPADRAO)
/

CREATE INDEX FIS00841_FK ON AJUSTFISCAUTO (CDFORMULA ASC)
/

ALTER TABLE AJUSTFISCAUTO
   ADD CONSTRAINT FKFIS00841_FORMAJUSTFISC FOREIGN KEY (CDFORMULA)
      REFERENCES FORMAJUSTFISC (CDFORMULA)
/

CREATE INDEX FIS00842_FK ON AJUSTFISCAUTO (CDIMPOSTONFE ASC)
/

ALTER TABLE AJUSTFISCAUTO
   ADD CONSTRAINT FKFIS00842_IMPOSTONFE FOREIGN KEY (CDIMPOSTONFE)
      REFERENCES IMPOSTONFE (CDIMPOSTONFE)
/