/* ============================================================ */
/*   Database name:  SISTEMAS_TEKNISA_5_01                      */
/*   DBMS name:      Microsoft SQL Server 6.x (double precision */
/*   Created on:     13/08/2008  13:46                          */
/* ============================================================ */

/*  Delete trigger "td_acertoorceve" for table "ACERTOORCEVE"  */
create trigger td_acertoorceve on ACERTOORCEVE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ACERTOQUEBRA"  */
    delete ACERTOQUEBRA
    from   ACERTOQUEBRA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.CDDEPTO = t1.CDDEPTO
    
    /*  Delete all children in "ACERTOSOBRA"  */
    delete ACERTOSOBRA
    from   ACERTOSOBRA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.CDDEPTO = t1.CDDEPTO
    
    /*  Delete all children in "ACERTOHREXTR"  */
    delete ACERTOHREXTR
    from   ACERTOHREXTR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.CDDEPTO = t1.CDDEPTO
    
    /*  Delete all children in "ACERTOEXTRA"  */
    delete ACERTOEXTRA
    from   ACERTOEXTRA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.CDDEPTO = t1.CDDEPTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_agencia" for table "AGENCIA"  */
create trigger td_agencia on AGENCIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "AGENCIA" if children still exist in "CONTCORR"  */
    if exists (select 1
               from   CONTCORR t2, deleted t1
               where  t2.CDBANCO = t1.CDBANCO
                and   t2.CDAGENCIA = t1.CDAGENCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTCORR". Cannot delete parent "AGENCIA".'
          goto error
       end
    
    /*  Delete all children in "CONTATO_AGEN"  */
    delete CONTATO_AGEN
    from   CONTATO_AGEN t2, deleted t1
    where  t2.CDBANCO = t1.CDBANCO
     and   t2.CDAGENCIA = t1.CDAGENCIA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_agendaevento" for table "AGENDAEVENTO"  */
create trigger td_agendaevento on AGENDAEVENTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXECEVENTO"  */
    delete EXECEVENTO
    from   EXECEVENTO t2, deleted t1
    where  t2.NRAGENDAEVEN = t1.NRAGENDAEVEN
    
    /*  Delete all children in "PARAMEXECEVENTO"  */
    delete PARAMEXECEVENTO
    from   PARAMEXECEVENTO t2, deleted t1
    where  t2.NRAGENDAEVEN = t1.NRAGENDAEVEN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ajbolpag" for table "AJBOLPAG"  */
create trigger td_ajbolpag on AJBOLPAG for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MOVFXAJBOLP"  */
    delete MOVFXAJBOLP
    from   MOVFXAJBOLP t2, deleted t1
    where  t2.CDFILIBOLPAG = t1.CDFILIAL
     and   t2.NRLANBOLPAG = t1.NRLANBOLPAG
     and   t2.CDAJUCTPAG = t1.CDAJUCTPAG
    
    /*  Delete all children in "ADIANTAPAGAR"  */
    delete ADIANTAPAGAR
    from   ADIANTAPAGAR t2, deleted t1
    where  t2.CDFILIAJBOLP = t1.CDFILIAL
     and   t2.NRLANBOLPAG = t1.NRLANBOLPAG
     and   t2.CDAJUCTPAG = t1.CDAJUCTPAG

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ajbolrec" for table "AJBOLREC"  */
create trigger td_ajbolrec on AJBOLREC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MOVFXAJBOLR"  */
    delete MOVFXAJBOLR
    from   MOVFXAJBOLR t2, deleted t1
    where  t2.CDFILIBOLREC = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC
     and   t2.CDAJUCTREC = t1.CDAJUCTREC
    
    /*  Delete all children in "ADIANTARECEB"  */
    delete ADIANTARECEB
    from   ADIANTARECEB t2, deleted t1
    where  t2.CDFILIAJBOLR = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC
     and   t2.CDAJUCTREC = t1.CDAJUCTREC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ala" for table "ALA"  */
create trigger td_ala on ALA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ALA" if children still exist in "SALA"  */
    if exists (select 1
               from   SALA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALA = t1.CDALA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALA". Cannot delete parent "ALA".'
          goto error
       end
    
    /*  Cannot delete parent "ALA" if children still exist in "ESTATDIET"  */
    if exists (select 1
               from   ESTATDIET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALA = t1.CDALA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTATDIET". Cannot delete parent "ALA".'
          goto error
       end
    
    /*  Delete all children in "ALANDAR"  */
    delete ALANDAR
    from   ALANDAR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDALA = t1.CDALA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_aliquota" for table "ALIQUOTA"  */
create trigger td_aliquota on ALIQUOTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MENSALIQU"  */
    delete MENSALIQU
    from   MENSALIQU t2, deleted t1
    where  t2.NRALIQUOTA = t1.NRALIQUOTA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_almoxarifado" for table "ALMOXARIFADO"  */
create trigger td_almoxarifado on ALMOXARIFADO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "LOCALESTOQUE"  */
    if exists (select 1
               from   LOCALESTOQUE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCALESTOQUE". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "ROTATRANS"  */
    if exists (select 1
               from   ROTATRANS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTATRANS". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "PARFILIALEMP"  */
    if exists (select 1
               from   PARFILIALEMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXEMPR = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALEMP". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXASAID = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXAENTR = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "PARFILIHOSP"  */
    if exists (select 1
               from   PARFILIHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXFARMA = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIHOSP". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIRECE = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "RESERPRODPED"  */
    if exists (select 1
               from   RESERPRODPED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERPRODPED". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "INVEROTA"  */
    if exists (select 1
               from   INVEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTA". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "CURVAABC"  */
    if exists (select 1
               from   CURVAABC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CURVAABC". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXENTR = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXSAID = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXSAID = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXENTR = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "PARFILIALVMN"  */
    if exists (select 1
               from   PARFILIALVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXPERDA = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALVMN". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "CODIGOBARRA"  */
    if exists (select 1
               from   CODIGOBARRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODIGOBARRA". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "PRODACABADO"  */
    if exists (select 1
               from   PRODACABADO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODACABADO". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "INVEROTALOTE"  */
    if exists (select 1
               from   INVEROTALOTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTALOTE". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Cannot delete parent "ALMOXARIFADO" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "ALMOXARIFADO".'
          goto error
       end
    
    /*  Delete all children in "OPERALMOX"  */
    delete OPERALMOX
    from   OPERALMOX t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
    
    /*  Delete all children in "GROPERALMOX"  */
    delete GROPERALMOX
    from   GROPERALMOX t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDALMOXARIFE = t1.CDALMOXARIFE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_amostraproc" for table "AMOSTRAPROC"  */
create trigger td_amostraproc on AMOSTRAPROC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "AMOSTRAPROC" if children still exist in "PROCESSAPROD"  */
    if exists (select 1
               from   PROCESSAPROD t2, deleted t1
               where  t2.NRAMOSTRA = t1.NRAMOSTRA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCESSAPROD". Cannot delete parent "AMOSTRAPROC".'
          goto error
       end
    
    /*  Delete all children in "ITEMAMOSTRA"  */
    delete ITEMAMOSTRA
    from   ITEMAMOSTRA t2, deleted t1
    where  t2.NRAMOSTRA = t1.NRAMOSTRA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_andar" for table "ANDAR"  */
create trigger td_andar on ANDAR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ANDAR" if children still exist in "ANDARSETO"  */
    if exists (select 1
               from   ANDARSETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDANDAR = t1.CDANDAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ANDARSETO". Cannot delete parent "ANDAR".'
          goto error
       end
    
    /*  Delete all children in "ALANDAR"  */
    delete ALANDAR
    from   ALANDAR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDANDAR = t1.CDANDAR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_aprerece" for table "APRERECE"  */
create trigger td_aprerece on APRERECE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "APREPROD"  */
    delete APREPROD
    from   APREPROD t2, deleted t1
    where  t2.CDAPRERECE = t1.CDAPRERECE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_atenhosp" for table "ATENHOSP"  */
create trigger td_atenhosp on ATENHOSP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "OBSATHOSP"  */
    if exists (select 1
               from   OBSATHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSATHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "ATENHOSP"  */
    if exists (select 1
               from   ATENHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENDHORIG = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATENHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "PROCATHOSP"  */
    if exists (select 1
               from   PROCATHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCATHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "FUNCATHOSP"  */
    if exists (select 1
               from   FUNCATHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCATHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "HONORATHOSP"  */
    if exists (select 1
               from   HONORATHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HONORATHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "REQUPROC"  */
    if exists (select 1
               from   REQUPROC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUPROC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "OCUPATHOSP"  */
    if exists (select 1
               from   OCUPATHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OCUPATHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "TRIAGDOAC"  */
    if exists (select 1
               from   TRIAGDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRIAGDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "TIPAGDOAC"  */
    if exists (select 1
               from   TIPAGDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPAGDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "COLETDOAC"  */
    if exists (select 1
               from   COLETDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "PRODUDOAC"  */
    if exists (select 1
               from   PRODUDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "PREPDOAC"  */
    if exists (select 1
               from   PREPDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "AFERESE"  */
    if exists (select 1
               from   AFERESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AFERESE". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "ITMOVPROTESE"  */
    if exists (select 1
               from   ITMOVPROTESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMOVPROTESE". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "ITAGENFUNC"  */
    if exists (select 1
               from   ITAGENFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATHOSPGERA = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITAGENFUNC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "ITAGENFUNC"  */
    if exists (select 1
               from   ITAGENFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATHOSPORIG = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITAGENFUNC". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "APURFATUGH"  */
    if exists (select 1
               from   APURFATUGH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUGH". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Cannot delete parent "ATENHOSP" if children still exist in "ENDEATEN"  */
    if exists (select 1
               from   ENDEATEN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRATENHOSP = t1.NRATENHOSP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ENDEATEN". Cannot delete parent "ATENHOSP".'
          goto error
       end
    
    /*  Delete all children in "VISITATHOSP"  */
    delete VISITATHOSP
    from   VISITATHOSP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRATENHOSP = t1.NRATENHOSP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_atenreal" for table "ATENREAL"  */
create trigger td_atenreal on ATENREAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITATENREAL"  */
    delete ITATENREAL
    from   ITATENREAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ativpessoal" for table "ATIVPESSOAL"  */
create trigger td_ativpessoal on ATIVPESSOAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "DESCATIVPESS"  */
    delete DESCATIVPESS
    from   DESCATIVPESS t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
     and   t2.NRATIVPESS = t1.NRATIVPESS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_auditoria" for table "AUDITORIA"  */
create trigger td_auditoria on AUDITORIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "AUDITORIA" if children still exist in "TRIAGDOAC"  */
    if exists (select 1
               from   TRIAGDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAUDITORIA = t1.NRAUDITORIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRIAGDOAC". Cannot delete parent "AUDITORIA".'
          goto error
       end
    
    /*  Cannot delete parent "AUDITORIA" if children still exist in "EXAMDOACAO"  */
    if exists (select 1
               from   EXAMDOACAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAUDITORIA = t1.NRAUDITORIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXAMDOACAO". Cannot delete parent "AUDITORIA".'
          goto error
       end
    
    /*  Cannot delete parent "AUDITORIA" if children still exist in "PREPDOAC"  */
    if exists (select 1
               from   PREPDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAUDITORIA = t1.NRAUDITORIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPDOAC". Cannot delete parent "AUDITORIA".'
          goto error
       end
    
    /*  Cannot delete parent "AUDITORIA" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAUDITORIA = t1.NRAUDITORIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "AUDITORIA".'
          goto error
       end
    
    /*  Cannot delete parent "AUDITORIA" if children still exist in "AUDITCAND"  */
    if exists (select 1
               from   AUDITCAND t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAUDITORIA = t1.NRAUDITORIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITCAND". Cannot delete parent "AUDITORIA".'
          goto error
       end
    
    /*  Delete all children in "INSPECAUDIT"  */
    delete INSPECAUDIT
    from   INSPECAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
    
    /*  Delete all children in "DESCAUDIT"  */
    delete DESCAUDIT
    from   DESCAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_autoforne" for table "AUTOFORNE"  */
create trigger td_autoforne on AUTOFORNE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "AUTOFORNE" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDFILDESTCON = t1.CDFILDEST
                and   t2.CDFILLANCCON = t1.CDFILLANC
                and   t2.NRAUTOFORCON = t1.NRAUTOFORN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "AUTOFORNE".'
          goto error
       end
    
    /*  Cannot delete parent "AUTOFORNE" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDFILDESTOL = t1.CDFILDEST
                and   t2.CDFILLANCOL = t1.CDFILLANC
                and   t2.NRAUTOFORNOL = t1.NRAUTOFORN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "AUTOFORNE".'
          goto error
       end
    
    /*  Delete all children in "MOVFLUXAF"  */
    delete MOVFLUXAF
    from   MOVFLUXAF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
    
    /*  Delete all children in "MENSAUTF"  */
    delete MENSAUTF
    from   MENSAUTF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
    
    /*  Delete all children in "ITEMAUTF"  */
    delete ITEMAUTF
    from   ITEMAUTF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
    
    /*  Delete all children in "PRAZOPGTAF"  */
    delete PRAZOPGTAF
    from   PRAZOPGTAF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
    
    /*  Delete all children in "ENCARGOAF"  */
    delete ENCARGOAF
    from   ENCARGOAF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
    
    /*  Delete all children in "RELACENTRAF"  */
    delete RELACENTRAF
    from   RELACENTRAF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_avalcand" for table "AVALCAND"  */
create trigger td_avalcand on AVALCAND for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PTFORTECAN"  */
    delete PTFORTECAN
    from   PTFORTECAN t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
     and   t2.DTAVALCAND = t1.DTAVALCAND
    
    /*  Delete all children in "PTFRACOCAN"  */
    delete PTFRACOCAN
    from   PTFRACOCAN t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
     and   t2.DTAVALCAND = t1.DTAVALCAND

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_baselineproj" for table "BASELINEPROJ"  */
create trigger td_baselineproj on BASELINEPROJ for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "ITEMTRABPROJ"  */
    if exists (select 1
               from   ITEMTRABPROJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTRABPROJ". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "ESCOPOPROJET"  */
    if exists (select 1
               from   ESCOPOPROJET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESCOPOPROJET". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "OPERPROJETO"  */
    if exists (select 1
               from   OPERPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERPROJETO". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "RISCOPROJETO"  */
    if exists (select 1
               from   RISCOPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RISCOPROJETO". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "SOLIBASELINE"  */
    if exists (select 1
               from   SOLIBASELINE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLIBASELINE". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "PRODBASELINE"  */
    if exists (select 1
               from   PRODBASELINE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODBASELINE". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "ENTBASELINE"  */
    if exists (select 1
               from   ENTBASELINE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ENTBASELINE". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "ASSUNTO"  */
    if exists (select 1
               from   ASSUNTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ASSUNTO". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "HABILPROJETO"  */
    if exists (select 1
               from   HABILPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABILPROJETO". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "HABILOPERPRJ"  */
    if exists (select 1
               from   HABILOPERPRJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABILOPERPRJ". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "MONITORAPROJ"  */
    if exists (select 1
               from   MONITORAPROJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MONITORAPROJ". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "CRONOPROJETO"  */
    if exists (select 1
               from   CRONOPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CRONOPROJETO". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Cannot delete parent "BASELINEPROJ" if children still exist in "SERVPREST"  */
    if exists (select 1
               from   SERVPREST t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPROJETO = t1.CDPROJETO
                and   t2.NRBASELINEPR = t1.NRBASELINEPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREST". Cannot delete parent "BASELINEPROJ".'
          goto error
       end
    
    /*  Delete all children in "TIPORECPROJ"  */
    delete TIPORECPROJ
    from   TIPORECPROJ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPROJETO = t1.CDPROJETO
     and   t2.NRBASELINEPR = t1.NRBASELINEPR
    
    /*  Delete all children in "ESTRECHUMPRJ"  */
    delete ESTRECHUMPRJ
    from   ESTRECHUMPRJ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPROJETO = t1.CDPROJETO
     and   t2.NRBASELINEPR = t1.NRBASELINEPR
    
    /*  Delete all children in "TIPOINDPROJ"  */
    delete TIPOINDPROJ
    from   TIPOINDPROJ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPROJETO = t1.CDPROJETO
     and   t2.NRBASELINEPR = t1.NRBASELINEPR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_bkgthread" for table "BKGTHREAD"  */
create trigger td_bkgthread on BKGTHREAD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "BKGTHREAD" if children still exist in "BKGTHREAD"  */
    if exists (select 1
               from   BKGTHREAD t2, deleted t1
               where  t2.IDREQTHREAD = t1.IDTHREAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BKGTHREAD". Cannot delete parent "BKGTHREAD".'
          goto error
       end
    
    /*  Cannot delete parent "BKGTHREAD" if children still exist in "EXECEVENTO"  */
    if exists (select 1
               from   EXECEVENTO t2, deleted t1
               where  t2.IDTHREAD = t1.IDTHREAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXECEVENTO". Cannot delete parent "BKGTHREAD".'
          goto error
       end
    
    /*  Delete all children in "BKGTHREADPARAM"  */
    delete BKGTHREADPARAM
    from   BKGTHREADPARAM t2, deleted t1
    where  t2.IDTHREAD = t1.IDTHREAD
    
    /*  Delete all children in "BKGTHREADSQL"  */
    delete BKGTHREADSQL
    from   BKGTHREADSQL t2, deleted t1
    where  t2.IDTHREAD = t1.IDTHREAD
    
    /*  Delete all children in "BKGTHREADDATA"  */
    delete BKGTHREADDATA
    from   BKGTHREADDATA t2, deleted t1
    where  t2.IDTHREAD = t1.IDTHREAD
    
    /*  Delete all children in "BKGSCREENPARAM"  */
    delete BKGSCREENPARAM
    from   BKGSCREENPARAM t2, deleted t1
    where  t2.IDTHREAD = t1.IDTHREAD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_boletpagar" for table "BOLETPAGAR"  */
create trigger td_boletpagar on BOLETPAGAR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "BOLETPAGAR" if children still exist in "CONTAPAGAR"  */
    if exists (select 1
               from   CONTAPAGAR t2, deleted t1
               where  t2.CDFILIBPAG = t1.CDFILIAL
                and   t2.NRLANBOLPAG = t1.NRLANBOLPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTAPAGAR". Cannot delete parent "BOLETPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "BOLETPAGAR" if children still exist in "AJBOLPAG"  */
    if exists (select 1
               from   AJBOLPAG t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANBOLPAG = t1.NRLANBOLPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJBOLPAG". Cannot delete parent "BOLETPAGAR".'
          goto error
       end
    
    /*  Delete all children in "ERRCNABBOLP"  */
    delete ERRCNABBOLP
    from   ERRCNABBOLP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANBOLPAG = t1.NRLANBOLPAG
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIBPAG = t1.CDFILIAL
     and   t2.NRLANBOLPAG = t1.NRLANBOLPAG

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_boletreceber" for table "BOLETRECEBER"  */
create trigger td_boletreceber on BOLETRECEBER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "BOLETRECEBER" if children still exist in "CONTARECEBER"  */
    if exists (select 1
               from   CONTARECEBER t2, deleted t1
               where  t2.CDFILIBOLREC = t1.CDFILIAL
                and   t2.NRLANBOLREC = t1.NRLANBOLREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTARECEBER". Cannot delete parent "BOLETRECEBER".'
          goto error
       end
    
    /*  Delete all children in "AJBOLREC"  */
    delete AJBOLREC
    from   AJBOLREC t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC
    
    /*  Delete all children in "ERRCNABBOLR"  */
    delete ERRCNABBOLR
    from   ERRCNABBOLR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC
    
    /*  Delete all children in "TPRECBOLETO"  */
    delete TPRECBOLETO
    from   TPRECBOLETO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIBREC = t1.CDFILIAL
     and   t2.NRLANBOLREC = t1.NRLANBOLREC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_candidato" for table "CANDIDATO"  */
create trigger td_candidato on CANDIDATO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CANDIDATO" if children still exist in "IDIOMACAND"  */
    if exists (select 1
               from   IDIOMACAND t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDCAND = t1.CDCAND)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IDIOMACAND". Cannot delete parent "CANDIDATO".'
          goto error
       end
    
    /*  Cannot delete parent "CANDIDATO" if children still exist in "AUDITCAND"  */
    if exists (select 1
               from   AUDITCAND t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDCAND = t1.CDCAND)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITCAND". Cannot delete parent "CANDIDATO".'
          goto error
       end
    
    /*  Cannot delete parent "CANDIDATO" if children still exist in "FOTOCAND"  */
    if exists (select 1
               from   FOTOCAND t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDCAND = t1.CDCAND)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOCAND". Cannot delete parent "CANDIDATO".'
          goto error
       end
    
    /*  Delete all children in "OBSERVCAND"  */
    delete OBSERVCAND
    from   OBSERVCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
    
    /*  Delete all children in "TESTCAND"  */
    delete TESTCAND
    from   TESTCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
    
    /*  Delete all children in "EXPEPROFCAND"  */
    delete EXPEPROFCAND
    from   EXPEPROFCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
    
    /*  Delete all children in "CURSOCAND"  */
    delete CURSOCAND
    from   CURSOCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
    
    /*  Delete all children in "LIONIBUSCAND"  */
    delete LIONIBUSCAND
    from   LIONIBUSCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND
    
    /*  Delete all children in "AVALCAND"  */
    delete AVALCAND
    from   AVALCAND t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDCAND = t1.CDCAND

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_capalote" for table "CAPALOTE"  */
create trigger td_capalote on CAPALOTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "LANCLOTE"  */
    delete LANCLOTE
    from   LANCLOTE t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDLOTE = t1.CDLOTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_cardadia" for table "CARDADIA"  */
create trigger td_cardadia on CARDADIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CARDADIA" if children still exist in "RETIRADA"  */
    if exists (select 1
               from   RETIRADA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRCARDAPIO = t1.NRCARDAPIO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADA". Cannot delete parent "CARDADIA".'
          goto error
       end
    
    /*  Delete all children in "COMPCARD"  */
    delete COMPCARD
    from   COMPCARD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCARDAPIO = t1.NRCARDAPIO
    
    /*  Delete all children in "CARDACLI"  */
    delete CARDACLI
    from   CARDACLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCARDAPIO = t1.NRCARDAPIO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_cardpaci" for table "CARDPACI"  */
create trigger td_cardpaci on CARDPACI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "NUTRNECE"  */
    delete NUTRNECE
    from   NUTRNECE t2, deleted t1
    where  t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.DTCARDPACI = t1.DTCARDPACI
    
    /*  Delete all children in "COMCADIAPACI"  */
    delete COMCADIAPACI
    from   COMCADIAPACI t2, deleted t1
    where  t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.DTCARDPACI = t1.DTCARDPACI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_cargo" for table "CARGO"  */
create trigger td_cargo on CARGO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CARGO" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "FUNCIONARIO"  */
    if exists (select 1
               from   FUNCIONARIO t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCIONARIO". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "MOVI_PESSOAL"  */
    if exists (select 1
               from   MOVI_PESSOAL t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVI_PESSOAL". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "CPREV_TEMP"  */
    if exists (select 1
               from   CPREV_TEMP t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CPREV_TEMP". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "TALENTO"  */
    if exists (select 1
               from   TALENTO t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TALENTO". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "MOVFUNCRH"  */
    if exists (select 1
               from   MOVFUNCRH t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFUNCRH". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDCARGOCNJCL = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDCARGOCLIE = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "OPERADOR"  */
    if exists (select 1
               from   OPERADOR t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERADOR". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Cannot delete parent "CARGO" if children still exist in "INDICADOR"  */
    if exists (select 1
               from   INDICADOR t2, deleted t1
               where  t2.CDCARGO = t1.CDCARGO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INDICADOR". Cannot delete parent "CARGO".'
          goto error
       end
    
    /*  Delete all children in "INSPRECRCARG"  */
    delete INSPRECRCARG
    from   INSPRECRCARG t2, deleted t1
    where  t2.CDCARGO = t1.CDCARGO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_centcust" for table "CENTCUST"  */
create trigger td_centcust on CENTCUST for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATECCUS"  */
    if exists (select 1
               from   RATECCUS t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATECCUS". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "ITEMRATEIO"  */
    if exists (select 1
               from   ITEMRATEIO t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRATEIO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "LANCCCUSTO"  */
    if exists (select 1
               from   LANCCCUSTO t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCCCUSTO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "SALDCCUS"  */
    if exists (select 1
               from   SALDCCUS t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDCCUS". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "ITEMORCA"  */
    if exists (select 1
               from   ITEMORCA t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCA". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATEIONF"  */
    if exists (select 1
               from   RATEIONF t2, deleted t1
               where  t2.CDCENTCUSNF = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATEIONF". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDCCUSTENTR = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDCCUSTSAID = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDCCUSTENTRF = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDCCUSTSAIDF = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "EMPRESA"  */
    if exists (select 1
               from   EMPRESA t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESA". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATCTRECEBER"  */
    if exists (select 1
               from   RATCTRECEBER t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATCTRECEBER". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATCTPAGAR"  */
    if exists (select 1
               from   RATCTPAGAR t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATCTPAGAR". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATTPDEBI"  */
    if exists (select 1
               from   RATTPDEBI t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPDEBI". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RATTPCRED"  */
    if exists (select 1
               from   RATTPCRED t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPCRED". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "SETOR"  */
    if exists (select 1
               from   SETOR t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOR". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RETIRADAESTO"  */
    if exists (select 1
               from   RETIRADAESTO t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAESTO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "RETIHOSP"  */
    if exists (select 1
               from   RETIHOSP t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIHOSP". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "APURFATUGH"  */
    if exists (select 1
               from   APURFATUGH t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUGH". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "ITREQUHOSP"  */
    if exists (select 1
               from   ITREQUHOSP t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUHOSP". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "ALMOXARIFADO"  */
    if exists (select 1
               from   ALMOXARIFADO t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALMOXARIFADO". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "LOCALESTOQUE"  */
    if exists (select 1
               from   LOCALESTOQUE t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCALESTOQUE". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "PRODCENTCUST"  */
    if exists (select 1
               from   PRODCENTCUST t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCENTCUST". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "CENTCUSTBEM"  */
    if exists (select 1
               from   CENTCUSTBEM t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CENTCUSTBEM". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "APRCUSTOP"  */
    if exists (select 1
               from   APRCUSTOP t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APRCUSTOP". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "IMPOCONTRETI"  */
    if exists (select 1
               from   IMPOCONTRETI t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOCONTRETI". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Cannot delete parent "CENTCUST" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDCENTCUST = t1.CDCENTCUST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "CENTCUST".'
          goto error
       end
    
    /*  Delete all children in "RATTPAJR"  */
    delete RATTPAJR
    from   RATTPAJR t2, deleted t1
    where  t2.CDCENTCUST = t1.CDCENTCUST
    
    /*  Delete all children in "RATTPAJP"  */
    delete RATTPAJP
    from   RATTPAJP t2, deleted t1
    where  t2.CDCENTCUST = t1.CDCENTCUST
    
    /*  Delete all children in "RATTPCTP"  */
    delete RATTPCTP
    from   RATTPCTP t2, deleted t1
    where  t2.CDCENTCUST = t1.CDCENTCUST
    
    /*  Delete all children in "RATTPCTR"  */
    delete RATTPCTR
    from   RATTPCTR t2, deleted t1
    where  t2.CDCENTCUST = t1.CDCENTCUST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ciafili" for table "CIAFILI"  */
create trigger td_ciafili on CIAFILI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PREFIXOCIA"  */
    if exists (select 1
               from   PREFIXOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREFIXOCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "CICLOCIA"  */
    if exists (select 1
               from   CICLOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CICLOCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "FRCICLOCIA"  */
    if exists (select 1
               from   FRCICLOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FRCICLOCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PRODUTOCIA"  */
    if exists (select 1
               from   PRODUTOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PERFECHAM"  */
    if exists (select 1
               from   PERFECHAM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERFECHAM". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "CUSTORECIA"  */
    if exists (select 1
               from   CUSTORECIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTORECIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "TABPRECOSE"  */
    if exists (select 1
               from   TABPRECOSE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TABPRECOSE". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "VOO"  */
    if exists (select 1
               from   VOO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VOO". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "CLASSECIA"  */
    if exists (select 1
               from   CLASSECIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLASSECIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "EQUIPCIA"  */
    if exists (select 1
               from   EQUIPCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "DESCMENSCIA"  */
    if exists (select 1
               from   DESCMENSCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCMENSCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "DESPERCIA"  */
    if exists (select 1
               from   DESPERCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPERCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PRECOSECIA"  */
    if exists (select 1
               from   PRECOSECIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOSECIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "SERVPLANO"  */
    if exists (select 1
               from   SERVPLANO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPLANO". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PADEQUIP"  */
    if exists (select 1
               from   PADEQUIP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADEQUIP". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PADPREFIXO"  */
    if exists (select 1
               from   PADPREFIXO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADPREFIXO". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PADVOOTRE"  */
    if exists (select 1
               from   PADVOOTRE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADVOOTRE". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "FRCICLOSER"  */
    if exists (select 1
               from   FRCICLOSER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FRCICLOSER". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "PROPSERVICO"  */
    if exists (select 1
               from   PROPSERVICO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPSERVICO". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "BREAKVOOTRE"  */
    if exists (select 1
               from   BREAKVOOTRE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BREAKVOOTRE". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "ESPEPROD"  */
    if exists (select 1
               from   ESPEPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROD". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "MANUALCIA"  */
    if exists (select 1
               from   MANUALCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MANUALCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Cannot delete parent "CIAFILI" if children still exist in "LOCALFATUCIA"  */
    if exists (select 1
               from   LOCALFATUCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCALFATUCIA". Cannot delete parent "CIAFILI".'
          goto error
       end
    
    /*  Delete all children in "TRABNUTRIREC"  */
    delete TRABNUTRIREC
    from   TRABNUTRIREC t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_clienfilial" for table "CLIENFILIAL"  */
create trigger td_clienfilial on CLIENFILIAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "GRUPSERVFAT"  */
    if exists (select 1
               from   GRUPSERVFAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPSERVFAT". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "SERVCLIE"  */
    if exists (select 1
               from   SERVCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIE". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "EFECLIFAT"  */
    if exists (select 1
               from   EFECLIFAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFECLIFAT". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "RESTITREFTUR"  */
    if exists (select 1
               from   RESTITREFTUR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTITREFTUR". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "CLIENROTA"  */
    if exists (select 1
               from   CLIENROTA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENROTA". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENFILIAL" if children still exist in "EFECLIFATTUR"  */
    if exists (select 1
               from   EFECLIFATTUR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFECLIFATTUR". Cannot delete parent "CLIENFILIAL".'
          goto error
       end
    
    /*  Delete all children in "ITEMREFECLIE"  */
    delete ITEMREFECLIE
    from   ITEMREFECLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "ITEMPRECOCLI"  */
    delete ITEMPRECOCLI
    from   ITEMPRECOCLI t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "CALEFATURVMN"  */
    delete CALEFATURVMN
    from   CALEFATURVMN t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_clienrota" for table "CLIENROTA"  */
create trigger td_clienrota on CLIENROTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EQUIPCLIROTA"  */
    delete EQUIPCLIROTA
    from   EQUIPCLIROTA t2, deleted t1
    where  t2.CDROTATRANS = t1.CDROTATRANS
     and   t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_cliente" for table "CLIENTE"  */
create trigger td_cliente on CLIENTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CLIENTE" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "EFETCLIE"  */
    if exists (select 1
               from   EFETCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "BOLETRECEBER"  */
    if exists (select 1
               from   BOLETRECEBER t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETRECEBER". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CLIENFILIAL"  */
    if exists (select 1
               from   CLIENFILIAL t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENFILIAL". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CTRLAPURFAT"  */
    if exists (select 1
               from   CTRLAPURFAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPURFAT". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "LICENCAUSO"  */
    if exists (select 1
               from   LICENCAUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LICENCAUSO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SISTEMACLIE"  */
    if exists (select 1
               from   SISTEMACLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SISTEMACLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "EXPORTCLIE"  */
    if exists (select 1
               from   EXPORTCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXPORTCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "DUPLICATA"  */
    if exists (select 1
               from   DUPLICATA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DUPLICATA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CONSUMIDOR"  */
    if exists (select 1
               from   CONSUMIDOR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSUMIDOR". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVICLIE"  */
    if exists (select 1
               from   MOVICLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDCLIENTEPDV = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "BANCOPREF"  */
    if exists (select 1
               from   BANCOPREF t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCOPREF". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PREVENTRADA"  */
    if exists (select 1
               from   PREVENTRADA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENTRADA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "FREQALUNO"  */
    if exists (select 1
               from   FREQALUNO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FREQALUNO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ESCOCLIE"  */
    if exists (select 1
               from   ESCOCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESCOCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "EMPRESTPROD"  */
    if exists (select 1
               from   EMPRESTPROD t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESTPROD". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "RESERVAPROD"  */
    if exists (select 1
               from   RESERVAPROD t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVAPROD". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "OPERADOR"  */
    if exists (select 1
               from   OPERADOR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERADOR". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARTATIVSALA"  */
    if exists (select 1
               from   PARTATIVSALA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTATIVSALA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "RESERVATURMA"  */
    if exists (select 1
               from   RESERVATURMA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVATURMA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ESTINTERECLI"  */
    if exists (select 1
               from   ESTINTERECLI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTINTERECLI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "DEPENDCLIE"  */
    if exists (select 1
               from   DEPENDCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEPENDCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "NEGOCIACA"  */
    if exists (select 1
               from   NEGOCIACA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NEGOCIACA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "COMENSAL"  */
    if exists (select 1
               from   COMENSAL t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMENSAL". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "DESTSRVCLI"  */
    if exists (select 1
               from   DESTSRVCLI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESTSRVCLI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CONVENIO"  */
    if exists (select 1
               from   CONVENIO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVENIO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CARDACLI"  */
    if exists (select 1
               from   CARDACLI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDACLI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SERVTRAN"  */
    if exists (select 1
               from   SERVTRAN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTRAN". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ETIQUETRAN"  */
    if exists (select 1
               from   ETIQUETRAN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETIQUETRAN". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "TIPOCONSCLI"  */
    if exists (select 1
               from   TIPOCONSCLI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCONSCLI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARCLIEPRVMN"  */
    if exists (select 1
               from   PARCLIEPRVMN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLIEPRVMN". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "REFECLIE"  */
    if exists (select 1
               from   REFECLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REFECLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PREVENDA"  */
    if exists (select 1
               from   PREVENDA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVCAIXA"  */
    if exists (select 1
               from   MOVCAIXA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCAIXA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARFILIALLOJ"  */
    if exists (select 1
               from   PARFILIALLOJ t2, deleted t1
               where  t2.CDCLIEPADORC = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALLOJ". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "RELACOPER"  */
    if exists (select 1
               from   RELACOPER t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACOPER". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PRODSUBCLIE"  */
    if exists (select 1
               from   PRODSUBCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODSUBCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "VENDAVMN"  */
    if exists (select 1
               from   VENDAVMN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAVMN". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARGERALVMN"  */
    if exists (select 1
               from   PARGERALVMN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARGERALVMN". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PREVENDAEQU"  */
    if exists (select 1
               from   PREVENDAEQU t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAEQU". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CHEQUERECE"  */
    if exists (select 1
               from   CHEQUERECE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHEQUERECE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PRECOBASECLI"  */
    if exists (select 1
               from   PRECOBASECLI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOBASECLI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "RELAT_MANDATO"  */
    if exists (select 1
               from   RELAT_MANDATO t2, deleted t1
               where  t2.CDCLIENTEMA = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_MANDATO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CTRLTPRECEB"  */
    if exists (select 1
               from   CTRLTPRECEB t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLTPRECEB". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "TROCA"  */
    if exists (select 1
               from   TROCA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TROCA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SOCIOCLIENTE"  */
    if exists (select 1
               from   SOCIOCLIENTE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOCIOCLIENTE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDCLIENTERAT = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "REAJSERVFAT"  */
    if exists (select 1
               from   REAJSERVFAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REAJSERVFAT". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CXAIMPFIS"  */
    if exists (select 1
               from   CXAIMPFIS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CXAIMPFIS". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PRODCLIE"  */
    if exists (select 1
               from   PRODCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PROSPECT"  */
    if exists (select 1
               from   PROSPECT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPECT". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SERVCLIEPME"  */
    if exists (select 1
               from   SERVCLIEPME t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIEPME". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SLACLIENTE"  */
    if exists (select 1
               from   SLACLIENTE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SLACLIENTE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PAR_FILIAL_GEP"  */
    if exists (select 1
               from   PAR_FILIAL_GEP t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_FILIAL_GEP". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "LOGSOLISLA"  */
    if exists (select 1
               from   LOGSOLISLA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGSOLISLA". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "TRADIDICLIE"  */
    if exists (select 1
               from   TRADIDICLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRADIDICLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "INVOICE"  */
    if exists (select 1
               from   INVOICE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVOICE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "VENDAREST"  */
    if exists (select 1
               from   VENDAREST t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAREST". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CONTRATOLINP"  */
    if exists (select 1
               from   CONTRATOLINP t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOLINP". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CONTRATOSERV"  */
    if exists (select 1
               from   CONTRATOSERV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOSERV". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PEDILINPCLIE"  */
    if exists (select 1
               from   PEDILINPCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDILINPCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PEDISERVCLIE"  */
    if exists (select 1
               from   PEDISERVCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDISERVCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "EFETDIET"  */
    if exists (select 1
               from   EFETDIET t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETDIET". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "EMPENHO"  */
    if exists (select 1
               from   EMPENHO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPENHO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARCLIEFAS"  */
    if exists (select 1
               from   PARCLIEFAS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLIEFAS". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ETLRESULTDES"  */
    if exists (select 1
               from   ETLRESULTDES t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLRESULTDES". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDCLICONTORC = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "APROATIVTARE"  */
    if exists (select 1
               from   APROATIVTARE t2, deleted t1
               where  t2.CDCLIEATEND = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APROATIVTARE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "AGENDASALAO"  */
    if exists (select 1
               from   AGENDASALAO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASALAO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARAMDESCOREV"  */
    if exists (select 1
               from   PARAMDESCOREV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMDESCOREV". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PERMUTAORE"  */
    if exists (select 1
               from   PERMUTAORE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERMUTAORE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "UTILBANCSOLU"  */
    if exists (select 1
               from   UTILBANCSOLU t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UTILBANCSOLU". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ETLFATUINVOICE"  */
    if exists (select 1
               from   ETLFATUINVOICE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLFATUINVOICE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVCREDBANC"  */
    if exists (select 1
               from   MOVCREDBANC t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCREDBANC". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVDEBBANC"  */
    if exists (select 1
               from   MOVDEBBANC t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVDEBBANC". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ETLCONTAREC"  */
    if exists (select 1
               from   ETLCONTAREC t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAREC". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "OBSRELACCLIE"  */
    if exists (select 1
               from   OBSRELACCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSRELACCLIE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PARAMINTEGRAIBB"  */
    if exists (select 1
               from   PARAMINTEGRAIBB t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRAIBB". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "NFRELACCDRS"  */
    if exists (select 1
               from   NFRELACCDRS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACCDRS". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "MOVCAIXADLV"  */
    if exists (select 1
               from   MOVCAIXADLV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCAIXADLV". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "ITCOMANDAPEND"  */
    if exists (select 1
               from   ITCOMANDAPEND t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDAPEND". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "CLIENTE" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDCLIEPRINCIPAL = t1.CDCLIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "CLIENTE".'
          goto error
       end
    
    /*  Delete all children in "COMPANHIA"  */
    delete COMPANHIA
    from   COMPANHIA t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "ENDECLIE"  */
    delete ENDECLIE
    from   ENDECLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "INTECLIE"  */
    delete INTECLIE
    from   INTECLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "OBSERVCLIE"  */
    delete OBSERVCLIE
    from   OBSERVCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "PRAZOPGTCLIE"  */
    delete PRAZOPGTCLIE
    from   PRAZOPGTCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "CCUSCLIE"  */
    delete CCUSCLIE
    from   CCUSCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "PARCLIEVMN"  */
    delete PARCLIEVMN
    from   PARCLIEVMN t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "OPERCLIE"  */
    delete OPERCLIE
    from   OPERCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "ITEMORCACLIE"  */
    delete ITEMORCACLIE
    from   ITEMORCACLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "HISTQUALCLIE"  */
    delete HISTQUALCLIE
    from   HISTQUALCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "EXPFILCLIE"  */
    delete EXPFILCLIE
    from   EXPFILCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "RESTPRODCLIE"  */
    delete RESTPRODCLIE
    from   RESTPRODCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
    
    /*  Delete all children in "PREFPRODCLIE"  */
    delete PREFPRODCLIE
    from   PREFPRODCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_codfiscoper" for table "CODFISCOPER"  */
create trigger td_codfiscoper on CODFISCOPER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MENSCFOP"  */
    delete MENSCFOP
    from   MENSCFOP t2, deleted t1
    where  t2.NRINCIDCFOP = t1.NRINCIDCFOP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_codsitutrib" for table "CODSITUTRIB"  */
create trigger td_codsitutrib on CODSITUTRIB for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CODSITUTRIB" if children still exist in "CODSITUTRIBNFE"  */
    if exists (select 1
               from   CODSITUTRIBNFE t2, deleted t1
               where  t2.NRINCIDCST = t1.NRINCIDCST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIBNFE". Cannot delete parent "CODSITUTRIB".'
          goto error
       end
    
    /*  Delete all children in "MENSCST"  */
    delete MENSCST
    from   MENSCST t2, deleted t1
    where  t2.NRINCIDCST = t1.NRINCIDCST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_coletapr" for table "COLETAPR"  */
create trigger td_coletapr on COLETAPR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "COLETAPR" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDFILCOLETA = t1.CDFILIAL
                and   t2.NRCOLEPRECO = t1.NRCOLEPRECO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "COLETAPR".'
          goto error
       end
    
    /*  Cannot delete parent "COLETAPR" if children still exist in "PARTFILI"  */
    if exists (select 1
               from   PARTFILI t2, deleted t1
               where  t2.CDFILCOTA = t1.CDFILIAL
                and   t2.NRCOLEPRECO = t1.NRCOLEPRECO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTFILI". Cannot delete parent "COLETAPR".'
          goto error
       end
    
    /*  Delete all children in "ITEMCOLE"  */
    delete ITEMCOLE
    from   ITEMCOLE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCOLEPRECO = t1.NRCOLEPRECO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_compra" for table "COMPRA"  */
create trigger td_compra on COMPRA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "COMPRA" if children still exist in "ITMAPURPREC"  */
    if exists (select 1
               from   ITMAPURPREC t2, deleted t1
               where  t2.NRCOMPRA = t1.NRCOMPRA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMAPURPREC". Cannot delete parent "COMPRA".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRA" if children still exist in "COLETAPR"  */
    if exists (select 1
               from   COLETAPR t2, deleted t1
               where  t2.NRCOMPRA = t1.NRCOMPRA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETAPR". Cannot delete parent "COMPRA".'
          goto error
       end
    
    /*  Delete all children in "PRODCOMPRA"  */
    delete PRODCOMPRA
    from   PRODCOMPRA t2, deleted t1
    where  t2.NRCOMPRA = t1.NRCOMPRA
    
    /*  Delete all children in "FORPRCOMPRA"  */
    delete FORPRCOMPRA
    from   FORPRCOMPRA t2, deleted t1
    where  t2.NRCOMPRA = t1.NRCOMPRA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_comprador" for table "COMPRADOR"  */
create trigger td_comprador on COMPRADOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "COLETAPR"  */
    if exists (select 1
               from   COLETAPR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETAPR". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "LIMPRODAF"  */
    if exists (select 1
               from   LIMPRODAF t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPRODAF". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "PREVLISTA"  */
    if exists (select 1
               from   PREVLISTA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVLISTA". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "PEDCOLETA"  */
    if exists (select 1
               from   PEDCOLETA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLETA". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "COMPROPER"  */
    if exists (select 1
               from   COMPROPER t2, deleted t1
               where  t2.CDCOMPRADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPROPER". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "APURPRECO"  */
    if exists (select 1
               from   APURPRECO t2, deleted t1
               where  t2.CDCOMPRADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURPRECO". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "COMPRA"  */
    if exists (select 1
               from   COMPRA t2, deleted t1
               where  t2.CDCOMPRADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPRA". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "PEDCOLGERAL"  */
    if exists (select 1
               from   PEDCOLGERAL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLGERAL". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Cannot delete parent "COMPRADOR" if children still exist in "LOCK_LISTA"  */
    if exists (select 1
               from   LOCK_LISTA t2, deleted t1
               where  t2.CDCOMPRADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCK_LISTA". Cannot delete parent "COMPRADOR".'
          goto error
       end
    
    /*  Delete all children in "PRODCOMP"  */
    delete PRODCOMP
    from   PRODCOMP t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "GERECOMP"  */
    delete GERECOMP
    from   GERECOMP t2, deleted t1
    where  t2.CDOPERCOMP = t1.CDOPERADOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_compreluser" for table "COMPRELUSER"  */
create trigger td_compreluser on COMPRELUSER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "COMPRELUSER" if children still exist in "RELACCAMPO"  */
    if exists (select 1
               from   RELACCAMPO t2, deleted t1
               where  t2.CDRELATORIO = t1.CDRELATORIO
                and   t2.CDCAMPRELAC = t1.CDCAMPO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACCAMPO". Cannot delete parent "COMPRELUSER".'
          goto error
       end
    
    /*  Delete all children in "CONSRELUSER"  */
    delete CONSRELUSER
    from   CONSRELUSER t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO
     and   t2.CDCAMPO = t1.CDCAMPO
    
    /*  Delete all children in "VALIDRELUSER"  */
    delete VALIDRELUSER
    from   VALIDRELUSER t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO
     and   t2.CDCAMPO = t1.CDCAMPO
    
    /*  Delete all children in "RELACCAMPO"  */
    delete RELACCAMPO
    from   RELACCAMPO t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO
     and   t2.CDCAMPO = t1.CDCAMPO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_confequippro" for table "CONFEQUIPPRO"  */
create trigger td_confequippro on CONFEQUIPPRO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEQUIPPRO"  */
    delete ITEQUIPPRO
    from   ITEQUIPPRO t2, deleted t1
    where  t2.CDEQUIPPRO = t1.CDEQUIPPRO
     and   t2.DTINVIGCNFEP = t1.DTINVIGCNFEP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_conspaci" for table "CONSPACI"  */
create trigger td_conspaci on CONSPACI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONSPACI" if children still exist in "METOAVAL"  */
    if exists (select 1
               from   METOAVAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTCONSPACI = t1.DTCONSPACI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "METOAVAL". Cannot delete parent "CONSPACI".'
          goto error
       end
    
    /*  Cannot delete parent "CONSPACI" if children still exist in "PLAEXERFISI"  */
    if exists (select 1
               from   PLAEXERFISI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTCONSPACI = t1.DTCONSPACI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLAEXERFISI". Cannot delete parent "CONSPACI".'
          goto error
       end
    
    /*  Cannot delete parent "CONSPACI" if children still exist in "PLCALORICO"  */
    if exists (select 1
               from   PLCALORICO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTCONSPACI = t1.DTCONSPACI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLCALORICO". Cannot delete parent "CONSPACI".'
          goto error
       end
    
    /*  Cannot delete parent "CONSPACI" if children still exist in "NUTRECOMPACI"  */
    if exists (select 1
               from   NUTRECOMPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTCONSPACI = t1.DTCONSPACI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRECOMPACI". Cannot delete parent "CONSPACI".'
          goto error
       end
    
    /*  Delete all children in "ATENREAL"  */
    delete ATENREAL
    from   ATENREAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "EXAMREAL"  */
    delete EXAMREAL
    from   EXAMREAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "AVALNUTR"  */
    delete AVALNUTR
    from   AVALNUTR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "DIAGPACI"  */
    delete DIAGPACI
    from   DIAGPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "CARDPACI"  */
    delete CARDPACI
    from   CARDPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "FREQALIMREAL"  */
    delete FREQALIMREAL
    from   FREQALIMREAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "LAUDO"  */
    delete LAUDO
    from   LAUDO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_consreluser" for table "CONSRELUSER"  */
create trigger td_consreluser on CONSRELUSER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "CAMPCONSREL"  */
    delete CAMPCONSREL
    from   CAMPCONSREL t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO
     and   t2.CDCAMPO = t1.CDCAMPO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_conssolol" for table "CONSSOLOL"  */
create trigger td_conssolol on CONSSOLOL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONSSOLOL" if children still exist in "ITEMSOLI"  */
    if exists (select 1
               from   ITEMSOLI t2, deleted t1
               where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLI". Cannot delete parent "CONSSOLOL".'
          goto error
       end
    
    /*  Delete all children in "PRODCONSSOLOL"  */
    delete PRODCONSSOLOL
    from   PRODCONSSOLOL t2, deleted t1
    where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL
    
    /*  Delete all children in "JUSTCONSSOLOL"  */
    delete JUSTCONSSOLOL
    from   JUSTCONSSOLOL t2, deleted t1
    where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_consultadw" for table "CONSULTADW"  */
create trigger td_consultadw on CONSULTADW for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONSULTADW" if children still exist in "CONFIGURADW"  */
    if exists (select 1
               from   CONFIGURADW t2, deleted t1
               where  t2.CDCONSULTADW = t1.CDCONSULTADW)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFIGURADW". Cannot delete parent "CONSULTADW".'
          goto error
       end
    
    /*  Delete all children in "CONSDWGRUPOP"  */
    delete CONSDWGRUPOP
    from   CONSDWGRUPOP t2, deleted t1
    where  t2.CDCONSULTADW = t1.CDCONSULTADW

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_consumidor" for table "CONSUMIDOR"  */
create trigger td_consumidor on CONSUMIDOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "MOVICLIE"  */
    if exists (select 1
               from   MOVICLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICLIE". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "PREVENDA"  */
    if exists (select 1
               from   PREVENDA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDA". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "SALDOCONS"  */
    if exists (select 1
               from   SALDOCONS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDOCONS". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "EXTRATOCONS"  */
    if exists (select 1
               from   EXTRATOCONS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXTRATOCONS". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "PEDICREDCONS"  */
    if exists (select 1
               from   PEDICREDCONS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDICREDCONS". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDCLIAUXVEND = t1.CDCLIENTE
                and   t2.CDCONAUXVEND = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "RALIMENT"  */
    if exists (select 1
               from   RALIMENT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RALIMENT". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "VENDAREST"  */
    if exists (select 1
               from   VENDAREST t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAREST". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "MOVCAIXADLV"  */
    if exists (select 1
               from   MOVCAIXADLV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCAIXADLV". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Cannot delete parent "CONSUMIDOR" if children still exist in "ITCOMANDAPEND"  */
    if exists (select 1
               from   ITCOMANDAPEND t2, deleted t1
               where  t2.CON_CDCLIENTE = t1.CDCLIENTE
                and   t2.CDCONSUMIDOR = t1.CDCONSUMIDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDAPEND". Cannot delete parent "CONSUMIDOR".'
          goto error
       end
    
    /*  Delete all children in "LOGTRMOVCONS"  */
    delete LOGTRMOVCONS
    from   LOGTRMOVCONS t2, deleted t1
    where  t2.CDCLIENDE = t1.CDCLIENTE
     and   t2.CDCONSDE = t1.CDCONSUMIDOR
    
    /*  Delete all children in "LOGTRMOVCONS"  */
    delete LOGTRMOVCONS
    from   LOGTRMOVCONS t2, deleted t1
    where  t2.CDCLIENPARA = t1.CDCLIENTE
     and   t2.CDCONSPARA = t1.CDCONSUMIDOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_contapagar" for table "CONTAPAGAR"  */
create trigger td_contapagar on CONTAPAGAR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONTAPAGAR" if children still exist in "AJCTPAGAR"  */
    if exists (select 1
               from   AJCTPAGAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQVENPAG = t1.NRSEQVENPAG
                and   t2.NRPARCELAPAG = t1.NRPARCELAPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTPAGAR". Cannot delete parent "CONTAPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTAPAGAR" if children still exist in "ADIANTAPAGAR"  */
    if exists (select 1
               from   ADIANTAPAGAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQVENPAG = t1.NRSEQVENPAG
                and   t2.NRPARCELAPAG = t1.NRPARCELAPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ADIANTAPAGAR". Cannot delete parent "CONTAPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTAPAGAR" if children still exist in "CONTAGERADA"  */
    if exists (select 1
               from   CONTAGERADA t2, deleted t1
               where  t2.CDFILIALPAG = t1.CDFILIAL
                and   t2.NRLANCTOPAG = t1.NRLANCTONF
                and   t2.NRSEQVENPAG = t1.NRSEQVENPAG
                and   t2.NRPARCELAPAG = t1.NRPARCELAPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTAGERADA". Cannot delete parent "CONTAPAGAR".'
          goto error
       end
    
    /*  Delete all children in "RATCTPAGAR"  */
    delete RATCTPAGAR
    from   RATCTPAGAR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQVENPAG = t1.NRSEQVENPAG
     and   t2.NRPARCELAPAG = t1.NRPARCELAPAG
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQVENPAG = t1.NRSEQVENPAG
     and   t2.NRPARCELAPAG = t1.NRPARCELAPAG

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_contareceber" for table "CONTARECEBER"  */
create trigger td_contareceber on CONTARECEBER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONTARECEBER" if children still exist in "AJCTRECEBER"  */
    if exists (select 1
               from   AJCTRECEBER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQVENREC = t1.NRSEQVENREC
                and   t2.NRPARCELAREC = t1.NRPARCELAREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTRECEBER". Cannot delete parent "CONTARECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "CONTARECEBER" if children still exist in "PARCELAVENDA"  */
    if exists (select 1
               from   PARCELAVENDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQVENREC = t1.NRSEQVENREC
                and   t2.NRPARCELAREC = t1.NRPARCELAREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCELAVENDA". Cannot delete parent "CONTARECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "CONTARECEBER" if children still exist in "ADIANTARECEB"  */
    if exists (select 1
               from   ADIANTARECEB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQVENREC = t1.NRSEQVENREC
                and   t2.NRPARCELAREC = t1.NRPARCELAREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ADIANTARECEB". Cannot delete parent "CONTARECEBER".'
          goto error
       end
    
    /*  Delete all children in "RATCTRECEBER"  */
    delete RATCTRECEBER
    from   RATCTRECEBER t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQVENREC = t1.NRSEQVENREC
     and   t2.NRPARCELAREC = t1.NRPARCELAREC
    
    /*  Delete all children in "CONTAGERADA"  */
    delete CONTAGERADA
    from   CONTAGERADA t2, deleted t1
    where  t2.CDFILIALREC = t1.CDFILIAL
     and   t2.NRLANCTOREC = t1.NRLANCTONF
     and   t2.NRSEQVENREC = t1.NRSEQVENREC
     and   t2.NRPARCELAREC = t1.NRPARCELAREC
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQVENREC = t1.NRSEQVENREC
     and   t2.NRPARCELAREC = t1.NRPARCELAREC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_contcorr" for table "CONTCORR"  */
create trigger td_contcorr on CONTCORR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BAIXARECEBIDA"  */
    if exists (select 1
               from   BAIXARECEBIDA t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXARECEBIDA". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "ITTRANSFBANC"  */
    if exists (select 1
               from   ITTRANSFBANC t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITTRANSFBANC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "SDOCCORR"  */
    if exists (select 1
               from   SDOCCORR t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SDOCCORR". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "MOVBANCARIO"  */
    if exists (select 1
               from   MOVBANCARIO t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVBANCARIO". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "MOVDEBBANC"  */
    if exists (select 1
               from   MOVDEBBANC t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVDEBBANC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "MOVCREDBANC"  */
    if exists (select 1
               from   MOVCREDBANC t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCREDBANC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BAIXAPAGA"  */
    if exists (select 1
               from   BAIXAPAGA t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXAPAGA". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CHEQUE"  */
    if exists (select 1
               from   CHEQUE t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHEQUE". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "TALAOCHEQUE"  */
    if exists (select 1
               from   TALAOCHEQUE t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TALAOCHEQUE". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "EXTRBANCO"  */
    if exists (select 1
               from   EXTRBANCO t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXTRBANCO". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BOLETPAGAR"  */
    if exists (select 1
               from   BOLETPAGAR t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETPAGAR". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BOLETRECEBER"  */
    if exists (select 1
               from   BOLETRECEBER t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETRECEBER". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CNABPAG"  */
    if exists (select 1
               from   CNABPAG t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABPAG". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CNABREC"  */
    if exists (select 1
               from   CNABREC t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABREC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARAMEMPRESA"  */
    if exists (select 1
               from   PARAMEMPRESA t2, deleted t1
               where  t2.CDREDUGBEMPR = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMEMPRESA". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDREDUGBFILI = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BANCOPREF"  */
    if exists (select 1
               from   BANCOPREF t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCOPREF". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "TIPOCTRECFIL"  */
    if exists (select 1
               from   TIPOCTRECFIL t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCTRECFIL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CODBARBOLETO"  */
    if exists (select 1
               from   CODBARBOLETO t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODBARBOLETO". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CHEQUERECE"  */
    if exists (select 1
               from   CHEQUERECE t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHEQUERECE". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARCTEMISBOL"  */
    if exists (select 1
               from   PARCTEMISBOL t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCTEMISBOL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CTRLEMISBOL"  */
    if exists (select 1
               from   CTRLEMISBOL t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLEMISBOL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CONCIMOVBANC"  */
    if exists (select 1
               from   CONCIMOVBANC t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONCIMOVBANC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "TIPORECE"  */
    if exists (select 1
               from   TIPORECE t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPORECE". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "BANCO"  */
    if exists (select 1
               from   BANCO t2, deleted t1
               where  t2.CDCONTCORVND = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCO". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDREDUCONT = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Cannot delete parent "CONTCORR" if children still exist in "CNABREC"  */
    if exists (select 1
               from   CNABREC t2, deleted t1
               where  t2.CDREDUCONTCOBR = t1.CDREDUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABREC". Cannot delete parent "CONTCORR".'
          goto error
       end
    
    /*  Delete all children in "JUROSCHEQUE"  */
    delete JUROSCHEQUE
    from   JUROSCHEQUE t2, deleted t1
    where  t2.CDREDUCONT = t1.CDREDUCONT
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDREDUCONT = t1.CDREDUCONT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_contratolinp" for table "CONTRATOLINP"  */
create trigger td_contratolinp on CONTRATOLINP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONTRATOLINP" if children still exist in "PEDILINPPROP"  */
    if exists (select 1
               from   PEDILINPPROP t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDLINPCLIE = t1.CDLINPCLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDILINPPROP". Cannot delete parent "CONTRATOLINP".'
          goto error
       end
    
    /*  Cannot delete parent "CONTRATOLINP" if children still exist in "LINPLUSO"  */
    if exists (select 1
               from   LINPLUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDLINPCLIE = t1.CDLINPCLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LINPLUSO". Cannot delete parent "CONTRATOLINP".'
          goto error
       end
    
    /*  Cannot delete parent "CONTRATOLINP" if children still exist in "PEDILINPCLIE"  */
    if exists (select 1
               from   PEDILINPCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDLINPCLIE = t1.CDLINPCLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDILINPCLIE". Cannot delete parent "CONTRATOLINP".'
          goto error
       end
    
    /*  Cannot delete parent "CONTRATOLINP" if children still exist in "MENSLINPCLIE"  */
    if exists (select 1
               from   MENSLINPCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDLINPCLIE = t1.CDLINPCLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MENSLINPCLIE". Cannot delete parent "CONTRATOLINP".'
          goto error
       end
    
    /*  Cannot delete parent "CONTRATOLINP" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDLINPCLIE = t1.CDLINPCLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "CONTRATOLINP".'
          goto error
       end
    
    /*  Delete all children in "SISTCONTLINP"  */
    delete SISTCONTLINP
    from   SISTCONTLINP t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDLINPCLIE = t1.CDLINPCLIE
    
    /*  Delete all children in "VALORLINP"  */
    delete VALORLINP
    from   VALORLINP t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDLINPCLIE = t1.CDLINPCLIE
    
    /*  Delete all children in "VENCLUSOCLIE"  */
    delete VENCLUSOCLIE
    from   VENCLUSOCLIE t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDLINPCLIE = t1.CDLINPCLIE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_contratoserv" for table "CONTRATOSERV"  */
create trigger td_contratoserv on CONTRATOSERV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CONTRATOSERV" if children still exist in "AGENDASERV"  */
    if exists (select 1
               from   AGENDASERV t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
                and   t2.CDTIPOSERV = t1.CDTIPOSERV
                and   t2.NRSERVCONT = t1.NRSERVCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASERV". Cannot delete parent "CONTRATOSERV".'
          goto error
       end
    
    /*  Cannot delete parent "CONTRATOSERV" if children still exist in "CHT"  */
    if exists (select 1
               from   CHT t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
                and   t2.CDTIPOSERV = t1.CDTIPOSERV
                and   t2.NRSERVCONT = t1.NRSERVCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHT". Cannot delete parent "CONTRATOSERV".'
          goto error
       end
    
    /*  Delete all children in "VALORSERV"  */
    delete VALORSERV
    from   VALORSERV t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
     and   t2.CDTIPOSERV = t1.CDTIPOSERV
     and   t2.NRSERVCONT = t1.NRSERVCONT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_critavafinal" for table "CRITAVAFINAL"  */
create trigger td_critavafinal on CRITAVAFINAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CRITAVAFINAL" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDTIPAUDIT = t1.CDTIPAUDIT
                and   t2.CDCRITAVAFIN = t1.CDCRITAVAFIN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "CRITAVAFINAL".'
          goto error
       end
    
    /*  Delete all children in "INSPAVAFINAL"  */
    delete INSPAVAFINAL
    from   INSPAVAFINAL t2, deleted t1
    where  t2.CDTIPAUDIT = t1.CDTIPAUDIT
     and   t2.CDCRITAVAFIN = t1.CDCRITAVAFIN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ctrlapurfat" for table "CTRLAPURFAT"  */
create trigger td_ctrlapurfat on CTRLAPURFAT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIAPFATU = t1.CDFILIAL
                and   t2.NRAPFATUTNF = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "EFECLIFAT"  */
    if exists (select 1
               from   EFECLIFAT t2, deleted t1
               where  t2.CDFILIFATU = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFECLIFAT". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "EFETCLIE"  */
    if exists (select 1
               from   EFETCLIE t2, deleted t1
               where  t2.CDFILIFATU = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCLIE". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "APURFATUGH"  */
    if exists (select 1
               from   APURFATUGH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUGH". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "MOVICLIE"  */
    if exists (select 1
               from   MOVICLIE t2, deleted t1
               where  t2.CDFILIAPFATU = t1.CDFILIAL
                and   t2.NRAPURACFATU = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICLIE". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "RELAT_MANDATO"  */
    if exists (select 1
               from   RELAT_MANDATO t2, deleted t1
               where  t2.CDFILIAPURMA = t1.CDFILIAL
                and   t2.NRAPURACFAMA = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_MANDATO". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "EFETCDRS"  */
    if exists (select 1
               from   EFETCDRS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCDRS". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "EFETRATCDRS"  */
    if exists (select 1
               from   EFETRATCDRS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETRATCDRS". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "EFETCLIEPME"  */
    if exists (select 1
               from   EFETCLIEPME t2, deleted t1
               where  t2.CDFILIFATU = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCLIEPME". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "CTRLAPCDFR"  */
    if exists (select 1
               from   CTRLAPCDFR t2, deleted t1
               where  t2.CDFILIALCENT = t1.CDFILIAL
                and   t2.NRAPURACFATU = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPCDFR". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDFILIAPFATU = t1.CDFILIAL
                and   t2.NRAPFATUTNF = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "ROMANEIO"  */
    if exists (select 1
               from   ROMANEIO t2, deleted t1
               where  t2.CDFILIFATUR = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIO". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "ITEMROMANEIO"  */
    if exists (select 1
               from   ITEMROMANEIO t2, deleted t1
               where  t2.CDFILIFATUR = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMROMANEIO". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Cannot delete parent "CTRLAPURFAT" if children still exist in "NFRELACCDRS"  */
    if exists (select 1
               from   NFRELACCDRS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRAPURACFAT = t1.NRAPURACFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACCDRS". Cannot delete parent "CTRLAPURFAT".'
          goto error
       end
    
    /*  Delete all children in "NOTAFISCAL"  */
    delete NOTAFISCAL
    from   NOTAFISCAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAPURACFAT = t1.NRAPURACFAT
    
    /*  Delete all children in "MRDPAPUR"  */
    delete MRDPAPUR
    from   MRDPAPUR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAPURACFAT = t1.NRAPURACFAT
    
    /*  Delete all children in "MRDPNTFI"  */
    delete MRDPNTFI
    from   MRDPNTFI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAPURACFAT = t1.NRAPURACFAT
    
    /*  Delete all children in "CTRAPUITPROD"  */
    delete CTRAPUITPROD
    from   CTRAPUITPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAPURACFAT = t1.NRAPURACFAT
    
    /*  Delete all children in "RELAT_CDRS"  */
    delete RELAT_CDRS
    from   RELAT_CDRS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAPURACFAT = t1.NRAPURACFAT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_custpadr" for table "CUSTPADR"  */
create trigger td_custpadr on CUSTPADR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "CUSTPADRTP"  */
    delete CUSTPADRTP
    from   CUSTPADRTP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.DTREFCUSPADR = t1.DTREFCUSPADR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_departamento" for table "DEPARTAMENTO"  */
create trigger td_departamento on DEPARTAMENTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "MOVFUNCRH"  */
    if exists (select 1
               from   MOVFUNCRH t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFUNCRH". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "SETOR"  */
    if exists (select 1
               from   SETOR t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOR". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "FUNCIONARIO"  */
    if exists (select 1
               from   FUNCIONARIO t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCIONARIO". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "OBSINTORCAEV"  */
    if exists (select 1
               from   OBSINTORCAEV t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSINTORCAEV". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDDEPTPEDORC = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "NFRELACORC"  */
    if exists (select 1
               from   NFRELACORC t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACORC". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Cannot delete parent "DEPARTAMENTO" if children still exist in "DEVOLORCAEVEN"  */
    if exists (select 1
               from   DEVOLORCAEVEN t2, deleted t1
               where  t2.CDDEPTO = t1.CDDEPTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEVOLORCAEVEN". Cannot delete parent "DEPARTAMENTO".'
          goto error
       end
    
    /*  Delete all children in "INSPRECRDEPT"  */
    delete INSPRECRDEPT
    from   INSPRECRDEPT t2, deleted t1
    where  t2.CDDEPTO = t1.CDDEPTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_diagnost" for table "DIAGNOST"  */
create trigger td_diagnost on DIAGNOST for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "DIAGPACI"  */
    if exists (select 1
               from   DIAGPACI t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAGPACI". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "DIAGINTER"  */
    if exists (select 1
               from   DIAGINTER t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAGINTER". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "RESTDIAG"  */
    if exists (select 1
               from   RESTDIAG t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTDIAG". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "AGRUCARD"  */
    if exists (select 1
               from   AGRUCARD t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGRUCARD". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "CONSNUTR"  */
    if exists (select 1
               from   CONSNUTR t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSNUTR". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Cannot delete parent "DIAGNOST" if children still exist in "PARADIETO"  */
    if exists (select 1
               from   PARADIETO t2, deleted t1
               where  t2.CDTABECDI = t1.CDTABECDI
                and   t2.CDDIAGNOST = t1.CDDIAGNOST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARADIETO". Cannot delete parent "DIAGNOST".'
          goto error
       end
    
    /*  Delete all children in "DIAGNUTR"  */
    delete DIAGNUTR
    from   DIAGNUTR t2, deleted t1
    where  t2.CDTABECDI = t1.CDTABECDI
     and   t2.CDDIAGNOST = t1.CDDIAGNOST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_dietpadr" for table "DIETPADR"  */
create trigger td_dietpadr on DIETPADR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "DIETPADR" if children still exist in "DIETSERV"  */
    if exists (select 1
               from   DIETSERV t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIETSERV". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "PRESDIET"  */
    if exists (select 1
               from   PRESDIET t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRESDIET". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "DIETPADR"  */
    if exists (select 1
               from   DIETPADR t2, deleted t1
               where  t2.CDDIETGRUP = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIETPADR". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "SERVINTE"  */
    if exists (select 1
               from   SERVINTE t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVINTE". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "SERVINTE"  */
    if exists (select 1
               from   SERVINTE t2, deleted t1
               where  t2.CDDIETGRUPSI = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVINTE". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "SETOPRODCARD"  */
    if exists (select 1
               from   SETOPRODCARD t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOPRODCARD". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "CONSPACI"  */
    if exists (select 1
               from   CONSPACI t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSPACI". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "EFETDIET"  */
    if exists (select 1
               from   EFETDIET t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETDIET". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Cannot delete parent "DIETPADR" if children still exist in "ESTATDIET"  */
    if exists (select 1
               from   ESTATDIET t2, deleted t1
               where  t2.CDDIETPAD = t1.CDDIETPAD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTATDIET". Cannot delete parent "DIETPADR".'
          goto error
       end
    
    /*  Delete all children in "RECDIEPAD"  */
    delete RECDIEPAD
    from   RECDIEPAD t2, deleted t1
    where  t2.CDDIETPAD = t1.CDDIETPAD
    
    /*  Delete all children in "CARACTPDIE"  */
    delete CARACTPDIE
    from   CARACTPDIE t2, deleted t1
    where  t2.CDDIETPAD = t1.CDDIETPAD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_dimedwgrupop" for table "DIMEDWGRUPOP"  */
create trigger td_dimedwgrupop on DIMEDWGRUPOP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FILDIMEGRUPOP"  */
    delete FILDIMEGRUPOP
    from   FILDIMEGRUPOP t2, deleted t1
    where  t2.NRSEQUEDIME = t1.NRSEQUEDIME
     and   t2.CDGRUPOPER = t1.CDGRUPOPER

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_dimensaodime" for table "DIMENSAODIME"  */
create trigger td_dimensaodime on DIMENSAODIME for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "DIMENSAODIME" if children still exist in "DIMENSAODW"  */
    if exists (select 1
               from   DIMENSAODW t2, deleted t1
               where  t2.NRSEQUEDIME = t1.NRSEQUEDIME)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIMENSAODW". Cannot delete parent "DIMENSAODIME".'
          goto error
       end
    
    /*  Cannot delete parent "DIMENSAODIME" if children still exist in "DIMENSAODRILL"  */
    if exists (select 1
               from   DIMENSAODRILL t2, deleted t1
               where  t2.NRSEQUEDIME = t1.NRSEQUEDIME)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIMENSAODRILL". Cannot delete parent "DIMENSAODIME".'
          goto error
       end
    
    /*  Cannot delete parent "DIMENSAODIME" if children still exist in "ATRIBEXTRADIME"  */
    if exists (select 1
               from   ATRIBEXTRADIME t2, deleted t1
               where  t2.NRSEQUEDIME = t1.NRSEQUEDIME)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATRIBEXTRADIME". Cannot delete parent "DIMENSAODIME".'
          goto error
       end
    
    /*  Delete all children in "DIMEODWOPER"  */
    delete DIMEODWOPER
    from   DIMEODWOPER t2, deleted t1
    where  t2.NRSEQUEDIME = t1.NRSEQUEDIME
    
    /*  Delete all children in "DIMEDWGRUPOP"  */
    delete DIMEDWGRUPOP
    from   DIMEDWGRUPOP t2, deleted t1
    where  t2.NRSEQUEDIME = t1.NRSEQUEDIME

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_dimeodwoper" for table "DIMEODWOPER"  */
create trigger td_dimeodwoper on DIMEODWOPER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FILDIMEOPER"  */
    delete FILDIMEOPER
    from   FILDIMEOPER t2, deleted t1
    where  t2.NRSEQUEDIME = t1.NRSEQUEDIME
     and   t2.CDOPERADOR = t1.CDOPERADOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_duplicata" for table "DUPLICATA"  */
create trigger td_duplicata on DUPLICATA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "DUPLICATA" if children still exist in "CONTARECEBER"  */
    if exists (select 1
               from   CONTARECEBER t2, deleted t1
               where  t2.CDFILIALDUP = t1.CDFILIAL
                and   t2.NRLANCTODUP = t1.NRLANCTODUP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTARECEBER". Cannot delete parent "DUPLICATA".'
          goto error
       end
    
    /*  Cannot delete parent "DUPLICATA" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIALDUP = t1.CDFILIAL
                and   t2.NRLANCTODUP = t1.NRLANCTODUP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "DUPLICATA".'
          goto error
       end
    
    /*  Delete all children in "AJDUPBOLET"  */
    delete AJDUPBOLET
    from   AJDUPBOLET t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTODUP = t1.NRLANCTODUP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_empresa" for table "EMPRESA"  */
create trigger td_empresa on EMPRESA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "EMPRESA" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "INCIDENCIA"  */
    if exists (select 1
               from   INCIDENCIA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCIDENCIA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ALIQUOTA"  */
    if exists (select 1
               from   ALIQUOTA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQUOTA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "INSCRMUNICIP"  */
    if exists (select 1
               from   INSCRMUNICIP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSCRMUNICIP". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "INSCRESTAD"  */
    if exists (select 1
               from   INSCRESTAD t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSCRESTAD". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CODFISCOPER"  */
    if exists (select 1
               from   CODFISCOPER t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODFISCOPER". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CODSITUTRIB"  */
    if exists (select 1
               from   CODSITUTRIB t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIB". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SALDEMPR"  */
    if exists (select 1
               from   SALDEMPR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDEMPR". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ORCAMENT"  */
    if exists (select 1
               from   ORCAMENT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAMENT". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "TERMOABERT"  */
    if exists (select 1
               from   TERMOABERT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TERMOABERT". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SALDCCUS"  */
    if exists (select 1
               from   SALDCCUS t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDCCUS". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LANCCONT"  */
    if exists (select 1
               from   LANCCONT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCCONT". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CONTOPECTBL"  */
    if exists (select 1
               from   CONTOPECTBL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTOPECTBL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SALDCOME"  */
    if exists (select 1
               from   SALDCOME t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDCOME". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CAPALOTE"  */
    if exists (select 1
               from   CAPALOTE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAPALOTE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "MOVFLUXO"  */
    if exists (select 1
               from   MOVFLUXO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFLUXO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "TRANSFBANC"  */
    if exists (select 1
               from   TRANSFBANC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRANSFBANC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "BAIXARECEBIDA"  */
    if exists (select 1
               from   BAIXARECEBIDA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXARECEBIDA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "BOLETRECEBER"  */
    if exists (select 1
               from   BOLETRECEBER t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETRECEBER". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CONTCORR"  */
    if exists (select 1
               from   CONTCORR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTCORR". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "BAIXAPAGA"  */
    if exists (select 1
               from   BAIXAPAGA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXAPAGA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "BOLETPAGAR"  */
    if exists (select 1
               from   BOLETPAGAR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETPAGAR". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "TERMABERENC"  */
    if exists (select 1
               from   TERMABERENC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TERMABERENC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "MAPAAPURAC"  */
    if exists (select 1
               from   MAPAAPURAC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPAAPURAC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "INVENTFISC"  */
    if exists (select 1
               from   INVENTFISC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVENTFISC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARAMEMPRESA"  */
    if exists (select 1
               from   PARAMEMPRESA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMEMPRESA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "EMITENTEAF"  */
    if exists (select 1
               from   EMITENTEAF t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITENTEAF". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARCMPSUP"  */
    if exists (select 1
               from   PARCMPSUP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCMPSUP". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SALDTEMP"  */
    if exists (select 1
               from   SALDTEMP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDTEMP". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "FUNCIONARIO"  */
    if exists (select 1
               from   FUNCIONARIO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCIONARIO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "EMPRCNTRFIL"  */
    if exists (select 1
               from   EMPRCNTRFIL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRCNTRFIL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CNABPAG"  */
    if exists (select 1
               from   CNABPAG t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABPAG". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CNABREC"  */
    if exists (select 1
               from   CNABREC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABREC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PRECOUTI"  */
    if exists (select 1
               from   PRECOUTI t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOUTI". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "DUPLICATA"  */
    if exists (select 1
               from   DUPLICATA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DUPLICATA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "SERVFATU"  */
    if exists (select 1
               from   SERVFATU t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVFATU". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "EMPRBANC"  */
    if exists (select 1
               from   EMPRBANC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRBANC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "EVENTO_EMPR"  */
    if exists (select 1
               from   EVENTO_EMPR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVENTO_EMPR". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "FOLHA"  */
    if exists (select 1
               from   FOLHA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOLHA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDEMPRESAPDV = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LANCCEXC"  */
    if exists (select 1
               from   LANCCEXC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCCEXC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LANCONTEX"  */
    if exists (select 1
               from   LANCONTEX t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCONTEX". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LANCOMEEX"  */
    if exists (select 1
               from   LANCOMEEX t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCOMEEX". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ORCAFINA"  */
    if exists (select 1
               from   ORCAFINA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAFINA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PREVENTRADA"  */
    if exists (select 1
               from   PREVENTRADA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENTRADA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PREVSAIDA"  */
    if exists (select 1
               from   PREVSAIDA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVSAIDA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LOGCOPECTBL"  */
    if exists (select 1
               from   LOGCOPECTBL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCOPECTBL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "EMITAFFORN"  */
    if exists (select 1
               from   EMITAFFORN t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITAFFORN". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "BEM"  */
    if exists (select 1
               from   BEM t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BEM". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "VRPRODBCIMPO"  */
    if exists (select 1
               from   VRPRODBCIMPO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VRPRODBCIMPO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CHEQUERECE"  */
    if exists (select 1
               from   CHEQUERECE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHEQUERECE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "GSCRESULTADO"  */
    if exists (select 1
               from   GSCRESULTADO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GSCRESULTADO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "INVOICE"  */
    if exists (select 1
               from   INVOICE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVOICE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PEDIDOEXOL"  */
    if exists (select 1
               from   PEDIDOEXOL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOEXOL". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "COMPRA"  */
    if exists (select 1
               from   COMPRA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPRA". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETL_RESUGER"  */
    if exists (select 1
               from   ETL_RESUGER t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUGER". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETL_RESUSERV"  */
    if exists (select 1
               from   ETL_RESUSERV t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUSERV". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "CONTRATOORCEVE"  */
    if exists (select 1
               from   CONTRATOORCEVE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOORCEVE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETLFATUINVOICE"  */
    if exists (select 1
               from   ETLFATUINVOICE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLFATUINVOICE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARAMSERIE"  */
    if exists (select 1
               from   PARAMSERIE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMSERIE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "PARCLIEFAS"  */
    if exists (select 1
               from   PARCLIEFAS t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLIEFAS". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETLDRO"  */
    if exists (select 1
               from   ETLDRO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLDRO". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETLCONTAPAG"  */
    if exists (select 1
               from   ETLCONTAPAG t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAPAG". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "ETLCONTAREC"  */
    if exists (select 1
               from   ETLCONTAREC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAREC". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Cannot delete parent "EMPRESA" if children still exist in "LOTENFE"  */
    if exists (select 1
               from   LOTENFE t2, deleted t1
               where  t2.CDEMPLOTENFE = t1.CDEMPRESA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOTENFE". Cannot delete parent "EMPRESA".'
          goto error
       end
    
    /*  Delete all children in "PARAMFOPAG"  */
    delete PARAMFOPAG
    from   PARAMFOPAG t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
    
    /*  Delete all children in "IMPOSEMPRE"  */
    delete IMPOSEMPRE
    from   IMPOSEMPRE t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_entregamerc" for table "ENTREGAMERC"  */
create trigger td_entregamerc on ENTREGAMERC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ENTREGAMERC" if children still exist in "ITEMENTREGA"  */
    if exists (select 1
               from   ITEMENTREGA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRENTREGA = t1.NRENTREGA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMENTREGA". Cannot delete parent "ENTREGAMERC".'
          goto error
       end
    
    /*  Delete all children in "RELACENTRAF"  */
    delete RELACENTRAF
    from   RELACENTRAF t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRENTREGA = t1.NRENTREGA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_equimedi" for table "EQUIMEDI"  */
create trigger td_equimedi on EQUIMEDI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "EQUIMEDI" if children still exist in "INTERNAC"  */
    if exists (select 1
               from   INTERNAC t2, deleted t1
               where  t2.CDEQUIMED = t1.CDEQUIMED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INTERNAC". Cannot delete parent "EQUIMEDI".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIMEDI" if children still exist in "CONSPACI"  */
    if exists (select 1
               from   CONSPACI t2, deleted t1
               where  t2.CDEQUIMED = t1.CDEQUIMED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSPACI". Cannot delete parent "EQUIMEDI".'
          goto error
       end
    
    /*  Delete all children in "EQUIMEDICO"  */
    delete EQUIMEDICO
    from   EQUIMEDICO t2, deleted t1
    where  t2.CDEQUIMED = t1.CDEQUIMED

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_equipaprod" for table "EQUIPAPROD"  */
create trigger td_equipaprod on EQUIPAPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "CONFEQUIPPRO"  */
    if exists (select 1
               from   CONFEQUIPPRO t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFEQUIPPRO". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "DISPONIBEQUI"  */
    if exists (select 1
               from   DISPONIBEQUI t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DISPONIBEQUI". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "EQUIPAPROC"  */
    if exists (select 1
               from   EQUIPAPROC t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROC". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "AGENEQUI"  */
    if exists (select 1
               from   AGENEQUI t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENEQUI". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "COLETDOAC"  */
    if exists (select 1
               from   COLETDOAC t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETDOAC". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "EQUIPCLIROTA"  */
    if exists (select 1
               from   EQUIPCLIROTA t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPCLIROTA". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "INSPETAPA"  */
    if exists (select 1
               from   INSPETAPA t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPETAPA". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "LOGUTEQUIP"  */
    if exists (select 1
               from   LOGUTEQUIP t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGUTEQUIP". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "PERDAOP"  */
    if exists (select 1
               from   PERDAOP t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERDAOP". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "AGENTARE"  */
    if exists (select 1
               from   AGENTARE t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENTARE". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "PERDAETAPA"  */
    if exists (select 1
               from   PERDAETAPA t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERDAETAPA". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "RECURETAPA"  */
    if exists (select 1
               from   RECURETAPA t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPA". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "RECURETAPAOP"  */
    if exists (select 1
               from   RECURETAPAOP t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPAOP". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "COMPOSINSPEC"  */
    if exists (select 1
               from   COMPOSINSPEC t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPOSINSPEC". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "PREVENDAEQU"  */
    if exists (select 1
               from   PREVENDAEQU t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAEQU". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "MOVIVALOR"  */
    if exists (select 1
               from   MOVIVALOR t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVIVALOR". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDEQUIPPAI = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "PAREQUIPAPRO"  */
    if exists (select 1
               from   PAREQUIPAPRO t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAREQUIPAPRO". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EQUIPAPROD" if children still exist in "VENDAVMN"  */
    if exists (select 1
               from   VENDAVMN t2, deleted t1
               where  t2.CDEQUIPPRO = t1.CDEQUIPPRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAVMN". Cannot delete parent "EQUIPAPROD".'
          goto error
       end
    
    /*  Delete all children in "EQUITPCONT"  */
    delete EQUITPCONT
    from   EQUITPCONT t2, deleted t1
    where  t2.CDEQUIPPRO = t1.CDEQUIPPRO
    
    /*  Delete all children in "COLETPCONT"  */
    delete COLETPCONT
    from   COLETPCONT t2, deleted t1
    where  t2.CDEQUIPPRO = t1.CDEQUIPPRO
    
    /*  Delete all children in "DESEQUIPAPRO"  */
    delete DESEQUIPAPRO
    from   DESEQUIPAPRO t2, deleted t1
    where  t2.CDEQUIPPRO = t1.CDEQUIPPRO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_espeprod" for table "ESPEPROD"  */
create trigger td_espeprod on ESPEPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ESPEPROD" if children still exist in "EQESPEPROD"  */
    if exists (select 1
               from   EQESPEPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.CDESPEPROD = t1.CDESPEPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQESPEPROD". Cannot delete parent "ESPEPROD".'
          goto error
       end
    
    /*  Cannot delete parent "ESPEPROD" if children still exist in "COMPPROD"  */
    if exists (select 1
               from   COMPPROD t2, deleted t1
               where  t2.CDPRODUTOAUX = t1.CDPRODUTO
                and   t2.CDESPPRODAUX = t1.CDESPEPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPPROD". Cannot delete parent "ESPEPROD".'
          goto error
       end
    
    /*  Delete all children in "DOCUPROD"  */
    delete DOCUPROD
    from   DOCUPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDESPEPROD = t1.CDESPEPROD
    
    /*  Delete all children in "TCOMPPRO"  */
    delete TCOMPPRO
    from   TCOMPPRO t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDESPEPROD = t1.CDESPEPROD
    
    /*  Delete all children in "VISESPEPRO"  */
    delete VISESPEPRO
    from   VISESPEPRO t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDESPEPROD = t1.CDESPEPROD
    
    /*  Delete all children in "FOTOESPEPROD"  */
    delete FOTOESPEPROD
    from   FOTOESPEPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDESPEPROD = t1.CDESPEPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_estruaten" for table "ESTRUATEN"  */
create trigger td_estruaten on ESTRUATEN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ESTRUATEN" if children still exist in "EXTRMARC"  */
    if exists (select 1
               from   EXTRMARC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC
                and   t2.CDESTRUATEN = t1.CDESTRUATEN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXTRMARC". Cannot delete parent "ESTRUATEN".'
          goto error
       end
    
    /*  Delete all children in "ETAPATEN"  */
    delete ETAPATEN
    from   ETAPATEN t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDESTRUATEN = t1.CDESTRUATEN
    
    /*  Delete all children in "RESTESTR"  */
    delete RESTESTR
    from   RESTESTR t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDESTRUATEN = t1.CDESTRUATEN
    
    /*  Delete all children in "ATENREALIZ"  */
    delete ATENREALIZ
    from   ATENREALIZ t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDESTRUATEN = t1.CDESTRUATEN
    
    /*  Delete all children in "RETOPROG"  */
    delete RETOPROG
    from   RETOPROG t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDESTRUATEN = t1.CDESTRUATEN
    
    /*  Delete all children in "LISTAESPR"  */
    delete LISTAESPR
    from   LISTAESPR t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDESTRUATEN = t1.CDESTRUATEN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_etafabric" for table "ETAFABRIC"  */
create trigger td_etafabric on ETAFABRIC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "INSUMETAPA"  */
    delete INSUMETAPA
    from   INSUMETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "CONETAPA"  */
    delete CONETAPA
    from   CONETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "PERDAETAPA"  */
    delete PERDAETAPA
    from   PERDAETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "INSPETAPA"  */
    delete INSPETAPA
    from   INSPETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "TEMPOETAFABR"  */
    delete TEMPOETAFABR
    from   TEMPOETAFABR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_etafabricop" for table "ETAFABRICOP"  */
create trigger td_etafabricop on ETAFABRICOP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ETAFABRICOP" if children still exist in "SUBPRETAPAOP"  */
    if exists (select 1
               from   SUBPRETAPAOP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPROD = t1.NRORDPROD
                and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
                and   t2.CDETAPAPROD = t1.CDETAPAPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SUBPRETAPAOP". Cannot delete parent "ETAFABRICOP".'
          goto error
       end
    
    /*  Delete all children in "CONETAPAOP"  */
    delete CONETAPAOP
    from   CONETAPAOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "PERDAOP"  */
    delete PERDAOP
    from   PERDAOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "AUDITORIA"  */
    delete AUDITORIA
    from   AUDITORIA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "RETIRADAESTO"  */
    delete RETIRADAESTO
    from   RETIRADAESTO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
    
    /*  Delete all children in "APRCUSTOP"  */
    delete APRCUSTOP
    from   APRCUSTOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDETAPAPROD = t1.CDETAPAPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_evolreqprod" for table "EVOLREQPROD"  */
create trigger td_evolreqprod on EVOLREQPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "EVOLREQPROD" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD
                and   t2.NRSEQREQPROD = t1.NRSEQREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "EVOLREQPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EVOLREQPROD" if children still exist in "EVOLREQSOLI"  */
    if exists (select 1
               from   EVOLREQSOLI t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD
                and   t2.NRSEQREQPROD = t1.NRSEQREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLREQSOLI". Cannot delete parent "EVOLREQPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EVOLREQPROD" if children still exist in "ORCAREQPROD"  */
    if exists (select 1
               from   ORCAREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD
                and   t2.NRSEQREQPROD = t1.NRSEQREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAREQPROD". Cannot delete parent "EVOLREQPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EVOLREQPROD" if children still exist in "ACORDOREQPRO"  */
    if exists (select 1
               from   ACORDOREQPRO t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD
                and   t2.NRSEQREQPROD = t1.NRSEQREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACORDOREQPRO". Cannot delete parent "EVOLREQPROD".'
          goto error
       end
    
    /*  Cannot delete parent "EVOLREQPROD" if children still exist in "LOGREQPROD"  */
    if exists (select 1
               from   LOGREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD
                and   t2.NRSEQREQPROD = t1.NRSEQREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGREQPROD". Cannot delete parent "EVOLREQPROD".'
          goto error
       end
    
    /*  Delete all children in "DESCREQPROD"  */
    delete DESCREQPROD
    from   DESCREQPROD t2, deleted t1
    where  t2.CDPRODGRUP = t1.CDPRODGRUP
     and   t2.CDREQPROD = t1.CDREQPROD
     and   t2.NRSEQREQPROD = t1.NRSEQREQPROD
    
    /*  Delete all children in "IMPACREQPROD"  */
    delete IMPACREQPROD
    from   IMPACREQPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODGRUP
     and   t2.CDREQPROD = t1.CDREQPROD
     and   t2.NRSEQREQPROD = t1.NRSEQREQPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_extrbanco" for table "EXTRBANCO"  */
create trigger td_extrbanco on EXTRBANCO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MOVREAL"  */
    delete MOVREAL
    from   MOVREAL t2, deleted t1
    where  t2.CDREDUCONT = t1.CDREDUCONT
     and   t2.CDSEQUEXTR = t1.CDSEQUEXTR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_fasetram" for table "FASETRAM"  */
create trigger td_fasetram on FASETRAM for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FASETRAM" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDFASETRAM = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "LOGFASETRAM"  */
    if exists (select 1
               from   LOGFASETRAM t2, deleted t1
               where  t2.CDFASETRAM = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGFASETRAM". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFASETRAM = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "FASETRAMDEST"  */
    if exists (select 1
               from   FASETRAMDEST t2, deleted t1
               where  t2.CDFASTRAMDES = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FASETRAMDEST". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFATRAMIWEB = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFATRAMENCE = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Cannot delete parent "FASETRAM" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFATRAMASSU = t1.CDFASETRAM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FASETRAM".'
          goto error
       end
    
    /*  Delete all children in "FASETRAMDEST"  */
    delete FASETRAMDEST
    from   FASETRAMDEST t2, deleted t1
    where  t2.CDFASETRAM = t1.CDFASETRAM

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_filial" for table "FILIAL"  */
create trigger td_filial on FILIAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FILIAL" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDFILIFISC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDFILLANC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INSCRESTAD"  */
    if exists (select 1
               from   INSCRESTAD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSCRESTAD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INSCRMUNICIP"  */
    if exists (select 1
               from   INSCRMUNICIP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSCRMUNICIP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INVEROTA"  */
    if exists (select 1
               from   INVEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FERIFILI"  */
    if exists (select 1
               from   FERIFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FERICOMP"  */
    if exists (select 1
               from   FERICOMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERICOMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPLANEJA"  */
    if exists (select 1
               from   PARAMPLANEJA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPLANEJA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SERVICO"  */
    if exists (select 1
               from   SERVICO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVICO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATECCUS"  */
    if exists (select 1
               from   RATECCUS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATECCUS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITEMRATEIO"  */
    if exists (select 1
               from   ITEMRATEIO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRATEIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCCCUSTO"  */
    if exists (select 1
               from   LANCCCUSTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCCCUSTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SALDCCUS"  */
    if exists (select 1
               from   SALDCCUS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDCCUS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "METARISL"  */
    if exists (select 1
               from   METARISL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "METARISL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRATO"  */
    if exists (select 1
               from   PRATO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RECEFILI"  */
    if exists (select 1
               from   RECEFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FOTOPRAFIL"  */
    if exists (select 1
               from   FOTOPRAFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOPRAFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRATFILI"  */
    if exists (select 1
               from   PRATFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PACIENTE"  */
    if exists (select 1
               from   PACIENTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PACIENTE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FILIOPER"  */
    if exists (select 1
               from   FILIOPER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIOPER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTNESTO"  */
    if exists (select 1
               from   CUSTNESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTNESTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREPPRAFIL"  */
    if exists (select 1
               from   PREPPRAFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPPRAFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITEMORCA"  */
    if exists (select 1
               from   ITEMORCA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVFLUXO"  */
    if exists (select 1
               from   MOVFLUXO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFLUXO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BAIXARECEBIDA"  */
    if exists (select 1
               from   BAIXARECEBIDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXARECEBIDA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BOLETRECEBER"  */
    if exists (select 1
               from   BOLETRECEBER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETRECEBER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BAIXAPAGA"  */
    if exists (select 1
               from   BAIXAPAGA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXAPAGA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BOLETPAGAR"  */
    if exists (select 1
               from   BOLETPAGAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETPAGAR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SETORCOZ"  */
    if exists (select 1
               from   SETORCOZ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETORCOZ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VAGA"  */
    if exists (select 1
               from   VAGA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VAGA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TIPONOTADEB"  */
    if exists (select 1
               from   TIPONOTADEB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPONOTADEB". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CLIENFILIAL"  */
    if exists (select 1
               from   CLIENFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFILILANCMO = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODCMPFILI"  */
    if exists (select 1
               from   PRODCMPFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCMPFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOPSOL"  */
    if exists (select 1
               from   MOVOPSOL t2, deleted t1
               where  t2.CDFILLANMOV = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPSOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVCOMPRA"  */
    if exists (select 1
               from   PREVCOMPRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMPRA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PERFILPROD"  */
    if exists (select 1
               from   PERFILPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERFILPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EMITENTEAF"  */
    if exists (select 1
               from   EMITENTEAF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITENTEAF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODNCMP"  */
    if exists (select 1
               from   PRODNCMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODNCMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RELFILTRAS"  */
    if exists (select 1
               from   RELFILTRAS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELFILTRAS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SETOTIPOPRAT"  */
    if exists (select 1
               from   SETOTIPOPRAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOTIPOPRAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MEDSOLI"  */
    if exists (select 1
               from   MEDSOLI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MEDSOLI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOPSOL"  */
    if exists (select 1
               from   MOVOPSOL t2, deleted t1
               where  t2.CDFILSOLMOV = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPSOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGNECECOMP"  */
    if exists (select 1
               from   LOGNECECOMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGNECECOMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLAPURFAT"  */
    if exists (select 1
               from   CTRLAPURFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPURFAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDFILSOLI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDFILLANCA = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDFILCENT = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVLISTA"  */
    if exists (select 1
               from   PREVLISTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVLISTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FUNCIONARIO"  */
    if exists (select 1
               from   FUNCIONARIO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCIONARIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ESCALAPADRAO"  */
    if exists (select 1
               from   ESCALAPADRAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESCALAPADRAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SIND_FILIAL"  */
    if exists (select 1
               from   SIND_FILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SIND_FILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PAR_FILIAL_RH"  */
    if exists (select 1
               from   PAR_FILIAL_RH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_FILIAL_RH". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVI_PESSOAL"  */
    if exists (select 1
               from   MOVI_PESSOAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVI_PESSOAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTOREAL"  */
    if exists (select 1
               from   CUSTOREAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOREAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTOPREVISTO"  */
    if exists (select 1
               from   CUSTOPREVISTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPREVISTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TALENTO"  */
    if exists (select 1
               from   TALENTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TALENTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FERIBANC"  */
    if exists (select 1
               from   FERIBANC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIBANC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESTSOL"  */
    if exists (select 1
               from   DESTSOL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESTSOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "META_FILIAL"  */
    if exists (select 1
               from   META_FILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "META_FILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SERVREAL"  */
    if exists (select 1
               from   SERVREAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVREAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RETIRADA_CONS"  */
    if exists (select 1
               from   RETIRADA_CONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADA_CONS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "HORBASFI"  */
    if exists (select 1
               from   HORBASFI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HORBASFI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATEIONF"  */
    if exists (select 1
               from   RATEIONF t2, deleted t1
               where  t2.CDFILICCUST = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATEIONF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INVFISCFIL"  */
    if exists (select 1
               from   INVFISCFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVFISCFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BEM"  */
    if exists (select 1
               from   BEM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BEM". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "REPLICDOC"  */
    if exists (select 1
               from   REPLICDOC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REPLICDOC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.CDFILIDESTRD = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.CDFILIORIGRD = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRECOUTI"  */
    if exists (select 1
               from   PRECOUTI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOUTI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CATRSERV"  */
    if exists (select 1
               from   CATRSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CATRSERV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "COMENSAL"  */
    if exists (select 1
               from   COMENSAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMENSAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DUPLICATA"  */
    if exists (select 1
               from   DUPLICATA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DUPLICATA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODUTOAP"  */
    if exists (select 1
               from   PRODUTOAP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOAP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDCOLETA"  */
    if exists (select 1
               from   PEDCOLETA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLETA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ROMANEIO"  */
    if exists (select 1
               from   ROMANEIO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ROMANEIO"  */
    if exists (select 1
               from   ROMANEIO t2, deleted t1
               where  t2.CDFILIRECERO = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPAJR"  */
    if exists (select 1
               from   RATTPAJR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPAJR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPAJP"  */
    if exists (select 1
               from   RATTPAJP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPAJP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPCTR"  */
    if exists (select 1
               from   RATTPCTR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPCTR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPCTP"  */
    if exists (select 1
               from   RATTPCTP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPCTP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATCTRECEBER"  */
    if exists (select 1
               from   RATCTRECEBER t2, deleted t1
               where  t2.CDFIRATCTREC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATCTRECEBER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATCTPAGAR"  */
    if exists (select 1
               from   RATCTPAGAR t2, deleted t1
               where  t2.CDFIRATCTPAG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATCTPAGAR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITEMREFE"  */
    if exists (select 1
               from   ITEMREFE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMREFE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GERANFROMA"  */
    if exists (select 1
               from   GERANFROMA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERANFROMA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GERANFROMA"  */
    if exists (select 1
               from   GERANFROMA t2, deleted t1
               where  t2.CDFILIRECEGE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERANFROMA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GERANFROMA"  */
    if exists (select 1
               from   GERANFROMA t2, deleted t1
               where  t2.CDFILIEMITGE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERANFROMA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODPREFMED"  */
    if exists (select 1
               from   PRODPREFMED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPREFMED". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ORDPRODUCAO"  */
    if exists (select 1
               from   ORDPRODUCAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORDPRODUCAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RETIRADAESTO"  */
    if exists (select 1
               from   RETIRADAESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAESTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDFCUSNFROMA = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RETIRADAPCP"  */
    if exists (select 1
               from   RETIRADAPCP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAPCP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "REQPROD"  */
    if exists (select 1
               from   REQPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLAPURFAT"  */
    if exists (select 1
               from   CTRLAPURFAT t2, deleted t1
               where  t2.CDFILIAPUR = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPURFAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGPRODUTOAP"  */
    if exists (select 1
               from   LOGPRODUTOAP t2, deleted t1
               where  t2.CDFILIAL1 = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPRODUTOAP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVFUNCRH"  */
    if exists (select 1
               from   MOVFUNCRH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFUNCRH". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDFILIALPDV = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROCESSAPROD"  */
    if exists (select 1
               from   PROCESSAPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCESSAPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DIAINVEFILI"  */
    if exists (select 1
               from   DIAINVEFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAINVEFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CLIEFILISAP"  */
    if exists (select 1
               from   CLIEFILISAP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIEFILISAP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDFILIALPED = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VISTEQFORN"  */
    if exists (select 1
               from   VISTEQFORN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISTEQFORN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGAPURQFO"  */
    if exists (select 1
               from   LOGAPURQFO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGAPURQFO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AVISQUALFOR"  */
    if exists (select 1
               from   AVISQUALFOR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AVISQUALFOR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AVISO"  */
    if exists (select 1
               from   AVISO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AVISO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ALMOXARIFADO"  */
    if exists (select 1
               from   ALMOXARIFADO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALMOXARIFADO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "APRHRTRAB"  */
    if exists (select 1
               from   APRHRTRAB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APRHRTRAB". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCDESPFATU"  */
    if exists (select 1
               from   LANCDESPFATU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCDESPFATU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "NUMERORNF"  */
    if exists (select 1
               from   NUMERORNF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUMERORNF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CIAFILI"  */
    if exists (select 1
               from   CIAFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CIAFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "OBSDELIVTMP"  */
    if exists (select 1
               from   OBSDELIVTMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSDELIVTMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVPAXTEMP"  */
    if exists (select 1
               from   PREVPAXTEMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVPAXTEMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RELPLANOVOO"  */
    if exists (select 1
               from   RELPLANOVOO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELPLANOVOO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RELPREVPAX"  */
    if exists (select 1
               from   RELPREVPAX t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELPREVPAX". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RELAT_VOOPER"  */
    if exists (select 1
               from   RELAT_VOOPER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_VOOPER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGPREVPAX2"  */
    if exists (select 1
               from   LOGPREVPAX2 t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVPAX2". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGPREVPAX"  */
    if exists (select 1
               from   LOGPREVPAX t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVPAX". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGSERVICO"  */
    if exists (select 1
               from   LOGSERVICO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGSERVICO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGVOO"  */
    if exists (select 1
               from   LOGVOO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGVOO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DELIVERY"  */
    if exists (select 1
               from   DELIVERY t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DELIVERY". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RECEITA2"  */
    if exists (select 1
               from   RECEITA2 t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEITA2". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTOPRODUTO"  */
    if exists (select 1
               from   CUSTOPRODUTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPRODUTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRECOMATDIDA"  */
    if exists (select 1
               from   PRECOMATDIDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOMATDIDA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SALA"  */
    if exists (select 1
               from   SALA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RESERVAPROD"  */
    if exists (select 1
               from   RESERVAPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVAPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EMPRESTPROD"  */
    if exists (select 1
               from   EMPRESTPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESTPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TURMAPERIODO"  */
    if exists (select 1
               from   TURMAPERIODO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAPERIODO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TIPOCTRECFIL"  */
    if exists (select 1
               from   TIPOCTRECFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCTRECFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROSPFILIAL"  */
    if exists (select 1
               from   PROSPFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPDEBI"  */
    if exists (select 1
               from   RATTPDEBI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPDEBI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RATTPCRED"  */
    if exists (select 1
               from   RATTPCRED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATTPCRED". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVDEBBANC"  */
    if exists (select 1
               from   MOVDEBBANC t2, deleted t1
               where  t2.CDFILILANDEB = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVDEBBANC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVCREDBANC"  */
    if exists (select 1
               from   MOVCREDBANC t2, deleted t1
               where  t2.CDFILILANCRE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCREDBANC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDFILORIGTRF = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDFILDESTTRF = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ATIVIDADE"  */
    if exists (select 1
               from   ATIVIDADE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVIDADE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DISPONIBFUNC"  */
    if exists (select 1
               from   DISPONIBFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DISPONIBFUNC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOTE"  */
    if exists (select 1
               from   LOTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOTE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CNABPAG"  */
    if exists (select 1
               from   CNABPAG t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABPAG". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CNABREC"  */
    if exists (select 1
               from   CNABREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABREC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ROMANEIO"  */
    if exists (select 1
               from   ROMANEIO t2, deleted t1
               where  t2.CDFILIEMITRO = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDFILORIGCLI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMETROQPR"  */
    if exists (select 1
               from   PARAMETROQPR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMETROQPR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGMATRICULA"  */
    if exists (select 1
               from   LOGMATRICULA t2, deleted t1
               where  t2.CDFILORIGLTR = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGMATRICULA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIEMIT = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIRECE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVESTMATDID"  */
    if exists (select 1
               from   MOVESTMATDID t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVESTMATDID". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ALIQIMPFIS"  */
    if exists (select 1
               from   ALIQIMPFIS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQIMPFIS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ROTFABLOCAL"  */
    if exists (select 1
               from   ROTFABLOCAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTFABLOCAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SUBGRPRODFIL"  */
    if exists (select 1
               from   SUBGRPRODFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SUBGRPRODFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ATIVAPRHR"  */
    if exists (select 1
               from   ATIVAPRHR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVAPRHR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SETORFIL"  */
    if exists (select 1
               from   SETORFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETORFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "HORAALTA"  */
    if exists (select 1
               from   HORAALTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HORAALTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EXEMPLAR"  */
    if exists (select 1
               from   EXEMPLAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXEMPLAR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PCHAVEPROD"  */
    if exists (select 1
               from   PCHAVEPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PCHAVEPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDFILIALCLIE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ATENHOSP"  */
    if exists (select 1
               from   ATENHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATENHOSP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ANDAR"  */
    if exists (select 1
               from   ANDAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ANDAR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ALA"  */
    if exists (select 1
               from   ALA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "POSTO"  */
    if exists (select 1
               from   POSTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESTFILI"  */
    if exists (select 1
               from   DESTFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESTFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DISPONIBEQUI"  */
    if exists (select 1
               from   DISPONIBEQUI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DISPONIBEQUI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AGENEQUI"  */
    if exists (select 1
               from   AGENEQUI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENEQUI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVABASEV"  */
    if exists (select 1
               from   PREVABASEV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVABASEV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AGENFUNC"  */
    if exists (select 1
               from   AGENFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENFUNC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TABEPREC"  */
    if exists (select 1
               from   TABEPREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TABEPREC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SEQNUMAVIENT"  */
    if exists (select 1
               from   SEQNUMAVIENT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SEQNUMAVIENT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESCROTLOCAL"  */
    if exists (select 1
               from   DESCROTLOCAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCROTLOCAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "HORVISITADEP"  */
    if exists (select 1
               from   HORVISITADEP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HORVISITADEP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TABEPROCFILI"  */
    if exists (select 1
               from   TABEPROCFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TABEPROCFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SIMULACAO"  */
    if exists (select 1
               from   SIMULACAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SIMULACAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INFOBRCONV"  */
    if exists (select 1
               from   INFOBRCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INFOBRCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FTFUNCMEDI"  */
    if exists (select 1
               from   FTFUNCMEDI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FTFUNCMEDI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREAUTPROD"  */
    if exists (select 1
               from   PREAUTPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREAUTPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESCPROGPR"  */
    if exists (select 1
               from   DESCPROGPR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCPROGPR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROAGRCONV"  */
    if exists (select 1
               from   PROAGRCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROAGRCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRNAUTCONV"  */
    if exists (select 1
               from   PRNAUTCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRNAUTCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TOLTMPCONV"  */
    if exists (select 1
               from   TOLTMPCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TOLTMPCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FTAUTOCONV"  */
    if exists (select 1
               from   FTAUTOCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FTAUTOCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOEDUTCONV"  */
    if exists (select 1
               from   MOEDUTCONV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOEDUTCONV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDFILIPROP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALEMP"  */
    if exists (select 1
               from   PARFILIALEMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALEMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROCEXAMDOAC"  */
    if exists (select 1
               from   PROCEXAMDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCEXAMDOAC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VISLIBPROD"  */
    if exists (select 1
               from   VISLIBPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISLIBPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMAN"  */
    if exists (select 1
               from   PARAMAN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMAN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "POOLHEMOPR"  */
    if exists (select 1
               from   POOLHEMOPR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POOLHEMOPR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDFILIPROP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROCPREPDOAC"  */
    if exists (select 1
               from   PROCPREPDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCPREPDOAC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FORMPRPROT"  */
    if exists (select 1
               from   FORMPRPROT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRPROT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIHOSP"  */
    if exists (select 1
               from   PARFILIHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIHOSP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVPROTESE"  */
    if exists (select 1
               from   MOVPROTESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVPROTESE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RETIHOSP"  */
    if exists (select 1
               from   RETIHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIHOSP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESCVALRPR"  */
    if exists (select 1
               from   DESCVALRPR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCVALRPR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVENDA"  */
    if exists (select 1
               from   PREVENDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALVMN"  */
    if exists (select 1
               from   PARFILIALVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ESPEPROD"  */
    if exists (select 1
               from   ESPEPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVIVALOR"  */
    if exists (select 1
               from   MOVIVALOR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVIVALOR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROCFABRICA"  */
    if exists (select 1
               from   PROCFABRICA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCFABRICA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AGENTARE"  */
    if exists (select 1
               from   AGENTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENTARE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGUTEQUIP"  */
    if exists (select 1
               from   LOGUTEQUIP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGUTEQUIP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVCOMP"  */
    if exists (select 1
               from   PREVCOMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TURNOFILI"  */
    if exists (select 1
               from   TURNOFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURNOFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "POSESTCONS"  */
    if exists (select 1
               from   POSESTCONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTCONS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTPROD"  */
    if exists (select 1
               from   CUSTPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGCATRACA"  */
    if exists (select 1
               from   LOGCATRACA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCATRACA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITCOMISSAO"  */
    if exists (select 1
               from   ITCOMISSAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMISSAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDFILIPADRAO = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRECCONCORR"  */
    if exists (select 1
               from   PRECCONCORR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECCONCORR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DIAUTILFIL"  */
    if exists (select 1
               from   DIAUTILFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAUTILFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALLOJ"  */
    if exists (select 1
               from   PARFILIALLOJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALLOJ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FORMPRECPROD"  */
    if exists (select 1
               from   FORMPRECPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRECPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOCORRPROD"  */
    if exists (select 1
               from   MOVOCORRPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORRPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CURVAABC"  */
    if exists (select 1
               from   CURVAABC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CURVAABC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VISITANTE"  */
    if exists (select 1
               from   VISITANTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISITANTE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ORCAFINA"  */
    if exists (select 1
               from   ORCAFINA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAFINA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CALCCUST"  */
    if exists (select 1
               from   CALCCUST t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALCCUST". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDFILICTRL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FORMPRECOCIA"  */
    if exists (select 1
               from   FORMPRECOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRECOCIA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PERCPRODFATU"  */
    if exists (select 1
               from   PERCPRODFATU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERCPRODFATU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TRABNUTRIREC"  */
    if exists (select 1
               from   TRABNUTRIREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRABNUTRIREC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVIVENDA"  */
    if exists (select 1
               from   PREVIVENDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVIVENDA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDFILIOPELOG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CODIGOBARRA"  */
    if exists (select 1
               from   CODIGOBARRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODIGOBARRA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICTROCO"  */
    if exists (select 1
               from   SOLICTROCO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICTROCO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TRAJETO"  */
    if exists (select 1
               from   TRAJETO t2, deleted t1
               where  t2.CDFILIALORI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRAJETO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TRAJETO"  */
    if exists (select 1
               from   TRAJETO t2, deleted t1
               where  t2.CDFILIALDES = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRAJETO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VENDAVMN"  */
    if exists (select 1
               from   VENDAVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VENDAVMN"  */
    if exists (select 1
               from   VENDAVMN t2, deleted t1
               where  t2.CDFILIALCLIE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVENDAEQU"  */
    if exists (select 1
               from   PREVENDAEQU t2, deleted t1
               where  t2.CDFILIALCLIE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAEQU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVENDAEQU"  */
    if exists (select 1
               from   PREVENDAEQU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAEQU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVENDAVMN"  */
    if exists (select 1
               from   PREVENDAVMN t2, deleted t1
               where  t2.CDFILIALCLIE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVENDAVMN"  */
    if exists (select 1
               from   PREVENDAVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EXPEDICAO"  */
    if exists (select 1
               from   EXPEDICAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXPEDICAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVABASEV"  */
    if exists (select 1
               from   PREVABASEV t2, deleted t1
               where  t2.CDFILIALCLIE = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVABASEV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EXPEDICAO"  */
    if exists (select 1
               from   EXPEDICAO t2, deleted t1
               where  t2.CDFILIALPED = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXPEDICAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "IMPITVENDAVM"  */
    if exists (select 1
               from   IMPITVENDAVM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPITVENDAVM". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALSAC"  */
    if exists (select 1
               from   PARFILIALSAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALSAC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ABASTEROTA"  */
    if exists (select 1
               from   ABASTEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ABASTEROTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "JUSTIFICAVMN"  */
    if exists (select 1
               from   JUSTIFICAVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTIFICAVMN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVPESQUISA"  */
    if exists (select 1
               from   MOVPESQUISA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVPESQUISA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROGPESQUISA"  */
    if exists (select 1
               from   PROGPESQUISA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGPESQUISA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TARIFADOR"  */
    if exists (select 1
               from   TARIFADOR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TARIFADOR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INVEROTALOTE"  */
    if exists (select 1
               from   INVEROTALOTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTALOTE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TAXAADMFATUR"  */
    if exists (select 1
               from   TAXAADMFATUR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAXAADMFATUR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALFAT"  */
    if exists (select 1
               from   PARFILIALFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALFAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DESPPREVFILI"  */
    if exists (select 1
               from   DESPPREVFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPPREVFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODFILIFATU"  */
    if exists (select 1
               from   PRODFILIFATU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILIFATU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLTPRECEB"  */
    if exists (select 1
               from   CTRLTPRECEB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLTPRECEB". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "EMITAFFORN"  */
    if exists (select 1
               from   EMITAFFORN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITAFFORN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RESERPRODPED"  */
    if exists (select 1
               from   RESERPRODPED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERPRODPED". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MAPARESUMECF"  */
    if exists (select 1
               from   MAPARESUMECF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPARESUMECF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREVISAOPDV"  */
    if exists (select 1
               from   PREVISAOPDV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVISAOPDV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RELAT_MANDATO"  */
    if exists (select 1
               from   RELAT_MANDATO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_MANDATO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMCMPPROD"  */
    if exists (select 1
               from   PARAMCMPPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCMPPROD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTOSUP"  */
    if exists (select 1
               from   CUSTOSUP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOSUP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FERIFUNC"  */
    if exists (select 1
               from   FERIFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIFUNC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PAREXPEDICAO"  */
    if exists (select 1
               from   PAREXPEDICAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAREXPEDICAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TROCA"  */
    if exists (select 1
               from   TROCA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TROCA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "VENDAREST"  */
    if exists (select 1
               from   VENDAREST t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAREST". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LIVROPRECO"  */
    if exists (select 1
               from   LIVROPRECO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIVROPRECO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CONFTELA"  */
    if exists (select 1
               from   CONFTELA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFTELA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AGRUPAF"  */
    if exists (select 1
               from   AGRUPAF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGRUPAF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ARQUIVIMPPOO"  */
    if exists (select 1
               from   ARQUIVIMPPOO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ARQUIVIMPPOO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "REAJSERVFAT"  */
    if exists (select 1
               from   REAJSERVFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REAJSERVFAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BREAKDOWN"  */
    if exists (select 1
               from   BREAKDOWN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BREAKDOWN". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TEMPOPREPVOO"  */
    if exists (select 1
               from   TEMPOPREPVOO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPVOO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TEMPOPREPSER"  */
    if exists (select 1
               from   TEMPOPREPSER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPSER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TEMPOPREPSUB"  */
    if exists (select 1
               from   TEMPOPREPSUB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPSUB". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TEMPOPREPREC"  */
    if exists (select 1
               from   TEMPOPREPREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPREC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "APURPRECO"  */
    if exists (select 1
               from   APURPRECO t2, deleted t1
               where  t2.CDFILCENTCMP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURPRECO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ESTOSEGU"  */
    if exists (select 1
               from   ESTOSEGU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTOSEGU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "APURPRECO"  */
    if exists (select 1
               from   APURPRECO t2, deleted t1
               where  t2.CDFILLANC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURPRECO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CALEPRODUCAO"  */
    if exists (select 1
               from   CALEPRODUCAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALEPRODUCAO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMPRODMRP"  */
    if exists (select 1
               from   PARAMPRODMRP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODMRP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROGPRODU"  */
    if exists (select 1
               from   PROGPRODU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGPRODU". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PREMATPPROGP"  */
    if exists (select 1
               from   PREMATPPROGP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREMATPPROGP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CALEPRODMRP"  */
    if exists (select 1
               from   CALEPRODMRP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALEPRODMRP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLACESPER"  */
    if exists (select 1
               from   CTRLACESPER t2, deleted t1
               where  t2.CDFILIPER = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLACESPER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ENTREGAMERC"  */
    if exists (select 1
               from   ENTREGAMERC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ENTREGAMERC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DEMAEXTRAMRP"  */
    if exists (select 1
               from   DEMAEXTRAMRP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEMAEXTRAMRP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "IMPOCONTRETI"  */
    if exists (select 1
               from   IMPOCONTRETI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOCONTRETI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCFATUDIVE"  */
    if exists (select 1
               from   LANCFATUDIVE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCFATUDIVE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INCONSMRP"  */
    if exists (select 1
               from   INCONSMRP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCONSMRP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LISTAINFLAC"  */
    if exists (select 1
               from   LISTAINFLAC t2, deleted t1
               where  t2.CDFILCENTCMP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LISTAINFLAC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GSCRESULTADO"  */
    if exists (select 1
               from   GSCRESULTADO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GSCRESULTADO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ARVTIPOCARD"  */
    if exists (select 1
               from   ARVTIPOCARD t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ARVTIPOCARD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CARDAPIOPME"  */
    if exists (select 1
               from   CARDAPIOPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDAPIOPME". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CARDIAPME"  */
    if exists (select 1
               from   CARDIAPME t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDIAPME". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GATILHO"  */
    if exists (select 1
               from   GATILHO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GATILHO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DIAFECMES"  */
    if exists (select 1
               from   DIAFECMES t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAFECMES". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARCTEMISBOL"  */
    if exists (select 1
               from   PARCTEMISBOL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCTEMISBOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLEMISBOL"  */
    if exists (select 1
               from   CTRLEMISBOL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLEMISBOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TIPOCARD"  */
    if exists (select 1
               from   TIPOCARD t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCARD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FILITIPOCARD"  */
    if exists (select 1
               from   FILITIPOCARD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILITIPOCARD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CMPCENTENDIR"  */
    if exists (select 1
               from   CMPCENTENDIR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CMPCENTENDIR". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITEMVENDMERE"  */
    if exists (select 1
               from   ITEMVENDMERE t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDMERE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODNECECOMP"  */
    if exists (select 1
               from   PRODNECECOMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODNECECOMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PAR_FILIAL_GEP"  */
    if exists (select 1
               from   PAR_FILIAL_GEP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_FILIAL_GEP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INVOICE"  */
    if exists (select 1
               from   INVOICE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVOICE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FAMSALDOFILI"  */
    if exists (select 1
               from   FAMSALDOFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FAMSALDOFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CONSSERVPME"  */
    if exists (select 1
               from   CONSSERVPME t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSERVPME". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DIASEMCONSPME"  */
    if exists (select 1
               from   DIASEMCONSPME t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIASEMCONSPME". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDIDOEXOL"  */
    if exists (select 1
               from   PEDIDOEXOL t2, deleted t1
               where  t2.CDFILLANC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOEXOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDIDOEXOL"  */
    if exists (select 1
               from   PEDIDOEXOL t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOEXOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "RALIMENT"  */
    if exists (select 1
               from   RALIMENT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RALIMENT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "UNIDMEDIFILI"  */
    if exists (select 1
               from   UNIDMEDIFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UNIDMEDIFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CTRLNUMEDELI"  */
    if exists (select 1
               from   CTRLNUMEDELI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLNUMEDELI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CUSTOLOGIST"  */
    if exists (select 1
               from   CUSTOLOGIST t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOLOGIST". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DEPARTAMENTO"  */
    if exists (select 1
               from   DEPARTAMENTO t2, deleted t1
               where  t2.CDFILIPEDORC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEPARTAMENTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDFILDESTENC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOPAF"  */
    if exists (select 1
               from   MOVOPAF t2, deleted t1
               where  t2.CDFILDESMOV = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPAF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MOVOPAF"  */
    if exists (select 1
               from   MOVOPAF t2, deleted t1
               where  t2.CDFILLANMOV = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPAF". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITEMSOLI"  */
    if exists (select 1
               from   ITEMSOLI t2, deleted t1
               where  t2.CDFILIOPELOG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CALENCARD"  */
    if exists (select 1
               from   CALENCARD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALENCARD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDIDOEXOL"  */
    if exists (select 1
               from   PEDIDOEXOL t2, deleted t1
               where  t2.CDFILIOPELOG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOEXOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CONSSOLOL"  */
    if exists (select 1
               from   CONSSOLOL t2, deleted t1
               where  t2.CDFILIOPELOG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSOLOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODCONSSOLOL"  */
    if exists (select 1
               from   PRODCONSSOLOL t2, deleted t1
               where  t2.CDFILLAN = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCONSSOLOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODCONSSOLOL"  */
    if exists (select 1
               from   PRODCONSSOLOL t2, deleted t1
               where  t2.CDFILSOLI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCONSSOLOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "OBSAPROVCARDFILI"  */
    if exists (select 1
               from   OBSAPROVCARDFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSAPROVCARDFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DATAEMBARQUEOL"  */
    if exists (select 1
               from   DATAEMBARQUEOL t2, deleted t1
               where  t2.CDFILIOPELOG = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DATAEMBARQUEOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "DATAEMBARQUEOL"  */
    if exists (select 1
               from   DATAEMBARQUEOL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DATAEMBARQUEOL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARFILIALROP"  */
    if exists (select 1
               from   PARFILIALROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALROP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "IMPOSTOROP"  */
    if exists (select 1
               from   IMPOSTOROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTOROP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TIPOOPERACROP"  */
    if exists (select 1
               from   TIPOOPERACROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERACROP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PRODUTOROP"  */
    if exists (select 1
               from   PRODUTOROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOROP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ROPFILIAL"  */
    if exists (select 1
               from   ROPFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROPFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LEITO"  */
    if exists (select 1
               from   LEITO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LEITO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETL_RESUGER"  */
    if exists (select 1
               from   ETL_RESUGER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUGER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETL_FORSALE"  */
    if exists (select 1
               from   ETL_FORSALE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_FORSALE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "CAIXAPERS"  */
    if exists (select 1
               from   CAIXAPERS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXAPERS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "IMPFISPERS"  */
    if exists (select 1
               from   IMPFISPERS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPFISPERS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "TIPOCONSFILI"  */
    if exists (select 1
               from   TIPOCONSFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCONSFILI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARPRODOPCONS"  */
    if exists (select 1
               from   PARPRODOPCONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARPRODOPCONS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOGESTATDIET"  */
    if exists (select 1
               from   LOGESTATDIET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGESTATDIET". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETL_RESUSERV"  */
    if exists (select 1
               from   ETL_RESUSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUSERV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "APURACAOROP"  */
    if exists (select 1
               from   APURACAOROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURACAOROP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PERMUTAORE"  */
    if exists (select 1
               from   PERMUTAORE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERMUTAORE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCROPFILIAL"  */
    if exists (select 1
               from   LANCROPFILIAL t2, deleted t1
               where  t2.CDFILCENTROP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCROPFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETLFATUINVOICE"  */
    if exists (select 1
               from   ETLFATUINVOICE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLFATUINVOICE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LANCATESTADO"  */
    if exists (select 1
               from   LANCATESTADO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCATESTADO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARAMSERIE"  */
    if exists (select 1
               from   PARAMSERIE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMSERIE". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "OBSERVREDZ"  */
    if exists (select 1
               from   OBSERVREDZ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSERVREDZ". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETLDRO"  */
    if exists (select 1
               from   ETLDRO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLDRO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITCOMMARKUP"  */
    if exists (select 1
               from   ITCOMMARKUP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMMARKUP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PESSOA"  */
    if exists (select 1
               from   PESSOA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PESSOA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GRTIPOPRRISL"  */
    if exists (select 1
               from   GRTIPOPRRISL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRTIPOPRRISL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "HISTFILIAL"  */
    if exists (select 1
               from   HISTFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTFILIAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PEDCOLGERAL"  */
    if exists (select 1
               from   PEDCOLGERAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLGERAL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETLCONTAPAG"  */
    if exists (select 1
               from   ETLCONTAPAG t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAPAG". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ETLCONTAREC"  */
    if exists (select 1
               from   ETLCONTAREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAREC". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "MAPDISTALMOX"  */
    if exists (select 1
               from   MAPDISTALMOX t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPDISTALMOX". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "BATIDAPONTO"  */
    if exists (select 1
               from   BATIDAPONTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BATIDAPONTO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "GRUPOFISCIMPFIL"  */
    if exists (select 1
               from   GRUPOFISCIMPFIL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPOFISCIMPFIL". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LAYOUTIMP"  */
    if exists (select 1
               from   LAYOUTIMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LAYOUTIMP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "NFRELACCDRS"  */
    if exists (select 1
               from   NFRELACCDRS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACCDRS". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "PARGERALSUV"  */
    if exists (select 1
               from   PARGERALSUV t2, deleted t1
               where  t2.CDFILSOLISUP = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARGERALSUV". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDFILISOLI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDFILAPROVSOLI = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FUNCIONARIO"  */
    if exists (select 1
               from   FUNCIONARIO t2, deleted t1
               where  t2.CDFILIALOCFUNC = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCIONARIO". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "FILIALROTA"  */
    if exists (select 1
               from   FILIALROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIALROTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "LOCK_LISTA"  */
    if exists (select 1
               from   LOCK_LISTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCK_LISTA". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "ITCOMANDAPEND"  */
    if exists (select 1
               from   ITCOMANDAPEND t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDAPEND". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "INVENTARIORETI"  */
    if exists (select 1
               from   INVENTARIORETI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVENTARIORETI". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Cannot delete parent "FILIAL" if children still exist in "AUTGRUPOP"  */
    if exists (select 1
               from   AUTGRUPOP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTGRUPOP". Cannot delete parent "FILIAL".'
          goto error
       end
    
    /*  Delete all children in "ENDEFILI"  */
    delete ENDEFILI
    from   ENDEFILI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "PARCMPSUP"  */
    delete PARCMPSUP
    from   PARCMPSUP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "PARASUPRI"  */
    delete PARASUPRI
    from   PARASUPRI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "FILINECE"  */
    delete FILINECE
    from   FILINECE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "FILINECE"  */
    delete FILINECE
    from   FILINECE t2, deleted t1
    where  t2.CDFILINECCMP = t1.CDFILIAL
    
    /*  Delete all children in "COLETAPR"  */
    delete COLETAPR
    from   COLETAPR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "PARTFILI"  */
    delete PARTFILI
    from   PARTFILI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "LIMISEGPROD"  */
    delete LIMISEGPROD
    from   LIMISEGPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "REFORFIL"  */
    delete REFORFIL
    from   REFORFIL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "CALECOMP"  */
    delete CALECOMP
    from   CALECOMP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "EMPRCNTRFIL"  */
    delete EMPRCNTRFIL
    from   EMPRCNTRFIL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "EXPAUTFILIAL"  */
    delete EXPAUTFILIAL
    from   EXPAUTFILIAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "LOGTRMOVCONS"  */
    delete LOGTRMOVCONS
    from   LOGTRMOVCONS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "FILGRUPFILI"  */
    delete FILGRUPFILI
    from   FILGRUPFILI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "INSPRECRPADR"  */
    delete INSPRECRPADR
    from   INSPRECRPADR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "INSPRECRDEPT"  */
    delete INSPRECRDEPT
    from   INSPRECRDEPT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "INSPRECRCARG"  */
    delete INSPRECRCARG
    from   INSPRECRCARG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "EXPFILCLIE"  */
    delete EXPFILCLIE
    from   EXPFILCLIE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "REGRAREVRECE"  */
    delete REGRAREVRECE
    from   REGRAREVRECE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
    
    /*  Delete all children in "PERCACRPROD"  */
    delete PERCACRPROD
    from   PERCACRPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_filioper" for table "FILIOPER"  */
create trigger td_filioper on FILIOPER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "SERVOPER"  */
    delete SERVOPER
    from   SERVOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
     and   t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_folha" for table "FOLHA"  */
create trigger td_folha on FOLHA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FUNC_FOLHA"  */
    delete FUNC_FOLHA
    from   FUNC_FOLHA t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.AAMECMPFOLHA = t1.AAMECMPFOLHA
     and   t2.NRVERSFOLHA = t1.NRVERSFOLHA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_formpgto" for table "FORMPGTO"  */
create trigger td_formpgto on FORMPGTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "PRODFORN"  */
    if exists (select 1
               from   PRODFORN t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFORN". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "RELFORMPGTO"  */
    if exists (select 1
               from   RELFORMPGTO t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELFORMPGTO". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "MODPGTO"  */
    if exists (select 1
               from   MODPGTO t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MODPGTO". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "FORMPRPROT"  */
    if exists (select 1
               from   FORMPRPROT t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRPROT". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "FORMPGTO" if children still exist in "ITMPREVLIST"  */
    if exists (select 1
               from   ITMPREVLIST t2, deleted t1
               where  t2.CDFORMPGTO = t1.CDFORMPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMPREVLIST". Cannot delete parent "FORMPGTO".'
          goto error
       end
    
    /*  Delete all children in "DIAPGTO"  */
    delete DIAPGTO
    from   DIAPGTO t2, deleted t1
    where  t2.CDFORMPGTO = t1.CDFORMPGTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_formulaorca" for table "FORMULAORCA"  */
create trigger td_formulaorca on FORMULAORCA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FORMULAORCA" if children still exist in "TIPOSERVEVEN"  */
    if exists (select 1
               from   TIPOSERVEVEN t2, deleted t1
               where  t2.CDFORMORCA = t1.CDFORMORCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOSERVEVEN". Cannot delete parent "FORMULAORCA".'
          goto error
       end
    
    /*  Cannot delete parent "FORMULAORCA" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDFORMORCA = t1.CDFORMORCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "FORMULAORCA".'
          goto error
       end
    
    /*  Delete all children in "FORMTIPOSERV"  */
    delete FORMTIPOSERV
    from   FORMTIPOSERV t2, deleted t1
    where  t2.CDFORMORCA = t1.CDFORMORCA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_fornecedor" for table "FORNECEDOR"  */
create trigger td_fornecedor on FORNECEDOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "IMPOSTO"  */
    if exists (select 1
               from   IMPOSTO t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTO". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "BOLETPAGAR"  */
    if exists (select 1
               from   BOLETPAGAR t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETPAGAR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "TESTEPROD"  */
    if exists (select 1
               from   TESTEPROD t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTEPROD". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PARTFILI"  */
    if exists (select 1
               from   PARTFILI t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTFILI". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "BEM"  */
    if exists (select 1
               from   BEM t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BEM". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "CCORFORN"  */
    if exists (select 1
               from   CCORFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CCORFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PEDCOLETA"  */
    if exists (select 1
               from   PEDCOLETA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLETA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "QFOPRODFORN"  */
    if exists (select 1
               from   QFOPRODFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QFOPRODFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "VISTEQFORN"  */
    if exists (select 1
               from   VISTEQFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISTEQFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "LOGAPURQFO"  */
    if exists (select 1
               from   LOGAPURQFO t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGAPURQFO". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PREVSAIDA"  */
    if exists (select 1
               from   PREVSAIDA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVSAIDA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ATENHOSP"  */
    if exists (select 1
               from   ATENHOSP t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATENHOSP". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "HONORATHOSP"  */
    if exists (select 1
               from   HONORATHOSP t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HONORATHOSP". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ITREQUPROC"  */
    if exists (select 1
               from   ITREQUPROC t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUPROC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDFORORIHEMO = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDFORDESHEMO = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "FORMPRPROT"  */
    if exists (select 1
               from   FORMPRPROT t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRPROT". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "VENDEDOR"  */
    if exists (select 1
               from   VENDEDOR t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDEDOR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFORNTRANSP = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "CODIGOBARRA"  */
    if exists (select 1
               from   CODIGOBARRA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODIGOBARRA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "EMITAFFORN"  */
    if exists (select 1
               from   EMITAFFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITAFFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "BEM"  */
    if exists (select 1
               from   BEM t2, deleted t1
               where  t2.CDFORNSEGURA = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BEM". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ATIVCLIEFORN"  */
    if exists (select 1
               from   ATIVCLIEFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVCLIEFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "TRANSPORT"  */
    if exists (select 1
               from   TRANSPORT t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRANSPORT". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MINFORNREG"  */
    if exists (select 1
               from   MINFORNREG t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MINFORNREG". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "FORPRCOMPRA"  */
    if exists (select 1
               from   FORPRCOMPRA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORPRCOMPRA". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDFORNOPELOG = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "CUSTOLOGIST"  */
    if exists (select 1
               from   CUSTOLOGIST t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOLOGIST". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MOBRAORCFORN"  */
    if exists (select 1
               from   MOBRAORCFORN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOBRAORCFORN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ITEMORCAEVEN"  */
    if exists (select 1
               from   ITEMORCAEVEN t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCAEVEN". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ACERTOHREXTR"  */
    if exists (select 1
               from   ACERTOHREXTR t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOHREXTR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDFORNPADORC = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "PERMUTAORE"  */
    if exists (select 1
               from   PERMUTAORE t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERMUTAORE". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDFORNERELAC = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MOVCREDBANC"  */
    if exists (select 1
               from   MOVCREDBANC t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCREDBANC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "MOVDEBBANC"  */
    if exists (select 1
               from   MOVDEBBANC t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVDEBBANC". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "FORNEPEDCOL"  */
    if exists (select 1
               from   FORNEPEDCOL t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNEPEDCOL". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "ETLCONTAPAG"  */
    if exists (select 1
               from   ETLCONTAPAG t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAPAG". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "FORNECEDOR" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDFORNBENEF = t1.CDFORNECED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "FORNECEDOR".'
          goto error
       end
    
    /*  Delete all children in "IMPOSFORN"  */
    delete IMPOSFORN
    from   IMPOSFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "ENDEFORN"  */
    delete ENDEFORN
    from   ENDEFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "MARCCOTA"  */
    delete MARCCOTA
    from   MARCCOTA t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "INTEFORN"  */
    delete INTEFORN
    from   INTEFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "REFORFIL"  */
    delete REFORFIL
    from   REFORFIL t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "REFOREG"  */
    delete REFOREG
    from   REFOREG t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "PRODFORN"  */
    delete PRODFORN
    from   PRODFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "OBSERVFORN"  */
    delete OBSERVFORN
    from   OBSERVFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "PRAZOPGTFORN"  */
    delete PRAZOPGTFORN
    from   PRAZOPGTFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
    
    /*  Delete all children in "HISTQUALFORN"  */
    delete HISTQUALFORN
    from   HISTQUALFORN t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_fornepedcol" for table "FORNEPEDCOL"  */
create trigger td_fornepedcol on FORNEPEDCOL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "REENVIOMAILPCOL"  */
    delete REENVIOMAILPCOL
    from   REENVIOMAILPCOL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDOCOL = t1.NRPEDIDOCOL
     and   t2.CDFORNECED = t1.CDFORNECED

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_func_folha" for table "FUNC_FOLHA"  */
create trigger td_func_folha on FUNC_FOLHA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "LANCAM_FOLHA"  */
    delete LANCAM_FOLHA
    from   LANCAM_FOLHA t2, deleted t1
    where  t2.AAMECMPFOLHA = t1.AAMECMPFOLHA
     and   t2.NRVERSFOLHA = t1.NRVERSFOLHA
     and   t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_funcionario" for table "FUNCIONARIO"  */
create trigger td_funcionario on FUNCIONARIO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ESTRUATEN"  */
    if exists (select 1
               from   ESTRUATEN t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTRUATEN". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ESPEPROFFUNC"  */
    if exists (select 1
               from   ESPEPROFFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROFFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "HORAPADR"  */
    if exists (select 1
               from   HORAPADR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HORAPADR". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AUSEFUNC"  */
    if exists (select 1
               from   AUSEFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUSEFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CONVACEITO"  */
    if exists (select 1
               from   CONVACEITO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVACEITO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ETAPATEN"  */
    if exists (select 1
               from   ETAPATEN t2, deleted t1
               where  t2.CDEMPRETAP = t1.CDEMPRESA
                and   t2.CDFUNCETAP = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETAPATEN". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "HOSPMEDI"  */
    if exists (select 1
               from   HOSPMEDI t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HOSPMEDI". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "RETOPROG"  */
    if exists (select 1
               from   RETOPROG t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETOPROG". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "LISTAESPR"  */
    if exists (select 1
               from   LISTAESPR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LISTAESPR". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "EXTRMARC"  */
    if exists (select 1
               from   EXTRMARC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXTRMARC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MARCACAO"  */
    if exists (select 1
               from   MARCACAO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCACAO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "APONTAMENTO"  */
    if exists (select 1
               from   APONTAMENTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APONTAMENTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "REMU_RH_FUNC"  */
    if exists (select 1
               from   REMU_RH_FUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REMU_RH_FUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ESC_TRABALHO"  */
    if exists (select 1
               from   ESC_TRABALHO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESC_TRABALHO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MOVI_PESSOAL"  */
    if exists (select 1
               from   MOVI_PESSOAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVI_PESSOAL". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "DESLIGAMENTO"  */
    if exists (select 1
               from   DESLIGAMENTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESLIGAMENTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FOTO_FUNC"  */
    if exists (select 1
               from   FOTO_FUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTO_FUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "DEPENDENTE"  */
    if exists (select 1
               from   DEPENDENTE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEPENDENTE". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "TESTEFUNC"  */
    if exists (select 1
               from   TESTEFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTEFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CPREV_DEMIT"  */
    if exists (select 1
               from   CPREV_DEMIT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CPREV_DEMIT". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "RESUMOMENSAL"  */
    if exists (select 1
               from   RESUMOMENSAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESUMOMENSAL". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CPREV_TEMP"  */
    if exists (select 1
               from   CPREV_TEMP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CPREV_TEMP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ESCALANORMAL"  */
    if exists (select 1
               from   ESCALANORMAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESCALANORMAL". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "HAB_BENEFICIO"  */
    if exists (select 1
               from   HAB_BENEFICIO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HAB_BENEFICIO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MARCSUSP"  */
    if exists (select 1
               from   MARCSUSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCSUSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AGENTEMP"  */
    if exists (select 1
               from   AGENTEMP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENTEMP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CONSMEDI"  */
    if exists (select 1
               from   CONSMEDI t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSMEDI". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "PONTO"  */
    if exists (select 1
               from   PONTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PONTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "SERVPREST"  */
    if exists (select 1
               from   SERVPREST t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREST". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FERIAS"  */
    if exists (select 1
               from   FERIAS t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIAS". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "EVENTO_FUNC"  */
    if exists (select 1
               from   EVENTO_FUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVENTO_FUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FUNC_FOLHA"  */
    if exists (select 1
               from   FUNC_FOLHA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNC_FOLHA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "VALE_TRANSP"  */
    if exists (select 1
               from   VALE_TRANSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALE_TRANSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ALTERAFUNC"  */
    if exists (select 1
               from   ALTERAFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALTERAFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "COMISSAOFUNC"  */
    if exists (select 1
               from   COMISSAOFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMISSAOFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FGTS_FUNC"  */
    if exists (select 1
               from   FGTS_FUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FGTS_FUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MOVFUNCRH"  */
    if exists (select 1
               from   MOVFUNCRH t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVFUNCRH". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FALTA"  */
    if exists (select 1
               from   FALTA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FALTA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MANUPONTO"  */
    if exists (select 1
               from   MANUPONTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MANUPONTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AUTOEXTRA"  */
    if exists (select 1
               from   AUTOEXTRA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOEXTRA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AGENDEMO"  */
    if exists (select 1
               from   AGENDEMO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDEMO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "TREINAMENT"  */
    if exists (select 1
               from   TREINAMENT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TREINAMENT". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "APRHRTRAB"  */
    if exists (select 1
               from   APRHRTRAB t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APRHRTRAB". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "BATIDAPONTO"  */
    if exists (select 1
               from   BATIDAPONTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BATIDAPONTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "NOTAAVAL"  */
    if exists (select 1
               from   NOTAAVAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAAVAL". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "TURMAPAD"  */
    if exists (select 1
               from   TURMAPAD t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAPAD". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "TURMAPERIODO"  */
    if exists (select 1
               from   TURMAPERIODO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAPERIODO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "DISPONIBFUNC"  */
    if exists (select 1
               from   DISPONIBFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DISPONIBFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "RESERVATURMA"  */
    if exists (select 1
               from   RESERVATURMA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVATURMA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "DISCIPLIESTA"  */
    if exists (select 1
               from   DISCIPLIESTA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DISCIPLIESTA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AULATURMA"  */
    if exists (select 1
               from   AULATURMA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AULATURMA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "PROSPECT"  */
    if exists (select 1
               from   PROSPECT t2, deleted t1
               where  t2.CDEMPFCAPPRO = t1.CDEMPRESA
                and   t2.CDFUNCCAPPRO = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPECT". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AGEATIVISALA"  */
    if exists (select 1
               from   AGEATIVISALA t2, deleted t1
               where  t2.CDEMPFSOLAGS = t1.CDEMPRESA
                and   t2.CDFUNCSOLAGS = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGEATIVISALA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "EMPRESTPROD"  */
    if exists (select 1
               from   EMPRESTPROD t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESTPROD". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "RESERVAPROD"  */
    if exists (select 1
               from   RESERVAPROD t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVAPROD". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "PARTATIVSALA"  */
    if exists (select 1
               from   PARTATIVSALA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTATIVSALA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDEMPFRESMAT = t1.CDEMPRESA
                and   t2.CDFUNCRESMAT = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MATRICULA"  */
    if exists (select 1
               from   MATRICULA t2, deleted t1
               where  t2.CDEMPFCAPMAT = t1.CDEMPRESA
                and   t2.CDFUNCCAPMAT = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATRICULA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDEMPFCAPCLI = t1.CDEMPRESA
                and   t2.CDFUNCCAPCLI = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDEMPFAPLTES = t1.CDEMPRESA
                and   t2.CDFUNCAPLTES = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FUNCESTA"  */
    if exists (select 1
               from   FUNCESTA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCESTA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "PROGRAMAULA"  */
    if exists (select 1
               from   PROGRAMAULA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGRAMAULA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "MOVESTMATDID"  */
    if exists (select 1
               from   MOVESTMATDID t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVESTMATDID". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ATIVAPRHR"  */
    if exists (select 1
               from   ATIVAPRHR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVAPRHR". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FUNCATHOSP"  */
    if exists (select 1
               from   FUNCATHOSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCATHOSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "HONORATHOSP"  */
    if exists (select 1
               from   HONORATHOSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HONORATHOSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "REQUPROC"  */
    if exists (select 1
               from   REQUPROC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUPROC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ITREQUPROC"  */
    if exists (select 1
               from   ITREQUPROC t2, deleted t1
               where  t2.CDEMPFUNCTEC = t1.CDEMPRESA
                and   t2.CDFUNRESPTEC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUPROC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "ITREQUPROC"  */
    if exists (select 1
               from   ITREQUPROC t2, deleted t1
               where  t2.CDEMPFUNCLAU = t1.CDEMPRESA
                and   t2.CDFUNRESPLAU = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUPROC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AGENFUNC"  */
    if exists (select 1
               from   AGENFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "EXAMDOACAO"  */
    if exists (select 1
               from   EXAMDOACAO t2, deleted t1
               where  t2.CDEMPFUNCEXD = t1.CDEMPRESA
                and   t2.CDFUNRESPEXD = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXAMDOACAO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AFERESE"  */
    if exists (select 1
               from   AFERESE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AFERESE". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FMEDFUNC"  */
    if exists (select 1
               from   FMEDFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FMEDFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "OPERADOR"  */
    if exists (select 1
               from   OPERADOR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERADOR". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "LIMCONVACE"  */
    if exists (select 1
               from   LIMCONVACE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMCONVACE". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "LOGPONTO"  */
    if exists (select 1
               from   LOGPONTO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPONTO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FERIFUNC"  */
    if exists (select 1
               from   FERIFUNC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIFUNC". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "FUNCRELEQUIP"  */
    if exists (select 1
               from   FUNCRELEQUIP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FUNCRELEQUIP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "AGENDASERV"  */
    if exists (select 1
               from   AGENDASERV t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASERV". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "RATECOMILINP"  */
    if exists (select 1
               from   RATECOMILINP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATECOMILINP". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "CHT"  */
    if exists (select 1
               from   CHT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHT". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "LANCATESTADO"  */
    if exists (select 1
               from   LANCATESTADO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCATESTADO". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Cannot delete parent "FUNCIONARIO" if children still exist in "PESSOA"  */
    if exists (select 1
               from   PESSOA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PESSOA". Cannot delete parent "FUNCIONARIO".'
          goto error
       end
    
    /*  Delete all children in "OBSERVFUNC"  */
    delete OBSERVFUNC
    from   OBSERVFUNC t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
    
    /*  Delete all children in "HORBASFU"  */
    delete HORBASFU
    from   HORBASFU t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
    
    /*  Delete all children in "AFASTAMENTO"  */
    delete AFASTAMENTO
    from   AFASTAMENTO t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_geranfroma" for table "GERANFROMA"  */
create trigger td_geranfroma on GERANFROMA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ROMANEIONF"  */
    delete ROMANEIONF
    from   ROMANEIONF t2, deleted t1
    where  t2.CDFILIALNFRO = t1.CDFILIAL
     and   t2.NRGERANFROMA = t1.NRGERANFROMA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_gerente" for table "GERENTE"  */
create trigger td_gerente on GERENTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GERENTE" if children still exist in "LIMGERPROD"  */
    if exists (select 1
               from   LIMGERPROD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMGERPROD". Cannot delete parent "GERENTE".'
          goto error
       end
    
    /*  Cannot delete parent "GERENTE" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDGERENTE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "GERENTE".'
          goto error
       end
    
    /*  Cannot delete parent "GERENTE" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDGEREORCFIN = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "GERENTE".'
          goto error
       end
    
    /*  Delete all children in "GERECOMP"  */
    delete GERECOMP
    from   GERECOMP t2, deleted t1
    where  t2.CDOPERGERE = t1.CDOPERADOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_grequivale" for table "GREQUIVALE"  */
create trigger td_grequivale on GREQUIVALE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GREQUIVALE" if children still exist in "CARDPADR"  */
    if exists (select 1
               from   CARDPADR t2, deleted t1
               where  t2.CDGREQUIVALE = t1.CDGREQUIVALE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDPADR". Cannot delete parent "GREQUIVALE".'
          goto error
       end
    
    /*  Cannot delete parent "GREQUIVALE" if children still exist in "CARDCONSNUTR"  */
    if exists (select 1
               from   CARDCONSNUTR t2, deleted t1
               where  t2.CDGREQUIVALE = t1.CDGREQUIVALE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDCONSNUTR". Cannot delete parent "GREQUIVALE".'
          goto error
       end
    
    /*  Delete all children in "ITEQUIVALE"  */
    delete ITEQUIVALE
    from   ITEQUIVALE t2, deleted t1
    where  t2.CDGREQUIVALE = t1.CDGREQUIVALE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_grupfili" for table "GRUPFILI"  */
create trigger td_grupfili on GRUPFILI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GRUPFILI" if children still exist in "PARFILIALVMN"  */
    if exists (select 1
               from   PARFILIALVMN t2, deleted t1
               where  t2.CDGRUPFILI = t1.CDGRUPFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALVMN". Cannot delete parent "GRUPFILI".'
          goto error
       end
    
    /*  Delete all children in "FILGRUPFILI"  */
    delete FILGRUPFILI
    from   FILGRUPFILI t2, deleted t1
    where  t2.CDGRUPFILI = t1.CDGRUPFILI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_grupoper" for table "GRUPOPER"  */
create trigger td_grupoper on GRUPOPER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "OPERGRUPOP"  */
    if exists (select 1
               from   OPERGRUPOP t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERGRUPOP". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMESTOQUE"  */
    if exists (select 1
               from   PARAMESTOQUE t2, deleted t1
               where  t2.CDGROPERRETI = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMESTOQUE". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERRESE = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERREEX = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMESTOQUE"  */
    if exists (select 1
               from   PARAMESTOQUE t2, deleted t1
               where  t2.CDGROPERFMES = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMESTOQUE". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERREPA = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARASUPRI"  */
    if exists (select 1
               from   PARASUPRI t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARASUPRI". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDGROPERAPRO = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMESTOQUE"  */
    if exists (select 1
               from   PARAMESTOQUE t2, deleted t1
               where  t2.CDGROPERSOLI = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMESTOQUE". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "GROPERVISTO"  */
    if exists (select 1
               from   GROPERVISTO t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GROPERVISTO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERPDV = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDGROPERDESC = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERPDV1 = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERPROP = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERPROR = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "GERAL_PAG"  */
    if exists (select 1
               from   GERAL_PAG t2, deleted t1
               where  t2.CDGROPERCNTP = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_PAG". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDGROPERCNTR = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDGROPERPROB = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "AUTGRUPOP"  */
    if exists (select 1
               from   AUTGRUPOP t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTGRUPOP". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARGERALVMN"  */
    if exists (select 1
               from   PARGERALVMN t2, deleted t1
               where  t2.CDGROPERABAS = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARGERALVMN". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "FASETRAM"  */
    if exists (select 1
               from   FASETRAM t2, deleted t1
               where  t2.CDGROPERRESP = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FASETRAM". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PAREXPEDICAO"  */
    if exists (select 1
               from   PAREXPEDICAO t2, deleted t1
               where  t2.CDGROPERTROC = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAREXPEDICAO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDGROPLIBINC = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARASUPRI"  */
    if exists (select 1
               from   PARASUPRI t2, deleted t1
               where  t2.CDGRUPAPRSOL = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARASUPRI". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDGROPCANDEL = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "GRUPOPROJETO"  */
    if exists (select 1
               from   GRUPOPROJETO t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPOPROJETO". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "CONSULTADW"  */
    if exists (select 1
               from   CONSULTADW t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSULTADW". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMESTOQUE"  */
    if exists (select 1
               from   PARAMESTOQUE t2, deleted t1
               where  t2.CDGROPERREQ = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMESTOQUE". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDGROPEPROPRAZO = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "ITEMDWGRUPOP"  */
    if exists (select 1
               from   ITEMDWGRUPOP t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMDWGRUPOP". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "DIMEDWGRUPOP"  */
    if exists (select 1
               from   DIMEDWGRUPOP t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIMEDWGRUPOP". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPER" if children still exist in "GROPMAPARESULT"  */
    if exists (select 1
               from   GROPMAPARESULT t2, deleted t1
               where  t2.CDGRUPOPER = t1.CDGRUPOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GROPMAPARESULT". Cannot delete parent "GRUPOPER".'
          goto error
       end
    
    /*  Delete all children in "PROGGRUPOP"  */
    delete PROGGRUPOP
    from   PROGGRUPOP t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER
    
    /*  Delete all children in "GROPERALMOX"  */
    delete GROPERALMOX
    from   GROPERALMOX t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER
    
    /*  Delete all children in "GROPERLESTOQ"  */
    delete GROPERLESTOQ
    from   GROPERLESTOQ t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER
    
    /*  Delete all children in "MENUGRUPOP"  */
    delete MENUGRUPOP
    from   MENUGRUPOP t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER
    
    /*  Delete all children in "CONSDWGRUPOP"  */
    delete CONSDWGRUPOP
    from   CONSDWGRUPOP t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_grupoprojeto" for table "GRUPOPROJETO"  */
create trigger td_grupoprojeto on GRUPOPROJETO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GRUPOPROJETO" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "GRUPOPROJETO".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPROJETO" if children still exist in "BASELINEGRPROJ"  */
    if exists (select 1
               from   BASELINEGRPROJ t2, deleted t1
               where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BASELINEGRPROJ". Cannot delete parent "GRUPOPROJETO".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPROJETO" if children still exist in "OPERGRPROJETO"  */
    if exists (select 1
               from   OPERGRPROJETO t2, deleted t1
               where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERGRPROJETO". Cannot delete parent "GRUPOPROJETO".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPROJETO" if children still exist in "ACORDOGRPROJETO"  */
    if exists (select 1
               from   ACORDOGRPROJETO t2, deleted t1
               where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACORDOGRPROJETO". Cannot delete parent "GRUPOPROJETO".'
          goto error
       end
    
    /*  Cannot delete parent "GRUPOPROJETO" if children still exist in "SOLGRACORDOPROJ"  */
    if exists (select 1
               from   SOLGRACORDOPROJ t2, deleted t1
               where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLGRACORDOPROJ". Cannot delete parent "GRUPOPROJETO".'
          goto error
       end
    
    /*  Delete all children in "DATAGRPROJETO"  */
    delete DATAGRPROJETO
    from   DATAGRPROJETO t2, deleted t1
    where  t2.CDGRUPOPROJ = t1.CDGRUPOPROJ

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_gruprocesso" for table "GRUPROCESSO"  */
create trigger td_gruprocesso on GRUPROCESSO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PARAMGPROC"  */
    delete PARAMGPROC
    from   PARAMGPROC t2, deleted t1
    where  t2.CDGRUPROCESSO = t1.CDGRUPROCESSO
    
    /*  Delete all children in "PROCESGPROC"  */
    delete PROCESGPROC
    from   PROCESGPROC t2, deleted t1
    where  t2.CDGRUPROCESSO = t1.CDGRUPROCESSO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_grupservfat" for table "GRUPSERVFAT"  */
create trigger td_grupservfat on GRUPSERVFAT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "GRUPSERVFAT" if children still exist in "SERVCLIE"  */
    if exists (select 1
               from   SERVCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDGRSERVFAT = t1.CDGRSERVFAT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIE". Cannot delete parent "GRUPSERVFAT".'
          goto error
       end
    
    /*  Delete all children in "CORRPRECGRUP"  */
    delete CORRPRECGRUP
    from   CORRPRECGRUP t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDGRSERVFAT = t1.CDGRSERVFAT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_habilidade" for table "HABILIDADE"  */
create trigger td_habilidade on HABILIDADE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "RECURETAPA"  */
    if exists (select 1
               from   RECURETAPA t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPA". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "RECURETAPAOP"  */
    if exists (select 1
               from   RECURETAPAOP t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPAOP". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "HABIPROCESSO"  */
    if exists (select 1
               from   HABIPROCESSO t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABIPROCESSO". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "COMPOSINSPEC"  */
    if exists (select 1
               from   COMPOSINSPEC t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPOSINSPEC". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "HABIGRUPCARG"  */
    if exists (select 1
               from   HABIGRUPCARG t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABIGRUPCARG". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "HABILPROJETO"  */
    if exists (select 1
               from   HABILPROJETO t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABILPROJETO". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Cannot delete parent "HABILIDADE" if children still exist in "HABILOPERPRJ"  */
    if exists (select 1
               from   HABILOPERPRJ t2, deleted t1
               where  t2.CDHABILIDADE = t1.CDHABILIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABILOPERPRJ". Cannot delete parent "HABILIDADE".'
          goto error
       end
    
    /*  Delete all children in "HABILOPER"  */
    delete HABILOPER
    from   HABILOPER t2, deleted t1
    where  t2.CDHABILIDADE = t1.CDHABILIDADE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_historcaeven" for table "HISTORCAEVEN"  */
create trigger td_historcaeven on HISTORCAEVEN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "OBSJUSTORCA"  */
    delete OBSJUSTORCA
    from   OBSJUSTORCA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.CDOPERADOR = t1.CDOPERADOR
     and   t2.DTHRHISTORCA = t1.DTHRHISTORCA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_histpadr" for table "HISTPADR"  */
create trigger td_histpadr on HISTPADR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "HISTPADR" if children still exist in "LANCLOTE"  */
    if exists (select 1
               from   LANCLOTE t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCLOTE". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "LANCCONT"  */
    if exists (select 1
               from   LANCCONT t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCCONT". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOAJRECEBER"  */
    if exists (select 1
               from   TIPOAJRECEBER t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOAJRECEBER". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TPCREDBANC"  */
    if exists (select 1
               from   TPCREDBANC t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPCREDBANC". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TPDEBIBANC"  */
    if exists (select 1
               from   TPDEBIBANC t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPDEBIBANC". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOAJPAGAR"  */
    if exists (select 1
               from   TIPOAJPAGAR t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOAJPAGAR". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "HISTINTEGRA"  */
    if exists (select 1
               from   HISTINTEGRA t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTINTEGRA". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOOPERAC"  */
    if exists (select 1
               from   TIPOOPERAC t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERAC". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPORETI"  */
    if exists (select 1
               from   TIPORETI t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPORETI". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "CUSTOINDI"  */
    if exists (select 1
               from   CUSTOINDI t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOINDI". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOCTPAGAR"  */
    if exists (select 1
               from   TIPOCTPAGAR t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCTPAGAR". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOCTRECEBER"  */
    if exists (select 1
               from   TIPOCTRECEBER t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCTRECEBER". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOBAIXAPAG"  */
    if exists (select 1
               from   TIPOBAIXAPAG t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOBAIXAPAG". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Cannot delete parent "HISTPADR" if children still exist in "TIPOBAIXAREC"  */
    if exists (select 1
               from   TIPOBAIXAREC t2, deleted t1
               where  t2.CDHISTPADR = t1.CDHISTPADR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOBAIXAREC". Cannot delete parent "HISTPADR".'
          goto error
       end
    
    /*  Delete all children in "HISTTIPOLANC"  */
    delete HISTTIPOLANC
    from   HISTTIPOLANC t2, deleted t1
    where  t2.CDHISTPADR = t1.CDHISTPADR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_hospital" for table "HOSPITAL"  */
create trigger td_hospital on HOSPITAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "HOSPMEDI"  */
    delete HOSPMEDI
    from   HOSPMEDI t2, deleted t1
    where  t2.CDHOSPITAL = t1.CDHOSPITAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_imposnftot" for table "IMPOSNFTOT"  */
create trigger td_imposnftot on IMPOSNFTOT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDIMPOSTOT = t1.CDIMPOSTO
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_imposto" for table "IMPOSTO"  */
create trigger td_imposto on IMPOSTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "ITEMPROIMPOS"  */
    if exists (select 1
               from   ITEMPROIMPOS t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPROIMPOS". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSTONF"  */
    if exists (select 1
               from   IMPOSTONF t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTONF". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "INCIDENCIA"  */
    if exists (select 1
               from   INCIDENCIA t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCIDENCIA". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "ALIQUOTA"  */
    if exists (select 1
               from   ALIQUOTA t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQUOTA". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSFORN"  */
    if exists (select 1
               from   IMPOSFORN t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSFORN". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "BASECALC"  */
    if exists (select 1
               from   BASECALC t2, deleted t1
               where  t2.CDIMPOBCALC = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BASECALC". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSNFTOT"  */
    if exists (select 1
               from   IMPOSNFTOT t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSNFTOT". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "TERMABERENC"  */
    if exists (select 1
               from   TERMABERENC t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TERMABERENC". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "MAPAAPURAC"  */
    if exists (select 1
               from   MAPAAPURAC t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPAAPURAC". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "RELATIMPOS"  */
    if exists (select 1
               from   RELATIMPOS t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELATIMPOS". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSERIE"  */
    if exists (select 1
               from   IMPOSERIE t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSERIE". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPITMAF"  */
    if exists (select 1
               from   IMPITMAF t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPITMAF". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPCOLET"  */
    if exists (select 1
               from   IMPCOLET t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPCOLET". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSTO"  */
    if exists (select 1
               from   IMPOSTO t2, deleted t1
               where  t2.CDIMPOSAPUR = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTO". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "RELACIMPOSTO"  */
    if exists (select 1
               from   RELACIMPOSTO t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACIMPOSTO". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSMRDP"  */
    if exists (select 1
               from   IMPOSMRDP t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSMRDP". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSINFUSU"  */
    if exists (select 1
               from   IMPOSINFUSU t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSINFUSU". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "PAREXPEDICAO"  */
    if exists (select 1
               from   PAREXPEDICAO t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAREXPEDICAO". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "ALIQIMPFIS"  */
    if exists (select 1
               from   ALIQIMPFIS t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQIMPFIS". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "ITVENDAIMPOS"  */
    if exists (select 1
               from   ITVENDAIMPOS t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDAIMPOS". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSTOREDZ"  */
    if exists (select 1
               from   IMPOSTOREDZ t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTOREDZ". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPITVENDAVM"  */
    if exists (select 1
               from   IMPITVENDAVM t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPITVENDAVM". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSTO"  */
    if exists (select 1
               from   IMPOSTO t2, deleted t1
               where  t2.CDIMPOSTRIB = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTO". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "PARAMFISCAL"  */
    if exists (select 1
               from   PARAMFISCAL t2, deleted t1
               where  t2.CDIMPATIVPER = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFISCAL". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "CTBIMPTPOPER"  */
    if exists (select 1
               from   CTBIMPTPOPER t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTBIMPTPOPER". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPOSTOROP"  */
    if exists (select 1
               from   IMPOSTOROP t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTOROP". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "IMPPEDCOLETA"  */
    if exists (select 1
               from   IMPPEDCOLETA t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPPEDCOLETA". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "TPIMPOSLIVRO"  */
    if exists (select 1
               from   TPIMPOSLIVRO t2, deleted t1
               where  t2.CDIMPOSTOLIVRO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPIMPOSLIVRO". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "GRUPOFISCIMPFIL"  */
    if exists (select 1
               from   GRUPOFISCIMPFIL t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPOFISCIMPFIL". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Cannot delete parent "IMPOSTO" if children still exist in "CODSITUTRIBNFE"  */
    if exists (select 1
               from   CODSITUTRIBNFE t2, deleted t1
               where  t2.CDIMPOSTO = t1.CDIMPOSTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIBNFE". Cannot delete parent "IMPOSTO".'
          goto error
       end
    
    /*  Delete all children in "BASECALC"  */
    delete BASECALC
    from   BASECALC t2, deleted t1
    where  t2.CDIMPOSTO = t1.CDIMPOSTO
    
    /*  Delete all children in "CCTBIMPFORN"  */
    delete CCTBIMPFORN
    from   CCTBIMPFORN t2, deleted t1
    where  t2.CDIMPOSTO = t1.CDIMPOSTO
    
    /*  Delete all children in "DIFEALIQ"  */
    delete DIFEALIQ
    from   DIFEALIQ t2, deleted t1
    where  t2.CDIMPOSTO = t1.CDIMPOSTO
    
    /*  Delete all children in "IMPOSEMPRE"  */
    delete IMPOSEMPRE
    from   IMPOSEMPRE t2, deleted t1
    where  t2.CDIMPOSTO = t1.CDIMPOSTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_impressora" for table "IMPRESSORA"  */
create trigger td_impressora on IMPRESSORA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "IMPRESSORA" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.CDIMPRESSORA = t1.CDIMPRESSORA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "IMPRESSORA".'
          goto error
       end
    
    /*  Cannot delete parent "IMPRESSORA" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.CDIMPRESSORA2 = t1.CDIMPRESSORA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "IMPRESSORA".'
          goto error
       end
    
    /*  Delete all children in "IMPRESSOOPER"  */
    delete IMPRESSOOPER
    from   IMPRESSOOPER t2, deleted t1
    where  t2.CDIMPRESSORA = t1.CDIMPRESSORA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_incidencia" for table "INCIDENCIA"  */
create trigger td_incidencia on INCIDENCIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MENSINCID"  */
    delete MENSINCID
    from   MENSINCID t2, deleted t1
    where  t2.NRINCIDENCIA = t1.NRINCIDENCIA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_indice" for table "INDICE"  */
create trigger td_indice on INDICE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INDICE" if children still exist in "CIAFILI"  */
    if exists (select 1
               from   CIAFILI t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CIAFILI". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "ORCAMENT"  */
    if exists (select 1
               from   ORCAMENT t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAMENT". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PARASUPRI"  */
    if exists (select 1
               from   PARASUPRI t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARASUPRI". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICEMF = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "TIPORECE"  */
    if exists (select 1
               from   TIPORECE t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPORECE". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "MOEDUTCONV"  */
    if exists (select 1
               from   MOEDUTCONV t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOEDUTCONV". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PARFILIALLOJ"  */
    if exists (select 1
               from   PARFILIALLOJ t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALLOJ". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PARAMGERAL"  */
    if exists (select 1
               from   PARAMGERAL t2, deleted t1
               where  t2.CDTPINDICE2 = t1.CDTPINDICE
                and   t2.IDPERVALOR2 = t1.IDPERVALOR
                and   t2.CDINDICEMF2 = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMGERAL". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "INVOICE"  */
    if exists (select 1
               from   INVOICE t2, deleted t1
               where  t2.CDTPINDICE = t1.CDTPINDICE
                and   t2.IDPERVALOR = t1.IDPERVALOR
                and   t2.CDINDICE = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVOICE". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDTPINDICE1 = t1.CDTPINDICE
                and   t2.IDPERVALOR1 = t1.IDPERVALOR
                and   t2.CDINDICE1 = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Cannot delete parent "INDICE" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDTPINDICE2 = t1.CDTPINDICE
                and   t2.IDPERVALOR2 = t1.IDPERVALOR
                and   t2.CDINDICE2 = t1.CDINDICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "INDICE".'
          goto error
       end
    
    /*  Delete all children in "MOVINDICE"  */
    delete MOVINDICE
    from   MOVINDICE t2, deleted t1
    where  t2.CDTPINDICE = t1.CDTPINDICE
     and   t2.IDPERVALOR = t1.IDPERVALOR
     and   t2.CDINDICE = t1.CDINDICE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inquerpaci" for table "INQUERPACI"  */
create trigger td_inquerpaci on INQUERPACI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITINQUERPACI"  */
    delete ITINQUERPACI
    from   ITINQUERPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTINIINQU = t1.DTINIINQU
     and   t2.DTFININQU = t1.DTFININQU

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inscrestad" for table "INSCRESTAD"  */
create trigger td_inscrestad on INSCRESTAD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "INCIDENCIA"  */
    if exists (select 1
               from   INCIDENCIA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCIDENCIA". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "ALIQUOTA"  */
    if exists (select 1
               from   ALIQUOTA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQUOTA". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "SERIENF"  */
    if exists (select 1
               from   SERIENF t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERIENF". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "CODFISCOPER"  */
    if exists (select 1
               from   CODFISCOPER t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODFISCOPER". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "CODSITUTRIB"  */
    if exists (select 1
               from   CODSITUTRIB t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIB". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "MAPAAPURAC"  */
    if exists (select 1
               from   MAPAAPURAC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPAAPURAC". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "INVENTFISC"  */
    if exists (select 1
               from   INVENTFISC t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVENTFISC". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "EMPRCNTRFIL"  */
    if exists (select 1
               from   EMPRCNTRFIL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRCNTRFIL". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "EMPRESA"  */
    if exists (select 1
               from   EMPRESA t2, deleted t1
               where  t2.CDEMPRIEST = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESA". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "CODTRIBSAP"  */
    if exists (select 1
               from   CODTRIBSAP t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODTRIBSAP". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "VRPRODBCIMPO"  */
    if exists (select 1
               from   VRPRODBCIMPO t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VRPRODBCIMPO". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Cannot delete parent "INSCRESTAD" if children still exist in "PARAMSERIE"  */
    if exists (select 1
               from   PARAMSERIE t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMSERIE". Cannot delete parent "INSCRESTAD".'
          goto error
       end
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDINSCESTA = t1.CDINSCESTA
    
    /*  Delete all children in "PARAMDIEFCE"  */
    delete PARAMDIEFCE
    from   PARAMDIEFCE t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDINSCESTA = t1.CDINSCESTA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inspecao" for table "INSPECAO"  */
create trigger td_inspecao on INSPECAO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPECPROD"  */
    if exists (select 1
               from   INSPECPROD t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECPROD". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPTIPAUDIT"  */
    if exists (select 1
               from   INSPTIPAUDIT t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPTIPAUDIT". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPECAUDIT"  */
    if exists (select 1
               from   INSPECAUDIT t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECAUDIT". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "ITEMINSAUDIT"  */
    if exists (select 1
               from   ITEMINSAUDIT t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMINSAUDIT". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "CRITAVAPARCI"  */
    if exists (select 1
               from   CRITAVAPARCI t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CRITAVAPARCI". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPAVAFINAL"  */
    if exists (select 1
               from   INSPAVAFINAL t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPAVAFINAL". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPPROC"  */
    if exists (select 1
               from   INSPPROC t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPPROC". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPETAPA"  */
    if exists (select 1
               from   INSPETAPA t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPETAPA". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPAGENTARE"  */
    if exists (select 1
               from   INSPAGENTARE t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPAGENTARE". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPRECRPADR"  */
    if exists (select 1
               from   INSPRECRPADR t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPRECRPADR". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPRECRDEPT"  */
    if exists (select 1
               from   INSPRECRDEPT t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPRECRDEPT". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPRECRCARG"  */
    if exists (select 1
               from   INSPRECRCARG t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPRECRCARG". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "RECURETAPAOP"  */
    if exists (select 1
               from   RECURETAPAOP t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPAOP". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "COMPOSINSPEC"  */
    if exists (select 1
               from   COMPOSINSPEC t2, deleted t1
               where  t2.CDINSPECAO = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPOSINSPEC". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPPROCESSO"  */
    if exists (select 1
               from   INSPPROCESSO t2, deleted t1
               where  t2.CDINSPGRUP = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPPROCESSO". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Cannot delete parent "INSPECAO" if children still exist in "INSPLANATIV"  */
    if exists (select 1
               from   INSPLANATIV t2, deleted t1
               where  t2.CDINSPPROD = t1.CDINSPECAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPLANATIV". Cannot delete parent "INSPECAO".'
          goto error
       end
    
    /*  Delete all children in "RESPOSINSPEC"  */
    delete RESPOSINSPEC
    from   RESPOSINSPEC t2, deleted t1
    where  t2.CDINSPECAO = t1.CDINSPECAO
    
    /*  Delete all children in "DESCINSPECAO"  */
    delete DESCINSPECAO
    from   DESCINSPECAO t2, deleted t1
    where  t2.CDINSPECAO = t1.CDINSPECAO
    
    /*  Delete all children in "ACAOINSPECAO"  */
    delete ACAOINSPECAO
    from   ACAOINSPECAO t2, deleted t1
    where  t2.CDINSPECAO = t1.CDINSPECAO
    
    /*  Delete all children in "VISTOINSPEC"  */
    delete VISTOINSPEC
    from   VISTOINSPEC t2, deleted t1
    where  t2.CDINSPECAO = t1.CDINSPECAO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inspecaudit" for table "INSPECAUDIT"  */
create trigger td_inspecaudit on INSPECAUDIT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMINSAUDIT"  */
    delete ITEMINSAUDIT
    from   ITEMINSAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
    
    /*  Delete all children in "VISTINSAUDIT"  */
    delete VISTINSAUDIT
    from   VISTINSAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
    
    /*  Delete all children in "PESSAUDIT"  */
    delete PESSAUDIT
    from   PESSAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
    
    /*  Delete all children in "MOVOCORRPROD"  */
    delete MOVOCORRPROD
    from   MOVOCORRPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
    
    /*  Delete all children in "RECURETAPAOP"  */
    delete RECURETAPAOP
    from   RECURETAPAOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
    
    /*  Delete all children in "DESCINSAUDIT"  */
    delete DESCINSAUDIT
    from   DESCINSAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inspetapa" for table "INSPETAPA"  */
create trigger td_inspetapa on INSPETAPA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "DESCINSETAPA"  */
    delete DESCINSETAPA
    from   DESCINSETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
     and   t2.CDINSPECAO = t1.CDINSPECAO
     and   t2.NRINSPETAPA = t1.NRINSPETAPA
    
    /*  Delete all children in "RECURETAPA"  */
    delete RECURETAPA
    from   RECURETAPA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDETAPAPROD = t1.CDETAPAPROD
     and   t2.CDINSPECAO = t1.CDINSPECAO
     and   t2.NRINSPETAPA = t1.NRINSPETAPA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inteclie" for table "INTECLIE"  */
create trigger td_inteclie on INTECLIE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INTECLIE" if children still exist in "SERVPREST"  */
    if exists (select 1
               from   SERVPREST t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDINTECLIE = t1.CDINTECLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREST". Cannot delete parent "INTECLIE".'
          goto error
       end
    
    /*  Cannot delete parent "INTECLIE" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDINTECLIE = t1.CDINTECLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "INTECLIE".'
          goto error
       end
    
    /*  Cannot delete parent "INTECLIE" if children still exist in "EVOLCONT"  */
    if exists (select 1
               from   EVOLCONT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDINTECLIE = t1.CDINTECLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLCONT". Cannot delete parent "INTECLIE".'
          goto error
       end
    
    /*  Cannot delete parent "INTECLIE" if children still exist in "MALADIRETA"  */
    if exists (select 1
               from   MALADIRETA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDINTECLIE = t1.CDINTECLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MALADIRETA". Cannot delete parent "INTECLIE".'
          goto error
       end
    
    /*  Cannot delete parent "INTECLIE" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDINTECLIE = t1.CDINTECLIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "INTECLIE".'
          goto error
       end
    
    /*  Delete all children in "CORINTECLI"  */
    delete CORINTECLI
    from   CORINTECLI t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDINTECLIE = t1.CDINTECLIE
    
    /*  Delete all children in "PROCINTECLI"  */
    delete PROCINTECLI
    from   PROCINTECLI t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDINTECLIE = t1.CDINTECLIE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inteforn" for table "INTEFORN"  */
create trigger td_inteforn on INTEFORN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INTEFORN" if children still exist in "EVOLCONT"  */
    if exists (select 1
               from   EVOLCONT t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED
                and   t2.CDINTERLOC = t1.CDINTERLOC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLCONT". Cannot delete parent "INTEFORN".'
          goto error
       end
    
    /*  Cannot delete parent "INTEFORN" if children still exist in "MALADIRETA"  */
    if exists (select 1
               from   MALADIRETA t2, deleted t1
               where  t2.CDFORNECED = t1.CDFORNECED
                and   t2.CDINTERLOC = t1.CDINTERLOC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MALADIRETA". Cannot delete parent "INTEFORN".'
          goto error
       end
    
    /*  Delete all children in "CORINTEFOR"  */
    delete CORINTEFOR
    from   CORINTEFOR t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDINTERLOC = t1.CDINTERLOC
    
    /*  Delete all children in "PROCINTEFOR"  */
    delete PROCINTEFOR
    from   PROCINTEFOR t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDINTERLOC = t1.CDINTERLOC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_inteprosp" for table "INTEPROSP"  */
create trigger td_inteprosp on INTEPROSP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INTEPROSP" if children still exist in "EVOLCONT"  */
    if exists (select 1
               from   EVOLCONT t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT
                and   t2.CDINTEPROS = t1.CDINTEPROS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLCONT". Cannot delete parent "INTEPROSP".'
          goto error
       end
    
    /*  Cannot delete parent "INTEPROSP" if children still exist in "MALADIRETA"  */
    if exists (select 1
               from   MALADIRETA t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT
                and   t2.CDINTEPROS = t1.CDINTEPROS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MALADIRETA". Cannot delete parent "INTEPROSP".'
          goto error
       end
    
    /*  Delete all children in "CORINTEPRO"  */
    delete CORINTEPRO
    from   CORINTEPRO t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT
     and   t2.CDINTEPROS = t1.CDINTEPROS
    
    /*  Delete all children in "PROCINTEPRO"  */
    delete PROCINTEPRO
    from   PROCINTEPRO t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT
     and   t2.CDINTEPROS = t1.CDINTEPROS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_internac" for table "INTERNAC"  */
create trigger td_internac on INTERNAC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INTERNAC" if children still exist in "SERVINTE"  */
    if exists (select 1
               from   SERVINTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTINTERNA = t1.DTINTERNA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVINTE". Cannot delete parent "INTERNAC".'
          goto error
       end
    
    /*  Cannot delete parent "INTERNAC" if children still exist in "INTERLEITO"  */
    if exists (select 1
               from   INTERLEITO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE
                and   t2.DTINTERNA = t1.DTINTERNA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INTERLEITO". Cannot delete parent "INTERNAC".'
          goto error
       end
    
    /*  Delete all children in "CONSPACI"  */
    delete CONSPACI
    from   CONSPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTINTERNA = t1.DTINTERNA
    
    /*  Delete all children in "DIAGINTER"  */
    delete DIAGINTER
    from   DIAGINTER t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTINTERNA = t1.DTINTERNA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_invoice" for table "INVOICE"  */
create trigger td_invoice on INVOICE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "INVOICE" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRINVOICE = t1.NRINVOICE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "INVOICE".'
          goto error
       end
    
    /*  Delete all children in "ITEMINVOICE"  */
    delete ITEMINVOICE
    from   ITEMINVOICE t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRINVOICE = t1.NRINVOICE
    
    /*  Delete all children in "DADOSADICINV"  */
    delete DADOSADICINV
    from   DADOSADICINV t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRINVOICE = t1.NRINVOICE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_irrf" for table "IRRF"  */
create trigger td_irrf on IRRF for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FAIXA_IRRF"  */
    delete FAIXA_IRRF
    from   FAIXA_IRRF t2, deleted t1
    where  t2.DTVIGEIRRF = t1.DTVIGEIRRF

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemautf" for table "ITEMAUTF"  */
create trigger td_itemautf on ITEMAUTF for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMAUTF" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILDEST
                and   t2.CDFILIAL = t1.CDFILLANC
                and   t2.NRAUTOFORN = t1.NRAUTOFORN
                and   t2.CDMARCA = t1.CDMARCA
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTAF = t1.DTPROGENTAF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "ITEMAUTF".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMAUTF" if children still exist in "NFRELACAF"  */
    if exists (select 1
               from   NFRELACAF t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILDEST
                and   t2.CDFILLANC = t1.CDFILLANC
                and   t2.NRAUTOFORN = t1.NRAUTOFORN
                and   t2.CDMARCA = t1.CDMARCA
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTAF = t1.DTPROGENTAF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACAF". Cannot delete parent "ITEMAUTF".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMAUTF" if children still exist in "NFRELACAFSAI"  */
    if exists (select 1
               from   NFRELACAFSAI t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILDEST
                and   t2.CDFILLANC = t1.CDFILLANC
                and   t2.NRAUTOFORN = t1.NRAUTOFORN
                and   t2.CDMARCA = t1.CDMARCA
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTAF = t1.DTPROGENTAF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACAFSAI". Cannot delete parent "ITEMAUTF".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMAUTF" if children still exist in "ITEMSAIDOL"  */
    if exists (select 1
               from   ITEMSAIDOL t2, deleted t1
               where  t2.CDFILDEST = t1.CDFILDEST
                and   t2.CDFILLANC = t1.CDFILLANC
                and   t2.NRAUTOFORN = t1.NRAUTOFORN
                and   t2.CDMARCA = t1.CDMARCA
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTAF = t1.DTPROGENTAF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSAIDOL". Cannot delete parent "ITEMAUTF".'
          goto error
       end
    
    /*  Delete all children in "IMPITMAF"  */
    delete IMPITMAF
    from   IMPITMAF t2, deleted t1
    where  t2.CDFILDEST = t1.CDFILDEST
     and   t2.CDFILLANC = t1.CDFILLANC
     and   t2.NRAUTOFORN = t1.NRAUTOFORN
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.DTPROGENTAF = t1.DTPROGENTAF

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemcole" for table "ITEMCOLE"  */
create trigger td_itemcole on ITEMCOLE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PZOPGTITMCOL"  */
    delete PZOPGTITMCOL
    from   PZOPGTITMCOL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCOLEPRECO = t1.NRCOLEPRECO
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "ITMPREVLIST"  */
    delete ITMPREVLIST
    from   ITMPREVLIST t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCOLEPRECO = t1.NRCOLEPRECO
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "IMPCOLET"  */
    delete IMPCOLET
    from   IMPCOLET t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCOLEPRECO = t1.NRCOLEPRECO
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemconssolol" for table "ITEMCONSSOLOL"  */
create trigger td_itemconssolol on ITEMCONSSOLOL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "RELACONSOLOL"  */
    delete RELACONSOLOL
    from   RELACONSOLOL t2, deleted t1
    where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL
     and   t2.CDFILSOLCONS = t1.CDFILSOLI
     and   t2.CDFILLANCONS = t1.CDFILLAN
     and   t2.CDPRODUTOCONS = t1.CDPRODUTO
     and   t2.DTENTRCONSOL = t1.DTENTRCONSOL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemdemof" for table "ITEMDEMOF"  */
create trigger td_itemdemof on ITEMDEMOF for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMAPUCTB"  */
    delete ITEMAPUCTB
    from   ITEMAPUCTB t2, deleted t1
    where  t2.CDTIPODEMO = t1.CDTIPODEMO
     and   t2.CDITEMSTRU = t1.CDITEMSTRU

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemdw" for table "ITEMDW"  */
create trigger td_itemdw on ITEMDW for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMDWOPER"  */
    delete ITEMDWOPER
    from   ITEMDWOPER t2, deleted t1
    where  t2.CDTABELADW = t1.CDTABELADW
     and   t2.CDITEMDW = t1.CDITEMDW
    
    /*  Delete all children in "ITEMDWGRUPOP"  */
    delete ITEMDWGRUPOP
    from   ITEMDWGRUPOP t2, deleted t1
    where  t2.CDTABELADW = t1.CDTABELADW
     and   t2.CDITEMDW = t1.CDITEMDW

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemencargo" for table "ITEMENCARGO"  */
create trigger td_itemencargo on ITEMENCARGO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.CDENCARGO = t1.CDENCARGO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_iteminsaudit" for table "ITEMINSAUDIT"  */
create trigger td_iteminsaudit on ITEMINSAUDIT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "RESULTAUDIT"  */
    delete RESULTAUDIT
    from   RESULTAUDIT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRAUDITORIA = t1.NRAUDITORIA
     and   t2.NRINSPECAUDI = t1.NRINSPECAUDI
     and   t2.CDINSPECAO = t1.CDINSPECAO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemnotafisc" for table "ITEMNOTAFISC"  */
create trigger td_itemnotafisc on ITEMNOTAFISC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQUITNOTA = t1.NRSEQUITNOTA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemorcaeven" for table "ITEMORCAEVEN"  */
create trigger td_itemorcaeven on ITEMORCAEVEN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMORCAEVEN" if children still exist in "HISTORCAEVEN"  */
    if exists (select 1
               from   HISTORCAEVEN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
                and   t2.NRITEMORCEVE = t1.NRITEMORCEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTORCAEVEN". Cannot delete parent "ITEMORCAEVEN".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMORCAEVEN" if children still exist in "ITEMPEDORE"  */
    if exists (select 1
               from   ITEMPEDORE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
                and   t2.NRITEMORCEVE = t1.NRITEMORCEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDORE". Cannot delete parent "ITEMORCAEVEN".'
          goto error
       end
    
    /*  Delete all children in "AGENSALITORC"  */
    delete AGENSALITORC
    from   AGENSALITORC t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.NRITEMORCEVE = t1.NRITEMORCEVE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itempedcol" for table "ITEMPEDCOL"  */
create trigger td_itempedcol on ITEMPEDCOL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "IMPPEDCOLETA"  */
    delete IMPPEDCOLETA
    from   IMPPEDCOLETA t2, deleted t1
    where  t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDCOLETA = t1.NRPEDCOLETA
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "MOVOPITEMPEDCOL"  */
    delete MOVOPITEMPEDCOL
    from   MOVOPITEMPEDCOL t2, deleted t1
    where  t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDCOLETA = t1.NRPEDCOLETA
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "LOGNGERAPEDCOL"  */
    delete LOGNGERAPEDCOL
    from   LOGNGERAPEDCOL t2, deleted t1
    where  t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDCOLETA = t1.NRPEDCOLETA
     and   t2.CDMARCA = t1.CDMARCA
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itempedido" for table "ITEMPEDIDO"  */
create trigger td_itempedido on ITEMPEDIDO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDFILIALPED = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "PROITPROP"  */
    if exists (select 1
               from   PROITPROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROITPROP". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDFILIALPED = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "INCONSPEDIDO"  */
    if exists (select 1
               from   INCONSPEDIDO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCONSPEDIDO". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "PEDIDOP"  */
    if exists (select 1
               from   PEDIDOP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOP". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "INSUPEDIDOP"  */
    if exists (select 1
               from   INSUPEDIDOP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSUPEDIDOP". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPEDIDO" if children still exist in "ITEMPEDORE"  */
    if exists (select 1
               from   ITEMPEDORE t2, deleted t1
               where  t2.CDFILIPEDORE = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO
                and   t2.NRITEMPED = t1.NRITEMPED)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDORE". Cannot delete parent "ITEMPEDIDO".'
          goto error
       end
    
    /*  Delete all children in "HISITEMPED"  */
    delete HISITEMPED
    from   HISITEMPED t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
     and   t2.NRITEMPED = t1.NRITEMPED
    
    /*  Delete all children in "VENDITEMPED"  */
    delete VENDITEMPED
    from   VENDITEMPED t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
     and   t2.NRITEMPED = t1.NRITEMPED
    
    /*  Delete all children in "RESERPRODPED"  */
    delete RESERPRODPED
    from   RESERPRODPED t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
     and   t2.NRITEMPED = t1.NRITEMPED
    
    /*  Delete all children in "COMPITEMPED"  */
    delete COMPITEMPED
    from   COMPITEMPED t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
     and   t2.NRITEMPED = t1.NRITEMPED

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itempreco" for table "ITEMPRECO"  */
create trigger td_itempreco on ITEMPRECO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMPRECO" if children still exist in "ITEMPRECODIA"  */
    if exists (select 1
               from   ITEMPRECODIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDTABEPREC = t1.CDTABEPREC
                and   t2.DTINIVGPREC = t1.DTINIVGPREC
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.CDPRPAITABPR = t1.CDPRPAITABPR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRECODIA". Cannot delete parent "ITEMPRECO".'
          goto error
       end
    
    /*  Delete all children in "ITEMPRECOCLI"  */
    delete ITEMPRECOCLI
    from   ITEMPRECOCLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDTABEPREC = t1.CDTABEPREC
     and   t2.DTINIVGPREC = t1.DTINIVGPREC
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDPRPAITABPR = t1.CDPRPAITABPR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemprecocli" for table "ITEMPRECOCLI"  */
create trigger td_itemprecocli on ITEMPRECOCLI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMPRECOCON"  */
    delete ITEMPRECOCON
    from   ITEMPRECOCON t2, deleted t1
    where  t2.CDTABEPREC = t1.CDTABEPREC
     and   t2.DTINIVGPREC = t1.DTINIVGPREC
     and   t2.CDPRPAITABPR = t1.CDPRPAITABPR
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemproduto" for table "ITEMPRODUTO"  */
create trigger td_itemproduto on ITEMPRODUTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "SERVFATU"  */
    if exists (select 1
               from   SERVFATU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVFATU". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "CTRAPUITPROD"  */
    if exists (select 1
               from   CTRAPUITPROD t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRAPUITPROD". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "ITEMVENDA"  */
    if exists (select 1
               from   ITEMVENDA t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDA". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "ITVENDAVMN"  */
    if exists (select 1
               from   ITVENDAVMN t2, deleted t1
               where  t2.CDFILIALNFV = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDAVMN". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "ITVENDAVMN"  */
    if exists (select 1
               from   ITVENDAVMN t2, deleted t1
               where  t2.CDFILIALNFV2 = t1.CDFILIAL
                and   t2.NRLANCTONF2 = t1.NRLANCTONF
                and   t2.NRSEQITPROD2 = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDAVMN". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "ITVENDAVMN"  */
    if exists (select 1
               from   ITVENDAVMN t2, deleted t1
               where  t2.CDFILIALNFV3 = t1.CDFILIAL
                and   t2.NRLANCTONF3 = t1.NRLANCTONF
                and   t2.NRSEQITPROD3 = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDAVMN". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMPRODUTO" if children still exist in "PARCLANCATIV"  */
    if exists (select 1
               from   PARCLANCATIV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF
                and   t2.NRSEQITPROD = t1.NRSEQITPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLANCATIV". Cannot delete parent "ITEMPRODUTO".'
          goto error
       end
    
    /*  Delete all children in "ITEMPROIMPOS"  */
    delete ITEMPROIMPOS
    from   ITEMPROIMPOS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD
    
    /*  Delete all children in "RATEIONF"  */
    delete RATEIONF
    from   RATEIONF t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD
    
    /*  Delete all children in "IMPOSINFUSU"  */
    delete IMPOSINFUSU
    from   IMPOSINFUSU t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD
    
    /*  Delete all children in "ITPRODORIEXP"  */
    delete ITPRODORIEXP
    from   ITPRODORIEXP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemproimpos" for table "ITEMPROIMPOS"  */
create trigger td_itemproimpos on ITEMPROIMPOS for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
     and   t2.NRSEQITPROD = t1.NRSEQITPROD
     and   t2.CDIMPOSTO = t1.CDIMPOSTO
     and   t2.NRITPROIMPOS = t1.NRITPROIMPOS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemrefe" for table "ITEMREFE"  */
create trigger td_itemrefe on ITEMREFE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMREFE" if children still exist in "ITMOVICOME"  */
    if exists (select 1
               from   ITMOVICOME t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDITEMREFE = t1.CDITEMREFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMOVICOME". Cannot delete parent "ITEMREFE".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMREFE" if children still exist in "RESTITREFTUR"  */
    if exists (select 1
               from   RESTITREFTUR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDITEMREFE = t1.CDITEMREFE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTITREFTUR". Cannot delete parent "ITEMREFE".'
          goto error
       end
    
    /*  Delete all children in "ITEMREFECLIE"  */
    delete ITEMREFECLIE
    from   ITEMREFECLIE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDITEMREFE = t1.CDITEMREFE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemreti" for table "ITEMRETI"  */
create trigger td_itemreti on ITEMRETI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMRETIPRAT"  */
    delete ITEMRETIPRAT
    from   ITEMRETIPRAT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCARDAPIO = t1.NRCARDAPIO
     and   t2.NRRETIRADA = t1.NRRETIRADA
     and   t2.NRSEQUITRETI = t1.NRSEQUITRETI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemretiest" for table "ITEMRETIEST"  */
create trigger td_itemretiest on ITEMRETIEST for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "LOTITRETIEST"  */
    delete LOTITRETIEST
    from   LOTITRETIEST t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRRETEST = t1.NRRETEST
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.NRSEQRETPROD = t1.NRSEQRETPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemsoli" for table "ITEMSOLI"  */
create trigger td_itemsoli on ITEMSOLI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMSOLI" if children still exist in "PRODCOMPRA"  */
    if exists (select 1
               from   PRODCOMPRA t2, deleted t1
               where  t2.CDFILSOLI = t1.CDFILSOLI
                and   t2.CDFILLAN = t1.CDFILLAN
                and   t2.NRSOLICMP = t1.NRSOLICMP
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTR = t1.DTPROGENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCOMPRA". Cannot delete parent "ITEMSOLI".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMSOLI" if children still exist in "ITEMSOLIDEL"  */
    if exists (select 1
               from   ITEMSOLIDEL t2, deleted t1
               where  t2.CDFILSOLI = t1.CDFILSOLI
                and   t2.CDFILLAN = t1.CDFILLAN
                and   t2.NRSOLICMP = t1.NRSOLICMP
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.DTPROGENTR = t1.DTPROGENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLIDEL". Cannot delete parent "ITEMSOLI".'
          goto error
       end
    
    /*  Delete all children in "NFRELACSOLI"  */
    delete NFRELACSOLI
    from   NFRELACSOLI t2, deleted t1
    where  t2.CDFILSOLI = t1.CDFILSOLI
     and   t2.CDFILLAN = t1.CDFILLAN
     and   t2.NRSOLICMP = t1.NRSOLICMP
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.DTPROGENTR = t1.DTPROGENTR
    
    /*  Delete all children in "RELACONSOLOL"  */
    delete RELACONSOLOL
    from   RELACONSOLOL t2, deleted t1
    where  t2.CDFILSOLI = t1.CDFILSOLI
     and   t2.CDFILLAN = t1.CDFILLAN
     and   t2.NRSOLICMP = t1.NRSOLICMP
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.DTPROGENTR = t1.DTPROGENTR
    
    /*  Set parent code of "ITEMSOLI" to NULL in child "ITEMSOLI"  */
    update ITEMSOLI
     set   CDFILSOLCONS = NULL,
           CDFILLANCONS = NULL,
           NRSOLCMPCONS = NULL,
           CDPRODUTCONS = NULL,
           DTPROGENCONS = NULL
    from   ITEMSOLI t2, deleted t1
    where  t2.CDFILSOLCONS = t1.CDFILSOLI
     and   t2.CDFILLANCONS = t1.CDFILLAN
     and   t2.NRSOLCMPCONS = t1.NRSOLICMP
     and   t2.CDPRODUTCONS = t1.CDPRODUTO
     and   t2.DTPROGENCONS = t1.DTPROGENTR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemtarefa" for table "ITEMTAREFA"  */
create trigger td_itemtarefa on ITEMTAREFA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMTAREFA" if children still exist in "PLATVRELITTR"  */
    if exists (select 1
               from   PLATVRELITTR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRSEQUITTARE = t1.NRSEQUITTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLATVRELITTR". Cannot delete parent "ITEMTAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTAREFA" if children still exist in "COMPALTITTAR"  */
    if exists (select 1
               from   COMPALTITTAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRSEQUITTARE = t1.NRSEQUITTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPALTITTAR". Cannot delete parent "ITEMTAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTAREFA" if children still exist in "VALIDATAREFA"  */
    if exists (select 1
               from   VALIDATAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRSEQUITTARE = t1.NRSEQUITTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALIDATAREFA". Cannot delete parent "ITEMTAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTAREFA" if children still exist in "ITEMVALIDATARE"  */
    if exists (select 1
               from   ITEMVALIDATARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRSEQUITTARE = t1.NRSEQUITTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVALIDATARE". Cannot delete parent "ITEMTAREFA".'
          goto error
       end
    
    /*  Delete all children in "DESCITEMTARE"  */
    delete DESCITEMTARE
    from   DESCITEMTARE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTAREFA = t1.NRTAREFA
     and   t2.NRSEQUITTARE = t1.NRSEQUITTARE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itemtrab" for table "ITEMTRAB"  */
create trigger td_itemtrab on ITEMTRAB for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "ITEMTRABPROJ"  */
    if exists (select 1
               from   ITEMTRABPROJ t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTRABPROJ". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "ITEMTRABPROD"  */
    if exists (select 1
               from   ITEMTRABPROD t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTRABPROD". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "ITEMTAREFA"  */
    if exists (select 1
               from   ITEMTAREFA t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTAREFA". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "COMPITTRAB"  */
    if exists (select 1
               from   COMPITTRAB t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPITTRAB". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "RASTREABTARE"  */
    if exists (select 1
               from   RASTREABTARE t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RASTREABTARE". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Cannot delete parent "ITEMTRAB" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDITEMTRAB = t1.CDITEMTRAB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "ITEMTRAB".'
          goto error
       end
    
    /*  Delete all children in "DESCITEMTRAB"  */
    delete DESCITEMTRAB
    from   DESCITEMTRAB t2, deleted t1
    where  t2.CDITEMTRAB = t1.CDITEMTRAB

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itequippro" for table "ITEQUIPPRO"  */
create trigger td_itequippro on ITEQUIPPRO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PRODITEQUIP"  */
    delete PRODITEQUIP
    from   PRODITEQUIP t2, deleted t1
    where  t2.CDEQUIPPRO = t1.CDEQUIPPRO
     and   t2.DTINVIGCNFEP = t1.DTINVIGCNFEP
     and   t2.CDITEQUIPPRO = t1.CDITEQUIPPRO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itformpreco" for table "ITFORMPRECO"  */
create trigger td_itformpreco on ITFORMPRECO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FORMPRECPROD"  */
    delete FORMPRECPROD
    from   FORMPRECPROD t2, deleted t1
    where  t2.CDITFORPRECO = t1.CDITFORPRECO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itfreqalim" for table "ITFREQALIM"  */
create trigger td_itfreqalim on ITFREQALIM for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FREQALIMREAL"  */
    delete FREQALIMREAL
    from   FREQALIMREAL t2, deleted t1
    where  t2.CDITFREQALIM = t1.CDITFREQALIM

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itlancropfilial" for table "ITLANCROPFILIAL"  */
create trigger td_itlancropfilial on ITLANCROPFILIAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITRELACITROP"  */
    delete ITRELACITROP
    from   ITRELACITROP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRROPFILIAL = t1.NRROPFILIAL
     and   t2.NRLANCROP = t1.NRLANCROP
     and   t2.NRITLANCROP = t1.NRITLANCROP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itmaoobraorca" for table "ITMAOOBRAORCA"  */
create trigger td_itmaoobraorca on ITMAOOBRAORCA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ITMAOOBRAORCA" if children still exist in "TURMAITMOBRA"  */
    if exists (select 1
               from   TURMAITMOBRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
                and   t2.NRTURNO = t1.NRTURNO
                and   t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAITMOBRA". Cannot delete parent "ITMAOOBRAORCA".'
          goto error
       end
    
    /*  Delete all children in "MOBRAORCFORN"  */
    delete MOBRAORCFORN
    from   MOBRAORCFORN t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
     and   t2.NRTURNO = t1.NRTURNO
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itproposta" for table "ITPROPOSTA"  */
create trigger td_itproposta on ITPROPOSTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PROITPROP"  */
    delete PROITPROP
    from   PROITPROP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPROPOSTA = t1.NRPROPOSTA
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRECCONCPROP"  */
    delete PRECCONCPROP
    from   PRECCONCPROP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPROPOSTA = t1.NRPROPOSTA
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_itrequesto" for table "ITREQUESTO"  */
create trigger td_itrequesto on ITREQUESTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MENSREQUESTO"  */
    delete MENSREQUESTO
    from   MENSREQUESTO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRREQUESTO = t1.NRREQUESTO
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_lanccont" for table "LANCCONT"  */
create trigger td_lanccont on LANCCONT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "LANCCONT" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.NRLANCCONT = t1.NRLANCCONT
                and   t2.NRSEQLANC = t1.NRSEQLANC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "LANCCONT".'
          goto error
       end
    
    /*  Cannot delete parent "LANCCONT" if children still exist in "AJCTPAGAR"  */
    if exists (select 1
               from   AJCTPAGAR t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.NRLANCCONT = t1.NRLANCCONT
                and   t2.NRSEQLANC = t1.NRSEQLANC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTPAGAR". Cannot delete parent "LANCCONT".'
          goto error
       end
    
    /*  Delete all children in "LANCCCUSTO"  */
    delete LANCCCUSTO
    from   LANCCCUSTO t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.NRLANCCONT = t1.NRLANCCONT
     and   t2.NRSEQLANC = t1.NRSEQLANC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_lanclote" for table "LANCLOTE"  */
create trigger td_lanclote on LANCLOTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "RATECCUS"  */
    delete RATECCUS
    from   RATECCUS t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDLOTE = t1.CDLOTE
     and   t2.NRLANCLOTE = t1.NRLANCLOTE
     and   t2.NRSEQLANLOTE = t1.NRSEQLANLOTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_lancropfilial" for table "LANCROPFILIAL"  */
create trigger td_lancropfilial on LANCROPFILIAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITLANCROPFILIAL"  */
    delete ITLANCROPFILIAL
    from   ITLANCROPFILIAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRROPFILIAL = t1.NRROPFILIAL
     and   t2.NRLANCROP = t1.NRLANCROP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_lanctoestoq" for table "LANCTOESTOQ"  */
create trigger td_lanctoestoq on LANCTOESTOQ for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "RETIRADA"  */
    if exists (select 1
               from   RETIRADA t2, deleted t1
               where  t2.CDFILANCESTQ = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADA". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "RETIRADAPCP"  */
    if exists (select 1
               from   RETIRADAPCP t2, deleted t1
               where  t2.CDFILIALLPCP = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAPCP". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "RETIRADAESTO"  */
    if exists (select 1
               from   RETIRADAESTO t2, deleted t1
               where  t2.CDFILIALLAN = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAESTO". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "AJURELACRET"  */
    if exists (select 1
               from   AJURELACRET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJURELACRET". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "AJURELACRET"  */
    if exists (select 1
               from   AJURELACRET t2, deleted t1
               where  t2.CDFILIALRET = t1.CDFILIAL
                and   t2.NRLANCESTRET = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJURELACRET". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIALENTR = t1.CDFILIAL
                and   t2.NRLANESTENTR = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIALSAID = t1.CDFILIAL
                and   t2.NRLANESTSAID = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "NFABASTEROTA"  */
    if exists (select 1
               from   NFABASTEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFABASTEROTA". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "PREVPDVLESTO"  */
    if exists (select 1
               from   PREVPDVLESTO t2, deleted t1
               where  t2.CDFILANCESTQ = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVPDVLESTO". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "COMPITVENDA"  */
    if exists (select 1
               from   COMPITVENDA t2, deleted t1
               where  t2.CDFILIALEST = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPITVENDA". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "CUSTOCMPPROD"  */
    if exists (select 1
               from   CUSTOCMPPROD t2, deleted t1
               where  t2.CDFILLANCEST = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOCMPPROD". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "ITEMVENDA"  */
    if exists (select 1
               from   ITEMVENDA t2, deleted t1
               where  t2.CDFILIALEST = t1.CDFILIAL
                and   t2.NRLANCESTQ = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDA". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Cannot delete parent "LANCTOESTOQ" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILLANCREQC = t1.CDFILIAL
                and   t2.NRLANCREQC = t1.NRLANCESTQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "LANCTOESTOQ".'
          goto error
       end
    
    /*  Delete all children in "ITLANCTOEST"  */
    delete ITLANCTOEST
    from   ITLANCTOEST t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCESTQ = t1.NRLANCESTQ
    
    /*  Delete all children in "PRODIMPOINVE"  */
    delete PRODIMPOINVE
    from   PRODIMPOINVE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCESTQ = t1.NRLANCESTQ
    
    /*  Delete all children in "NFRELACRETIR"  */
    delete NFRELACRETIR
    from   NFRELACRETIR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCESTQ = t1.NRLANCESTQ
    
    /*  Delete all children in "NFRELACRETIR"  */
    delete NFRELACRETIR
    from   NFRELACRETIR t2, deleted t1
    where  t2.CDFILIALRET = t1.CDFILIAL
     and   t2.NRLANCESTRET = t1.NRLANCESTQ

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_licencauso" for table "LICENCAUSO"  */
create trigger td_licencauso on LICENCAUSO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "SERVPREST"  */
    if exists (select 1
               from   SERVPREST t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREST". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "SERVFATU"  */
    if exists (select 1
               from   SERVFATU t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVFATU". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "SISTLUSO"  */
    if exists (select 1
               from   SISTLUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SISTLUSO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "BOLETRECEBER"  */
    if exists (select 1
               from   BOLETRECEBER t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BOLETRECEBER". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "MALADIRETA"  */
    if exists (select 1
               from   MALADIRETA t2, deleted t1
               where  t2.CDCLIELICE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MALADIRETA". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "CORRLINUSO"  */
    if exists (select 1
               from   CORRLINUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CORRLINUSO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "LINPLUSO"  */
    if exists (select 1
               from   LINPLUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LINPLUSO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "CONTRATOLINP"  */
    if exists (select 1
               from   CONTRATOLINP t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLUSOFATLU = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOLINP". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "CONTRATOLINP"  */
    if exists (select 1
               from   CONTRATOLINP t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLUSOFATMAN = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOLINP". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "CAIXALUSO"  */
    if exists (select 1
               from   CAIXALUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXALUSO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "IMPRLUSO"  */
    if exists (select 1
               from   IMPRLUSO t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPRLUSO". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Cannot delete parent "LICENCAUSO" if children still exist in "MENSLINPCLIE"  */
    if exists (select 1
               from   MENSLINPCLIE t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MENSLINPCLIE". Cannot delete parent "LICENCAUSO".'
          goto error
       end
    
    /*  Delete all children in "LOGLIBEPERS"  */
    delete LOGLIBEPERS
    from   LOGLIBEPERS t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.NRLICENCAUSO = t1.NRLICENCAUSO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_localestoque" for table "LOCALESTOQUE"  */
create trigger td_localestoque on LOCALESTOQUE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "POSESTDIA"  */
    if exists (select 1
               from   POSESTDIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTDIA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "MOVILOTE"  */
    if exists (select 1
               from   MOVILOTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVILOTE". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODLCESTOQ"  */
    if exists (select 1
               from   PRODLCESTOQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODLCESTOQ". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.CDFILIORIGRD = t1.CDFILIAL
                and   t2.CDALMOORIGRD = t1.CDALMOXARIFE
                and   t2.CDLESTORIGRD = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.CDFILIDESTRD = t1.CDFILIAL
                and   t2.CDALMODESTRD = t1.CDALMOXARIFE
                and   t2.CDLESTDESTRD = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ROTATRANS"  */
    if exists (select 1
               from   ROTATRANS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTATRANS". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXASAID = t1.CDALMOXARIFE
                and   t2.CDLESTOQSAID = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXAENTR = t1.CDALMOXARIFE
                and   t2.CDLESTOQENTR = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXINSPR = t1.CDALMOXARIFE
                and   t2.CDLOCALINSPR = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXPRACA = t1.CDALMOXARIFE
                and   t2.CDLOCALPRACA = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDFILIRECE = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "POSESTATU"  */
    if exists (select 1
               from   POSESTATU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTATU". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXPRCLI = t1.CDALMOXARIFE
                and   t2.CDLOCALPRCLI = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "RESERPRODPED"  */
    if exists (select 1
               from   RESERPRODPED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERPRODPED". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "INVEROTA"  */
    if exists (select 1
               from   INVEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CURVAABC"  */
    if exists (select 1
               from   CURVAABC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CURVAABC". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "LOTITRETIEST"  */
    if exists (select 1
               from   LOTITRETIEST t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOTITRETIEST". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXENTR = t1.CDALMOXARIFE
                and   t2.CDLCESTENTR = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXSAID = t1.CDALMOXARIFE
                and   t2.CDLCESTSAID = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXSAID = t1.CDALMOXARIFE
                and   t2.CDLCESTSAID = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXENTR = t1.CDALMOXARIFE
                and   t2.CDLCESTENTR = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PARFILIALVMN"  */
    if exists (select 1
               from   PARFILIALVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXPERDA = t1.CDALMOXARIFE
                and   t2.CDLOCALPERDA = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALVMN". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CODIGOBARRA"  */
    if exists (select 1
               from   CODIGOBARRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODIGOBARRA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODACABADO"  */
    if exists (select 1
               from   PRODACABADO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODACABADO". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "INVEROTALOTE"  */
    if exists (select 1
               from   INVEROTALOTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTALOTE". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXPISTA = t1.CDALMOXARIFE
                and   t2.CDLOCESTPIST = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "ITEMENTREGA"  */
    if exists (select 1
               from   ITEMENTREGA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMENTREGA". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "CUSTOCMPPROD"  */
    if exists (select 1
               from   CUSTOCMPPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
                and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOCMPPROD". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Cannot delete parent "LOCALESTOQUE" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDALMOXQUAR = t1.CDALMOXARIFE
                and   t2.CDLOCALQUAR = t1.CDLOCALESTOQ)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "LOCALESTOQUE".'
          goto error
       end
    
    /*  Delete all children in "OPERLESTOQ"  */
    delete OPERLESTOQ
    from   OPERLESTOQ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
     and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ
    
    /*  Delete all children in "GROPERLESTOQ"  */
    delete GROPERLESTOQ
    from   GROPERLESTOQ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDALMOXARIFE = t1.CDALMOXARIFE
     and   t2.CDLOCALESTOQ = t1.CDLOCALESTOQ

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_logapurqfo" for table "LOGAPURQFO"  */
create trigger td_logapurqfo on LOGAPURQFO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "OCOAPURQFO"  */
    delete OCOAPURQFO
    from   OCOAPURQFO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLOGAPURQFO = t1.NRLOGAPURQFO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_mapaapurac" for table "MAPAAPURAC"  */
create trigger td_mapaapurac on MAPAAPURAC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MAPAAPURAC" if children still exist in "IMPOSTONF"  */
    if exists (select 1
               from   IMPOSTONF t2, deleted t1
               where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTONF". Cannot delete parent "MAPAAPURAC".'
          goto error
       end
    
    /*  Cannot delete parent "MAPAAPURAC" if children still exist in "IMPOSTONF"  */
    if exists (select 1
               from   IMPOSTONF t2, deleted t1
               where  t2.NRSEQMAPOIMP = t1.NRSEQUMAPAPU)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTONF". Cannot delete parent "MAPAAPURAC".'
          goto error
       end
    
    /*  Cannot delete parent "MAPAAPURAC" if children still exist in "PARCLANCATIV"  */
    if exists (select 1
               from   PARCLANCATIV t2, deleted t1
               where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLANCATIV". Cannot delete parent "MAPAAPURAC".'
          goto error
       end
    
    /*  Cannot delete parent "MAPAAPURAC" if children still exist in "PARGERGIASC"  */
    if exists (select 1
               from   PARGERGIASC t2, deleted t1
               where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARGERGIASC". Cannot delete parent "MAPAAPURAC".'
          goto error
       end
    
    /*  Cannot delete parent "MAPAAPURAC" if children still exist in "LANCINFCOMP"  */
    if exists (select 1
               from   LANCINFCOMP t2, deleted t1
               where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCINFCOMP". Cannot delete parent "MAPAAPURAC".'
          goto error
       end
    
    /*  Delete all children in "MAPARECUP"  */
    delete MAPARECUP
    from   MAPARECUP t2, deleted t1
    where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_mapabasev" for table "MAPABASEV"  */
create trigger td_mapabasev on MAPABASEV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "TROCOMAPABAS"  */
    if exists (select 1
               from   TROCOMAPABAS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TROCOMAPABAS". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "OBSMAPABASEV"  */
    if exists (select 1
               from   OBSMAPABASEV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSMAPABASEV". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "RECITEMAPABA"  */
    if exists (select 1
               from   RECITEMAPABA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECITEMAPABA". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "SOLICTROCO"  */
    if exists (select 1
               from   SOLICTROCO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICTROCO". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "VENDAVMN"  */
    if exists (select 1
               from   VENDAVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAVMN". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "MOVIVALOR"  */
    if exists (select 1
               from   MOVIVALOR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVIVALOR". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "ITPREVABAMAP"  */
    if exists (select 1
               from   ITPREVABAMAP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVABAMAP". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Cannot delete parent "MAPABASEV" if children still exist in "ITPREVABASAU"  */
    if exists (select 1
               from   ITPREVABASAU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRMAPABASEV = t1.NRMAPABASEV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVABASAU". Cannot delete parent "MAPABASEV".'
          goto error
       end
    
    /*  Delete all children in "ITEMAPABASEV"  */
    delete ITEMAPABASEV
    from   ITEMAPABASEV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRMAPABASEV = t1.NRMAPABASEV
    
    /*  Delete all children in "NFRELACMAPAB"  */
    delete NFRELACMAPAB
    from   NFRELACMAPAB t2, deleted t1
    where  t2.CDFILMAPABAS = t1.CDFILIAL
     and   t2.NRMAPABASEV = t1.NRMAPABASEV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_maparecup" for table "MAPARECUP"  */
create trigger td_maparecup on MAPARECUP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITMAPARECUP"  */
    delete ITMAPARECUP
    from   ITMAPARECUP t2, deleted t1
    where  t2.NRSEQUMAPAPU = t1.NRSEQUMAPAPU

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_marca" for table "MARCA"  */
create trigger td_marca on MARCA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDSEGMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "TESTEPROD"  */
    if exists (select 1
               from   TESTEPROD t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTEPROD". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "PARTFILI"  */
    if exists (select 1
               from   PARTFILI t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTFILI". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "QFOMARPROD"  */
    if exists (select 1
               from   QFOMARPROD t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QFOMARPROD". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "MARCCOTA"  */
    if exists (select 1
               from   MARCCOTA t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCCOTA". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "PARASUPRI"  */
    if exists (select 1
               from   PARASUPRI t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARASUPRI". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMPEDCOL"  */
    if exists (select 1
               from   ITEMPEDCOL t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDCOL". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDSEGMARCCOL = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "MARCTESTPR"  */
    if exists (select 1
               from   MARCTESTPR t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCTESTPR". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "COLETDOAC"  */
    if exists (select 1
               from   COLETDOAC t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETDOAC". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMENTREGA"  */
    if exists (select 1
               from   ITEMENTREGA t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMENTREGA". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDMARCAOL = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "RELACPRODOL"  */
    if exists (select 1
               from   RELACPRODOL t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACPRODOL". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "MARCPRODUTO"  */
    if exists (select 1
               from   MARCPRODUTO t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCPRODUTO". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "ITEMPEDEXOL"  */
    if exists (select 1
               from   ITEMPEDEXOL t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDEXOL". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "MOVOPAF"  */
    if exists (select 1
               from   MOVOPAF t2, deleted t1
               where  t2.CDMARCMOV = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPAF". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Cannot delete parent "MARCA" if children still exist in "PRMARCPEDCOL"  */
    if exists (select 1
               from   PRMARCPEDCOL t2, deleted t1
               where  t2.CDMARCA = t1.CDMARCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRMARCPEDCOL". Cannot delete parent "MARCA".'
          goto error
       end
    
    /*  Delete all children in "MARCPROD"  */
    delete MARCPROD
    from   MARCPROD t2, deleted t1
    where  t2.CDMARCA = t1.CDMARCA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_marcprod" for table "MARCPROD"  */
create trigger td_marcprod on MARCPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MARCPRODCOMP"  */
    delete MARCPRODCOMP
    from   MARCPRODCOMP t2, deleted t1
    where  t2.CDMARCA = t1.CDMARCA
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_medicamento" for table "MEDICAMENTO"  */
create trigger td_medicamento on MEDICAMENTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "MEDICAPACI"  */
    delete MEDICAPACI
    from   MEDICAPACI t2, deleted t1
    where  t2.CDMEDICAMENT = t1.CDMEDICAMENT
    
    /*  Delete all children in "PRINATIVMEDI"  */
    delete PRINATIVMEDI
    from   PRINATIVMEDI t2, deleted t1
    where  t2.CDMEDICAMENT = t1.CDMEDICAMENT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_medicase" for table "MEDICASE"  */
create trigger td_medicase on MEDICASE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MEDICASE" if children still exist in "ITINQUERPACI"  */
    if exists (select 1
               from   ITINQUERPACI t2, deleted t1
               where  t2.CDMEDICASE = t1.CDMEDICASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITINQUERPACI". Cannot delete parent "MEDICASE".'
          goto error
       end
    
    /*  Cannot delete parent "MEDICASE" if children still exist in "COMCADIAPACI"  */
    if exists (select 1
               from   COMCADIAPACI t2, deleted t1
               where  t2.CDMEDICASE = t1.CDMEDICASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMCADIAPACI". Cannot delete parent "MEDICASE".'
          goto error
       end
    
    /*  Cannot delete parent "MEDICASE" if children still exist in "PRODMECA"  */
    if exists (select 1
               from   PRODMECA t2, deleted t1
               where  t2.CDMEDICASE = t1.CDMEDICASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODMECA". Cannot delete parent "MEDICASE".'
          goto error
       end
    
    /*  Cannot delete parent "MEDICASE" if children still exist in "CARDCONSNUTR"  */
    if exists (select 1
               from   CARDCONSNUTR t2, deleted t1
               where  t2.CDMEDICASE = t1.CDMEDICASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDCONSNUTR". Cannot delete parent "MEDICASE".'
          goto error
       end
    
    /*  Cannot delete parent "MEDICASE" if children still exist in "ITEQUIVALE"  */
    if exists (select 1
               from   ITEQUIVALE t2, deleted t1
               where  t2.CDMEDICASE = t1.CDMEDICASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEQUIVALE". Cannot delete parent "MEDICASE".'
          goto error
       end
    
    /*  Delete all children in "MEDIPRATO"  */
    delete MEDIPRATO
    from   MEDIPRATO t2, deleted t1
    where  t2.CDMEDICASE = t1.CDMEDICASE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_menugrupop" for table "MENUGRUPOP"  */
create trigger td_menugrupop on MENUGRUPOP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FGMENUGRUPOP"  */
    delete FGMENUGRUPOP
    from   FGMENUGRUPOP t2, deleted t1
    where  t2.CDGRUPOPER = t1.CDGRUPOPER
     and   t2.CDSISTGRUP = t1.CDSISTGRUP
     and   t2.CDMENUITGRUP = t1.CDMENUITGRUP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_menuoper" for table "MENUOPER"  */
create trigger td_menuoper on MENUOPER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "FGMENUOPER"  */
    delete FGMENUOPER
    from   FGMENUOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
     and   t2.CDSISTOPER = t1.CDSISTOPER
     and   t2.CDMENUITOPER = t1.CDMENUITOPER

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_metoaval" for table "METOAVAL"  */
create trigger td_metoaval on METOAVAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "INFOAVAL"  */
    delete INFOAVAL
    from   INFOAVAL t2, deleted t1
    where  t2.CDTBMETOAVAL = t1.CDTBMETOAVAL
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI
    
    /*  Delete all children in "RESUAVAL"  */
    delete RESUAVAL
    from   RESUAVAL t2, deleted t1
    where  t2.CDTBMETOAVAL = t1.CDTBMETOAVAL
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_metorefe" for table "METOREFE"  */
create trigger td_metorefe on METOREFE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "TABEVRDIET"  */
    delete TABEVRDIET
    from   TABEVRDIET t2, deleted t1
    where  t2.CDREFEBIBL = t1.CDREFEBIBL
     and   t2.CDTBMETOAVAL = t1.CDTBMETOAVAL
    
    /*  Delete all children in "TABVRPREGAS"  */
    delete TABVRPREGAS
    from   TABVRPREGAS t2, deleted t1
    where  t2.CDREFEBIBL = t1.CDREFEBIBL
     and   t2.CDTBMETOAVAL = t1.CDTBMETOAVAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_modpgto" for table "MODPGTO"  */
create trigger td_modpgto on MODPGTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MODPGTO" if children still exist in "PRECOUTI"  */
    if exists (select 1
               from   PRECOUTI t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOUTI". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDMODPGTO = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Cannot delete parent "MODPGTO" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDMODPGMOBRA = t1.CDMODPGTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "MODPGTO".'
          goto error
       end
    
    /*  Delete all children in "PRAZOMODPGTO"  */
    delete PRAZOMODPGTO
    from   PRAZOMODPGTO t2, deleted t1
    where  t2.CDMODPGTO = t1.CDMODPGTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_moedeiro" for table "MOEDEIRO"  */
create trigger td_moedeiro on MOEDEIRO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MOEDEIRO" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDMOEDEIRO = t1.CDMOEDEIRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "MOEDEIRO".'
          goto error
       end
    
    /*  Cannot delete parent "MOEDEIRO" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDMOEDEIRO = t1.CDMOEDEIRO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "MOEDEIRO".'
          goto error
       end
    
    /*  Delete all children in "NUMEMOEDEIRO"  */
    delete NUMEMOEDEIRO
    from   NUMEMOEDEIRO t2, deleted t1
    where  t2.CDMOEDEIRO = t1.CDMOEDEIRO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movfluxo" for table "MOVFLUXO"  */
create trigger td_movfluxo on MOVFLUXO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "TRANSFBANC"  */
    if exists (select 1
               from   TRANSFBANC t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRANSFBANC". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "MOVCREDBANC"  */
    if exists (select 1
               from   MOVCREDBANC t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCREDBANC". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "MOVDEBBANC"  */
    if exists (select 1
               from   MOVDEBBANC t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVDEBBANC". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "CONTAPAGAR"  */
    if exists (select 1
               from   CONTAPAGAR t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFIMOVFLPAG = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTAPAGAR". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "CONTARECEBER"  */
    if exists (select 1
               from   CONTARECEBER t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFIMOVFLREC = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTARECEBER". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "AJCTPAGAR"  */
    if exists (select 1
               from   AJCTPAGAR t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMMFAJCNTP = t1.CDEMPRESA
                and   t2.CDFIMFAJCNTP = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTPAGAR". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "AJBOLPAG"  */
    if exists (select 1
               from   AJBOLPAG t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMMFAJBOLP = t1.CDEMPRESA
                and   t2.CDFIMFAJBOLP = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJBOLPAG". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "AJBOLREC"  */
    if exists (select 1
               from   AJBOLREC t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMMFAJBOLR = t1.CDEMPRESA
                and   t2.CDFIMFAJBOLR = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJBOLREC". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Cannot delete parent "MOVFLUXO" if children still exist in "AJCTRECEBER"  */
    if exists (select 1
               from   AJCTRECEBER t2, deleted t1
               where  t2.CDCLASSFINA = t1.CDCLASSFINA
                and   t2.CDEMMFAJCNTR = t1.CDEMPRESA
                and   t2.CDFIMFAJCNTR = t1.CDFILIAL
                and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
                and   t2.IDMOVFLUXO = t1.IDMOVFLUXO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTRECEBER". Cannot delete parent "MOVFLUXO".'
          goto error
       end
    
    /*  Delete all children in "MOVFXAJBOLP"  */
    delete MOVFXAJBOLP
    from   MOVFXAJBOLP t2, deleted t1
    where  t2.CDCLASSFINA = t1.CDCLASSFINA
     and   t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
     and   t2.IDMOVFLUXO = t1.IDMOVFLUXO
    
    /*  Delete all children in "MOVFXAJBOLR"  */
    delete MOVFXAJBOLR
    from   MOVFXAJBOLR t2, deleted t1
    where  t2.CDCLASSFINA = t1.CDCLASSFINA
     and   t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.DTMOVFLUXO = t1.DTMOVFLUXO
     and   t2.IDMOVFLUXO = t1.IDMOVFLUXO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movicome" for table "MOVICOME"  */
create trigger td_movicome on MOVICOME for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITMOVICOME"  */
    delete ITMOVICOME
    from   ITMOVICOME t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCOMENSAL = t1.CDCOMENSAL
     and   t2.DTHRMOVICOME = t1.DTHRMOVICOME

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movivalor" for table "MOVIVALOR"  */
create trigger td_movivalor on MOVIVALOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MOVIVALOR" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.DTMOVIVALOR = t1.DTMOVIVALOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "MOVIVALOR".'
          goto error
       end
    
    /*  Cannot delete parent "MOVIVALOR" if children still exist in "JUSTIFICAVMN"  */
    if exists (select 1
               from   JUSTIFICAVMN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.DTMOVIVALOR = t1.DTMOVIVALOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTIFICAVMN". Cannot delete parent "MOVIVALOR".'
          goto error
       end
    
    /*  Delete all children in "ITMOVIVALOR"  */
    delete ITMOVIVALOR
    from   ITMOVIVALOR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.DTMOVIVALOR = t1.DTMOVIVALOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movocorr" for table "MOVOCORR"  */
create trigger td_movocorr on MOVOCORR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "MOVOCORR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDFILILANCMO = t1.CDFILILANCMO
                and   t2.NROCORRMOV = t1.NROCORRMOV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "MOVOCORR".'
          goto error
       end
    
    /*  Delete all children in "PARECER"  */
    delete PARECER
    from   PARECER t2, deleted t1
    where  t2.CDFILILANCMO = t1.CDFILILANCMO
     and   t2.NROCORRMOV = t1.NROCORRMOV
    
    /*  Delete all children in "OCOAPURQFO"  */
    delete OCOAPURQFO
    from   OCOAPURQFO t2, deleted t1
    where  t2.CDFILILANCMO = t1.CDFILILANCMO
     and   t2.NROCORRMOV = t1.NROCORRMOV
    
    /*  Delete all children in "DESCMOVOCORR"  */
    delete DESCMOVOCORR
    from   DESCMOVOCORR t2, deleted t1
    where  t2.CDFILILANCMO = t1.CDFILILANCMO
     and   t2.NROCORRMOV = t1.NROCORRMOV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movocorrprod" for table "MOVOCORRPROD"  */
create trigger td_movocorrprod on MOVOCORRPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ACAOOCORPROD"  */
    delete ACAOOCORPROD
    from   ACAOOCORPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRMOVOCOPROD = t1.NRMOVOCOPROD
    
    /*  Delete all children in "PAREOCORPROD"  */
    delete PAREOCORPROD
    from   PAREOCORPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRMOVOCOPROD = t1.NRMOVOCOPROD
    
    /*  Delete all children in "AVALOCORPROD"  */
    delete AVALOCORPROD
    from   AVALOCORPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRMOVOCOPROD = t1.NRMOVOCOPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_movreal" for table "MOVREAL"  */
create trigger td_movreal on MOVREAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "LOGMOVREAL"  */
    delete LOGMOVREAL
    from   LOGMOVREAL t2, deleted t1
    where  t2.CDREDUCONT = t1.CDREDUCONT
     and   t2.CDSEQUEXTR = t1.CDSEQUEXTR
     and   t2.NRSEQMOVREAL = t1.NRSEQMOVREAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_natucont" for table "NATUCONT"  */
create trigger td_natucont on NATUCONT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "NATUCONT" if children still exist in "MOTICONT"  */
    if exists (select 1
               from   MOTICONT t2, deleted t1
               where  t2.CDNATUCONT = t1.CDNATUCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOTICONT". Cannot delete parent "NATUCONT".'
          goto error
       end
    
    /*  Delete all children in "NATUCONTOPER"  */
    delete NATUCONTOPER
    from   NATUCONTOPER t2, deleted t1
    where  t2.CDNATUCONT = t1.CDNATUCONT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_notafiscal" for table "NOTAFISCAL"  */
create trigger td_notafiscal on NOTAFISCAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "SERVPLANO"  */
    if exists (select 1
               from   SERVPLANO t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPLANO". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "ITEMNOTAFISC"  */
    if exists (select 1
               from   ITEMNOTAFISC t2, deleted t1
               where  t2.CDFILIALIT = t1.CDFILIAL
                and   t2.NRLANCTONFIT = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMNOTAFISC". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "CONTAPAGAR"  */
    if exists (select 1
               from   CONTAPAGAR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTAPAGAR". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDFILLANC = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "MAPAAPURAC"  */
    if exists (select 1
               from   MAPAAPURAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPAAPURAC". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "ROMANEIONF"  */
    if exists (select 1
               from   ROMANEIONF t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIONF". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "AMOSTRAPROC"  */
    if exists (select 1
               from   AMOSTRAPROC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AMOSTRAPROC". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "LANCDESPFATU"  */
    if exists (select 1
               from   LANCDESPFATU t2, deleted t1
               where  t2.CDFILILANC = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCDESPFATU". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "PREVRETI"  */
    if exists (select 1
               from   PREVRETI t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETI". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "FORMULARIONF"  */
    if exists (select 1
               from   FORMULARIONF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMULARIONF". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFRELACAF"  */
    if exists (select 1
               from   NFRELACAF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACAF". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFABASTEROTA"  */
    if exists (select 1
               from   NFABASTEROTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFABASTEROTA". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "MAPARESUMECF"  */
    if exists (select 1
               from   MAPARESUMECF t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPARESUMECF". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "EXPNFFRETE"  */
    if exists (select 1
               from   EXPNFFRETE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXPNFFRETE". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "ITREQUESTO"  */
    if exists (select 1
               from   ITREQUESTO t2, deleted t1
               where  t2.CDFILIALRE = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUESTO". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "ITEMROMANEIO"  */
    if exists (select 1
               from   ITEMROMANEIO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMROMANEIO". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "OCORRERFATUR"  */
    if exists (select 1
               from   OCORRERFATUR t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OCORRERFATUR". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFRELACAFSAI"  */
    if exists (select 1
               from   NFRELACAFSAI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACAFSAI". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "INVOICE"  */
    if exists (select 1
               from   INVOICE t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVOICE". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFRELACPEXOL"  */
    if exists (select 1
               from   NFRELACPEXOL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACPEXOL". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFRELACORC"  */
    if exists (select 1
               from   NFRELACORC t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACORC". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "MOVCAIXA"  */
    if exists (select 1
               from   MOVCAIXA t2, deleted t1
               where  t2.CDFILIALNF = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVCAIXA". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Cannot delete parent "NOTAFISCAL" if children still exist in "NFRELACCDRS"  */
    if exists (select 1
               from   NFRELACCDRS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRLANCTONF = t1.NRLANCTONF)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACCDRS". Cannot delete parent "NOTAFISCAL".'
          goto error
       end
    
    /*  Delete all children in "ITEMENCARGO"  */
    delete ITEMENCARGO
    from   ITEMENCARGO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "ITEMNOTAFISC"  */
    delete ITEMNOTAFISC
    from   ITEMNOTAFISC t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "IMPOSTONF"  */
    delete IMPOSTONF
    from   IMPOSTONF t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "ITEMPRODUTO"  */
    delete ITEMPRODUTO
    from   ITEMPRODUTO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "DADOSADIC"  */
    delete DADOSADIC
    from   DADOSADIC t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "LANCTOESTOQ"  */
    delete LANCTOESTOQ
    from   LANCTOESTOQ t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "IMPOSNFTOT"  */
    delete IMPOSNFTOT
    from   IMPOSNFTOT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "CONTARECEBER"  */
    delete CONTARECEBER
    from   CONTARECEBER t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "NFRELACSOLI"  */
    delete NFRELACSOLI
    from   NFRELACSOLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "NFRELACMAPAB"  */
    delete NFRELACMAPAB
    from   NFRELACMAPAB t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "RELAT_CDRS"  */
    delete RELAT_CDRS
    from   RELAT_CDRS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "DADOSNF"  */
    delete DADOSNF
    from   DADOSNF t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF
    
    /*  Delete all children in "DADOSORIGNF"  */
    delete DADOSORIGNF
    from   DADOSORIGNF t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRLANCTONF = t1.NRLANCTONF

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_numerornf" for table "NUMERORNF"  */
create trigger td_numerornf on NUMERORNF for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDFILIALRE = t1.CDFILIAL
     and   t2.DTNUMERNF = t1.DTNUMERNF
     and   t2.IDENTRSAID = t1.IDENTRSAID
     and   t2.IDTIPOOPERNF = t1.IDTIPOOPERNF
     and   t2.NRNUMERNF = t1.NRNUMERNF

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_nutriente" for table "NUTRIENTE"  */
create trigger td_nutriente on NUTRIENTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "LIMNUTRI"  */
    if exists (select 1
               from   LIMNUTRI t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMNUTRI". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "NUTRPROD"  */
    if exists (select 1
               from   NUTRPROD t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRPROD". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "DIAGNUTR"  */
    if exists (select 1
               from   DIAGNUTR t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIAGNUTR". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "NUTRECOM"  */
    if exists (select 1
               from   NUTRECOM t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRECOM". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "NUTRNECE"  */
    if exists (select 1
               from   NUTRNECE t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRNECE". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "GREQUIVALE"  */
    if exists (select 1
               from   GREQUIVALE t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GREQUIVALE". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "NUTRIENTE" if children still exist in "NUTRECOMPACI"  */
    if exists (select 1
               from   NUTRECOMPACI t2, deleted t1
               where  t2.CDNUTRIENTE = t1.CDNUTRIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRECOMPACI". Cannot delete parent "NUTRIENTE".'
          goto error
       end
    
    /*  Delete all children in "PRINATIVNUTR"  */
    delete PRINATIVNUTR
    from   PRINATIVNUTR t2, deleted t1
    where  t2.CDNUTRIENTE = t1.CDNUTRIENTE
    
    /*  Delete all children in "TRABNUTRIREC"  */
    delete TRABNUTRIREC
    from   TRABNUTRIREC t2, deleted t1
    where  t2.CDNUTRIENTE = t1.CDNUTRIENTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_operador" for table "OPERADOR"  */
create trigger td_operador on OPERADOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONSPACI"  */
    if exists (select 1
               from   CONSPACI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSPACI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVOPSOL"  */
    if exists (select 1
               from   MOVOPSOL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPSOL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGNECECOMP"  */
    if exists (select 1
               from   LOGNECECOMP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGNECECOMP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGPREVCONS"  */
    if exists (select 1
               from   LOGPREVCONS t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVCONS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGPREVRETI"  */
    if exists (select 1
               from   LOGPREVRETI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVRETI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVSOLEXT"  */
    if exists (select 1
               from   MOVSOLEXT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVSOLEXT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TALAOCHEQUE"  */
    if exists (select 1
               from   TALAOCHEQUE t2, deleted t1
               where  t2.CDRESPRECEB = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TALAOCHEQUE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDADMINISTRA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGSERVTRAN"  */
    if exists (select 1
               from   LOGSERVTRAN t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGSERVTRAN". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROGOPER"  */
    if exists (select 1
               from   PROGOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGLIBEPERS"  */
    if exists (select 1
               from   LOGLIBEPERS t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGLIBEPERS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TARESERV"  */
    if exists (select 1
               from   TARESERV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TARESERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGFASETRAM"  */
    if exists (select 1
               from   LOGFASETRAM t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGFASETRAM". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CNABPAG"  */
    if exists (select 1
               from   CNABPAG t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABPAG". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CNABREC"  */
    if exists (select 1
               from   CNABREC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABREC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TARESERV"  */
    if exists (select 1
               from   TARESERV t2, deleted t1
               where  t2.CDOPLIBETEST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TARESERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VERSGSIS"  */
    if exists (select 1
               from   VERSGSIS t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VERSGSIS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TURCAIXA"  */
    if exists (select 1
               from   TURCAIXA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURCAIXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TESTTARE"  */
    if exists (select 1
               from   TESTTARE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTTARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CNABBCO"  */
    if exists (select 1
               from   CNABBCO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNABBCO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGDATACOMP"  */
    if exists (select 1
               from   LOGDATACOMP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGDATACOMP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPANALISE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PREVRETI"  */
    if exists (select 1
               from   PREVRETI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PAROPERSERV"  */
    if exists (select 1
               from   PAROPERSERV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAROPERSERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "GERENTE"  */
    if exists (select 1
               from   GERENTE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERENTE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMPRADOR"  */
    if exists (select 1
               from   COMPRADOR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPRADOR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MALADIRETA"  */
    if exists (select 1
               from   MALADIRETA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MALADIRETA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ATUALIZABD"  */
    if exists (select 1
               from   ATUALIZABD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATUALIZABD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSACAO"  */
    if exists (select 1
               from   OBSACAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSACAO"  */
    if exists (select 1
               from   OBSACAO t2, deleted t1
               where  t2.CDOPEVISTOAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGTRAMACAO"  */
    if exists (select 1
               from   LOGTRAMACAO t2, deleted t1
               where  t2.CDOPERECEBAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGTRAMACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGTRAMACAO"  */
    if exists (select 1
               from   LOGTRAMACAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGTRAMACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETAPACAO"  */
    if exists (select 1
               from   ETAPACAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETAPACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDOPERABERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDOPERRESPAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGPREVACAO"  */
    if exists (select 1
               from   LOGPREVACAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PARAMFILIAL"  */
    if exists (select 1
               from   PARAMFILIAL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGAPURQFO"  */
    if exists (select 1
               from   LOGAPURQFO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGAPURQFO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDOPERAVAL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "NUTRICIO"  */
    if exists (select 1
               from   NUTRICIO t2, deleted t1
               where  t2.CDNUTRICIO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRICIO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGDATAPREV"  */
    if exists (select 1
               from   LOGDATAPREV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGDATAPREV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "DELIVERY"  */
    if exists (select 1
               from   DELIVERY t2, deleted t1
               where  t2.CDOPCANCDELI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DELIVERY". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CUSTOPRODUTO"  */
    if exists (select 1
               from   CUSTOPRODUTO t2, deleted t1
               where  t2.CDOPERCPRO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPRODUTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDOPERPCIA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PLANOPGESTA"  */
    if exists (select 1
               from   PLANOPGESTA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANOPGESTA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGEATIVISALA"  */
    if exists (select 1
               from   AGEATIVISALA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGEATIVISALA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGCOPECTBL"  */
    if exists (select 1
               from   LOGCOPECTBL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCOPECTBL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGMATRICULA"  */
    if exists (select 1
               from   LOGMATRICULA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGMATRICULA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TURCAIXA"  */
    if exists (select 1
               from   TURCAIXA t2, deleted t1
               where  t2.CDOPERABER = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURCAIXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TURCAIXA"  */
    if exists (select 1
               from   TURCAIXA t2, deleted t1
               where  t2.CDOPERFECH = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURCAIXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGEQUIPPRO"  */
    if exists (select 1
               from   LOGEQUIPPRO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEQUIPPRO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGENEQUI"  */
    if exists (select 1
               from   AGENEQUI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENEQUI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SEQNUMAVIENT"  */
    if exists (select 1
               from   SEQNUMAVIENT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SEQNUMAVIENT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDOPEINCAUDI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "INSPECAUDIT"  */
    if exists (select 1
               from   INSPECAUDIT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECAUDIT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMINSAUDIT"  */
    if exists (select 1
               from   ITEMINSAUDIT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMINSAUDIT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VISTINSAUDIT"  */
    if exists (select 1
               from   VISTINSAUDIT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISTINSAUDIT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PESSAUDIT"  */
    if exists (select 1
               from   PESSAUDIT t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PESSAUDIT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SIMULACAO"  */
    if exists (select 1
               from   SIMULACAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SIMULACAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VISHEMOPRDOA"  */
    if exists (select 1
               from   VISHEMOPRDOA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISHEMOPRDOA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "POOLHEMOPR"  */
    if exists (select 1
               from   POOLHEMOPR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POOLHEMOPR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDOPEDEVDOAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDOPEINUHEMO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDOPEENVHEMO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDOPERECHEMO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVPROTESE"  */
    if exists (select 1
               from   MOVPROTESE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVPROTESE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "REQUHOSP"  */
    if exists (select 1
               from   REQUHOSP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUHOSP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RETIHOSP"  */
    if exists (select 1
               from   RETIHOSP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIHOSP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITRETIHOSP"  */
    if exists (select 1
               from   ITRETIHOSP t2, deleted t1
               where  t2.CDOPERECRETI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITRETIHOSP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VENDEDOR"  */
    if exists (select 1
               from   VENDEDOR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDEDOR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BANCSOLU"  */
    if exists (select 1
               from   BANCSOLU t2, deleted t1
               where  t2.CDOPINCLSOLU = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCSOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BANCSOLU"  */
    if exists (select 1
               from   BANCSOLU t2, deleted t1
               where  t2.CDOPAPROSOLU = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCSOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVICLIE"  */
    if exists (select 1
               from   MOVICLIE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICLIE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVICLIE"  */
    if exists (select 1
               from   MOVICLIE t2, deleted t1
               where  t2.CDOPERECSALD = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICLIE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PREVENDA"  */
    if exists (select 1
               from   PREVENDA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ESPEPROD"  */
    if exists (select 1
               from   ESPEPROD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SALDOCAIXA"  */
    if exists (select 1
               from   SALDOCAIXA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALDOCAIXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVIVALOR"  */
    if exists (select 1
               from   MOVIVALOR t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVIVALOR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROCFABRICA"  */
    if exists (select 1
               from   PROCFABRICA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCFABRICA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGUTEQUIP"  */
    if exists (select 1
               from   LOGUTEQUIP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGUTEQUIP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VISTOINSPEC"  */
    if exists (select 1
               from   VISTOINSPEC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISTOINSPEC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PREVCOMP"  */
    if exists (select 1
               from   PREVCOMP t2, deleted t1
               where  t2.CDOPGERPRCOM = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PREVCOMP"  */
    if exists (select 1
               from   PREVCOMP t2, deleted t1
               where  t2.CDOPAPRPRCOM = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VISESPEPRO"  */
    if exists (select 1
               from   VISESPEPRO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISESPEPRO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITAGENFUNC"  */
    if exists (select 1
               from   ITAGENFUNC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITAGENFUNC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONETAPAOP"  */
    if exists (select 1
               from   CONETAPAOP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONETAPAOP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGCATRACA"  */
    if exists (select 1
               from   LOGCATRACA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCATRACA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGMOVREAL"  */
    if exists (select 1
               from   LOGMOVREAL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGMOVREAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERCLIE"  */
    if exists (select 1
               from   OPERCLIE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERCLIE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CANDIDATO"  */
    if exists (select 1
               from   CANDIDATO t2, deleted t1
               where  t2.CDOPEENTCAND = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CANDIDATO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RECURETAPAOP"  */
    if exists (select 1
               from   RECURETAPAOP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECURETAPAOP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGINDICE"  */
    if exists (select 1
               from   LOGINDICE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGINDICE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPERSOLI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TARESERV"  */
    if exists (select 1
               from   TARESERV t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TARESERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDOPERCONT = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EVOLCONT"  */
    if exists (select 1
               from   EVOLCONT t2, deleted t1
               where  t2.CDOPEREVOL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLCONT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGFASETRAM"  */
    if exists (select 1
               from   LOGFASETRAM t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGFASETRAM". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPVALIIMPL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVIVALOR"  */
    if exists (select 1
               from   MOVIVALOR t2, deleted t1
               where  t2.CDOPERSOLVLR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVIVALOR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ROTATRANS"  */
    if exists (select 1
               from   ROTATRANS t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTATRANS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EXEMPLAR"  */
    if exists (select 1
               from   EXEMPLAR t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXEMPLAR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EXEMPLAR"  */
    if exists (select 1
               from   EXEMPLAR t2, deleted t1
               where  t2.CDOPERVISTO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXEMPLAR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDOPVERIIMPL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDOPCONCCONT = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "REVIPUBLPROD"  */
    if exists (select 1
               from   REVIPUBLPROD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REVIPUBLPROD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOTE"  */
    if exists (select 1
               from   LOTE t2, deleted t1
               where  t2.CDOPELIBLOTE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOTE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RELACOPER"  */
    if exists (select 1
               from   RELACOPER t2, deleted t1
               where  t2.CDOPERELAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RELACOPER"  */
    if exists (select 1
               from   RELACOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSERVPROSP"  */
    if exists (select 1
               from   OBSERVPROSP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSERVPROSP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSERVCLIE"  */
    if exists (select 1
               from   OBSERVCLIE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSERVCLIE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDOPERABER = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDOPERCONC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMTAREFA"  */
    if exists (select 1
               from   ITEMTAREFA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "APROATIVTARE"  */
    if exists (select 1
               from   APROATIVTARE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APROATIVTARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ATIVPESSOAL"  */
    if exists (select 1
               from   ATIVPESSOAL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVPESSOAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "INSPECAUDIT"  */
    if exists (select 1
               from   INSPECAUDIT t2, deleted t1
               where  t2.CDOPECONCINS = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECAUDIT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "REQUESTO"  */
    if exists (select 1
               from   REQUESTO t2, deleted t1
               where  t2.CDOPERREQU = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUESTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CODIGOBARRA"  */
    if exists (select 1
               from   CODIGOBARRA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODIGOBARRA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERPROJETO"  */
    if exists (select 1
               from   OPERPROJETO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACORDOREQPRO"  */
    if exists (select 1
               from   ACORDOREQPRO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACORDOREQPRO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDOPERABASEV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BAIXARECEBIDA"  */
    if exists (select 1
               from   BAIXARECEBIDA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXARECEBIDA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BAIXAPAGA"  */
    if exists (select 1
               from   BAIXAPAGA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXAPAGA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BASELINEPROJ"  */
    if exists (select 1
               from   BASELINEPROJ t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BASELINEPROJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BASELINEPROJ"  */
    if exists (select 1
               from   BASELINEPROJ t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BASELINEPROJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RISCOPROJETO"  */
    if exists (select 1
               from   RISCOPROJETO t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RISCOPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RISCOPROJETO"  */
    if exists (select 1
               from   RISCOPROJETO t2, deleted t1
               where  t2.CDOPEOCORISC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RISCOPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGREQPROD"  */
    if exists (select 1
               from   LOGREQPROD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGREQPROD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERENTITELE"  */
    if exists (select 1
               from   OPERENTITELE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERENTITELE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ABASTEROTA"  */
    if exists (select 1
               from   ABASTEROTA t2, deleted t1
               where  t2.CDOPERABASEV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ABASTEROTA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "JUSTIFICAVMN"  */
    if exists (select 1
               from   JUSTIFICAVMN t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTIFICAVMN". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMPREBASE"  */
    if exists (select 1
               from   ITEMPREBASE t2, deleted t1
               where  t2.CDOPERATZIPB = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPREBASE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDOPERABER = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDOPERCONC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDOPERLANC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMSOLI"  */
    if exists (select 1
               from   ITEMSOLI t2, deleted t1
               where  t2.CDOPERAPROV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVPESQUISA"  */
    if exists (select 1
               from   MOVPESQUISA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVPESQUISA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "INCONSPEDIDO"  */
    if exists (select 1
               from   INCONSPEDIDO t2, deleted t1
               where  t2.CDOPERLIBE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCONSPEDIDO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGMENSVEND"  */
    if exists (select 1
               from   LOGMENSVEND t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGMENSVEND". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HISITEMPED"  */
    if exists (select 1
               from   HISITEMPED t2, deleted t1
               where  t2.CDOPEMANHIST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISITEMPED". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PREVISAOPDV"  */
    if exists (select 1
               from   PREVISAOPDV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVISAOPDV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGPONTO"  */
    if exists (select 1
               from   LOGPONTO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPONTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGMANUTBD"  */
    if exists (select 1
               from   LOGMANUTBD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGMANUTBD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ASSINATAREFA"  */
    if exists (select 1
               from   ASSINATAREFA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ASSINATAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDOPERGERA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RESPONSAVBEM"  */
    if exists (select 1
               from   RESPONSAVBEM t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESPONSAVBEM". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGCHEQUEREC"  */
    if exists (select 1
               from   LOGCHEQUEREC t2, deleted t1
               where  t2.CDOPEBORCREC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCHEQUEREC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMTAREFA"  */
    if exists (select 1
               from   ITEMTAREFA t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMTAREFA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MENUOPER"  */
    if exists (select 1
               from   MENUOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MENUOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VENDAREST"  */
    if exists (select 1
               from   VENDAREST t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAREST". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMANDACXA"  */
    if exists (select 1
               from   COMANDACXA t2, deleted t1
               where  t2.CDOPERCOMCXA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMANDACXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMANDACXA"  */
    if exists (select 1
               from   COMANDACXA t2, deleted t1
               where  t2.CDOPECANCOMC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMANDACXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGRUPAF"  */
    if exists (select 1
               from   AGRUPAF t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGRUPAF". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDOPERBAIXA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "RESERVAMESA"  */
    if exists (select 1
               from   RESERVAMESA t2, deleted t1
               where  t2.CDOPERINCRES = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVAMESA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COLETAPR"  */
    if exists (select 1
               from   COLETAPR t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETAPR". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BREAKDOWN"  */
    if exists (select 1
               from   BREAKDOWN t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BREAKDOWN". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SERVBREAK"  */
    if exists (select 1
               from   SERVBREAK t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVBREAK". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMPROPER"  */
    if exists (select 1
               from   COMPROPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPROPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TEMPOPREPVOO"  */
    if exists (select 1
               from   TEMPOPREPVOO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPVOO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "APURPRECO"  */
    if exists (select 1
               from   APURPRECO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURPRECO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BKGTHREAD"  */
    if exists (select 1
               from   BKGTHREAD t2, deleted t1
               where  t2.CDOPERTHREAD = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BKGTHREAD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSOLIOPER"  */
    if exists (select 1
               from   OBSOLIOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSOLIOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERADORURA"  */
    if exists (select 1
               from   OPERADORURA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERADORURA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CTRLACESPER"  */
    if exists (select 1
               from   CTRLACESPER t2, deleted t1
               where  t2.CDOPERPER = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLACESPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LISTAINFLAC"  */
    if exists (select 1
               from   LISTAINFLAC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LISTAINFLAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGLISINFLAC"  */
    if exists (select 1
               from   LOGLISINFLAC t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGLISINFLAC". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "GSCRESULTADO"  */
    if exists (select 1
               from   GSCRESULTADO t2, deleted t1
               where  t2.CDGOPEGSCRES = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GSCRESULTADO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "GATILHO"  */
    if exists (select 1
               from   GATILHO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GATILHO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BANCSOLU"  */
    if exists (select 1
               from   BANCSOLU t2, deleted t1
               where  t2.CDOPREVISOLU = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BANCSOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "UTILBANCSOLU"  */
    if exists (select 1
               from   UTILBANCSOLU t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UTILBANCSOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITCOMANDACXA"  */
    if exists (select 1
               from   ITCOMANDACXA t2, deleted t1
               where  t2.CDOPECAITCXA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDACXA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROCESSO"  */
    if exists (select 1
               from   PROCESSO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCESSO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDOPERAPUR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDOPECANCNF = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGACESSO"  */
    if exists (select 1
               from   LOGACESSO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGACESSO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONTROLSENHA"  */
    if exists (select 1
               from   CONTROLSENHA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTROLSENHA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.CDOPERESPRJ = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMPEDEXOL"  */
    if exists (select 1
               from   ITEMPEDEXOL t2, deleted t1
               where  t2.CDOPERBAIXA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDEXOL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HABILOPERPRJ"  */
    if exists (select 1
               from   HABILOPERPRJ t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABILOPERPRJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CRONOPROJETO"  */
    if exists (select 1
               from   CRONOPROJETO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CRONOPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDOPEINCOREV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMPRA"  */
    if exists (select 1
               from   COMPRA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPRA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMPEDIDO"  */
    if exists (select 1
               from   ITEMPEDIDO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDIDO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONSULTADW"  */
    if exists (select 1
               from   CONSULTADW t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSULTADW". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "FERIOPER"  */
    if exists (select 1
               from   FERIOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FERIOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETLRESULTDES"  */
    if exists (select 1
               from   ETLRESULTDES t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLRESULTDES". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVOPAF"  */
    if exists (select 1
               from   MOVOPAF t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPAF". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HISTORCAEVEN"  */
    if exists (select 1
               from   HISTORCAEVEN t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTORCAEVEN". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDOPEJUSTORC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CTRLAPROVCARD"  */
    if exists (select 1
               from   CTRLAPROVCARD t2, deleted t1
               where  t2.CDOPERLIBECARDMATR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPROVCARD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CTRLAPROVCARD"  */
    if exists (select 1
               from   CTRLAPROVCARD t2, deleted t1
               where  t2.CDOPERAPROCARDMATR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPROVCARD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CTRLAPROVCARD"  */
    if exists (select 1
               from   CTRLAPROVCARD t2, deleted t1
               where  t2.CDOPERAPROCARDFILI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPROVCARD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGAPROVCARD"  */
    if exists (select 1
               from   LOGAPROVCARD t2, deleted t1
               where  t2.CDOPERLOGAPROVCARD = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGAPROVCARD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGCALENCARD"  */
    if exists (select 1
               from   LOGCALENCARD t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGCALENCARD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONSSOLOL"  */
    if exists (select 1
               from   CONSSOLOL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSOLOL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "COMANDAFOS"  */
    if exists (select 1
               from   COMANDAFOS t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMANDAFOS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "JUSTCONSSOLOL"  */
    if exists (select 1
               from   JUSTCONSSOLOL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTCONSSOLOL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OCUPAOPERPROJ"  */
    if exists (select 1
               from   OCUPAOPERPROJ t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OCUPAOPERPROJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EVENTOAEV"  */
    if exists (select 1
               from   EVENTOAEV t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVENTOAEV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGENDAEVENTO"  */
    if exists (select 1
               from   AGENDAEVENTO t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDAEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EXECEVENTO"  */
    if exists (select 1
               from   EXECEVENTO t2, deleted t1
               where  t2.CDOPERDEST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXECEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "APROATIVTARE"  */
    if exists (select 1
               from   APROATIVTARE t2, deleted t1
               where  t2.CDOPERATEND = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APROATIVTARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMVALIDATARE"  */
    if exists (select 1
               from   ITEMVALIDATARE t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVALIDATARE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "EQUIPETRAB"  */
    if exists (select 1
               from   EQUIPETRAB t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPETRAB". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ROPFILIAL"  */
    if exists (select 1
               from   ROPFILIAL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROPFILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUGER"  */
    if exists (select 1
               from   ETL_RESUGER t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUGER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUGER"  */
    if exists (select 1
               from   ETL_RESUGER t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUGER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDOPERSOLI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGPREVCONSPDV"  */
    if exists (select 1
               from   LOGPREVCONSPDV t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVCONSPDV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONTRATOLINP"  */
    if exists (select 1
               from   CONTRATOLINP t2, deleted t1
               where  t2.CDOPERLINPRESC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOLINP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CONTRATOSERV"  */
    if exists (select 1
               from   CONTRATOSERV t2, deleted t1
               where  t2.CDOPERSERVRESC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOSERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TESTCAND"  */
    if exists (select 1
               from   TESTCAND t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTCAND". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "APURCOMISOLU"  */
    if exists (select 1
               from   APURCOMISOLU t2, deleted t1
               where  t2.CDOPERAPUR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURCOMISOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMCOMISOLU"  */
    if exists (select 1
               from   ITEMCOMISOLU t2, deleted t1
               where  t2.CDOPERRECECOMI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOMISOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGENDAREL"  */
    if exists (select 1
               from   AGENDAREL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDAREL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOGESTATDIET"  */
    if exists (select 1
               from   LOGESTATDIET t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGESTATDIET". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDOPERAPROV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUSERV"  */
    if exists (select 1
               from   ETL_RESUSERV t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUSERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETL_RESUSERV"  */
    if exists (select 1
               from   ETL_RESUSERV t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUSERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDOPECORORE1 = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDOPECORORE2 = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "AGENDASALAO"  */
    if exists (select 1
               from   AGENDASALAO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASALAO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMORCAEVEN"  */
    if exists (select 1
               from   ITEMORCAEVEN t2, deleted t1
               where  t2.CDOPLIBPRORE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCAEVEN". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "BAIXARECEBIDA"  */
    if exists (select 1
               from   BAIXARECEBIDA t2, deleted t1
               where  t2.CDOPCFBAIREC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BAIXARECEBIDA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PERMUTAORE"  */
    if exists (select 1
               from   PERMUTAORE t2, deleted t1
               where  t2.CDOPINCPEORE = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERMUTAORE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VALORLINP"  */
    if exists (select 1
               from   VALORLINP t2, deleted t1
               where  t2.CDOPERREAJLINP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALORLINP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMDWOPER"  */
    if exists (select 1
               from   ITEMDWOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMDWOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "DIMEODWOPER"  */
    if exists (select 1
               from   DIMEODWOPER t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIMEODWOPER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ITEMCOMISOLU"  */
    if exists (select 1
               from   ITEMCOMISOLU t2, deleted t1
               where  t2.CDOPERUTILSOLU = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOMISOLU". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "CHT"  */
    if exists (select 1
               from   CHT t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CHT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "VALORSERV"  */
    if exists (select 1
               from   VALORSERV t2, deleted t1
               where  t2.CDOPERREAJSERV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALORSERV". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETLDRO"  */
    if exists (select 1
               from   ETLDRO t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLDRO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETLDRO"  */
    if exists (select 1
               from   ETLDRO t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLDRO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HISTFILIAL"  */
    if exists (select 1
               from   HISTFILIAL t2, deleted t1
               where  t2.CDGOPERACHIST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTFILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "HISTFILIAL"  */
    if exists (select 1
               from   HISTFILIAL t2, deleted t1
               where  t2.CDSUPERVHIST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTFILIAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PRODBLOQVND"  */
    if exists (select 1
               from   PRODBLOQVND t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODBLOQVND". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDSUPERVISOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDGOPERAC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDOPERRESP = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERGRPROJETO"  */
    if exists (select 1
               from   OPERGRPROJETO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERGRPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "ACORDOGRPROJETO"  */
    if exists (select 1
               from   ACORDOGRPROJETO t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACORDOGRPROJETO". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OBSRELACCLIE"  */
    if exists (select 1
               from   OBSRELACCLIE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSRELACCLIE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PEDCOLGERAL"  */
    if exists (select 1
               from   PEDCOLGERAL t2, deleted t1
               where  t2.CDOPERLANC = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLGERAL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "TRABLIMPEZA"  */
    if exists (select 1
               from   TRABLIMPEZA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRABLIMPEZA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MOVOPITEMPEDCOL"  */
    if exists (select 1
               from   MOVOPITEMPEDCOL t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPITEMPEDCOL". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLGRACORDOPROJ"  */
    if exists (select 1
               from   SOLGRACORDOPROJ t2, deleted t1
               where  t2.CDOPEREXCLUSAO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLGRACORDOPROJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLGRACORDOPROJ"  */
    if exists (select 1
               from   SOLGRACORDOPROJ t2, deleted t1
               where  t2.CDOPERINCLUSAO = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLGRACORDOPROJ". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERPRESTCONTA"  */
    if exists (select 1
               from   OPERPRESTCONTA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERPRESTCONTA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERTIPOSUPRI"  */
    if exists (select 1
               from   OPERTIPOSUPRI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERTIPOSUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERAPROVSUPRI"  */
    if exists (select 1
               from   OPERAPROVSUPRI t2, deleted t1
               where  t2.CDGERENTEAPROV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERAPROVSUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERAPROVSUPRI"  */
    if exists (select 1
               from   OPERAPROVSUPRI t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERAPROVSUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDOPERSOLI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDOPERBENEF = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SOLICITASUPRI"  */
    if exists (select 1
               from   SOLICITASUPRI t2, deleted t1
               where  t2.CDOPERAPROV = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITASUPRI". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOTENFE"  */
    if exists (select 1
               from   LOTENFE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOTENFE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PLANEJAEQUIPE"  */
    if exists (select 1
               from   PLANEJAEQUIPE t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANEJAEQUIPE". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PARCLIEFAS"  */
    if exists (select 1
               from   PARCLIEFAS t2, deleted t1
               where  t2.CDGERENTEIMPCLI = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLIEFAS". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "OPERPRECOPROP"  */
    if exists (select 1
               from   OPERPRECOPROP t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPERPRECOPROP". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "LOCK_LISTA"  */
    if exists (select 1
               from   LOCK_LISTA t2, deleted t1
               where  t2.CDOPERADOR = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOCK_LISTA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "MAPARESULT"  */
    if exists (select 1
               from   MAPARESULT t2, deleted t1
               where  t2.CDOPERINCL = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPARESULT". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "PRESTACONTA"  */
    if exists (select 1
               from   PRESTACONTA t2, deleted t1
               where  t2.CDOPERCONCPREST = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRESTACONTA". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Cannot delete parent "OPERADOR" if children still exist in "SERVPREST"  */
    if exists (select 1
               from   SERVPREST t2, deleted t1
               where  t2.CDOPERINCLCSA = t1.CDOPERADOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREST". Cannot delete parent "OPERADOR".'
          goto error
       end
    
    /*  Delete all children in "FILIOPER"  */
    delete FILIOPER
    from   FILIOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "OPERGRUPOP"  */
    delete OPERGRUPOP
    from   OPERGRUPOP t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "PROGOPEROPC"  */
    delete PROGOPEROPC
    from   PROGOPEROPC t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "LOGTRMOVCONS"  */
    delete LOGTRMOVCONS
    from   LOGTRMOVCONS t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "HABILOPER"  */
    delete HABILOPER
    from   HABILOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "NATUCONTOPER"  */
    delete NATUCONTOPER
    from   NATUCONTOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "NATUASSUOPER"  */
    delete NATUASSUOPER
    from   NATUASSUOPER t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "OPERALMOX"  */
    delete OPERALMOX
    from   OPERALMOX t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR
    
    /*  Delete all children in "OPERLESTOQ"  */
    delete OPERLESTOQ
    from   OPERLESTOQ t2, deleted t1
    where  t2.CDOPERADOR = t1.CDOPERADOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_orcaevento" for table "ORCAEVENTO"  */
create trigger td_orcaevento on ORCAEVENTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "OBSINTORCAEV"  */
    if exists (select 1
               from   OBSINTORCAEV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSINTORCAEV". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDFILIALORC = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "NFRELACORC"  */
    if exists (select 1
               from   NFRELACORC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACORC". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "AGENDASALAO"  */
    if exists (select 1
               from   AGENDASALAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASALAO". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "CONTRATOORCEVE"  */
    if exists (select 1
               from   CONTRATOORCEVE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOORCEVE". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "PERMUTAORE"  */
    if exists (select 1
               from   PERMUTAORE t2, deleted t1
               where  t2.CDFILIALOREV = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERMUTAORE". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "ORCAEVENTO" if children still exist in "DEVOLORCAEVEN"  */
    if exists (select 1
               from   DEVOLORCAEVEN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
                and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEVOLORCAEVEN". Cannot delete parent "ORCAEVENTO".'
          goto error
       end
    
    /*  Delete all children in "ITEMORCAEVEN"  */
    delete ITEMORCAEVEN
    from   ITEMORCAEVEN t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "ITMAOOBRAORCA"  */
    delete ITMAOOBRAORCA
    from   ITMAOOBRAORCA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "DEGUSTACAO"  */
    delete DEGUSTACAO
    from   DEGUSTACAO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "OBSEXTORCAEV"  */
    delete OBSEXTORCAEV
    from   OBSEXTORCAEV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "ACERTOORCEVE"  */
    delete ACERTOORCEVE
    from   ACERTOORCEVE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "ENDCLIORCEVE"  */
    delete ENDCLIORCEVE
    from   ENDCLIORCEVE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE
    
    /*  Delete all children in "HISTORCAEVEN"  */
    delete HISTORCAEVEN
    from   HISTORCAEVEN t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORCAEVENTO = t1.NRORCAEVENTO
     and   t2.NRSEQORCAEVE = t1.NRSEQORCAEVE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_orcament" for table "ORCAMENT"  */
create trigger td_orcament on ORCAMENT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMORCA"  */
    delete ITEMORCA
    from   ITEMORCA t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDORCAMENT = t1.CDORCAMENT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ordproducao" for table "ORDPRODUCAO"  */
create trigger td_ordproducao on ORDPRODUCAO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ORDPRODUCAO" if children still exist in "ORDPRODUCAO"  */
    if exists (select 1
               from   ORDPRODUCAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPRODPAI = t1.NRORDPROD
                and   t2.NRSUBOPPAI = t1.NRSUBORDPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORDPRODUCAO". Cannot delete parent "ORDPRODUCAO".'
          goto error
       end
    
    /*  Cannot delete parent "ORDPRODUCAO" if children still exist in "RESERPRODPED"  */
    if exists (select 1
               from   RESERPRODPED t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPROD = t1.NRORDPROD
                and   t2.NRSUBORDPROD = t1.NRSUBORDPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERPRODPED". Cannot delete parent "ORDPRODUCAO".'
          goto error
       end
    
    /*  Cannot delete parent "ORDPRODUCAO" if children still exist in "SERVBREAK"  */
    if exists (select 1
               from   SERVBREAK t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPROD = t1.NRORDPROD
                and   t2.NRSUBORDPROD = t1.NRSUBORDPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVBREAK". Cannot delete parent "ORDPRODUCAO".'
          goto error
       end
    
    /*  Cannot delete parent "ORDPRODUCAO" if children still exist in "ORDPRODUCAO"  */
    if exists (select 1
               from   ORDPRODUCAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPRODCON = t1.NRORDPROD
                and   t2.NRSUBOPCON = t1.NRSUBORDPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORDPRODUCAO". Cannot delete parent "ORDPRODUCAO".'
          goto error
       end
    
    /*  Cannot delete parent "ORDPRODUCAO" if children still exist in "INSUPEDIDOP"  */
    if exists (select 1
               from   INSUPEDIDOP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRORDPROD = t1.NRORDPROD
                and   t2.NRSUBORDPROD = t1.NRSUBORDPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSUPEDIDOP". Cannot delete parent "ORDPRODUCAO".'
          goto error
       end
    
    /*  Delete all children in "AUDITORIA"  */
    delete AUDITORIA
    from   AUDITORIA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
    
    /*  Delete all children in "ETAFABRICOP"  */
    delete ETAFABRICOP
    from   ETAFABRICOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
    
    /*  Delete all children in "SUBPRETAPAOP"  */
    delete SUBPRETAPAOP
    from   SUBPRETAPAOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
    
    /*  Delete all children in "PEDIDOP"  */
    delete PEDIDOP
    from   PEDIDOP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_paciente" for table "PACIENTE"  */
create trigger td_paciente on PACIENTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PACIENTE" if children still exist in "HABALIMT"  */
    if exists (select 1
               from   HABALIMT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABALIMT". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "INTERNAC"  */
    if exists (select 1
               from   INTERNAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INTERNAC". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "CONSPACI"  */
    if exists (select 1
               from   CONSPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSPACI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "INQUERPACI"  */
    if exists (select 1
               from   INQUERPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INQUERPACI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "MARCACAO"  */
    if exists (select 1
               from   MARCACAO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCACAO". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "ATENREALIZ"  */
    if exists (select 1
               from   ATENREALIZ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATENREALIZ". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "RETOPROG"  */
    if exists (select 1
               from   RETOPROG t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETOPROG". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "LISTAESPR"  */
    if exists (select 1
               from   LISTAESPR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LISTAESPR". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "EXTRMARC"  */
    if exists (select 1
               from   EXTRMARC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXTRMARC". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "CONSMEDI"  */
    if exists (select 1
               from   CONSMEDI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSMEDI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "SERVINTE"  */
    if exists (select 1
               from   SERVINTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVINTE". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "PRESDIET"  */
    if exists (select 1
               from   PRESDIET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRESDIET". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "FREQCONSALIM"  */
    if exists (select 1
               from   FREQCONSALIM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FREQCONSALIM". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "DESCHABIPACI"  */
    if exists (select 1
               from   DESCHABIPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCHABIPACI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "DESCINTOPACI"  */
    if exists (select 1
               from   DESCINTOPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCINTOPACI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "INTOALIM"  */
    if exists (select 1
               from   INTOALIM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INTOALIM". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "OBSERVPACI"  */
    if exists (select 1
               from   OBSERVPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSERVPACI". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "ATENHOSP"  */
    if exists (select 1
               from   ATENHOSP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATENHOSP". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "CONSNUTR"  */
    if exists (select 1
               from   CONSNUTR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSNUTR". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "PARTATIVSALA"  */
    if exists (select 1
               from   PARTATIVSALA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTATIVSALA". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "TRIAGDOAC"  */
    if exists (select 1
               from   TRIAGDOAC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIRECEPT = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRIAGDOAC". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "ITAGENFUNC"  */
    if exists (select 1
               from   ITAGENFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITAGENFUNC". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "APURFATUGH"  */
    if exists (select 1
               from   APURFATUGH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIENTE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUGH". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Cannot delete parent "PACIENTE" if children still exist in "PACIENTE"  */
    if exists (select 1
               from   PACIENTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDPACIMAE = t1.CDPACIENTE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PACIENTE". Cannot delete parent "PACIENTE".'
          goto error
       end
    
    /*  Delete all children in "ENDEPACI"  */
    delete ENDEPACI
    from   ENDEPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
    
    /*  Delete all children in "INDIPACI"  */
    delete INDIPACI
    from   INDIPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
    
    /*  Delete all children in "RESPPACI"  */
    delete RESPPACI
    from   RESPPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
    
    /*  Delete all children in "MEDICAPACI"  */
    delete MEDICAPACI
    from   MEDICAPACI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_padrateio" for table "PADRATEIO"  */
create trigger td_padrateio on PADRATEIO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PADRATEIO" if children still exist in "CONTCTBL"  */
    if exists (select 1
               from   CONTCTBL t2, deleted t1
               where  t2.CDRATEIO = t1.CDRATEIO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTCTBL". Cannot delete parent "PADRATEIO".'
          goto error
       end
    
    /*  Delete all children in "ITEMRATEIO"  */
    delete ITEMRATEIO
    from   ITEMRATEIO t2, deleted t1
    where  t2.CDRATEIO = t1.CDRATEIO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_palm" for table "PALM"  */
create trigger td_palm on PALM for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "HISTPALM"  */
    delete HISTPALM
    from   HISTPALM t2, deleted t1
    where  t2.CDPALM = t1.CDPALM

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_pedcoleta" for table "PEDCOLETA"  */
create trigger td_pedcoleta on PEDCOLETA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PEDCOLETA" if children still exist in "COLETAPR"  */
    if exists (select 1
               from   COLETAPR t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO
                and   t2.CDFORNECED = t1.CDFORNECED
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDCOLETA = t1.NRPEDCOLETA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETAPR". Cannot delete parent "PEDCOLETA".'
          goto error
       end
    
    /*  Delete all children in "ITEMPEDCOL"  */
    delete ITEMPEDCOL
    from   ITEMPEDCOL t2, deleted t1
    where  t2.CDREGIAO = t1.CDREGIAO
     and   t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDCOLETA = t1.NRPEDCOLETA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_pedido" for table "PEDIDO"  */
create trigger td_pedido on PEDIDO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PEDIDO" if children still exist in "PARPEDIDO"  */
    if exists (select 1
               from   PARPEDIDO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARPEDIDO". Cannot delete parent "PEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "PEDIDO" if children still exist in "INCONSPEDIDO"  */
    if exists (select 1
               from   INCONSPEDIDO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCONSPEDIDO". Cannot delete parent "PEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "PEDIDO" if children still exist in "SOLICITA"  */
    if exists (select 1
               from   SOLICITA t2, deleted t1
               where  t2.CDFILIALPED = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICITA". Cannot delete parent "PEDIDO".'
          goto error
       end
    
    /*  Cannot delete parent "PEDIDO" if children still exist in "GRADEDISTR"  */
    if exists (select 1
               from   GRADEDISTR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPEDIDO = t1.NRPEDIDO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRADEDISTR". Cannot delete parent "PEDIDO".'
          goto error
       end
    
    /*  Delete all children in "ITEMPEDIDO"  */
    delete ITEMPEDIDO
    from   ITEMPEDIDO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
    
    /*  Delete all children in "PRAZOPGTOPED"  */
    delete PRAZOPGTOPED
    from   PRAZOPGTOPED t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO
    
    /*  Delete all children in "OBSPEDIDO"  */
    delete OBSPEDIDO
    from   OBSPEDIDO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPEDIDO = t1.NRPEDIDO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_pedilinpclie" for table "PEDILINPCLIE"  */
create trigger td_pedilinpclie on PEDILINPCLIE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "RATECOMILINP"  */
    delete RATECOMILINP
    from   RATECOMILINP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
     and   t2.NRLINPPEDP = t1.NRLINPPEDP
     and   t2.CDCLIENTE = t1.CDCLIENTE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_perientr" for table "PERIENTR"  */
create trigger td_perientr on PERIENTR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PERIENTR" if children still exist in "PERFILPROD"  */
    if exists (select 1
               from   PERFILPROD t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERFILPROD". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "CALECOMP"  */
    if exists (select 1
               from   CALECOMP t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALECOMP". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "AGENTARE"  */
    if exists (select 1
               from   AGENTARE t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENTARE". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "REFORFIL"  */
    if exists (select 1
               from   REFORFIL t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REFORFIL". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "REFOREG"  */
    if exists (select 1
               from   REFOREG t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REFOREG". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "ROTATRANS"  */
    if exists (select 1
               from   ROTATRANS t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTATRANS". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "CLIENFILIAL"  */
    if exists (select 1
               from   CLIENFILIAL t2, deleted t1
               where  t2.CDPERIENTR = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENFILIAL". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Cannot delete parent "PERIENTR" if children still exist in "ROTATRANS"  */
    if exists (select 1
               from   ROTATRANS t2, deleted t1
               where  t2.CDPERIENTEN = t1.CDPERIENTR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTATRANS". Cannot delete parent "PERIENTR".'
          goto error
       end
    
    /*  Delete all children in "DIASENTR"  */
    delete DIASENTR
    from   DIASENTR t2, deleted t1
    where  t2.CDPERIENTR = t1.CDPERIENTR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_pessoa" for table "PESSOA"  */
create trigger td_pessoa on PESSOA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PESSOA" if children still exist in "REGISCATRACA"  */
    if exists (select 1
               from   REGISCATRACA t2, deleted t1
               where  t2.NRPESSOA = t1.NRPESSOA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REGISCATRACA". Cannot delete parent "PESSOA".'
          goto error
       end
    
    /*  Delete all children in "FOTODIGITAL"  */
    delete FOTODIGITAL
    from   FOTODIGITAL t2, deleted t1
    where  t2.NRPESSOA = t1.NRPESSOA
    
    /*  Delete all children in "FOTOPESSOA"  */
    delete FOTOPESSOA
    from   FOTOPESSOA t2, deleted t1
    where  t2.NRPESSOA = t1.NRPESSOA
    
    /*  Delete all children in "DIGITALCATRACA"  */
    delete DIGITALCATRACA
    from   DIGITALCATRACA t2, deleted t1
    where  t2.NRPESSOA = t1.NRPESSOA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_planativtare" for table "PLANATIVTARE"  */
create trigger td_planativtare on PLANATIVTARE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "RELATIVTARE"  */
    if exists (select 1
               from   RELATIVTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELATIVTARE". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "RELATIVTARE"  */
    if exists (select 1
               from   RELATIVTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTARERELAC = t1.NRTAREFA
                and   t2.NRATIVRELAC = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELATIVTARE". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "APROATIVTARE"  */
    if exists (select 1
               from   APROATIVTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APROATIVTARE". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "RESPPLATIV"  */
    if exists (select 1
               from   RESPPLATIV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESPPLATIV". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "INSPLANATIV"  */
    if exists (select 1
               from   INSPLANATIV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPLANATIV". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "ARQATIVTARE"  */
    if exists (select 1
               from   ARQATIVTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ARQATIVTARE". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "CRONOPROJETO"  */
    if exists (select 1
               from   CRONOPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CRONOPROJETO". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Cannot delete parent "PLANATIVTARE" if children still exist in "VALIDATAREFA"  */
    if exists (select 1
               from   VALIDATAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA
                and   t2.NRPLATIVTARE = t1.NRPLATIVTARE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALIDATAREFA". Cannot delete parent "PLANATIVTARE".'
          goto error
       end
    
    /*  Delete all children in "DESCPLATIV"  */
    delete DESCPLATIV
    from   DESCPLATIV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTAREFA = t1.NRTAREFA
     and   t2.NRPLATIVTARE = t1.NRPLATIVTARE
    
    /*  Delete all children in "PLATVRELITTR"  */
    delete PLATVRELITTR
    from   PLATVRELITTR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTAREFA = t1.NRTAREFA
     and   t2.NRPLATIVTARE = t1.NRPLATIVTARE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_planovoo" for table "PLANOVOO"  */
create trigger td_planovoo on PLANOVOO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PLANOVOO" if children still exist in "DELIVERY"  */
    if exists (select 1
               from   DELIVERY t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA
                and   t2.NRVOO = t1.NRVOO
                and   t2.CDTRECHO = t1.CDTRECHO
                and   t2.DTHRPLAVOOPV = t1.DTHRPLAVOOPV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DELIVERY". Cannot delete parent "PLANOVOO".'
          goto error
       end
    
    /*  Cannot delete parent "PLANOVOO" if children still exist in "PREVCOMPRA"  */
    if exists (select 1
               from   PREVCOMPRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIACOMP = t1.CDCIA
                and   t2.NRVOOCOMP = t1.NRVOO
                and   t2.CDTRECHOCOMP = t1.CDTRECHO
                and   t2.DTPREVCOMP = t1.DTHRPLAVOOPV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMPRA". Cannot delete parent "PLANOVOO".'
          goto error
       end
    
    /*  Cannot delete parent "PLANOVOO" if children still exist in "DELIVERYCANC"  */
    if exists (select 1
               from   DELIVERYCANC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA
                and   t2.NRVOO = t1.NRVOO
                and   t2.CDTRECHO = t1.CDTRECHO
                and   t2.DTHRPLAVOOPV = t1.DTHRPLAVOOPV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DELIVERYCANC". Cannot delete parent "PLANOVOO".'
          goto error
       end
    
    /*  Delete all children in "PREVISAOPAX"  */
    delete PREVISAOPAX
    from   PREVISAOPAX t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRVOO = t1.NRVOO
     and   t2.CDTRECHO = t1.CDTRECHO
     and   t2.DTHRPLAVOOPV = t1.DTHRPLAVOOPV
    
    /*  Delete all children in "SERVPLANO"  */
    delete SERVPLANO
    from   SERVPLANO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRVOO = t1.NRVOO
     and   t2.CDTRECHO = t1.CDTRECHO
     and   t2.DTHRPLAVOOPV = t1.DTHRPLAVOOPV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_plcalorico" for table "PLCALORICO"  */
create trigger td_plcalorico on PLCALORICO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PLCALOSERV"  */
    delete PLCALOSERV
    from   PLCALOSERV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTCONSPACI = t1.DTCONSPACI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prato" for table "PRATO"  */
create trigger td_prato on PRATO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PRATO" if children still exist in "FOTOPRAFIL"  */
    if exists (select 1
               from   FOTOPRAFIL t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOPRAFIL". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "FOTOPRASER"  */
    if exists (select 1
               from   FOTOPRASER t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOPRASER". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PRATFILI"  */
    if exists (select 1
               from   PRATFILI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATFILI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PRATSERV"  */
    if exists (select 1
               from   PRATSERV t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATSERV". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "FOTOPRAPAD"  */
    if exists (select 1
               from   FOTOPRAPAD t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOPRAPAD". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PRATPADR"  */
    if exists (select 1
               from   PRATPADR t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATPADR". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "RECEFILI"  */
    if exists (select 1
               from   RECEFILI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEFILI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "RECESERV"  */
    if exists (select 1
               from   RECESERV t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECESERV". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "COMPCARD"  */
    if exists (select 1
               from   COMPCARD t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPCARD". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "MEDIPRATO"  */
    if exists (select 1
               from   MEDIPRATO t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MEDIPRATO". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "HABALIMT"  */
    if exists (select 1
               from   HABALIMT t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABALIMT". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDFILAUXPRRI = t1.CDFILAUXPRAT
                and   t2.CDMAIORPRERI = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDFILAUXPRSL = t1.CDFILAUXPRAT
                and   t2.CDMAIORPRESL = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PREPPRASER"  */
    if exists (select 1
               from   PREPPRASER t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPPRASER". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PREPPRAFIL"  */
    if exists (select 1
               from   PREPPRAFIL t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPPRAFIL". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PREPPRAPAD"  */
    if exists (select 1
               from   PREPPRAPAD t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPPRAPAD". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "RECDIEPAD"  */
    if exists (select 1
               from   RECDIEPAD t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECDIEPAD". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "FREQALIM"  */
    if exists (select 1
               from   FREQALIM t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FREQALIM". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "RECEPADR"  */
    if exists (select 1
               from   RECEPADR t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEPADR". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "UTILPRATSEM"  */
    if exists (select 1
               from   UTILPRATSEM t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UTILPRATSEM". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "SERVTRAN"  */
    if exists (select 1
               from   SERVTRAN t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTRAN". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "CONFESCO"  */
    if exists (select 1
               from   CONFESCO t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFESCO". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PREVRETIPRAT"  */
    if exists (select 1
               from   PREVRETIPRAT t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETIPRAT". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "ITEMRETIPRAT"  */
    if exists (select 1
               from   ITEMRETIPRAT t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETIPRAT". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "ITEQUIVALE"  */
    if exists (select 1
               from   ITEQUIVALE t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEQUIVALE". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "COMCADIAPACI"  */
    if exists (select 1
               from   COMCADIAPACI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMCADIAPACI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "ITINQUERPACI"  */
    if exists (select 1
               from   ITINQUERPACI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITINQUERPACI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "CARDCONSNUTR"  */
    if exists (select 1
               from   CARDCONSNUTR t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDCONSNUTR". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "EFETDIET"  */
    if exists (select 1
               from   EFETDIET t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETDIET". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PARAMPLANEJA"  */
    if exists (select 1
               from   PARAMPLANEJA t2, deleted t1
               where  t2.CDFILCONSFIX = t1.CDFILAUXPRAT
                and   t2.CDPRACONSFIX = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPLANEJA". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PARAMPLANEJA"  */
    if exists (select 1
               from   PARAMPLANEJA t2, deleted t1
               where  t2.CDFILCONSPER = t1.CDFILAUXPRAT
                and   t2.CDPRACONSPER = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPLANEJA". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "ITEMRETI"  */
    if exists (select 1
               from   ITEMRETI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PREVRETI"  */
    if exists (select 1
               from   PREVRETI t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETI". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "COMPCARDPME"  */
    if exists (select 1
               from   COMPCARDPME t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATO = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPCARDPME". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "COMPCARD"  */
    if exists (select 1
               from   COMPCARD t2, deleted t1
               where  t2.CDFILAUXPRATMAT = t1.CDFILAUXPRAT
                and   t2.CDPRATOMAT = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPCARD". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Cannot delete parent "PRATO" if children still exist in "PARADIETO"  */
    if exists (select 1
               from   PARADIETO t2, deleted t1
               where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
                and   t2.CDPRATOENTPAR = t1.CDPRATO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARADIETO". Cannot delete parent "PRATO".'
          goto error
       end
    
    /*  Delete all children in "LIMPEPRA"  */
    delete LIMPEPRA
    from   LIMPEPRA t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO
    
    /*  Delete all children in "RESTOPCA"  */
    delete RESTOPCA
    from   RESTOPCA t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO
    
    /*  Delete all children in "FREQOPCA"  */
    delete FREQOPCA
    from   FREQOPCA t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO
    
    /*  Delete all children in "RESTPRAT"  */
    delete RESTPRAT
    from   RESTPRAT t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO
    
    /*  Delete all children in "FREQPRAT"  */
    delete FREQPRAT
    from   FREQPRAT t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_precobase" for table "PRECOBASE"  */
create trigger td_precobase on PRECOBASE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PRECOUTI"  */
    if exists (select 1
               from   PRECOUTI t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOUTI". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "VENDEDOR"  */
    if exists (select 1
               from   VENDEDOR t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDEDOR". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PRECOBASECLI"  */
    if exists (select 1
               from   PRECOBASECLI t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOBASECLI". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "TIPOOPERAC"  */
    if exists (select 1
               from   TIPOOPERAC t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERAC". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "TROCA"  */
    if exists (select 1
               from   TROCA t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TROCA". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ITDESPREBASE"  */
    if exists (select 1
               from   ITDESPREBASE t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITDESPREBASE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDPRECOBASE = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDPREBAMOBRA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDPREBAMOBRA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDPRBAQUEBRA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDPRBAHREXTR = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDPRBASOBRA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "ACERTOORCEVE"  */
    if exists (select 1
               from   ACERTOORCEVE t2, deleted t1
               where  t2.CDPRBAEXTRA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOORCEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDPBASEPRATA = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Cannot delete parent "PRECOBASE" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDPBASEREPMT = t1.CDPRECOBASE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "PRECOBASE".'
          goto error
       end
    
    /*  Delete all children in "ITEMPREBASE"  */
    delete ITEMPREBASE
    from   ITEMPREBASE t2, deleted t1
    where  t2.CDPRECOBASE = t1.CDPRECOBASE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_precoproposta" for table "PRECOPROPOSTA"  */
create trigger td_precoproposta on PRECOPROPOSTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "SISTPRECOPROP"  */
    delete SISTPRECOPROP
    from   SISTPRECOPROP t2, deleted t1
    where  t2.CDPRECOPROP = t1.CDPRECOPROP
    
    /*  Delete all children in "USUPRECOPROP"  */
    delete USUPRECOPROP
    from   USUPRECOPROP t2, deleted t1
    where  t2.CDPRECOPROP = t1.CDPRECOPROP
    
    /*  Delete all children in "OPERPRECOPROP"  */
    delete OPERPRECOPROP
    from   OPERPRECOPROP t2, deleted t1
    where  t2.CDPRECOPROP = t1.CDPRECOPROP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_precosecia" for table "PRECOSECIA"  */
create trigger td_precosecia on PRECOSECIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PROPRECOSE"  */
    delete PROPRECOSE
    from   PROPRECOSE t2, deleted t1
    where  t2.NRTABPRECO = t1.NRTABPRECO
     and   t2.CDSERVICO = t1.CDSERVICO
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRSEQPRECO = t1.NRSEQPRECO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_precustoserv" for table "PRECUSTOSERV"  */
create trigger td_precustoserv on PRECUSTOSERV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITCUSTOSERV"  */
    delete ITCUSTOSERV
    from   ITCUSTOSERV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.CDMANCIA = t1.CDMANCIA
     and   t2.IDTPMANUAL = t1.IDTPMANUAL
     and   t2.CDSERVICO = t1.CDSERVICO
     and   t2.CDSUBGRUPO = t1.CDSUBGRUPO
     and   t2.DTCUSTOSERV = t1.DTCUSTOSERV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prevcomp" for table "PREVCOMP"  */
create trigger td_prevcomp on PREVCOMP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITPREVCOMP"  */
    delete ITPREVCOMP
    from   ITPREVCOMP t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPREVCOMP = t1.NRPREVCOMP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prevlista" for table "PREVLISTA"  */
create trigger td_prevlista on PREVLISTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PREVLISTA" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSEQPRELIST = t1.NRSEQPRELIST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "PREVLISTA".'
          goto error
       end
    
    /*  Cannot delete parent "PREVLISTA" if children still exist in "ITEXCLISPREC"  */
    if exists (select 1
               from   ITEXCLISPREC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSEQPRELIST = t1.NRSEQPRELIST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEXCLISPREC". Cannot delete parent "PREVLISTA".'
          goto error
       end
    
    /*  Delete all children in "ITMPREVLIST"  */
    delete ITMPREVLIST
    from   ITMPREVLIST t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSEQPRELIST = t1.NRSEQPRELIST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prevpaxprod" for table "PREVPAXPROD"  */
create trigger td_prevpaxprod on PREVPAXPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "SERVPREVPAXPROD"  */
    delete SERVPREVPAXPROD
    from   SERVPREVPAXPROD t2, deleted t1
    where  t2.CDCLASSE = t1.CDCLASSE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRVOO = t1.NRVOO
     and   t2.CDTRECHO = t1.CDTRECHO
     and   t2.DTVOO = t1.DTVOO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prevreti" for table "PREVRETI"  */
create trigger td_prevreti on PREVRETI for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PREVRETIPRAT"  */
    delete PREVRETIPRAT
    from   PREVRETIPRAT t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.DTPREVRETI = t1.DTPREVRETI
     and   t2.NRREQUEXTR = t1.NRREQUEXTR
     and   t2.IDPREVRETI = t1.IDPREVRETI
     and   t2.DTLIMITERETI = t1.DTLIMITERETI
     and   t2.HRLIMITERETI = t1.HRLIMITERETI
     and   t2.NRPRPREVRETI = t1.NRPRPREVRETI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prinativ" for table "PRINATIV"  */
create trigger td_prinativ on PRINATIV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PRINATIV" if children still exist in "PRODPRINATIV"  */
    if exists (select 1
               from   PRODPRINATIV t2, deleted t1
               where  t2.CDPRINATIV = t1.CDPRINATIV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPRINATIV". Cannot delete parent "PRINATIV".'
          goto error
       end
    
    /*  Delete all children in "PRINATIVMEDI"  */
    delete PRINATIVMEDI
    from   PRINATIVMEDI t2, deleted t1
    where  t2.CDPRINATIV = t1.CDPRINATIV
    
    /*  Delete all children in "PRINATIVNUTR"  */
    delete PRINATIVNUTR
    from   PRINATIVNUTR t2, deleted t1
    where  t2.CDPRINATIV = t1.CDPRINATIV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_processaprod" for table "PROCESSAPROD"  */
create trigger td_processaprod on PROCESSAPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROCESSAPROD" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPROCPROD = t1.NRPROCPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "PROCESSAPROD".'
          goto error
       end
    
    /*  Delete all children in "ITEMPROCPROD"  */
    delete ITEMPROCPROD
    from   ITEMPROCPROD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPROCPROD = t1.NRPROCPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_processo" for table "PROCESSO"  */
create trigger td_processo on PROCESSO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROCESSO" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "FASETRAMDEST"  */
    if exists (select 1
               from   FASETRAMDEST t2, deleted t1
               where  t2.CDPROCESGRUP = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FASETRAMDEST". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PROCINTECLI"  */
    if exists (select 1
               from   PROCINTECLI t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCINTECLI". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PROCINTEFOR"  */
    if exists (select 1
               from   PROCINTEFOR t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCINTEFOR". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PROCINTEPRO"  */
    if exists (select 1
               from   PROCINTEPRO t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCINTEPRO". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PARFILIALSAC"  */
    if exists (select 1
               from   PARFILIALSAC t2, deleted t1
               where  t2.CDGRUPPRDEVO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALSAC". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "INSPPROCESSO"  */
    if exists (select 1
               from   INSPPROCESSO t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPPROCESSO". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "ASSUNTO"  */
    if exists (select 1
               from   ASSUNTO t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ASSUNTO". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "QUALIFPROC"  */
    if exists (select 1
               from   QUALIFPROC t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QUALIFPROC". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDPROCMONIT = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "ETLRESULTDES"  */
    if exists (select 1
               from   ETLRESULTDES t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLRESULTDES". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDPROCATEINT = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDPROCATEDES = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PLANEJAEQUIPE"  */
    if exists (select 1
               from   PLANEJAEQUIPE t2, deleted t1
               where  t2.CDPROCESSO = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANEJAEQUIPE". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Cannot delete parent "PROCESSO" if children still exist in "PARFILIALGEP"  */
    if exists (select 1
               from   PARFILIALGEP t2, deleted t1
               where  t2.CDPROATIVEXTFLP = t1.CDPROCESSO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARFILIALGEP". Cannot delete parent "PROCESSO".'
          goto error
       end
    
    /*  Delete all children in "HABIPROCESSO"  */
    delete HABIPROCESSO
    from   HABIPROCESSO t2, deleted t1
    where  t2.CDPROCESSO = t1.CDPROCESSO
    
    /*  Delete all children in "DESCPROCESSO"  */
    delete DESCPROCESSO
    from   DESCPROCESSO t2, deleted t1
    where  t2.CDPROCESSO = t1.CDPROCESSO
    
    /*  Delete all children in "PROCESGPROC"  */
    delete PROCESGPROC
    from   PROCESGPROC t2, deleted t1
    where  t2.CDPROCESSO = t1.CDPROCESSO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_procfabrica" for table "PROCFABRICA"  */
create trigger td_procfabrica on PROCFABRICA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROCFABRICA" if children still exist in "ORDPRODUCAO"  */
    if exists (select 1
               from   ORDPRODUCAO t2, deleted t1
               where  t2.CDPROCFABRIC = t1.CDPROCFABRIC
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORDPRODUCAO". Cannot delete parent "PROCFABRICA".'
          goto error
       end
    
    /*  Cannot delete parent "PROCFABRICA" if children still exist in "PARPRODOPCONS"  */
    if exists (select 1
               from   PARPRODOPCONS t2, deleted t1
               where  t2.CDPROCFABRIC = t1.CDPROCFABRIC
                and   t2.CDPRODUTO = t1.CDPRODUTO
                and   t2.CDFILIAL = t1.CDFILIAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARPRODOPCONS". Cannot delete parent "PROCFABRICA".'
          goto error
       end
    
    /*  Delete all children in "ETAFABRIC"  */
    delete ETAFABRIC
    from   ETAFABRIC t2, deleted t1
    where  t2.CDPROCFABRIC = t1.CDPROCFABRIC
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDFILIAL = t1.CDFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prodconssolol" for table "PRODCONSSOLOL"  */
create trigger td_prodconssolol on PRODCONSSOLOL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMCONSSOLOL"  */
    delete ITEMCONSSOLOL
    from   ITEMCONSSOLOL t2, deleted t1
    where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDFILSOLI = t1.CDFILSOLI
     and   t2.CDFILLAN = t1.CDFILLAN
    
    /*  Delete all children in "ESTOCONSSOLOL"  */
    delete ESTOCONSSOLOL
    from   ESTOCONSSOLOL t2, deleted t1
    where  t2.NRSEQUCONSOL = t1.NRSEQUCONSOL
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDFILSOLI = t1.CDFILSOLI
     and   t2.CDFILLAN = t1.CDFILLAN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prodforn" for table "PRODFORN"  */
create trigger td_prodforn on PRODFORN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PZOPGTPRFO"  */
    delete PZOPGTPRFO
    from   PZOPGTPRFO t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODFORNCOMP"  */
    delete PRODFORNCOMP
    from   PRODFORNCOMP t2, deleted t1
    where  t2.CDFORNECED = t1.CDFORNECED
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_produto" for table "PRODUTO"  */
create trigger td_produto on PRODUTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTOCIA"  */
    if exists (select 1
               from   PRODUTOCIA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONVPRODUTO"  */
    if exists (select 1
               from   CONVPRODUTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INCIDENCIA"  */
    if exists (select 1
               from   INCIDENCIA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCIDENCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODFILI"  */
    if exists (select 1
               from   PRODFILI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INVEROTA"  */
    if exists (select 1
               from   INVEROTA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVEROTA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "POSESTDIA"  */
    if exists (select 1
               from   POSESTDIA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTDIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ALIQUOTA"  */
    if exists (select 1
               from   ALIQUOTA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQUOTA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CODFISCOPER"  */
    if exists (select 1
               from   CODFISCOPER t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODFISCOPER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CODSITUTRIB"  */
    if exists (select 1
               from   CODSITUTRIB t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIB". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECEFILI"  */
    if exists (select 1
               from   RECEFILI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEFILI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECESERV"  */
    if exists (select 1
               from   RECESERV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECESERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RESTPROD"  */
    if exists (select 1
               from   RESTPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LIMPESER"  */
    if exists (select 1
               from   LIMPESER t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPESER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LIMPEPRA"  */
    if exists (select 1
               from   LIMPEPRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPEPRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONS"  */
    if exists (select 1
               from   PREVCONS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVRETI"  */
    if exists (select 1
               from   PREVRETI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONSSERV"  */
    if exists (select 1
               from   CONSSERV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "NDPCAL"  */
    if exists (select 1
               from   NDPCAL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NDPCAL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "NUTRPROD"  */
    if exists (select 1
               from   NUTRPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.CDPRODMOVI = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RATENESTO"  */
    if exists (select 1
               from   RATENESTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATENESTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTNESTO"  */
    if exists (select 1
               from   CUSTNESTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTNESTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMRETI"  */
    if exists (select 1
               from   ITEMRETI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONVPROD"  */
    if exists (select 1
               from   CONVPROD t2, deleted t1
               where  t2.CDPRODESTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONVPROD"  */
    if exists (select 1
               from   CONVPROD t2, deleted t1
               where  t2.CDPRODCMPR = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SETOPRODCONS"  */
    if exists (select 1
               from   SETOPRODCONS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOPRODCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SETOPRODCARD"  */
    if exists (select 1
               from   SETOPRODCARD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SETOPRODCARD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVICO"  */
    if exists (select 1
               from   SERVICO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVICO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVCLIE"  */
    if exists (select 1
               from   SERVCLIE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MOVOPSOL"  */
    if exists (select 1
               from   MOVOPSOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPSOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMAUTF"  */
    if exists (select 1
               from   ITEMAUTF t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAUTF". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMSOLI"  */
    if exists (select 1
               from   ITEMSOLI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARTFILI"  */
    if exists (select 1
               from   PARTFILI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARTFILI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TESTEPROD"  */
    if exists (select 1
               from   TESTEPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTEPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODCMPFILI"  */
    if exists (select 1
               from   PRODCMPFILI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCMPFILI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCOMPRA"  */
    if exists (select 1
               from   PREVCOMPRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMPRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EMITENTEAF"  */
    if exists (select 1
               from   EMITENTEAF t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITENTEAF". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PERFILPROD"  */
    if exists (select 1
               from   PERFILPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERFILPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODNCMP"  */
    if exists (select 1
               from   PRODNCMP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODNCMP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MEDSOLI"  */
    if exists (select 1
               from   MEDSOLI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MEDSOLI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECEPADR"  */
    if exists (select 1
               from   RECEPADR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEPADR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LIMPRODAF"  */
    if exists (select 1
               from   LIMPRODAF t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPRODAF". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LIMGERPROD"  */
    if exists (select 1
               from   LIMGERPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMGERPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPEDIDO"  */
    if exists (select 1
               from   ITEMPEDIDO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDIDO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RETIRADA_CONS"  */
    if exists (select 1
               from   RETIRADA_CONS t2, deleted t1
               where  t2.CDPRODRCONS = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADA_CONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDPRODGENE = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDPRODPSER = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDPRODDESP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDPRODREFE = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONSERVREAL"  */
    if exists (select 1
               from   CONSERVREAL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSERVREAL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CURVAABC"  */
    if exists (select 1
               from   CURVAABC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CURVAABC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INVFISCFIL"  */
    if exists (select 1
               from   INVFISCFIL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVFISCFIL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INVENTFISC"  */
    if exists (select 1
               from   INVENTFISC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INVENTFISC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMVENDA"  */
    if exists (select 1
               from   ITEMVENDA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "QFOMARPROD"  */
    if exists (select 1
               from   QFOMARPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QFOMARPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MARCCOTA"  */
    if exists (select 1
               from   MARCCOTA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCCOTA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTOAP"  */
    if exists (select 1
               from   PRODUTOAP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOAP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPRECO"  */
    if exists (select 1
               from   ITEMPRECO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRECO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPEDCOL"  */
    if exists (select 1
               from   ITEMPEDCOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDCOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMROMANEIO"  */
    if exists (select 1
               from   ITEMROMANEIO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMROMANEIO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODPREFMED"  */
    if exists (select 1
               from   PRODPREFMED t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPREFMED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODPREFMED"  */
    if exists (select 1
               from   PRODPREFMED t2, deleted t1
               where  t2.CDPRODPREF = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPREFMED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ROTFABRICA"  */
    if exists (select 1
               from   ROTFABRICA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTFABRICA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ROTFABRICA"  */
    if exists (select 1
               from   ROTFABRICA t2, deleted t1
               where  t2.CDSUBPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTFABRICA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESCROTEIRO"  */
    if exists (select 1
               from   DESCROTEIRO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCROTEIRO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ORDPRODUCAO"  */
    if exists (select 1
               from   ORDPRODUCAO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORDPRODUCAO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TRABEVOLPREC"  */
    if exists (select 1
               from   TRABEVOLPREC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRABEVOLPREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMRETIEST"  */
    if exists (select 1
               from   ITEMRETIEST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETIEST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMRETPCP"  */
    if exists (select 1
               from   ITEMRETPCP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETPCP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LOGPRODUTOAP"  */
    if exists (select 1
               from   LOGPRODUTOAP t2, deleted t1
               where  t2.CDPRODUTO1 = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPRODUTOAP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AMOSTRAPROC"  */
    if exists (select 1
               from   AMOSTRAPROC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AMOSTRAPROC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMAMOSTRA"  */
    if exists (select 1
               from   ITEMAMOSTRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAMOSTRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROCESSAPROD"  */
    if exists (select 1
               from   PROCESSAPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCESSAPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPROCPROD"  */
    if exists (select 1
               from   ITEMPROCPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPROCPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "QFOPRODFORN"  */
    if exists (select 1
               from   QFOPRODFORN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QFOPRODFORN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.CDPRODPADDOS = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODFATOR"  */
    if exists (select 1
               from   PRODFATOR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFATOR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TPDOCTRANSAP"  */
    if exists (select 1
               from   TPDOCTRANSAP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPDOCTRANSAP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "HABALIMT"  */
    if exists (select 1
               from   HABALIMT t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HABALIMT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CODTRIBSAP"  */
    if exists (select 1
               from   CODTRIBSAP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODTRIBSAP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODMECA"  */
    if exists (select 1
               from   PRODMECA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODMECA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "VISTEQFORN"  */
    if exists (select 1
               from   VISTEQFORN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VISTEQFORN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONFESCO"  */
    if exists (select 1
               from   CONFESCO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFESCO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEQUIVALE"  */
    if exists (select 1
               from   ITEQUIVALE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEQUIVALE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMCADIAPACI"  */
    if exists (select 1
               from   COMCADIAPACI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMCADIAPACI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEQUIPPRO"  */
    if exists (select 1
               from   ITEQUIPPRO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEQUIPPRO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ALIQIMPFIS"  */
    if exists (select 1
               from   ALIQIMPFIS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ALIQIMPFIS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESPADMFATUR"  */
    if exists (select 1
               from   DESPADMFATUR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPADMFATUR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDPRODORIG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CNVPRODUNEST"  */
    if exists (select 1
               from   CNVPRODUNEST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNVPRODUNEST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CNVPRODUNEST"  */
    if exists (select 1
               from   CNVPRODUNEST t2, deleted t1
               where  t2.CDPRUNEST = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNVPRODUNEST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODIMPOINVE"  */
    if exists (select 1
               from   PRODIMPOINVE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODIMPOINVE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDPRAEROIN = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPRODUTO"  */
    if exists (select 1
               from   CUSTOPRODUTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MATDIDATICO"  */
    if exists (select 1
               from   MATDIDATICO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MATDIDATICO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ESTAGIO"  */
    if exists (select 1
               from   ESTAGIO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTAGIO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPRECOMATDI"  */
    if exists (select 1
               from   ITPRECOMATDI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPRECOMATDI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EMPRESTPROD"  */
    if exists (select 1
               from   EMPRESTPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRESTPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RESERVAPROD"  */
    if exists (select 1
               from   RESERVAPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERVAPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MOVILOTE"  */
    if exists (select 1
               from   MOVILOTE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVILOTE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INSPECPROD"  */
    if exists (select 1
               from   INSPECPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITINQUERPACI"  */
    if exists (select 1
               from   ITINQUERPACI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITINQUERPACI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMISSAO"  */
    if exists (select 1
               from   ITEMISSAO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMISSAO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ROTFABLOCAL"  */
    if exists (select 1
               from   ROTFABLOCAL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTFABLOCAL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ROTFABLOCAL"  */
    if exists (select 1
               from   ROTFABLOCAL t2, deleted t1
               where  t2.CDSUBPRODLOC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROTFABLOCAL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "NIVELPRODSEG"  */
    if exists (select 1
               from   NIVELPRODSEG t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NIVELPRODSEG". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPRECO"  */
    if exists (select 1
               from   ITEMPRECO t2, deleted t1
               where  t2.CDPRPAITABPR = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRECO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONSPDV"  */
    if exists (select 1
               from   PREVCONSPDV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSPDV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EXEMPLAR"  */
    if exists (select 1
               from   EXEMPLAR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EXEMPLAR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PCHAVEPROD"  */
    if exists (select 1
               from   PCHAVEPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PCHAVEPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AUTORPROD"  */
    if exists (select 1
               from   AUTORPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTORPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMAPABASEV"  */
    if exists (select 1
               from   ITEMAPABASEV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAPABASEV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TIPODEPE"  */
    if exists (select 1
               from   TIPODEPE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPODEPE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRESATHOSP"  */
    if exists (select 1
               from   PRESATHOSP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRESATHOSP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROGPRATHOSP"  */
    if exists (select 1
               from   PROGPRATHOSP t2, deleted t1
               where  t2.CDPRODSUBST = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGPRATHOSP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODPROC"  */
    if exists (select 1
               from   PRODPROC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPROC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPREVABASEV"  */
    if exists (select 1
               from   ITPREVABASEV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVABASEV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROCED"  */
    if exists (select 1
               from   PROCED t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODMETO"  */
    if exists (select 1
               from   PRODMETO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODMETO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CARDCONSNUTR"  */
    if exists (select 1
               from   CARDCONSNUTR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDCONSNUTR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESCROTLOCAL"  */
    if exists (select 1
               from   DESCROTLOCAL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCROTLOCAL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PUBLPROD"  */
    if exists (select 1
               from   PUBLPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PUBLPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREAUTPROD"  */
    if exists (select 1
               from   PREAUTPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREAUTPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESCPROGPR"  */
    if exists (select 1
               from   DESCPROGPR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCPROGPR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROAGRCONV"  */
    if exists (select 1
               from   PROAGRCONV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROAGRCONV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRNAUTCONV"  */
    if exists (select 1
               from   PRNAUTCONV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRNAUTCONV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "FTAUTOCONV"  */
    if exists (select 1
               from   FTAUTOCONV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FTAUTOCONV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MOEDUTCONV"  */
    if exists (select 1
               from   MOEDUTCONV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOEDUTCONV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "VALIHEMOCOMP"  */
    if exists (select 1
               from   VALIHEMOCOMP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VALIHEMOCOMP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COLETDOAC"  */
    if exists (select 1
               from   COLETDOAC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COLETDOAC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "HEMOPRDOAC"  */
    if exists (select 1
               from   HEMOPRDOAC t2, deleted t1
               where  t2.CDPRANTICOAG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HEMOPRDOAC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECEBDOAC"  */
    if exists (select 1
               from   RECEBDOAC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEBDOAC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROTESE"  */
    if exists (select 1
               from   PROTESE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROTESE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "FORMPRPROT"  */
    if exists (select 1
               from   FORMPRPROT t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRPROT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARCLIEPRVMN"  */
    if exists (select 1
               from   PARCLIEPRVMN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCLIEPRVMN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MEDICAMENTO"  */
    if exists (select 1
               from   MEDICAMENTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MEDICAMENTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITREQUHOSP"  */
    if exists (select 1
               from   ITREQUHOSP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUHOSP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONSPDV"  */
    if exists (select 1
               from   PREVCONSPDV t2, deleted t1
               where  t2.CDPRPAIPCPDV = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSPDV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESCVALRPR"  */
    if exists (select 1
               from   DESCVALRPR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESCVALRPR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPREVENDA"  */
    if exists (select 1
               from   ITPREVENDA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVENDA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ESPEPROD"  */
    if exists (select 1
               from   ESPEPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMPPROD"  */
    if exists (select 1
               from   COMPPROD t2, deleted t1
               where  t2.CDPRODINSUMO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROCFABRICA"  */
    if exists (select 1
               from   PROCFABRICA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROCFABRICA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INSUMETAPA"  */
    if exists (select 1
               from   INSUMETAPA t2, deleted t1
               where  t2.CDPRODINSUMO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSUMETAPA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPREVCOMP"  */
    if exists (select 1
               from   ITPREVCOMP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVCOMP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMAPABASEV"  */
    if exists (select 1
               from   ITEMAPABASEV t2, deleted t1
               where  t2.CDPRODORIGTR = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMAPABASEV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODTURCAIXA"  */
    if exists (select 1
               from   PRODTURCAIXA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODTURCAIXA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "POSESTCONS"  */
    if exists (select 1
               from   POSESTCONS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SUBPRETAPAOP"  */
    if exists (select 1
               from   SUBPRETAPAOP t2, deleted t1
               where  t2.CDSUBPRODETA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SUBPRETAPAOP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTPROD"  */
    if exists (select 1
               from   CUSTPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PERDAOP"  */
    if exists (select 1
               from   PERDAOP t2, deleted t1
               where  t2.CDPRODPERDA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERDAOP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "APURFATUGH"  */
    if exists (select 1
               from   APURFATUGH t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUGH". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RESPOSINSPEC"  */
    if exists (select 1
               from   RESPOSINSPEC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESPOSINSPEC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PERDAETAPA"  */
    if exists (select 1
               from   PERDAETAPA t2, deleted t1
               where  t2.CDPRODPERDA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERDAETAPA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPROPOSTA"  */
    if exists (select 1
               from   ITPROPOSTA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPROPOSTA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRECCONCORR"  */
    if exists (select 1
               from   PRECCONCORR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECCONCORR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "POSESTATU"  */
    if exists (select 1
               from   POSESTATU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "POSESTATU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "FORMPRECPROD"  */
    if exists (select 1
               from   FORMPRECPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRECPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RESERPRODPED"  */
    if exists (select 1
               from   RESERPRODPED t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESERPRODPED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CALCCUST"  */
    if exists (select 1
               from   CALCCUST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALCCUST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVPADR"  */
    if exists (select 1
               from   SERVPADR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPADR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESPHOSP"  */
    if exists (select 1
               from   DESPHOSP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPHOSP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PERCPRODFATU"  */
    if exists (select 1
               from   PERCPRODFATU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERCPRODFATU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODACABADO"  */
    if exists (select 1
               from   PRODACABADO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODACABADO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDPRODANTE = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVIVENDA"  */
    if exists (select 1
               from   PREVIVENDA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVIVENDA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "VERSAOPROD"  */
    if exists (select 1
               from   VERSAOPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VERSAOPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODLIBPATCH"  */
    if exists (select 1
               from   PRODLIBPATCH t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODLIBPATCH". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODLIBEVERS"  */
    if exists (select 1
               from   PRODLIBEVERS t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODLIBEVERS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "REQUISITPROD"  */
    if exists (select 1
               from   REQUISITPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUISITPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODREQPROD"  */
    if exists (select 1
               from   PRODREQPROD t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODREQPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODSUBCLIE"  */
    if exists (select 1
               from   PRODSUBCLIE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODSUBCLIE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODTIPCONS"  */
    if exists (select 1
               from   PRODTIPCONS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODTIPCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITREQUESTO"  */
    if exists (select 1
               from   ITREQUESTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREQUESTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODFABRIC"  */
    if exists (select 1
               from   PRODFABRIC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFABRIC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODPRINATIV"  */
    if exists (select 1
               from   PRODPRINATIV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODPRINATIV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODCENTCUST"  */
    if exists (select 1
               from   PRODCENTCUST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCENTCUST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECITEMAPABA"  */
    if exists (select 1
               from   RECITEMAPABA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECITEMAPABA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITVENDAVMN"  */
    if exists (select 1
               from   ITVENDAVMN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDAVMN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMPOSINSPEC"  */
    if exists (select 1
               from   COMPOSINSPEC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPOSINSPEC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMPOSINSPEC"  */
    if exists (select 1
               from   COMPOSINSPEC t2, deleted t1
               where  t2.CDPRODINSPEC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPOSINSPEC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODBASELINE"  */
    if exists (select 1
               from   PRODBASELINE t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODBASELINE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPREVABASAU"  */
    if exists (select 1
               from   ITPREVABASAU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVABASAU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVENDAEQU"  */
    if exists (select 1
               from   PREVENDAEQU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAEQU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVENDAVMN"  */
    if exists (select 1
               from   PREVENDAVMN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVENDAVMN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONSVMN"  */
    if exists (select 1
               from   PREVCONSVMN t2, deleted t1
               where  t2.CDSUBPRODVMN = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSVMN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "IMPITVENDAVM"  */
    if exists (select 1
               from   IMPITVENDAVM t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPITVENDAVM". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITABASTEROTA"  */
    if exists (select 1
               from   ITABASTEROTA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITABASTEROTA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPREVABAMAP"  */
    if exists (select 1
               from   ITPREVABAMAP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPREVABAMAP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "JUSTIFICAVMN"  */
    if exists (select 1
               from   JUSTIFICAVMN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTIFICAVMN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TAXAADMFATUR"  */
    if exists (select 1
               from   TAXAADMFATUR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAXAADMFATUR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESPPREV"  */
    if exists (select 1
               from   DESPPREV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPPREV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DESPPREVFILI"  */
    if exists (select 1
               from   DESPPREVFILI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESPPREVFILI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODFILIFATU"  */
    if exists (select 1
               from   PRODFILIFATU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFILIFATU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EMITAFFORN"  */
    if exists (select 1
               from   EMITAFFORN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMITAFFORN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "HISITEMPED"  */
    if exists (select 1
               from   HISITEMPED t2, deleted t1
               where  t2.CDPRODORIG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISITEMPED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODMETAVEND"  */
    if exists (select 1
               from   PRODMETAVEND t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODMETAVEND". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELAT_MANDATO"  */
    if exists (select 1
               from   RELAT_MANDATO t2, deleted t1
               where  t2.CDPRODCMPMAN = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_MANDATO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELAT_MANDATO"  */
    if exists (select 1
               from   RELAT_MANDATO t2, deleted t1
               where  t2.CDPRODCMPDES = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_MANDATO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMCMPPROD"  */
    if exists (select 1
               from   PARAMCMPPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCMPPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ARVOREQPROD"  */
    if exists (select 1
               from   ARVOREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ARVOREQPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOSUP"  */
    if exists (select 1
               from   CUSTOSUP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOSUP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "GRUPSIST"  */
    if exists (select 1
               from   GRUPSIST t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPSIST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SISTEMA"  */
    if exists (select 1
               from   SISTEMA t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SISTEMA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODTPMANUT"  */
    if exists (select 1
               from   PRODTPMANUT t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODTPMANUT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODSOLISERV"  */
    if exists (select 1
               from   PRODSOLISERV t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODSOLISERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITCOMANDAVEN"  */
    if exists (select 1
               from   ITCOMANDAVEN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDAVEN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CTBPRDTPOPER"  */
    if exists (select 1
               from   CTBPRDTPOPER t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTBPRDTPOPER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROLIVROPREC"  */
    if exists (select 1
               from   PROLIVROPREC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROLIVROPREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDRECPRIPC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTORECIA"  */
    if exists (select 1
               from   CUSTORECIA t2, deleted t1
               where  t2.CDRECPRIC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTORECIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTORECEITA"  */
    if exists (select 1
               from   CUSTORECEITA t2, deleted t1
               where  t2.CDRECPRI = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTORECEITA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPRODUTO"  */
    if exists (select 1
               from   CUSTOPRODUTO t2, deleted t1
               where  t2.CDRECPRIP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTORECIA"  */
    if exists (select 1
               from   CUSTORECIA t2, deleted t1
               where  t2.CDSUBGRU = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTORECIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDSUBGRU = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRECOSECIA"  */
    if exists (select 1
               from   PRECOSECIA t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECOSECIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROPSERVICO"  */
    if exists (select 1
               from   PROPSERVICO t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPSERVICO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PADEQUIP"  */
    if exists (select 1
               from   PADEQUIP t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADEQUIP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PADPREFIXO"  */
    if exists (select 1
               from   PADPREFIXO t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADPREFIXO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVPLANO"  */
    if exists (select 1
               from   SERVPLANO t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPLANO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PADVOOTRE"  */
    if exists (select 1
               from   PADVOOTRE t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PADVOOTRE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTORECIA"  */
    if exists (select 1
               from   CUSTORECIA t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTORECIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOPROCIA"  */
    if exists (select 1
               from   CUSTOPROCIA t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOPROCIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "FRCICLOSER"  */
    if exists (select 1
               from   FRCICLOSER t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FRCICLOSER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCOMPRA"  */
    if exists (select 1
               from   PREVCOMPRA t2, deleted t1
               where  t2.CDSERVCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMPRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMRETPCP"  */
    if exists (select 1
               from   ITEMRETPCP t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMRETPCP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDPRODCOUVER = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDPRODCONSUM = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITCOMANDACXA"  */
    if exists (select 1
               from   ITCOMANDACXA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDACXA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITVENDACAN"  */
    if exists (select 1
               from   ITVENDACAN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITVENDACAN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOSEMA"  */
    if exists (select 1
               from   CUSTOSEMA t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOSEMA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVBREAK"  */
    if exists (select 1
               from   SERVBREAK t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVBREAK". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDPRODCOMVEN = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODOCORRE"  */
    if exists (select 1
               from   PRODOCORRE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODOCORRE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "VRPRODBCIMPO"  */
    if exists (select 1
               from   VRPRODBCIMPO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VRPRODBCIMPO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TEMPOPREPSER"  */
    if exists (select 1
               from   TEMPOPREPSER t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPSER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TEMPOPREPSUB"  */
    if exists (select 1
               from   TEMPOPREPSUB t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPSUB". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TEMPOPREPSUB"  */
    if exists (select 1
               from   TEMPOPREPSUB t2, deleted t1
               where  t2.CDSUBGRU = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPSUB". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TEMPOPREPREC"  */
    if exists (select 1
               from   TEMPOPREPREC t2, deleted t1
               where  t2.CDRECEITA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TEMPOPREPREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ESTOSEGU"  */
    if exists (select 1
               from   ESTOSEGU t2, deleted t1
               where  t2.CDPRODESTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTOSEGU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELACEXPPROD"  */
    if exists (select 1
               from   RELACEXPPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACEXPPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDPRODFATURA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CDBARRAPROD"  */
    if exists (select 1
               from   CDBARRAPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CDBARRAPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMPRODMRP"  */
    if exists (select 1
               from   PARAMPRODMRP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODMRP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROGPRODU"  */
    if exists (select 1
               from   PROGPRODU t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGPRODU". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREMATPPROGP"  */
    if exists (select 1
               from   PREMATPPROGP t2, deleted t1
               where  t2.CDPRODPREVMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREMATPPROGP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CALEPRODMRP"  */
    if exists (select 1
               from   CALEPRODMRP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CALEPRODMRP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELAT_CDRS"  */
    if exists (select 1
               from   RELAT_CDRS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_CDRS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EFETCDRS"  */
    if exists (select 1
               from   EFETCDRS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCDRS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "EFETRATCDRS"  */
    if exists (select 1
               from   EFETRATCDRS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETRATCDRS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMENTREGA"  */
    if exists (select 1
               from   ITEMENTREGA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMENTREGA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODLOJA"  */
    if exists (select 1
               from   PRODLOJA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODLOJA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DEMAEXTRAMRP"  */
    if exists (select 1
               from   DEMAEXTRAMRP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEMAEXTRAMRP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "IMPOCONTRETI"  */
    if exists (select 1
               from   IMPOCONTRETI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOCONTRETI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "VRMAXVENPROD"  */
    if exists (select 1
               from   VRMAXVENPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VRMAXVENPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INCONSMRP"  */
    if exists (select 1
               from   INCONSMRP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCONSMRP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PERLISINFLAC"  */
    if exists (select 1
               from   PERLISINFLAC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERLISINFLAC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ESPEPROD"  */
    if exists (select 1
               from   ESPEPROD t2, deleted t1
               where  t2.CDPRODUTOPRI = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESPEPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TPSUBPRODPRO"  */
    if exists (select 1
               from   TPSUBPRODPRO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPSUBPRODPRO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDPRODCONSUF = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMPITVENDA"  */
    if exists (select 1
               from   COMPITVENDA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPITVENDA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITPRODORIEXP"  */
    if exists (select 1
               from   ITPRODORIEXP t2, deleted t1
               where  t2.CDPRODORIEXP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPRODORIEXP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVTIPOCARD"  */
    if exists (select 1
               from   SERVTIPOCARD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTIPOCARD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ASSUNTO"  */
    if exists (select 1
               from   ASSUNTO t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ASSUNTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODCLIE"  */
    if exists (select 1
               from   PRODCLIE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCLIE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONSPME"  */
    if exists (select 1
               from   PREVCONSPME t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSPME". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CMPCENTENDIR"  */
    if exists (select 1
               from   CMPCENTENDIR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CMPCENTENDIR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "OCORRERFATUR"  */
    if exists (select 1
               from   OCORRERFATUR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OCORRERFATUR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMVENDMERE"  */
    if exists (select 1
               from   ITEMVENDMERE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDMERE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMVENDMERE"  */
    if exists (select 1
               from   ITEMVENDMERE t2, deleted t1
               where  t2.CDPRODFATUR = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMVENDMERE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELACPRODOL"  */
    if exists (select 1
               from   RELACPRODOL t2, deleted t1
               where  t2.CDPRODARVOL = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACPRODOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RELACPRODOL"  */
    if exists (select 1
               from   RELACPRODOL t2, deleted t1
               where  t2.CDPRARVORIG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACPRODOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMSAIDOL"  */
    if exists (select 1
               from   ITEMSAIDOL t2, deleted t1
               where  t2.CDPRODSAIDOL = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSAIDOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "OPCPRODREC"  */
    if exists (select 1
               from   OPCPRODREC t2, deleted t1
               where  t2.CDPRORIGREC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPCPRODREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "OPCPRODREC"  */
    if exists (select 1
               from   OPCPRODREC t2, deleted t1
               where  t2.CDPROPCREC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OPCPRODREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODNECECOMP"  */
    if exists (select 1
               from   PRODNECECOMP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODNECECOMP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMINVOICE"  */
    if exists (select 1
               from   ITEMINVOICE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMINVOICE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "FAMSALDOPROD"  */
    if exists (select 1
               from   FAMSALDOPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FAMSALDOPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTOFAM"  */
    if exists (select 1
               from   PRODUTOFAM t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOFAM". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TIPOSERV"  */
    if exists (select 1
               from   TIPOSERV t2, deleted t1
               where  t2.CDPRODFATU = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOSERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "APURFATUFAS"  */
    if exists (select 1
               from   APURFATUFAS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APURFATUFAS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PAR_GERAL_FAS"  */
    if exists (select 1
               from   PAR_GERAL_FAS t2, deleted t1
               where  t2.CDPROFATLUSO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_GERAL_FAS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PAR_GERAL_FAS"  */
    if exists (select 1
               from   PAR_GERAL_FAS t2, deleted t1
               where  t2.CDPROFATMANU = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_GERAL_FAS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PAR_GERAL_FAS"  */
    if exists (select 1
               from   PAR_GERAL_FAS t2, deleted t1
               where  t2.CDPROFATLOCA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_GERAL_FAS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MARCPRODUTO"  */
    if exists (select 1
               from   MARCPRODUTO t2, deleted t1
               where  t2.CDPRODCOMP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MARCPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDPRODTAXSER = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CONSSERVPME"  */
    if exists (select 1
               from   CONSSERVPME t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSERVPME". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODRALIMENT"  */
    if exists (select 1
               from   PRODRALIMENT t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODRALIMENT". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMPEDEXOL"  */
    if exists (select 1
               from   ITEMPEDEXOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPEDEXOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMORCAEVEN"  */
    if exists (select 1
               from   ITEMORCAEVEN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCAEVEN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRECUSTOSERV"  */
    if exists (select 1
               from   PRECUSTOSERV t2, deleted t1
               where  t2.CDSUBGRUPO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECUSTOSERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRECUSTOSERV"  */
    if exists (select 1
               from   PRECUSTOSERV t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRECUSTOSERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITCUSTOSERV"  */
    if exists (select 1
               from   ITCUSTOSERV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCUSTOSERV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITMAOOBRAORCA"  */
    if exists (select 1
               from   ITMAOOBRAORCA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMAOOBRAORCA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITDEGUSTACAO"  */
    if exists (select 1
               from   ITDEGUSTACAO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITDEGUSTACAO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "CUSTOLOGIST"  */
    if exists (select 1
               from   CUSTOLOGIST t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTOLOGIST". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMSOLI"  */
    if exists (select 1
               from   ITEMSOLI t2, deleted t1
               where  t2.CDPRODORIG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AJUSTEQTDSOL"  */
    if exists (select 1
               from   AJUSTEQTDSOL t2, deleted t1
               where  t2.CDPRODORIG = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJUSTEQTDSOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "AJUSTEQTDSOL"  */
    if exists (select 1
               from   AJUSTEQTDSOL t2, deleted t1
               where  t2.CDPRODDEST = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJUSTEQTDSOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ACERTOQUEBRA"  */
    if exists (select 1
               from   ACERTOQUEBRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOQUEBRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ACERTOSOBRA"  */
    if exists (select 1
               from   ACERTOSOBRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOSOBRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ACERTOHREXTR"  */
    if exists (select 1
               from   ACERTOHREXTR t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOHREXTR". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ACERTOEXTRA"  */
    if exists (select 1
               from   ACERTOEXTRA t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACERTOEXTRA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODSERVTCON"  */
    if exists (select 1
               from   PRODSERVTCON t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODSERVTCON". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEXCLISPREC"  */
    if exists (select 1
               from   ITEXCLISPREC t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEXCLISPREC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MOVOPAF"  */
    if exists (select 1
               from   MOVOPAF t2, deleted t1
               where  t2.CDPRODMOV = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOPAF". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVPLANOCANC"  */
    if exists (select 1
               from   SERVPLANOCANC t2, deleted t1
               where  t2.CDSERVICOCANC = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPLANOCANC". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SALAOEVENTO"  */
    if exists (select 1
               from   SALAOEVENTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SALAOEVENTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TURNOPRODUTO"  */
    if exists (select 1
               from   TURNOPRODUTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURNOPRODUTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODCONSSOLOL"  */
    if exists (select 1
               from   PRODCONSSOLOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODCONSSOLOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ESTOCONSSOLOL"  */
    if exists (select 1
               from   ESTOCONSSOLOL t2, deleted t1
               where  t2.CDPRODARVOL = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTOCONSSOLOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "JUSTCONSSOLOL"  */
    if exists (select 1
               from   JUSTCONSSOLOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "JUSTCONSSOLOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DATAEMBARQUEOL"  */
    if exists (select 1
               from   DATAEMBARQUEOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DATAEMBARQUEOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "TIPOSERVPROD"  */
    if exists (select 1
               from   TIPOSERVPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOSERVPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PAR_GERAL_FAS"  */
    if exists (select 1
               from   PAR_GERAL_FAS t2, deleted t1
               where  t2.CDPROFATSOLI = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAR_GERAL_FAS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODUTOROP"  */
    if exists (select 1
               from   PRODUTOROP t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTOROP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ETL_FORSALE"  */
    if exists (select 1
               from   ETL_FORSALE t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_FORSALE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PREVCONSPDVDIA"  */
    if exists (select 1
               from   PREVCONSPDVDIA t2, deleted t1
               where  t2.CDPRPAIPCPDV = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSPDVDIA". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARPRODOPCONS"  */
    if exists (select 1
               from   PARPRODOPCONS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARPRODOPCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARPRODOPCONS"  */
    if exists (select 1
               from   PARPRODOPCONS t2, deleted t1
               where  t2.CDMATPRIMA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARPRODOPCONS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "INSUPEDIDOP"  */
    if exists (select 1
               from   INSUPEDIDOP t2, deleted t1
               where  t2.CDPRODINSUMO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSUPEDIDOP". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "DEVOLORCAEVEN"  */
    if exists (select 1
               from   DEVOLORCAEVEN t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DEVOLORCAEVEN". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDPRODFRETE = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITCOMISSAO"  */
    if exists (select 1
               from   ITCOMISSAO t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMISSAO". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODBLOQVND"  */
    if exists (select 1
               from   PRODBLOQVND t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODBLOQVND". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PARAMDESCOREV"  */
    if exists (select 1
               from   PARAMDESCOREV t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMDESCOREV". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRMARCPEDCOL"  */
    if exists (select 1
               from   PRMARCPEDCOL t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRMARCPEDCOL". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "MAPDISTALMOX"  */
    if exists (select 1
               from   MAPDISTALMOX t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPDISTALMOX". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "NFRELACCDRS"  */
    if exists (select 1
               from   NFRELACCDRS t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NFRELACCDRS". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PRODTIPOSUPRI"  */
    if exists (select 1
               from   PRODTIPOSUPRI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODTIPOSUPRI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMSOLISUPRI"  */
    if exists (select 1
               from   ITEMSOLISUPRI t2, deleted t1
               where  t2.CDPRODSOLISUP = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMSOLISUPRI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "COMPITEMPED"  */
    if exists (select 1
               from   COMPITEMPED t2, deleted t1
               where  t2.CDPRODCOMPED = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPITEMPED". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITCOMANDAPEND"  */
    if exists (select 1
               from   ITCOMANDAPEND t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITCOMANDAPEND". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "ITEMINVENTRETI"  */
    if exists (select 1
               from   ITEMINVENTRETI t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMINVENTRETI". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROPSGPROD"  */
    if exists (select 1
               from   PROPSGPROD t2, deleted t1
               where  t2.CDSERVICO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPSGPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "PROPSGPROD"  */
    if exists (select 1
               from   PROPSGPROD t2, deleted t1
               where  t2.CDSUBGRUPO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPSGPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SERVPREVPAXPROD"  */
    if exists (select 1
               from   SERVPREVPAXPROD t2, deleted t1
               where  t2.CDPRODUTO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVPREVPAXPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "SGPREVPAXPROD"  */
    if exists (select 1
               from   SGPREVPAXPROD t2, deleted t1
               where  t2.CDSUBGRUPO = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SGPREVPAXPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTO" if children still exist in "RECEPREVPAXPROD"  */
    if exists (select 1
               from   RECEPREVPAXPROD t2, deleted t1
               where  t2.CDRECEITA = t1.CDPRODUTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEPREVPAXPROD". Cannot delete parent "PRODUTO".'
          goto error
       end
    
    /*  Delete all children in "PRODFORN"  */
    delete PRODFORN
    from   PRODFORN t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODCOMP"  */
    delete PRODCOMP
    from   PRODCOMP t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "LIMISEGPROD"  */
    delete LIMISEGPROD
    from   LIMISEGPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "MARCPROD"  */
    delete MARCPROD
    from   MARCPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "ITEMPREBASE"  */
    delete ITEMPREBASE
    from   ITEMPREBASE t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODINTDOS"  */
    delete PRODINTDOS
    from   PRODINTDOS t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "APREPROD"  */
    delete APREPROD
    from   APREPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "NVSUBGRPROD"  */
    delete NVSUBGRPROD
    from   NVSUBGRPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRSUBGRPROD"  */
    delete PRSUBGRPROD
    from   PRSUBGRPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODFORNCOMP"  */
    delete PRODFORNCOMP
    from   PRODFORNCOMP t2, deleted t1
    where  t2.CDPRODCOMPL = t1.CDPRODUTO
    
    /*  Delete all children in "MARCPRODCOMP"  */
    delete MARCPRODCOMP
    from   MARCPRODCOMP t2, deleted t1
    where  t2.CDPRMARCOMPL = t1.CDPRODUTO
    
    /*  Delete all children in "PRODLCESTOQ"  */
    delete PRODLCESTOQ
    from   PRODLCESTOQ t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "LOTE"  */
    delete LOTE
    from   LOTE t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODUTOSEG"  */
    delete PRODUTOSEG
    from   PRODUTOSEG t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PRODITEMORCA"  */
    delete PRODITEMORCA
    from   PRODITEMORCA t2, deleted t1
    where  t2.CDPRODGRUP = t1.CDPRODUTO
    
    /*  Delete all children in "ITEMTRABPROD"  */
    delete ITEMTRABPROD
    from   ITEMTRABPROD t2, deleted t1
    where  t2.CDPRODCOMP = t1.CDPRODUTO
    
    /*  Delete all children in "ITDESPREBASE"  */
    delete ITDESPREBASE
    from   ITDESPREBASE t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "CUSTORECIA"  */
    delete CUSTORECIA
    from   CUSTORECIA t2, deleted t1
    where  t2.CDRECEITA = t1.CDPRODUTO
    
    /*  Delete all children in "CUSTORECEITA"  */
    delete CUSTORECEITA
    from   CUSTORECEITA t2, deleted t1
    where  t2.CDRECEITA = t1.CDPRODUTO
    
    /*  Delete all children in "TRABNUTRIREC"  */
    delete TRABNUTRIREC
    from   TRABNUTRIREC t2, deleted t1
    where  t2.CDRECEITA = t1.CDPRODUTO
    
    /*  Delete all children in "REGRAREVRECE"  */
    delete REGRAREVRECE
    from   REGRAREVRECE t2, deleted t1
    where  t2.CDPRODORIGRE = t1.CDPRODUTO
    
    /*  Delete all children in "REGRAREVRECE"  */
    delete REGRAREVRECE
    from   REGRAREVRECE t2, deleted t1
    where  t2.CDPRODDESTRE = t1.CDPRODUTO
    
    /*  Delete all children in "PERCACRPROD"  */
    delete PERCACRPROD
    from   PERCACRPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "RESTPRODCLIE"  */
    delete RESTPRODCLIE
    from   RESTPRODCLIE t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PREFPRODCLIE"  */
    delete PREFPRODCLIE
    from   PREFPRODCLIE t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_produtocia" for table "PRODUTOCIA"  */
create trigger td_produtocia on PRODUTOCIA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PRODUTOCIA" if children still exist in "EQESPEPROD"  */
    if exists (select 1
               from   EQESPEPROD t2, deleted t1
               where  t2.CDPRODEQUIP = t1.CDPRODUTO
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQESPEPROD". Cannot delete parent "PRODUTOCIA".'
          goto error
       end
    
    /*  Cannot delete parent "PRODUTOCIA" if children still exist in "SUBPRODCIA"  */
    if exists (select 1
               from   SUBPRODCIA t2, deleted t1
               where  t2.CDSUBPROCIA = t1.CDPRODUTO
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SUBPRODCIA". Cannot delete parent "PRODUTOCIA".'
          goto error
       end
    
    /*  Delete all children in "SUBPRODCIA"  */
    delete SUBPRODCIA
    from   SUBPRODCIA t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_programa" for table "PROGRAMA"  */
create trigger td_programa on PROGRAMA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROGRAMA" if children still exist in "PROGGRUPOP"  */
    if exists (select 1
               from   PROGGRUPOP t2, deleted t1
               where  t2.CDTELAPROG = t1.CDTELAPROG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGGRUPOP". Cannot delete parent "PROGRAMA".'
          goto error
       end
    
    /*  Cannot delete parent "PROGRAMA" if children still exist in "PROGOPER"  */
    if exists (select 1
               from   PROGOPER t2, deleted t1
               where  t2.CDTELAPROG = t1.CDTELAPROG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROGOPER". Cannot delete parent "PROGRAMA".'
          goto error
       end
    
    /*  Delete all children in "PROGOPEROPC"  */
    delete PROGOPEROPC
    from   PROGOPEROPC t2, deleted t1
    where  t2.CDTELAPROG = t1.CDTELAPROG

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_proposta" for table "PROPOSTA"  */
create trigger td_proposta on PROPOSTA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROPOSTA" if children still exist in "ITPROPOSTA"  */
    if exists (select 1
               from   ITPROPOSTA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRPROPOSTA = t1.NRPROPOSTA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITPROPOSTA". Cannot delete parent "PROPOSTA".'
          goto error
       end
    
    /*  Delete all children in "OBSPROPOSTA"  */
    delete OBSPROPOSTA
    from   OBSPROPOSTA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRPROPOSTA = t1.NRPROPOSTA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_propostafas" for table "PROPOSTAFAS"  */
create trigger td_propostafas on PROPOSTAFAS for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROPOSTAFAS" if children still exist in "CONTRATOLINP"  */
    if exists (select 1
               from   CONTRATOLINP t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOLINP". Cannot delete parent "PROPOSTAFAS".'
          goto error
       end
    
    /*  Cannot delete parent "PROPOSTAFAS" if children still exist in "CONTRATOSERV"  */
    if exists (select 1
               from   CONTRATOSERV t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTRATOSERV". Cannot delete parent "PROPOSTAFAS".'
          goto error
       end
    
    /*  Cannot delete parent "PROPOSTAFAS" if children still exist in "PROJETO"  */
    if exists (select 1
               from   PROJETO t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROJETO". Cannot delete parent "PROPOSTAFAS".'
          goto error
       end
    
    /*  Delete all children in "TXTPROPFAS"  */
    delete TXTPROPFAS
    from   TXTPROPFAS t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
    
    /*  Delete all children in "PROPOSTALINP"  */
    delete PROPOSTALINP
    from   PROPOSTALINP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
    
    /*  Delete all children in "PEDILINPPROP"  */
    delete PEDILINPPROP
    from   PEDILINPPROP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
    
    /*  Delete all children in "PEDISERVPROP"  */
    delete PEDISERVPROP
    from   PEDISERVPROP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
    
    /*  Delete all children in "PROPOSTASERV"  */
    delete PROPOSTASERV
    from   PROPOSTASERV t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
    
    /*  Delete all children in "OBSPROPFAS"  */
    delete OBSPROPFAS
    from   OBSPROPFAS t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_propostalinp" for table "PROPOSTALINP"  */
create trigger td_propostalinp on PROPOSTALINP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROPOSTALINP" if children still exist in "PEDILINPPROP"  */
    if exists (select 1
               from   PEDILINPPROP t2, deleted t1
               where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
                and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
                and   t2.CDLINPPROP = t1.CDLINPPROP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDILINPPROP". Cannot delete parent "PROPOSTALINP".'
          goto error
       end
    
    /*  Delete all children in "SISTPROPLINP"  */
    delete SISTPROPLINP
    from   SISTPROPLINP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
     and   t2.CDLINPPROP = t1.CDLINPPROP
    
    /*  Delete all children in "VENCLUSOPROP"  */
    delete VENCLUSOPROP
    from   VENCLUSOPROP t2, deleted t1
    where  t2.NRPROPOSFAS = t1.NRPROPOSFAS
     and   t2.NRVERPROPFAS = t1.NRVERPROPFAS
     and   t2.CDLINPPROP = t1.CDLINPPROP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prospect" for table "PROSPECT"  */
create trigger td_prospect on PROSPECT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "PROSPECT" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "ESTINTERESSE"  */
    if exists (select 1
               from   ESTINTERESSE t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTINTERESSE". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "ESCOPROS"  */
    if exists (select 1
               from   ESCOPROS t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESCOPROS". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "PROSPFILIAL"  */
    if exists (select 1
               from   PROSPFILIAL t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPFILIAL". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "AUDITORIA"  */
    if exists (select 1
               from   AUDITORIA t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUDITORIA". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "PROPOSTAFAS"  */
    if exists (select 1
               from   PROPOSTAFAS t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTAFAS". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "CURSOPROSPECT"  */
    if exists (select 1
               from   CURSOPROSPECT t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CURSOPROSPECT". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "QUALIFPROSP"  */
    if exists (select 1
               from   QUALIFPROSP t2, deleted t1
               where  t2.CDPROSPECT = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QUALIFPROSP". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Cannot delete parent "PROSPECT" if children still exist in "QUALIFPROSP"  */
    if exists (select 1
               from   QUALIFPROSP t2, deleted t1
               where  t2.CDPROSPECMATRIZ = t1.CDPROSPECT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QUALIFPROSP". Cannot delete parent "PROSPECT".'
          goto error
       end
    
    /*  Delete all children in "ENDEPROSPECT"  */
    delete ENDEPROSPECT
    from   ENDEPROSPECT t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT
    
    /*  Delete all children in "OBSERVPROSP"  */
    delete OBSERVPROSP
    from   OBSERVPROSP t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT
    
    /*  Delete all children in "INTEPROSP"  */
    delete INTEPROSP
    from   INTEPROSP t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT
    
    /*  Delete all children in "HISTQUALPROS"  */
    delete HISTQUALPROS
    from   HISTQUALPROS t2, deleted t1
    where  t2.CDPROSPECT = t1.CDPROSPECT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_prsubgrprod" for table "PRSUBGRPROD"  */
create trigger td_prsubgrprod on PRSUBGRPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PARTFILI"  */
    delete PARTFILI
    from   PARTFILI t2, deleted t1
    where  t2.CDGRUPPROD = t1.CDGRUPPROD
     and   t2.CDSUBGRPROD = t1.CDSUBGRPROD
     and   t2.CDFILISUBGR = t1.CDFILISUBGR
     and   t2.CDPRODUTO = t1.CDPRODUTO
    
    /*  Delete all children in "PARAMPRODSUP"  */
    delete PARAMPRODSUP
    from   PARAMPRODSUP t2, deleted t1
    where  t2.CDGRUPPROD = t1.CDGRUPPROD
     and   t2.CDSUBGRPROD = t1.CDSUBGRPROD
     and   t2.CDFILISUBGR = t1.CDFILISUBGR
     and   t2.CDPRODUTO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_refenutrecom" for table "REFENUTRECOM"  */
create trigger td_refenutrecom on REFENUTRECOM for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "REFENUTRECOM" if children still exist in "NUTRECOMPACI"  */
    if exists (select 1
               from   NUTRECOMPACI t2, deleted t1
               where  t2.CDRFNUTRECOM = t1.CDRFNUTRECOM)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRECOMPACI". Cannot delete parent "REFENUTRECOM".'
          goto error
       end
    
    /*  Delete all children in "NUTRECOM"  */
    delete NUTRECOM
    from   NUTRECOM t2, deleted t1
    where  t2.CDRFNUTRECOM = t1.CDRFNUTRECOM

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_regiao" for table "REGIAO"  */
create trigger td_regiao on REGIAO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "REGIAO" if children still exist in "ITEMCOLE"  */
    if exists (select 1
               from   ITEMCOLE t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMCOLE". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "PEDCOLETA"  */
    if exists (select 1
               from   PEDCOLETA t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDCOLETA". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "MUNICIPIO"  */
    if exists (select 1
               from   MUNICIPIO t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MUNICIPIO". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "PARAMPRODSUP"  */
    if exists (select 1
               from   PARAMPRODSUP t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPRODSUP". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "MINFORNREG"  */
    if exists (select 1
               from   MINFORNREG t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MINFORNREG". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "GSCRESULTADO"  */
    if exists (select 1
               from   GSCRESULTADO t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GSCRESULTADO". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "ITEXCLISPREC"  */
    if exists (select 1
               from   ITEXCLISPREC t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEXCLISPREC". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "REGIAOPEDCOL"  */
    if exists (select 1
               from   REGIAOPEDCOL t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REGIAOPEDCOL". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Cannot delete parent "REGIAO" if children still exist in "ETLCOMPRASGER"  */
    if exists (select 1
               from   ETLCOMPRASGER t2, deleted t1
               where  t2.CDREGIAO = t1.CDREGIAO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCOMPRASGER". Cannot delete parent "REGIAO".'
          goto error
       end
    
    /*  Delete all children in "REFOREG"  */
    delete REFOREG
    from   REFOREG t2, deleted t1
    where  t2.CDREGIAO = t1.CDREGIAO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_relat_cdrs" for table "RELAT_CDRS"  */
create trigger td_relat_cdrs on RELAT_CDRS for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "NFRELACCDRS"  */
    delete NFRELACCDRS
    from   NFRELACCDRS t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.NRCDRS = t1.NRCDRS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_relatuser" for table "RELATUSER"  */
create trigger td_relatuser on RELATUSER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "COMPRELUSER"  */
    delete COMPRELUSER
    from   COMPRELUSER t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO
    
    /*  Delete all children in "RELATIMPOS"  */
    delete RELATIMPOS
    from   RELATIMPOS t2, deleted t1
    where  t2.CDRELATORIO = t1.CDRELATORIO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_requesto" for table "REQUESTO"  */
create trigger td_requesto on REQUESTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "REQUESTO" if children still exist in "RETIRADAESTO"  */
    if exists (select 1
               from   RETIRADAESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRREQUESTO = t1.NRREQUESTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RETIRADAESTO". Cannot delete parent "REQUESTO".'
          goto error
       end
    
    /*  Cannot delete parent "REQUESTO" if children still exist in "CTRLMOVIESTQ"  */
    if exists (select 1
               from   CTRLMOVIESTQ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRREQUESTO = t1.NRREQUESTO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLMOVIESTQ". Cannot delete parent "REQUESTO".'
          goto error
       end
    
    /*  Delete all children in "ITREQUESTO"  */
    delete ITREQUESTO
    from   ITREQUESTO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRREQUESTO = t1.NRREQUESTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_requisitprod" for table "REQUISITPROD"  */
create trigger td_requisitprod on REQUISITPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "REQUISITPROD" if children still exist in "RELACREQPROD"  */
    if exists (select 1
               from   RELACREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACREQPROD". Cannot delete parent "REQUISITPROD".'
          goto error
       end
    
    /*  Cannot delete parent "REQUISITPROD" if children still exist in "RELACREQPROD"  */
    if exists (select 1
               from   RELACREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPRELAC = t1.CDREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACREQPROD". Cannot delete parent "REQUISITPROD".'
          goto error
       end
    
    /*  Cannot delete parent "REQUISITPROD" if children still exist in "PRODREQPROD"  */
    if exists (select 1
               from   PRODREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODREQPROD". Cannot delete parent "REQUISITPROD".'
          goto error
       end
    
    /*  Cannot delete parent "REQUISITPROD" if children still exist in "EVOLREQPROD"  */
    if exists (select 1
               from   EVOLREQPROD t2, deleted t1
               where  t2.CDPRODGRUP = t1.CDPRODGRUP
                and   t2.CDREQPROD = t1.CDREQPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLREQPROD". Cannot delete parent "REQUISITPROD".'
          goto error
       end
    
    /*  Delete all children in "PCHAVREQPROD"  */
    delete PCHAVREQPROD
    from   PCHAVREQPROD t2, deleted t1
    where  t2.CDPRODUTO = t1.CDPRODGRUP
     and   t2.CDREQPROD = t1.CDREQPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_retirada" for table "RETIRADA"  */
create trigger td_retirada on RETIRADA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMRETI"  */
    delete ITEMRETI
    from   ITEMRETI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRCARDAPIO = t1.NRCARDAPIO
     and   t2.NRRETIRADA = t1.NRRETIRADA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_retiradaesto" for table "RETIRADAESTO"  */
create trigger td_retiradaesto on RETIRADAESTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMRETIEST"  */
    delete ITEMRETIEST
    from   ITEMRETIEST t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRRETEST = t1.NRRETEST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_romaneio" for table "ROMANEIO"  */
create trigger td_romaneio on ROMANEIO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ROMANEIO" if children still exist in "ROMANEIONF"  */
    if exists (select 1
               from   ROMANEIONF t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRROMANEIO = t1.NRROMANEIO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ROMANEIONF". Cannot delete parent "ROMANEIO".'
          goto error
       end
    
    /*  Cannot delete parent "ROMANEIO" if children still exist in "OCORRERFATUR"  */
    if exists (select 1
               from   OCORRERFATUR t2, deleted t1
               where  t2.CDFILIROMA = t1.CDFILIAL
                and   t2.NRROMANEIO = t1.NRROMANEIO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OCORRERFATUR". Cannot delete parent "ROMANEIO".'
          goto error
       end
    
    /*  Delete all children in "ITEMROMANEIO"  */
    delete ITEMROMANEIO
    from   ITEMROMANEIO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRROMANEIO = t1.NRROMANEIO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_ropfilial" for table "ROPFILIAL"  */
create trigger td_ropfilial on ROPFILIAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "LANCROPFILIAL"  */
    delete LANCROPFILIAL
    from   LANCROPFILIAL t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRROPFILIAL = t1.NRROPFILIAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_rotatrans" for table "ROTATRANS"  */
create trigger td_rotatrans on ROTATRANS for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "SERVICO"  */
    if exists (select 1
               from   SERVICO t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVICO". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "CLIENROTA"  */
    if exists (select 1
               from   CLIENROTA t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENROTA". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "SOLICTROCO"  */
    if exists (select 1
               from   SOLICTROCO t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICTROCO". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "TRAJETOROTA"  */
    if exists (select 1
               from   TRAJETOROTA t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRAJETOROTA". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "MUNICIPIROTA"  */
    if exists (select 1
               from   MUNICIPIROTA t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MUNICIPIROTA". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "ABASTEROTA"  */
    if exists (select 1
               from   ABASTEROTA t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ABASTEROTA". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "PREVABASEV"  */
    if exists (select 1
               from   PREVABASEV t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVABASEV". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Cannot delete parent "ROTATRANS" if children still exist in "FILIALROTA"  */
    if exists (select 1
               from   FILIALROTA t2, deleted t1
               where  t2.CDROTATRANS = t1.CDROTATRANS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIALROTA". Cannot delete parent "ROTATRANS".'
          goto error
       end
    
    /*  Delete all children in "CALEABAS"  */
    delete CALEABAS
    from   CALEABAS t2, deleted t1
    where  t2.CDROTATRANS = t1.CDROTATRANS
    
    /*  Delete all children in "CALENTREGA"  */
    delete CALENTREGA
    from   CALENTREGA t2, deleted t1
    where  t2.CDROTATRANS = t1.CDROTATRANS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_salaoevento" for table "SALAOEVENTO"  */
create trigger td_salaoevento on SALAOEVENTO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SALAOEVENTO" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDSALAOEVEN = t1.CDSALAOEVEN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "SALAOEVENTO".'
          goto error
       end
    
    /*  Cannot delete parent "SALAOEVENTO" if children still exist in "AGENDASALAO"  */
    if exists (select 1
               from   AGENDASALAO t2, deleted t1
               where  t2.CDSALAOEVEN = t1.CDSALAOEVEN)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGENDASALAO". Cannot delete parent "SALAOEVENTO".'
          goto error
       end
    
    /*  Delete all children in "ENDESALAOEVEN"  */
    delete ENDESALAOEVEN
    from   ENDESALAOEVEN t2, deleted t1
    where  t2.CDSALAOEVEN = t1.CDSALAOEVEN
    
    /*  Delete all children in "SALAOSUBGRPROD"  */
    delete SALAOSUBGRPROD
    from   SALAOSUBGRPROD t2, deleted t1
    where  t2.CDSALAOEVEN = t1.CDSALAOEVEN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_segmercado" for table "SEGMERCADO"  */
create trigger td_segmercado on SEGMERCADO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "PROSPECT"  */
    if exists (select 1
               from   PROSPECT t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPECT". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "PRATO"  */
    if exists (select 1
               from   PRATO t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATO". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "NIVELPRODSEG"  */
    if exists (select 1
               from   NIVELPRODSEG t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NIVELPRODSEG". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "ETL_RESUGER"  */
    if exists (select 1
               from   ETL_RESUGER t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUGER". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "ETL_RESUFOOD"  */
    if exists (select 1
               from   ETL_RESUFOOD t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUFOOD". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "ETL_RESUESTO"  */
    if exists (select 1
               from   ETL_RESUESTO t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUESTO". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "ETL_RESUSERV"  */
    if exists (select 1
               from   ETL_RESUSERV t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETL_RESUSERV". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Cannot delete parent "SEGMERCADO" if children still exist in "ETLDRO"  */
    if exists (select 1
               from   ETLDRO t2, deleted t1
               where  t2.CDSEGMER = t1.CDSEGMER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLDRO". Cannot delete parent "SEGMERCADO".'
          goto error
       end
    
    /*  Delete all children in "PRODUTOSEG"  */
    delete PRODUTOSEG
    from   PRODUTOSEG t2, deleted t1
    where  t2.CDSEGMER = t1.CDSEGMER

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_serienf" for table "SERIENF"  */
create trigger td_serienf on SERIENF for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERIENF" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "CIAFILI"  */
    if exists (select 1
               from   CIAFILI t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CIAFILI". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "SERVCONTRAT"  */
    if exists (select 1
               from   SERVCONTRAT t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCONTRAT". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "GERANFROMA"  */
    if exists (select 1
               from   GERANFROMA t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERANFROMA". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "EMPRCNTRFIL"  */
    if exists (select 1
               from   EMPRCNTRFIL t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EMPRCNTRFIL". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "FILIAL"  */
    if exists (select 1
               from   FILIAL t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FILIAL". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIEEMIS = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "PARAMSERIE"  */
    if exists (select 1
               from   PARAMSERIE t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIE = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMSERIE". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Cannot delete parent "SERIENF" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDINSCMUNI = t1.CDINSCMUNI
                and   t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDINSCESTA = t1.CDINSCESTA
                and   t2.CDSERIECNTREC = t1.CDSERIE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "SERIENF".'
          goto error
       end
    
    /*  Delete all children in "IMPOSERIE"  */
    delete IMPOSERIE
    from   IMPOSERIE t2, deleted t1
    where  t2.CDINSCMUNI = t1.CDINSCMUNI
     and   t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDINSCESTA = t1.CDINSCESTA
     and   t2.CDSERIE = t1.CDSERIE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servclie" for table "SERVCLIE"  */
create trigger td_servclie on SERVCLIE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERVCLIE" if children still exist in "HISTSERVFATU"  */
    if exists (select 1
               from   HISTSERVFATU t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISTSERVFATU". Cannot delete parent "SERVCLIE".'
          goto error
       end
    
    /*  Cannot delete parent "SERVCLIE" if children still exist in "RELAT_CDRS"  */
    if exists (select 1
               from   RELAT_CDRS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELAT_CDRS". Cannot delete parent "SERVCLIE".'
          goto error
       end
    
    /*  Cannot delete parent "SERVCLIE" if children still exist in "EFETCDRS"  */
    if exists (select 1
               from   EFETCDRS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCDRS". Cannot delete parent "SERVCLIE".'
          goto error
       end
    
    /*  Cannot delete parent "SERVCLIE" if children still exist in "EFETRATCDRS"  */
    if exists (select 1
               from   EFETRATCDRS t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETRATCDRS". Cannot delete parent "SERVCLIE".'
          goto error
       end
    
    /*  Cannot delete parent "SERVCLIE" if children still exist in "SERVCLIETURN"  */
    if exists (select 1
               from   SERVCLIETURN t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIETURN". Cannot delete parent "SERVCLIE".'
          goto error
       end
    
    /*  Delete all children in "CORRPRECSERV"  */
    delete CORRPRECSERV
    from   CORRPRECSERV t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servcontrat" for table "SERVCONTRAT"  */
create trigger td_servcontrat on SERVCONTRAT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERVCONTRAT" if children still exist in "PREPSERVFATU"  */
    if exists (select 1
               from   PREPSERVFATU t2, deleted t1
               where  t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.CDSERVCONT = t1.CDSERVCONT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPSERVFATU". Cannot delete parent "SERVCONTRAT".'
          goto error
       end
    
    /*  Delete all children in "PZPGSERCONT"  */
    delete PZPGSERCONT
    from   PZPGSERCONT t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDSERVCONT = t1.CDSERVCONT
    
    /*  Delete all children in "VALSERVCONT"  */
    delete VALSERVCONT
    from   VALSERVCONT t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDSERVCONT = t1.CDSERVCONT
    
    /*  Delete all children in "SERVFATU"  */
    delete SERVFATU
    from   SERVFATU t2, deleted t1
    where  t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.CDSERVCONT = t1.CDSERVCONT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servico" for table "SERVICO"  */
create trigger td_servico on SERVICO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVCLIE"  */
    if exists (select 1
               from   SERVCLIE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIE". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "FOTOPRASER"  */
    if exists (select 1
               from   FOTOPRASER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FOTOPRASER". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PRATSERV"  */
    if exists (select 1
               from   PRATSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RECESERV"  */
    if exists (select 1
               from   RECESERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECESERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "EFETCLIE"  */
    if exists (select 1
               from   EFETCLIE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETCLIE". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LIMPESER"  */
    if exists (select 1
               from   LIMPESER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPESER". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RESTPROD"  */
    if exists (select 1
               from   RESTPROD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTPROD". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LIMNUTRI"  */
    if exists (select 1
               from   LIMNUTRI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMNUTRI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LIMPEPRA"  */
    if exists (select 1
               from   LIMPEPRA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LIMPEPRA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RESTOPCA"  */
    if exists (select 1
               from   RESTOPCA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTOPCA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "FREQOPCA"  */
    if exists (select 1
               from   FREQOPCA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FREQOPCA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RESTPRAT"  */
    if exists (select 1
               from   RESTPRAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTPRAT". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "FREQPRAT"  */
    if exists (select 1
               from   FREQPRAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FREQPRAT". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PREVCONS"  */
    if exists (select 1
               from   PREVCONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONS". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PREVRETI"  */
    if exists (select 1
               from   PREVRETI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVRETI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "DIASEMCONS"  */
    if exists (select 1
               from   DIASEMCONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIASEMCONS". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CONSSERV"  */
    if exists (select 1
               from   CONSSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CNFCAMES"  */
    if exists (select 1
               from   CNFCAMES t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CNFCAMES". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDADIA"  */
    if exists (select 1
               from   CARDADIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDADIA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVOPER"  */
    if exists (select 1
               from   SERVOPER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVOPER". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RATENESTO"  */
    if exists (select 1
               from   RATENESTO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RATENESTO". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PREPPRASER"  */
    if exists (select 1
               from   PREPPRASER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREPPRASER". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "GRUPSERVFAT"  */
    if exists (select 1
               from   GRUPSERVFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERAJEFEGSF = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GRUPSERVFAT". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "EFECLIFAT"  */
    if exists (select 1
               from   EFECLIFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFECLIFAT". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "DIETSERV"  */
    if exists (select 1
               from   DIETSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIETSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LOGPREVRETI"  */
    if exists (select 1
               from   LOGPREVRETI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGPREVRETI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "UTILPRATSEM"  */
    if exists (select 1
               from   UTILPRATSEM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UTILPRATSEM". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDPACI"  */
    if exists (select 1
               from   CARDPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDPACI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LOGSERVTRAN"  */
    if exists (select 1
               from   LOGSERVTRAN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGSERVTRAN". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVTRAN"  */
    if exists (select 1
               from   SERVTRAN t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTRAN". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVREAL"  */
    if exists (select 1
               from   SERVREAL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVREAL". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CATRSERV"  */
    if exists (select 1
               from   CATRSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CATRSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "MENSREQU"  */
    if exists (select 1
               from   MENSREQU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MENSREQU". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "ITINQUERPACI"  */
    if exists (select 1
               from   ITINQUERPACI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITINQUERPACI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "MOVICOME"  */
    if exists (select 1
               from   MOVICOME t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVICOME". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PAROPERSERV"  */
    if exists (select 1
               from   PAROPERSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAROPERSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVINTE"  */
    if exists (select 1
               from   SERVINTE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVINTE". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PLCALOSERV"  */
    if exists (select 1
               from   PLCALOSERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLCALOSERV". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "RESTITREFTUR"  */
    if exists (select 1
               from   RESTITREFTUR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTITREFTUR". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "DESTSRVCLI"  */
    if exists (select 1
               from   DESTSRVCLI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DESTSRVCLI". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDPADR"  */
    if exists (select 1
               from   CARDPADR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDPADR". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDCONSNUTR"  */
    if exists (select 1
               from   CARDCONSNUTR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDCONSNUTR". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVTURCAIXA"  */
    if exists (select 1
               from   SERVTURCAIXA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTURCAIXA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SUBSTIPOCONS"  */
    if exists (select 1
               from   SUBSTIPOCONS t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SUBSTIPOCONS". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDIAPME"  */
    if exists (select 1
               from   CARDIAPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDIAPME". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CONFCARDPME"  */
    if exists (select 1
               from   CONFCARDPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFCARDPME". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVTIPOCARD"  */
    if exists (select 1
               from   SERVTIPOCARD t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVTIPOCARD". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "PARAMPLANEJA"  */
    if exists (select 1
               from   PARAMPLANEJA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVRETPME = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPLANEJA". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "ITEMPRODUTO"  */
    if exists (select 1
               from   ITEMPRODUTO t2, deleted t1
               where  t2.CDFILIAPUR = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMPRODUTO". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "EFETDIET"  */
    if exists (select 1
               from   EFETDIET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EFETDIET". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CONSSERVPME"  */
    if exists (select 1
               from   CONSSERVPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSSERVPME". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "DIASEMCONSPME"  */
    if exists (select 1
               from   DIASEMCONSPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DIASEMCONSPME". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "FATORESUFAT"  */
    if exists (select 1
               from   FATORESUFAT t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FATORESUFAT". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVICO"  */
    if exists (select 1
               from   SERVICO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVPREQUI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVICO". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CTRLAPROVCARD"  */
    if exists (select 1
               from   CTRLAPROVCARD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTRLAPROVCARD". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "LOGAPROVCARD"  */
    if exists (select 1
               from   LOGAPROVCARD t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGAPROVCARD". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "COMPCARDDEL"  */
    if exists (select 1
               from   COMPCARDDEL t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPCARDDEL". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "SERVICOROP"  */
    if exists (select 1
               from   SERVICOROP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVICOROP". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "ESTATDIET"  */
    if exists (select 1
               from   ESTATDIET t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ESTATDIET". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Cannot delete parent "SERVICO" if children still exist in "CARDAMES"  */
    if exists (select 1
               from   CARDAMES t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CARDAMES". Cannot delete parent "SERVICO".'
          goto error
       end
    
    /*  Delete all children in "TURNOSERV"  */
    delete TURNOSERV
    from   TURNOSERV t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
    
    /*  Delete all children in "CUSTPADR"  */
    delete CUSTPADR
    from   CUSTPADR t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
    
    /*  Delete all children in "CONFCARD"  */
    delete CONFCARD
    from   CONFCARD t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
    
    /*  Delete all children in "LOGPREVCONS"  */
    delete LOGPREVCONS
    from   LOGPREVCONS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servinte" for table "SERVINTE"  */
create trigger td_servinte on SERVINTE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "CONFESCO"  */
    delete CONFESCO
    from   CONFESCO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDPACIENTE = t1.CDPACIENTE
     and   t2.DTSERVINTE = t1.DTSERVINTE
     and   t2.NRSEQUSERV = t1.NRSEQUSERV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servprest" for table "SERVPREST"  */
create trigger td_servprest on SERVPREST for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERVPREST" if children still exist in "APROATIVCSA"  */
    if exists (select 1
               from   APROATIVCSA t2, deleted t1
               where  t2.CDEMPRESA = t1.CDEMPRESA
                and   t2.CDFUNC = t1.CDFUNC
                and   t2.CDCLIENTE = t1.CDCLIENTE
                and   t2.NRLICENCAUSO = t1.NRLICENCAUSO
                and   t2.DTSERVPREST = t1.DTSERVPREST)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APROATIVCSA". Cannot delete parent "SERVPREST".'
          goto error
       end
    
    /*  Delete all children in "DESCSERVPRES"  */
    delete DESCSERVPRES
    from   DESCSERVPRES t2, deleted t1
    where  t2.CDEMPRESA = t1.CDEMPRESA
     and   t2.CDFUNC = t1.CDFUNC
     and   t2.CDCLIENTE = t1.CDCLIENTE
     and   t2.NRLICENCAUSO = t1.NRLICENCAUSO
     and   t2.DTSERVPREST = t1.DTSERVPREST

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servprevpaxprod" for table "SERVPREVPAXPROD"  */
create trigger td_servprevpaxprod on SERVPREVPAXPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "SGPREVPAXPROD"  */
    delete SGPREVPAXPROD
    from   SGPREVPAXPROD t2, deleted t1
    where  t2.CDCLASSE = t1.CDCLASSE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRVOO = t1.NRVOO
     and   t2.CDTRECHO = t1.CDTRECHO
     and   t2.DTVOO = t1.DTVOO
     and   t2.CDSERVICO = t1.CDPRODUTO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servtipocard" for table "SERVTIPOCARD"  */
create trigger td_servtipocard on SERVTIPOCARD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SERVTIPOCARD" if children still exist in "CONFPADTCARD"  */
    if exists (select 1
               from   CONFPADTCARD t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILCENTPME
                and   t2.CDTIPOCARD = t1.CDTIPOCARD
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONFPADTCARD". Cannot delete parent "SERVTIPOCARD".'
          goto error
       end
    
    /*  Cannot delete parent "SERVTIPOCARD" if children still exist in "CUSTPADTCARD"  */
    if exists (select 1
               from   CUSTPADTCARD t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILCENTPME
                and   t2.CDTIPOCARD = t1.CDTIPOCARD
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CUSTPADTCARD". Cannot delete parent "SERVTIPOCARD".'
          goto error
       end
    
    /*  Cannot delete parent "SERVTIPOCARD" if children still exist in "SERVCLIEPME"  */
    if exists (select 1
               from   SERVCLIEPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILCENTPME
                and   t2.CDTIPOCARD = t1.CDTIPOCARD
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVCLIEPME". Cannot delete parent "SERVTIPOCARD".'
          goto error
       end
    
    /*  Cannot delete parent "SERVTIPOCARD" if children still exist in "PREVCONSPME"  */
    if exists (select 1
               from   PREVCONSPME t2, deleted t1
               where  t2.CDFILCENTPME = t1.CDFILCENTPME
                and   t2.CDTIPOCARD = t1.CDTIPOCARD
                and   t2.CDSERVFILI = t1.CDSERVFILI)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCONSPME". Cannot delete parent "SERVTIPOCARD".'
          goto error
       end
    
    /*  Delete all children in "COMESERVTCAR"  */
    delete COMESERVTCAR
    from   COMESERVTCAR t2, deleted t1
    where  t2.CDFILCENTPME = t1.CDFILCENTPME
     and   t2.CDTIPOCARD = t1.CDTIPOCARD
     and   t2.CDSERVFILI = t1.CDSERVFILI

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_servtran" for table "SERVTRAN"  */
create trigger td_servtran on SERVTRAN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ETIQUETRAN"  */
    delete ETIQUETRAN
    from   ETIQUETRAN t2, deleted t1
    where  t2.CDFILAUXPRAT = t1.CDFILAUXPRAT
     and   t2.CDPRATO = t1.CDPRATO
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDSERVFILI = t1.CDSERVFILI
     and   t2.CDDIETPADST = t1.CDDIETPADST
     and   t2.DTSERVTRAN = t1.DTSERVTRAN
     and   t2.NRSERVTRAN = t1.NRSERVTRAN

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_sgprevpaxprod" for table "SGPREVPAXPROD"  */
create trigger td_sgprevpaxprod on SGPREVPAXPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "RECEPREVPAXPROD"  */
    delete RECEPREVPAXPROD
    from   RECEPREVPAXPROD t2, deleted t1
    where  t2.CDCLASSE = t1.CDCLASSE
     and   t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRVOO = t1.NRVOO
     and   t2.CDTRECHO = t1.CDTRECHO
     and   t2.DTVOO = t1.DTVOO
     and   t2.CDSERVICO = t1.CDSERVICO
     and   t2.CDSUBGRUPO = t1.CDSUBGRUPO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_solicita" for table "SOLICITA"  */
create trigger td_solicita on SOLICITA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SOLICITA" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDFILLANCA = t1.CDFILLANCA
                and   t2.CDFILSOLI = t1.CDFILSOLI
                and   t2.NRSOLICMP = t1.NRSOLICMP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "SOLICITA".'
          goto error
       end
    
    /*  Delete all children in "ITEMSOLI"  */
    delete ITEMSOLI
    from   ITEMSOLI t2, deleted t1
    where  t2.CDFILLAN = t1.CDFILLANCA
     and   t2.CDFILSOLI = t1.CDFILSOLI
     and   t2.NRSOLICMP = t1.NRSOLICMP
    
    /*  Delete all children in "MOVSOLEXT"  */
    delete MOVSOLEXT
    from   MOVSOLEXT t2, deleted t1
    where  t2.CDFILLANCEXT = t1.CDFILLANCA
     and   t2.CDFILSOLEXT = t1.CDFILSOLI
     and   t2.NRSOLIMOVEXT = t1.NRSOLICMP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_soliserv" for table "SOLISERV"  */
create trigger td_soliserv on SOLISERV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "SOLISERV" if children still exist in "TARESERV"  */
    if exists (select 1
               from   TARESERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TARESERV". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "LOGFASETRAM"  */
    if exists (select 1
               from   LOGFASETRAM t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGFASETRAM". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "LOGDATACOMP"  */
    if exists (select 1
               from   LOGDATACOMP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGDATACOMP". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "COMISSAOFUNC"  */
    if exists (select 1
               from   COMISSAOFUNC t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMISSAOFUNC". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SERVFATU"  */
    if exists (select 1
               from   SERVFATU t2, deleted t1
               where  t2.CDFILISOLI = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SERVFATU". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "LOGDATAPREV"  */
    if exists (select 1
               from   LOGDATAPREV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGDATAPREV". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "APRHRTRAB"  */
    if exists (select 1
               from   APRHRTRAB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APRHRTRAB". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "ATIVAPRHR"  */
    if exists (select 1
               from   ATIVAPRHR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVAPRHR". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLIPATCH"  */
    if exists (select 1
               from   SOLIPATCH t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLIPATCH". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLIPATCHPRO"  */
    if exists (select 1
               from   SOLIPATCHPRO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLIPATCHPRO". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "EVOLREQSOLI"  */
    if exists (select 1
               from   EVOLREQSOLI t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EVOLREQSOLI". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "ORCASOLICITA"  */
    if exists (select 1
               from   ORCASOLICITA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCASOLICITA". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLIBASELINE"  */
    if exists (select 1
               from   SOLIBASELINE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLIBASELINE". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "PRODSOLISERV"  */
    if exists (select 1
               from   PRODSOLISERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODSOLISERV". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "OBSOLIOPER"  */
    if exists (select 1
               from   OBSOLIOPER t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "OBSOLIOPER". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "LOGSOLISLA"  */
    if exists (select 1
               from   LOGSOLISLA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGSOLISLA". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLICGERTARE"  */
    if exists (select 1
               from   SOLICGERTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICGERTARE". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "PARCSOLISERV"  */
    if exists (select 1
               from   PARCSOLISERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCSOLISERV". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "CRONOPROJETO"  */
    if exists (select 1
               from   CRONOPROJETO t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CRONOPROJETO". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLIORIG = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "ETLRESULTDES"  */
    if exists (select 1
               from   ETLRESULTDES t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLRESULTDES". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "LOGEMAILGEP"  */
    if exists (select 1
               from   LOGEMAILGEP t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGEMAILGEP". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Cannot delete parent "SOLISERV" if children still exist in "SOLGRACORDOPROJ"  */
    if exists (select 1
               from   SOLGRACORDOPROJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLGRACORDOPROJ". Cannot delete parent "SOLISERV".'
          goto error
       end
    
    /*  Delete all children in "DESCSOLI"  */
    delete DESCSOLI
    from   DESCSOLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSOLISERV = t1.NRSOLISERV
    
    /*  Delete all children in "IMPLSOLI"  */
    delete IMPLSOLI
    from   IMPLSOLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSOLISERV = t1.NRSOLISERV
    
    /*  Delete all children in "ESPESOLI"  */
    delete ESPESOLI
    from   ESPESOLI t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSOLISERV = t1.NRSOLISERV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_subpretapaop" for table "SUBPRETAPAOP"  */
create trigger td_subpretapaop on SUBPRETAPAOP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PRODACABADO"  */
    delete PRODACABADO
    from   PRODACABADO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRORDPROD = t1.NRORDPROD
     and   t2.NRSUBORDPROD = t1.NRSUBORDPROD
     and   t2.CDSUBPRODETA = t1.CDSUBPRODETA
     and   t2.NRSUBPRETAOP = t1.NRSUBPRETAOP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tabelaamb" for table "TABELAAMB"  */
create trigger td_tabelaamb on TABELAAMB for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TABELAAMB" if children still exist in "CONVENIO"  */
    if exists (select 1
               from   CONVENIO t2, deleted t1
               where  t2.CDTABEAMB = t1.CDTABEAMB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVENIO". Cannot delete parent "TABELAAMB".'
          goto error
       end
    
    /*  Delete all children in "VALORCH"  */
    delete VALORCH
    from   VALORCH t2, deleted t1
    where  t2.CDTABEAMB = t1.CDTABEAMB
    
    /*  Delete all children in "VALORESPECH"  */
    delete VALORESPECH
    from   VALORESPECH t2, deleted t1
    where  t2.CDTABEAMB = t1.CDTABEAMB

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tabeladw" for table "TABELADW"  */
create trigger td_tabeladw on TABELADW for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TABELADW" if children still exist in "ITEMDW"  */
    if exists (select 1
               from   ITEMDW t2, deleted t1
               where  t2.CDTABDRILL = t1.CDTABELADW)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMDW". Cannot delete parent "TABELADW".'
          goto error
       end
    
    /*  Delete all children in "ITEMDW"  */
    delete ITEMDW
    from   ITEMDW t2, deleted t1
    where  t2.CDTABELADW = t1.CDTABELADW
    
    /*  Delete all children in "DIMENSAODW"  */
    delete DIMENSAODW
    from   DIMENSAODW t2, deleted t1
    where  t2.CDTABELADW = t1.CDTABELADW
    
    /*  Delete all children in "ITEMDRILL"  */
    delete ITEMDRILL
    from   ITEMDRILL t2, deleted t1
    where  t2.CDTABDRILL = t1.CDTABELADW
    
    /*  Delete all children in "DIMENSAODRILL"  */
    delete DIMENSAODRILL
    from   DIMENSAODRILL t2, deleted t1
    where  t2.CDTABDRILL = t1.CDTABELADW

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tabevend" for table "TABEVEND"  */
create trigger td_tabevend on TABEVEND for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMPRECO"  */
    delete ITEMPRECO
    from   ITEMPRECO t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDTABEPREC = t1.CDTABEPREC
     and   t2.DTINIVGPREC = t1.DTINIVGPREC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tabprecose" for table "TABPRECOSE"  */
create trigger td_tabprecose on TABPRECOSE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TABPRECOSE" if children still exist in "FORMPRECOCIA"  */
    if exists (select 1
               from   FORMPRECOCIA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.CDCIA = t1.CDCIA
                and   t2.NRTABPRECO = t1.NRTABPRECO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORMPRECOCIA". Cannot delete parent "TABPRECOSE".'
          goto error
       end
    
    /*  Delete all children in "PRECOSECIA"  */
    delete PRECOSECIA
    from   PRECOSECIA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.CDCIA = t1.CDCIA
     and   t2.NRTABPRECO = t1.NRTABPRECO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tarefa" for table "TAREFA"  */
create trigger td_tarefa on TAREFA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TAREFA" if children still exist in "PLANATIVTARE"  */
    if exists (select 1
               from   PLANATIVTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PLANATIVTARE". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "MOVOCORR"  */
    if exists (select 1
               from   MOVOCORR t2, deleted t1
               where  t2.CDFILILANCMO = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVOCORR". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "TAREFA"  */
    if exists (select 1
               from   TAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFAORIG = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TAREFA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "RELACTAREFA"  */
    if exists (select 1
               from   RELACTAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACTAREFA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "RELACTAREFA"  */
    if exists (select 1
               from   RELACTAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTARERELAC = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACTAREFA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "ASSINATAREFA"  */
    if exists (select 1
               from   ASSINATAREFA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ASSINATAREFA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "UTILBANCSOLU"  */
    if exists (select 1
               from   UTILBANCSOLU t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UTILBANCSOLU". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "SOLICGERTARE"  */
    if exists (select 1
               from   SOLICGERTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLICGERTARE". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "INDMONITPROJ"  */
    if exists (select 1
               from   INDMONITPROJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INDMONITPROJ". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "RISCOMONIPRJ"  */
    if exists (select 1
               from   RISCOMONIPRJ t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RISCOMONIPRJ". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "AGEATIVISALA"  */
    if exists (select 1
               from   AGEATIVISALA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AGEATIVISALA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "RASTREABTARE"  */
    if exists (select 1
               from   RASTREABTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RASTREABTARE". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Cannot delete parent "TAREFA" if children still exist in "MOVPESQUISA"  */
    if exists (select 1
               from   MOVPESQUISA t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRTAREFA = t1.NRTAREFA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MOVPESQUISA". Cannot delete parent "TAREFA".'
          goto error
       end
    
    /*  Delete all children in "DESCTAREFA"  */
    delete DESCTAREFA
    from   DESCTAREFA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTAREFA = t1.NRTAREFA
    
    /*  Delete all children in "ITEMTAREFA"  */
    delete ITEMTAREFA
    from   ITEMTAREFA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTAREFA = t1.NRTAREFA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tareserv" for table "TARESERV"  */
create trigger td_tareserv on TARESERV for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TARESERV" if children still exist in "TESTTARE"  */
    if exists (select 1
               from   TESTTARE t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV
                and   t2.NRTARESERV = t1.NRTARESERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTTARE". Cannot delete parent "TARESERV".'
          goto error
       end
    
    /*  Cannot delete parent "TARESERV" if children still exist in "APRHRTRAB"  */
    if exists (select 1
               from   APRHRTRAB t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV
                and   t2.NRTARESERV = t1.NRTARESERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "APRHRTRAB". Cannot delete parent "TARESERV".'
          goto error
       end
    
    /*  Cannot delete parent "TARESERV" if children still exist in "ATIVAPRHR"  */
    if exists (select 1
               from   ATIVAPRHR t2, deleted t1
               where  t2.CDFILIAL = t1.CDFILIAL
                and   t2.NRSOLISERV = t1.NRSOLISERV
                and   t2.NRTARESERV = t1.NRTARESERV)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ATIVAPRHR". Cannot delete parent "TARESERV".'
          goto error
       end
    
    /*  Delete all children in "DESCTARE"  */
    delete DESCTARE
    from   DESCTARE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSOLISERV = t1.NRSOLISERV
     and   t2.NRTARESERV = t1.NRTARESERV
    
    /*  Delete all children in "IMPLTARE"  */
    delete IMPLTARE
    from   IMPLTARE t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRSOLISERV = t1.NRSOLISERV
     and   t2.NRTARESERV = t1.NRTARESERV

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tbmetoaval" for table "TBMETOAVAL"  */
create trigger td_tbmetoaval on TBMETOAVAL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TBMETOAVAL" if children still exist in "DADOAVAL"  */
    if exists (select 1
               from   DADOAVAL t2, deleted t1
               where  t2.CDTBMETOAVAL = t1.CDTBMETOAVAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "DADOAVAL". Cannot delete parent "TBMETOAVAL".'
          goto error
       end
    
    /*  Cannot delete parent "TBMETOAVAL" if children still exist in "METOREFE"  */
    if exists (select 1
               from   METOREFE t2, deleted t1
               where  t2.CDTBMETOAVAL = t1.CDTBMETOAVAL)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "METOREFE". Cannot delete parent "TBMETOAVAL".'
          goto error
       end
    
    /*  Delete all children in "METOAVAL"  */
    delete METOAVAL
    from   METOAVAL t2, deleted t1
    where  t2.CDTBMETOAVAL = t1.CDTBMETOAVAL

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tcomppro" for table "TCOMPPRO"  */
create trigger td_tcomppro on TCOMPPRO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "COMPPROD"  */
    delete COMPPROD
    from   COMPPROD t2, deleted t1
    where  t2.CDTIPOCOMP = t1.CDTIPOCOMP
     and   t2.CDPRODUTO = t1.CDPRODUTO
     and   t2.CDESPEPROD = t1.CDESPEPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_testeprod" for table "TESTEPROD"  */
create trigger td_testeprod on TESTEPROD for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TESTEPROD" if children still exist in "TESTEPROD"  */
    if exists (select 1
               from   TESTEPROD t2, deleted t1
               where  t2.NRTESPRDORIG = t1.NRTESTEPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TESTEPROD". Cannot delete parent "TESTEPROD".'
          goto error
       end
    
    /*  Cannot delete parent "TESTEPROD" if children still exist in "QFOMARPROD"  */
    if exists (select 1
               from   QFOMARPROD t2, deleted t1
               where  t2.NRTESTEPROD = t1.NRTESTEPROD)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "QFOMARPROD". Cannot delete parent "TESTEPROD".'
          goto error
       end
    
    /*  Delete all children in "TPPRATTEST"  */
    delete TPPRATTEST
    from   TPPRATTEST t2, deleted t1
    where  t2.NRTESTEPROD = t1.NRTESTEPROD
    
    /*  Delete all children in "MARCTESTPR"  */
    delete MARCTESTPR
    from   MARCTESTPR t2, deleted t1
    where  t2.NRTESTEPROD = t1.NRTESTEPROD
    
    /*  Delete all children in "CUSBENTEST"  */
    delete CUSBENTEST
    from   CUSBENTEST t2, deleted t1
    where  t2.NRTESTEPROD = t1.NRTESTEPROD
    
    /*  Delete all children in "OBSTESTPROD"  */
    delete OBSTESTPROD
    from   OBSTESTPROD t2, deleted t1
    where  t2.NRTESTEPROD = t1.NRTESTEPROD

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipoajpagar" for table "TIPOAJPAGAR"  */
create trigger td_tipoajpagar on TIPOAJPAGAR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "AJCTPAGAR"  */
    if exists (select 1
               from   AJCTPAGAR t2, deleted t1
               where  t2.CDAJUCTPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTPAGAR". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "AJBOLPAG"  */
    if exists (select 1
               from   AJBOLPAG t2, deleted t1
               where  t2.CDAJUCTPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJBOLPAG". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "GERAL_PAG"  */
    if exists (select 1
               from   GERAL_PAG t2, deleted t1
               where  t2.CDAJMORAPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_PAG". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "GERAL_PAG"  */
    if exists (select 1
               from   GERAL_PAG t2, deleted t1
               where  t2.CDAJMULTAPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_PAG". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "GERAL_PAG"  */
    if exists (select 1
               from   GERAL_PAG t2, deleted t1
               where  t2.CDAJACREPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_PAG". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "GERAL_PAG"  */
    if exists (select 1
               from   GERAL_PAG t2, deleted t1
               where  t2.CDAJDESCPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_PAG". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJPAGAR" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDAJUCTPAG = t1.CDAJUCTPAG)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "TIPOAJPAGAR".'
          goto error
       end
    
    /*  Delete all children in "RATTPAJP"  */
    delete RATTPAJP
    from   RATTPAJP t2, deleted t1
    where  t2.CDAJUCTPAG = t1.CDAJUCTPAG

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipoajreceber" for table "TIPOAJRECEBER"  */
create trigger td_tipoajreceber on TIPOAJRECEBER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "AJCTRECEBER"  */
    if exists (select 1
               from   AJCTRECEBER t2, deleted t1
               where  t2.CDAJUCTREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJCTRECEBER". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "AJBOLREC"  */
    if exists (select 1
               from   AJBOLREC t2, deleted t1
               where  t2.CDAJUCTREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJBOLREC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDAJMULTAREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDAJMORAREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDAJACREREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDAJDESCREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "IMPOSTO"  */
    if exists (select 1
               from   IMPOSTO t2, deleted t1
               where  t2.CDAJUCTREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTO". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "AJDUPBOLET"  */
    if exists (select 1
               from   AJDUPBOLET t2, deleted t1
               where  t2.CDAJUCTREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AJDUPBOLET". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "CONVENIO"  */
    if exists (select 1
               from   CONVENIO t2, deleted t1
               where  t2.CDAJUCTRECDE = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVENIO". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDAJUCTRECCR = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDAJCTRECCON = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "GERAL_REC"  */
    if exists (select 1
               from   GERAL_REC t2, deleted t1
               where  t2.CDAJDESCBDES = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERAL_REC". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "TIPORECE"  */
    if exists (select 1
               from   TIPORECE t2, deleted t1
               where  t2.CDAJTXADMREC = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPORECE". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDAJCTRECSOB = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOAJRECEBER" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.CDAJRECFECHA = t1.CDAJUCTREC)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOAJRECEBER".'
          goto error
       end
    
    /*  Delete all children in "RATTPAJR"  */
    delete RATTPAJR
    from   RATTPAJR t2, deleted t1
    where  t2.CDAJUCTREC = t1.CDAJUCTREC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipocomp" for table "TIPOCOMP"  */
create trigger td_tipocomp on TIPOCOMP for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOCOMP" if children still exist in "PARAMCOMIS"  */
    if exists (select 1
               from   PARAMCOMIS t2, deleted t1
               where  t2.CDTIPOCOMP = t1.CDTIPOCOMP)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMCOMIS". Cannot delete parent "TIPOCOMP".'
          goto error
       end
    
    /*  Delete all children in "TCOMPPRO"  */
    delete TCOMPPRO
    from   TCOMPPRO t2, deleted t1
    where  t2.CDTIPOCOMP = t1.CDTIPOCOMP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipoctpagar" for table "TIPOCTPAGAR"  */
create trigger td_tipoctpagar on TIPOCTPAGAR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "CONTAPAGAR"  */
    if exists (select 1
               from   CONTAPAGAR t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTAPAGAR". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "TIPOOPERAC"  */
    if exists (select 1
               from   TIPOOPERAC t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERAC". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "PARAMFISCAL"  */
    if exists (select 1
               from   PARAMFISCAL t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMFISCAL". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "AUTOFORNE"  */
    if exists (select 1
               from   AUTOFORNE t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AUTOFORNE". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "IMPOSTO"  */
    if exists (select 1
               from   IMPOSTO t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "IMPOSTO". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "PAREXPEDICAO"  */
    if exists (select 1
               from   PAREXPEDICAO t2, deleted t1
               where  t2.CDTPCTPAGFRE = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PAREXPEDICAO". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "PEDIDOEXOL"  */
    if exists (select 1
               from   PEDIDOEXOL t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDOEXOL". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "COMPRA"  */
    if exists (select 1
               from   COMPRA t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPRA". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTPAGAR" if children still exist in "ETLCONTAPAG"  */
    if exists (select 1
               from   ETLCONTAPAG t2, deleted t1
               where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAPAG". Cannot delete parent "TIPOCTPAGAR".'
          goto error
       end
    
    /*  Delete all children in "RATTPCTP"  */
    delete RATTPCTP
    from   RATTPCTP t2, deleted t1
    where  t2.CDTPCTPAGAR = t1.CDTPCTPAGAR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipoctreceber" for table "TIPOCTRECEBER"  */
create trigger td_tipoctreceber on TIPOCTRECEBER for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "CONTARECEBER"  */
    if exists (select 1
               from   CONTARECEBER t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTARECEBER". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "TIPOOPERAC"  */
    if exists (select 1
               from   TIPOOPERAC t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERAC". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECMATR = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECTRNF = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECMDID = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECTRNC = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECMENS = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECREMA = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECCANC = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECRENH = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "TIPOCTRECFIL"  */
    if exists (select 1
               from   TIPOCTRECFIL t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOCTRECFIL". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECTRTU = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.CDTCTRECCHA = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "TIPORECE"  */
    if exists (select 1
               from   TIPORECE t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPORECE". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOCTRECEBER" if children still exist in "ETLCONTAREC"  */
    if exists (select 1
               from   ETLCONTAREC t2, deleted t1
               where  t2.CDTPCTRECEB = t1.CDTPCTRECEB)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ETLCONTAREC". Cannot delete parent "TIPOCTRECEBER".'
          goto error
       end
    
    /*  Delete all children in "RATTPCTR"  */
    delete RATTPCTR
    from   RATTPCTR t2, deleted t1
    where  t2.CDTPCTRECEB = t1.CDTPCTRECEB

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipodemo" for table "TIPODEMO"  */
create trigger td_tipodemo on TIPODEMO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ITEMDEMOF"  */
    delete ITEMDEMOF
    from   ITEMDEMOF t2, deleted t1
    where  t2.CDTIPODEMO = t1.CDTIPODEMO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipolanc" for table "TIPOLANC"  */
create trigger td_tipolanc on TIPOLANC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "HISTTIPOLANC"  */
    delete HISTTIPOLANC
    from   HISTTIPOLANC t2, deleted t1
    where  t2.CDTIPOLANC = t1.CDTIPOLANC

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tipooperac" for table "TIPOOPERAC"  */
create trigger td_tipooperac on TIPOOPERAC for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "INCIDENCIA"  */
    if exists (select 1
               from   INCIDENCIA t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INCIDENCIA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CODFISCOPER"  */
    if exists (select 1
               from   CODFISCOPER t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODFISCOPER". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CODSITUTRIB"  */
    if exists (select 1
               from   CODSITUTRIB t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODSITUTRIB". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARCMPSUP"  */
    if exists (select 1
               from   PARCMPSUP t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARCMPSUP". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "RELACTIPOPER"  */
    if exists (select 1
               from   RELACTIPOPER t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELACTIPOPER". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "REPLICDOC"  */
    if exists (select 1
               from   REPLICDOC t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REPLICDOC". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "ITREPLICDOC"  */
    if exists (select 1
               from   ITREPLICDOC t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREPLICDOC". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "ITLANCTOEST"  */
    if exists (select 1
               from   ITLANCTOEST t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITLANCTOEST". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "GERANFROMA"  */
    if exists (select 1
               from   GERANFROMA t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "GERANFROMA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.IDENTSAIOPDV = t1.IDENTRSAIDOP
                and   t2.CDTPOPERPDV = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "TPDOCTRANSAP"  */
    if exists (select 1
               from   TPDOCTRANSAP t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPDOCTRANSAP". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CODTRIBSAP"  */
    if exists (select 1
               from   CODTRIBSAP t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CODTRIBSAP". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMACA"  */
    if exists (select 1
               from   PARAMACA t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMACA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CAIXA"  */
    if exists (select 1
               from   CAIXA t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CAIXA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMINTEGRA"  */
    if exists (select 1
               from   PARAMINTEGRA t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMINTEGRA". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CTBIMPTPOPER"  */
    if exists (select 1
               from   CTBIMPTPOPER t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTBIMPTPOPER". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "CTBPRDTPOPER"  */
    if exists (select 1
               from   CTBPRDTPOPER t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CTBPRDTPOPER". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "VRPRODBCIMPO"  */
    if exists (select 1
               from   VRPRODBCIMPO t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VRPRODBCIMPO". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "COMPFATCGO"  */
    if exists (select 1
               from   COMPFATCGO t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPFATCGO". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.IDESOPEROREV = t1.IDENTRSAIDOP
                and   t2.CDTPOPEROREV = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.IDESOPENTACE = t1.IDENTRSAIDOP
                and   t2.CDTPOPENTACE = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.IDESOPSAIACE = t1.IDENTRSAIDOP
                and   t2.CDTPOPSAIACE = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMORCAEVE"  */
    if exists (select 1
               from   PARAMORCAEVE t2, deleted t1
               where  t2.IDESOPFINACE = t1.IDENTRSAIDOP
                and   t2.CDTPOPFINACE = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMORCAEVE". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "TIPOOPERACROP"  */
    if exists (select 1
               from   TIPOOPERACROP t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOOPERACROP". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAMSERIE"  */
    if exists (select 1
               from   PARAMSERIE t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTIPOOPER = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMSERIE". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOOPERAC" if children still exist in "PARAVEND"  */
    if exists (select 1
               from   PARAVEND t2, deleted t1
               where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
                and   t2.CDTPOPERCNTREC = t1.CDTIPOOPER)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAVEND". Cannot delete parent "TIPOOPERAC".'
          goto error
       end
    
    /*  Delete all children in "EXPORTACAOTG"  */
    delete EXPORTACAOTG
    from   EXPORTACAOTG t2, deleted t1
    where  t2.IDENTRSAIDOP = t1.IDENTRSAIDOP
     and   t2.CDTIPOOPER = t1.CDTIPOOPER

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tiporequisit" for table "TIPOREQUISIT"  */
create trigger td_tiporequisit on TIPOREQUISIT for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TIPOREQUISIT" if children still exist in "REQUISITPROD"  */
    if exists (select 1
               from   REQUISITPROD t2, deleted t1
               where  t2.CDTPREQUISIT = t1.CDTPREQUISIT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REQUISITPROD". Cannot delete parent "TIPOREQUISIT".'
          goto error
       end
    
    /*  Cannot delete parent "TIPOREQUISIT" if children still exist in "VARITEMORCA"  */
    if exists (select 1
               from   VARITEMORCA t2, deleted t1
               where  t2.CDTPREQUISIT = t1.CDTPREQUISIT)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VARITEMORCA". Cannot delete parent "TIPOREQUISIT".'
          goto error
       end
    
    /*  Delete all children in "DESCTIPOREQU"  */
    delete DESCTIPOREQU
    from   DESCTIPOREQU t2, deleted t1
    where  t2.CDTPREQUISIT = t1.CDTPREQUISIT

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_tpcontrole" for table "TPCONTROLE"  */
create trigger td_tpcontrole on TPCONTROLE for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TPCONTROLE" if children still exist in "CONETAPA"  */
    if exists (select 1
               from   CONETAPA t2, deleted t1
               where  t2.CDTPCONTROLE = t1.CDTPCONTROLE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONETAPA". Cannot delete parent "TPCONTROLE".'
          goto error
       end
    
    /*  Delete all children in "EQUITPCONT"  */
    delete EQUITPCONT
    from   EQUITPCONT t2, deleted t1
    where  t2.CDTPCONTROLE = t1.CDTPCONTROLE
    
    /*  Delete all children in "COLETPCONT"  */
    delete COLETPCONT
    from   COLETPCONT t2, deleted t1
    where  t2.CDTPCONTROLE = t1.CDTPCONTROLE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_trabcabelancctbl" for table "TRABCABELANCCTBL"  */
create trigger td_trabcabelancctbl on TRABCABELANCCTBL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "ERROLANCCTBL"  */
    delete ERROLANCCTBL
    from   ERROLANCCTBL t2, deleted t1
    where  t2.CDEMPRLCTBCABE = t1.CDEMPRLCTBCABE
     and   t2.NRLANCCTBLCABE = t1.NRLANCCTBLCABE
    
    /*  Delete all children in "TRABLANCCTBL"  */
    delete TRABLANCCTBL
    from   TRABLANCCTBL t2, deleted t1
    where  t2.CDEMPRLCTBCABE = t1.CDEMPRLCTBCABE
     and   t2.NRLANCCTBLCABE = t1.NRLANCCTBLCABE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_trablancctbl" for table "TRABLANCCTBL"  */
create trigger td_trablancctbl on TRABLANCCTBL for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "TRABLANCCUSTO"  */
    delete TRABLANCCUSTO
    from   TRABLANCCUSTO t2, deleted t1
    where  t2.CDEMPRLCTBCABE = t1.CDEMPRLCTBCABE
     and   t2.NRLANCCTBLCABE = t1.NRLANCCTBLCABE
     and   t2.NRSEQULCTBTEMP = t1.NRSEQULCTBTEMP

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_trablimpeza" for table "TRABLIMPEZA"  */
create trigger td_trablimpeza on TRABLIMPEZA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "TRABPROCLIMPEZA"  */
    delete TRABPROCLIMPEZA
    from   TRABPROCLIMPEZA t2, deleted t1
    where  t2.IDLIMPEZA = t1.IDLIMPEZA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_troca" for table "TROCA"  */
create trigger td_troca on TROCA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TROCA" if children still exist in "LANCTOESTOQ"  */
    if exists (select 1
               from   LANCTOESTOQ t2, deleted t1
               where  t2.CDFILIMOVI = t1.CDFILIAL
                and   t2.NRTROCA = t1.NRTROCA)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LANCTOESTOQ". Cannot delete parent "TROCA".'
          goto error
       end
    
    /*  Delete all children in "DESCTROCA"  */
    delete DESCTROCA
    from   DESCTROCA t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRTROCA = t1.NRTROCA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_turno" for table "TURNO"  */
create trigger td_turno on TURNO for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "TURNO" if children still exist in "TRANSVEIC"  */
    if exists (select 1
               from   TRANSVEIC t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TRANSVEIC". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "TURMAPAD"  */
    if exists (select 1
               from   TURMAPAD t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAPAD". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "TURMAPERIODO"  */
    if exists (select 1
               from   TURMAPERIODO t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURMAPERIODO". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "PROSPECT"  */
    if exists (select 1
               from   PROSPECT t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROSPECT". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "RESTITREFTUR"  */
    if exists (select 1
               from   RESTITREFTUR t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESTITREFTUR". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "AULATURMA"  */
    if exists (select 1
               from   AULATURMA t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "AULATURMA". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "ITREGRACONS"  */
    if exists (select 1
               from   ITREGRACONS t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITREGRACONS". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "TURNOFILI"  */
    if exists (select 1
               from   TURNOFILI t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TURNOFILI". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "MAPABASEV"  */
    if exists (select 1
               from   MAPABASEV t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MAPABASEV". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "ITMAOOBRAORCA"  */
    if exists (select 1
               from   ITMAOOBRAORCA t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITMAOOBRAORCA". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Cannot delete parent "TURNO" if children still exist in "TIPOSERVTURNO"  */
    if exists (select 1
               from   TIPOSERVTURNO t2, deleted t1
               where  t2.NRTURNO = t1.NRTURNO)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TIPOSERVTURNO". Cannot delete parent "TURNO".'
          goto error
       end
    
    /*  Delete all children in "TURNOPRODUTO"  */
    delete TURNOPRODUTO
    from   TURNOPRODUTO t2, deleted t1
    where  t2.NRTURNO = t1.NRTURNO

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_unidmedida" for table "UNIDMEDIDA"  */
create trigger td_unidmedida on UNIDMEDIDA for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PRODUTO"  */
    if exists (select 1
               from   PRODUTO t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODUTO". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "CONVUNID"  */
    if exists (select 1
               from   CONVUNID t2, deleted t1
               where  t2.SGUNIDPARA = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVUNID". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "CONVPRODUTO"  */
    if exists (select 1
               from   CONVPRODUTO t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONVPRODUTO". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PRATO"  */
    if exists (select 1
               from   PRATO t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRATO". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "RECEFILI"  */
    if exists (select 1
               from   RECEFILI t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEFILI". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "RECESERV"  */
    if exists (select 1
               from   RECESERV t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECESERV". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "NUTRIENTE"  */
    if exists (select 1
               from   NUTRIENTE t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NUTRIENTE". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "ITATENREAL"  */
    if exists (select 1
               from   ITATENREAL t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITATENREAL". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PRODFORN"  */
    if exists (select 1
               from   PRODFORN t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRODFORN". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PREVCOMPRA"  */
    if exists (select 1
               from   PREVCOMPRA t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PREVCOMPRA". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "RECEPADR"  */
    if exists (select 1
               from   RECEPADR t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEPADR". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "FORNECEDOR"  */
    if exists (select 1
               from   FORNECEDOR t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FORNECEDOR". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "RECEITA2"  */
    if exists (select 1
               from   RECEITA2 t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RECEITA2". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "INSPECAO"  */
    if exists (select 1
               from   INSPECAO t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INSPECAO". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PRINATIVMEDI"  */
    if exists (select 1
               from   PRINATIVMEDI t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PRINATIVMEDI". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "TPCONTROLE"  */
    if exists (select 1
               from   TPCONTROLE t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "TPCONTROLE". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "COMPPROD"  */
    if exists (select 1
               from   COMPPROD t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "COMPPROD". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "RESPOSINSPEC"  */
    if exists (select 1
               from   RESPOSINSPEC t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RESPOSINSPEC". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "ITEMINSAUDIT"  */
    if exists (select 1
               from   ITEMINSAUDIT t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMINSAUDIT". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "FICHAFPR"  */
    if exists (select 1
               from   FICHAFPR t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "FICHAFPR". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "ITFICHAFPR"  */
    if exists (select 1
               from   ITFICHAFPR t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITFICHAFPR". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PERDAOP"  */
    if exists (select 1
               from   PERDAOP t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PERDAOP". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "ITEMORCAPROD"  */
    if exists (select 1
               from   ITEMORCAPROD t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ITEMORCAPROD". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "METAVENDA"  */
    if exists (select 1
               from   METAVENDA t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "METAVENDA". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "EQUIPAPROD"  */
    if exists (select 1
               from   EQUIPAPROD t2, deleted t1
               where  t2.SGUNIDCONSEP = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "EQUIPAPROD". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "PARAMETRO"  */
    if exists (select 1
               from   PARAMETRO t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMETRO". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "BEM"  */
    if exists (select 1
               from   BEM t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "BEM". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "UNIDMEDIFILI"  */
    if exists (select 1
               from   UNIDMEDIFILI t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "UNIDMEDIFILI". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Cannot delete parent "UNIDMEDIDA" if children still exist in "INDICADOR"  */
    if exists (select 1
               from   INDICADOR t2, deleted t1
               where  t2.SGUNIDADE = t1.SGUNIDADE)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "INDICADOR". Cannot delete parent "UNIDMEDIDA".'
          goto error
       end
    
    /*  Delete all children in "CONVUNID"  */
    delete CONVUNID
    from   CONVUNID t2, deleted t1
    where  t2.SGUNIDDE = t1.SGUNIDADE

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_vendedor" for table "VENDEDOR"  */
create trigger td_vendedor on VENDEDOR for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "CLIENTE"  */
    if exists (select 1
               from   CLIENTE t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CLIENTE". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "CONTATO"  */
    if exists (select 1
               from   CONTATO t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONTATO". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "VENDA"  */
    if exists (select 1
               from   VENDA t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDA". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "PARAMPEDIDO"  */
    if exists (select 1
               from   PARAMPEDIDO t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PARAMPEDIDO". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "ACAO"  */
    if exists (select 1
               from   ACAO t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ACAO". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "VENDITEMPED"  */
    if exists (select 1
               from   VENDITEMPED t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDITEMPED". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "PROPOSTA"  */
    if exists (select 1
               from   PROPOSTA t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PROPOSTA". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "NOTAFISCAL"  */
    if exists (select 1
               from   NOTAFISCAL t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "NOTAFISCAL". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "PEDIDO"  */
    if exists (select 1
               from   PEDIDO t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "PEDIDO". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "MENSVEND"  */
    if exists (select 1
               from   MENSVEND t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MENSVEND". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "HISITEMPED"  */
    if exists (select 1
               from   HISITEMPED t2, deleted t1
               where  t2.CDVENDORIG = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISITEMPED". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "METAVENDEDOR"  */
    if exists (select 1
               from   METAVENDEDOR t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "METAVENDEDOR". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "HISITEMPED"  */
    if exists (select 1
               from   HISITEMPED t2, deleted t1
               where  t2.CDVENDATUAL = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "HISITEMPED". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "REGIAOPEDVEN"  */
    if exists (select 1
               from   REGIAOPEDVEN t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "REGIAOPEDVEN". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "MESAVENDE"  */
    if exists (select 1
               from   MESAVENDE t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "MESAVENDE". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "VENDAREST"  */
    if exists (select 1
               from   VENDAREST t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "VENDAREST". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "LOJA"  */
    if exists (select 1
               from   LOJA t2, deleted t1
               where  t2.CDVENDPADRAO = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOJA". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "CONSUMIDOR"  */
    if exists (select 1
               from   CONSUMIDOR t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "CONSUMIDOR". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Cannot delete parent "VENDEDOR" if children still exist in "ORCAEVENTO"  */
    if exists (select 1
               from   ORCAEVENTO t2, deleted t1
               where  t2.CDVENDEDOR = t1.CDVENDEDOR)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "ORCAEVENTO". Cannot delete parent "VENDEDOR".'
          goto error
       end
    
    /*  Delete all children in "ENDEVENDE"  */
    delete ENDEVENDE
    from   ENDEVENDE t2, deleted t1
    where  t2.CDVENDEDOR = t1.CDVENDEDOR

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_versgsis" for table "VERSGSIS"  */
create trigger td_versgsis on VERSGSIS for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Cannot delete parent "VERSGSIS" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDGRUPSIST = t1.CDGRUPSIST
                and   t2.CDVERSLIBE = t1.CDVERSGSIS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "VERSGSIS".'
          goto error
       end
    
    /*  Cannot delete parent "VERSGSIS" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDGRUPSIST = t1.CDGRUPSIST
                and   t2.CDVERSPLAN = t1.CDVERSGSIS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "VERSGSIS".'
          goto error
       end
    
    /*  Cannot delete parent "VERSGSIS" if children still exist in "SOLISERV"  */
    if exists (select 1
               from   SOLISERV t2, deleted t1
               where  t2.CDGRUPSIST = t1.CDGRUPSIST
                and   t2.CDVERSORIG = t1.CDVERSGSIS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "SOLISERV". Cannot delete parent "VERSGSIS".'
          goto error
       end
    
    /*  Cannot delete parent "VERSGSIS" if children still exist in "LOGDATACOMP"  */
    if exists (select 1
               from   LOGDATACOMP t2, deleted t1
               where  t2.CDGRUPSIST = t1.CDGRUPSIST
                and   t2.CDVERSGSIS = t1.CDVERSGSIS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "LOGDATACOMP". Cannot delete parent "VERSGSIS".'
          goto error
       end
    
    /*  Cannot delete parent "VERSGSIS" if children still exist in "RELEASEVERS"  */
    if exists (select 1
               from   RELEASEVERS t2, deleted t1
               where  t2.CDGRUPSIST = t1.CDGRUPSIST
                and   t2.CDVERSGSIS = t1.CDVERSGSIS)
       begin
          select @errno  = 30006,
                 @errmsg = 'Children still exist in "RELEASEVERS". Cannot delete parent "VERSGSIS".'
          goto error
       end
    
    /*  Delete all children in "SISTLIBEVERS"  */
    delete SISTLIBEVERS
    from   SISTLIBEVERS t2, deleted t1
    where  t2.CDGRUPSIST = t1.CDGRUPSIST
     and   t2.CDVERSGSIS = t1.CDVERSGSIS

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

/*  Delete trigger "td_visteqforn" for table "VISTEQFORN"  */
create trigger td_visteqforn on VISTEQFORN for delete as
begin
    declare
       @numrows  int,
       @errno    int,
       @errmsg   varchar(255)

    select  @numrows = @@rowcount
    if @numrows = 0
       return

    
    /*  Delete all children in "PERGRESPVIS"  */
    delete PERGRESPVIS
    from   PERGRESPVIS t2, deleted t1
    where  t2.CDFILIAL = t1.CDFILIAL
     and   t2.NRVISITA = t1.NRVISITA

    return

/*  Errors handling  */
error:
    raiserror @errno @errmsg
    rollback  transaction
end
go

